/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.commands;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import cn.edu.whu.lmars.utils.HandleTool;

/**
 * This is an analog command stack which supports last-in-first-out.The usual
 * push and pop operations are provided.
 * 
 * @author Mingda Zhang
 * 
 */

public class CommandStack {
	/** Command list. */
	private List<GeoCommand> commands = new ArrayList<GeoCommand>();
	/** The size of the stack. */
	private int num = 0;
	/**
	 * The max size of the stack.
	 */
	private int max = 15;
	/** The index of current command. */
	private int present = 0;
	/** Logger */
	private Logger logger = Logger.getLogger(CommandStack.class);

	/**
	 * Pushes a command to the stack. When the maximum size is exceeded, will
	 * delete the first to join.
	 * 
	 * @param command
	 *            Command to be added.
	 */
	public void addCommand(GeoCommand command) {
		if (num > max) {
			commands.remove(0);
			num--;
		}
		commands.add(command);
		logger.info("Add a command to the commandstack");
		num++;
		present = num;
	}

	/**
	 * Pops one command to execute redo operation.
	 */
	public void redo() {
		if (num == 0) {
			logger.warn("command stack is null,unable to redo");
			return;
		}
		if (present >= num) {
			logger.warn("This is the last command, can't redo");
			return;
		}
		GeoCommand command = commands.get(present);
		command.redo();
		HandleTool.refreshFigShow();
		present++;
	}

	/**
	 * Pops one command to execute undo operation.
	 */
	public void undo() {
		if (num == 0) {
			logger.warn("Command stack is null,unable to undo");
			return;
		}
		if (present < 1) {
			logger.warn("This is the last command, unable to undo");
			return;
		}

		GeoCommand command = commands.get(present - 1);
		command.undo();
		present--;
		HandleTool.refreshFigShow();
	}
}
