/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.commands;
import org.apache.log4j.Logger;

import cn.edu.whu.lmars.figure.beans.DataFigure;
import cn.edu.whu.lmars.figure.beans.FlowFigure;
import cn.edu.whu.lmars.figure.beans.LinkFigure;
import cn.edu.whu.lmars.figure.beans.ProcessFigure;
import cn.edu.whu.lmars.figure.utils.FigureStore;
import cn.edu.whu.lmars.model.AbstractParam;
import cn.edu.whu.lmars.model.AtomProcess;
import cn.edu.whu.lmars.model.ChainFlow;
import cn.edu.whu.lmars.model.util.PureModelStore;

/**
 * Controls flow model and flow figure in consistent.
 * 
 * @author Mingda Zhang
 *
 */
public class FlowCommand extends GeoCommand {

	private PureModelStore modelStore;
	private FigureStore figureStore;
	private DataFigure dataFigure;
	private ProcessFigure processFigure;
	private String paramName;
	private DataFigure stateFigure;
	private String fromUuid,toUuid;
	private ChainFlow chainFlow;
	private FlowFigure flowFigure;
	private LinkFigure linkFigure;
	private String oldValue,newValue;
	private AbstractParam param;
	private Logger logger = Logger.getLogger(FlowCommand.class);
	public FlowCommand(PureModelStore modelStore,FigureStore figureStore,DataFigure dataFigure,
			ProcessFigure handleFigure,String identifier){
		this.modelStore = modelStore;
		this.figureStore = figureStore;
		this.dataFigure = dataFigure;
		this.paramName = identifier;
		this.processFigure = handleFigure;
		this.fromUuid = dataFigure.getUuid().split("#")[0];
	
		this.toUuid = handleFigure.getUuid();
		this.processFigure = handleFigure;
		AtomProcess atomProcess = modelStore.getAtomProcessByUuid(toUuid);
		param = atomProcess.getParamByName(identifier);
		newValue = dataFigure.getUuid();
		oldValue = param.getValue();
	}
	
	/**
	 * Creates flow model and flow figure and adds them to container.
	 */
	@Override
	public boolean execute() {
		
		stateFigure = processFigure.getDataFigureByUuid(toUuid+"#"+paramName);
		stateFigure.setState("ref");
		 flowFigure = new FlowFigure(dataFigure.pointR,processFigure.pointL,dataFigure,processFigure);
		 flowFigure.setUuid(fromUuid+"#"+toUuid+"#"+paramName);
		figureStore.addFlowFigure(flowFigure);
		 chainFlow = new ChainFlow();
		chainFlow.setUuid(fromUuid+"#"+toUuid+"#"+paramName);
		param.setValue(newValue);
		modelStore.addChainFlow(chainFlow);
		return true;
	}

	/**
	 * Adds the flow model and flow figure to container.
	 */
	@Override
	public void redo() {
		stateFigure.setState("ref");
		if(linkFigure != null){
			processFigure.removeLinkFigure(linkFigure);
		}

		figureStore.addFlowFigure(flowFigure);
		modelStore.addChainFlow(chainFlow);
		param.setValue(newValue);
		logger.info("redo creating flow operation");
	}

	/**
	 * Removes the flow model and flow figure from container.
	 */
	@Override
	public void undo() {
		if (processFigure.isNul()) {
			stateFigure.setState("nul");
			if (linkFigure != null) {
				processFigure.removeLinkFigure(linkFigure);
			}
		}else {
			stateFigure.setState("val");
			if (linkFigure == null) {
				linkFigure = new LinkFigure(stateFigure.pointR, processFigure.pointL, stateFigure,processFigure);
				linkFigure.setUuid(stateFigure.getUuid());
			}
			processFigure.addLinkFigure(linkFigure);
		}
		figureStore.removeFlowFigure(flowFigure);
		modelStore.removeChainFlow(chainFlow);
		param.setValue(oldValue);
		logger.info("undo creating flow operation");
	}
}


