/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.commands;

import java.util.ArrayList;
import java.util.List;

import cn.edu.whu.lmars.figure.beans.FlowFigure;
import cn.edu.whu.lmars.figure.beans.ProcessFigure;
import cn.edu.whu.lmars.figure.utils.FigureStore;
import cn.edu.whu.lmars.model.AtomProcess;
import cn.edu.whu.lmars.model.ChainFlow;
import cn.edu.whu.lmars.model.util.PureModelStore;

/**
 * This command is used to clear the figures and models.
 * 
 * @author Mingda Zhang
 * 
 */
public class GeoDeleteAllCommand extends GeoCommand {

	private FigureStore figureStore;
	private PureModelStore modelStore;
	private List<AtomProcess> atomProcesses, newAtomProcesses;
	private List<ChainFlow> chainFlows, newChainFlows;
	private List<ProcessFigure> processFigures, newProcessFigures;
	private List<FlowFigure> flowFigures, newFlowFigures;

	/**
	 * Instantiates a GeoDeleteAllCommand. 
	 * 
	 * @param figureStore
	 *            figure container.
	 * @param modelStore
	 *            model container.
	 */
	public GeoDeleteAllCommand(FigureStore figureStore,
			PureModelStore modelStore) {
		this.figureStore = figureStore;
		this.modelStore = modelStore;
		this.atomProcesses = modelStore.getAtomProcesses();
		this.chainFlows = modelStore.getChainFlows();
		this.flowFigures = figureStore.getFlowFigures();
		this.processFigures = figureStore.getProcessFigures();
		newAtomProcesses = new ArrayList<AtomProcess>();
		newChainFlows = new ArrayList<ChainFlow>();
		newProcessFigures = new ArrayList<ProcessFigure>();
		newFlowFigures = new ArrayList<FlowFigure>();
	}

	/**
	 * Clears the figure container and model container.
	 */
	@Override
	public boolean execute() {
		figureStore.setFlowFigures(newFlowFigures);
		figureStore.setProcessFigures(newProcessFigures);
		modelStore.setChainFlows(newChainFlows);
		modelStore.setAtomProcesses(newAtomProcesses);
		return true;
	}

	/**
	 * Redo the GeoDeleteAllCommand.
	 */
	@Override
	public void redo() {
		execute();
	}

	/**
	 * Undo the GeoDeleteAllCommand.
	 */
	@Override
	public void undo() {
		figureStore.setProcessFigures(processFigures);
		figureStore.setFlowFigures(flowFigures);
		modelStore.setAtomProcesses(atomProcesses);
		modelStore.setChainFlows(chainFlows);
	}

}
