/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.commands;

import org.apache.log4j.Logger;

import cn.edu.whu.lmars.figure.beans.DataFigure;
import cn.edu.whu.lmars.figure.beans.FlowFigure;
import cn.edu.whu.lmars.figure.beans.LinkFigure;
import cn.edu.whu.lmars.figure.beans.ProcessFigure;
import cn.edu.whu.lmars.figure.utils.FigureStore;
import cn.edu.whu.lmars.model.AbstractParam;
import cn.edu.whu.lmars.model.ChainFlow;
import cn.edu.whu.lmars.model.util.PureModelStore;

/**
 * This command is used to delete flow figure and flow model in consistent.
 * 
 * @author Mingda Zhang
 * 
 */
public class GeoDeleteFlowCommand extends GeoCommand {
	private Logger logger = Logger.getLogger(GeoDeleteCommand.class);
	private FigureStore figureStore;
	private PureModelStore modelStore;
	private DataFigure dataFigure;
	private FlowFigure flowFigure;
	private LinkFigure linkFigure;
	private ChainFlow chainFlow;
	private ProcessFigure processFigure;
	private String paramValue;
	private AbstractParam parameter;
	private String dataUuid, startUuid;

	/**
	 * Instantiates a GeoDeleteFlowCommand.
	 * 
	 * @param modelStore
	 *            model container.
	 * @param figureStore
	 *            figure container.
	 * @param flowFigure
	 *            the flow figure to be deleted
	 */
	public GeoDeleteFlowCommand(PureModelStore modelStore,
			FigureStore figureStore, FlowFigure flowFigure) {
		this.modelStore = modelStore;
		this.figureStore = figureStore;
		this.flowFigure = flowFigure;
	}

	/**
	 * Removes flow figure and flow model from container.
	 */
	@Override
	public boolean execute() {
		if (processFigure == null || dataFigure == null) {
			String id = flowFigure.getUuid();
			String[] uuids = id.split("#");
			processFigure = figureStore.getProcessFigureByUuid(uuids[1]);
			dataUuid = uuids[1] + "#" + uuids[2];
			dataFigure = processFigure.getDataFigureByUuid(dataUuid);
			if (parameter == null) {
				parameter = (AbstractParam)dataFigure.getModel();
				paramValue = parameter.getValue();
			}
			
			startUuid = uuids[0];
		}
		parameter.setValue("");
		if (processFigure.isNul()) {
			dataFigure.setState("nul");
		} else {
			if (linkFigure == null) {
				linkFigure = dataFigure.getLinkFigure();
				if (linkFigure == null) {
					linkFigure = new LinkFigure(dataFigure.pointR,
							processFigure.pointL, dataFigure, processFigure);
					linkFigure.setUuid(dataUuid);
				}
			}
			dataFigure.setState("val");
			processFigure.addLinkFigure(linkFigure);
		}

		figureStore.removeFlowFigure(flowFigure);
		if (chainFlow == null) {
			chainFlow = modelStore.getChainFlowByStartUuid(startUuid);
		}
		modelStore.removeChainFlow(chainFlow);
		logger.info("Execute GeoDeleteFlowCommand");
		return true;
	}

	/**
	 * Redo GeoDeleteFlowCommand
	 */
	@Override
	public void redo() {
		execute();

	}

	/**
	 * Undo GeoDeleteFlowCommand
	 */
	@Override
	public void undo() {
		processFigure.removeLinkFigure(linkFigure);
		dataFigure.setState("ref");
		figureStore.addFlowFigure(flowFigure);
		modelStore.addChainFlow(chainFlow);
		parameter.setValue(paramValue);
		logger.info("undo GeoDeleteFlowCommand");
	}
}
