/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.commands;

import java.util.ArrayList;
import java.util.List;

import cn.edu.whu.lmars.figure.beans.FlowFigure;
import cn.edu.whu.lmars.figure.beans.ProcessFigure;
import cn.edu.whu.lmars.figure.utils.FigureStore;
import cn.edu.whu.lmars.model.AtomProcess;
import cn.edu.whu.lmars.model.util.PureModelStore;

/**
 * This command is used to delete a process figure and model in consistent.
 * 
 * @author Mingda Zhang
 * 
 */
public class GeoDeleteProcessCommand extends GeoCommand {
	private PureModelStore modelStore;
	private FigureStore figureStore;
	private List<GeoDeleteFlowCommand> deleteFlowCommands = new ArrayList<GeoDeleteFlowCommand>(
			4);
	private ProcessFigure processFigure;
	private String uuid;
	private AtomProcess atomProcess = null;

	/**
	 * Instantiates a GeoDeleteProcessCommand.
	 * 
	 * @param modelStore
	 *            model container.
	 * @param figureStore
	 *            figure container.
	 * @param processFigure
	 *            the process to be deleted.
	 */
	public GeoDeleteProcessCommand(PureModelStore modelStore,
			FigureStore figureStore, ProcessFigure processFigure) {
		this.modelStore = modelStore;
		this.figureStore = figureStore;
		this.processFigure = processFigure;
		this.uuid = processFigure.getUuid();
	}

	/**
	 * Removes process model and figure from container.
	 */
	@Override
	public boolean execute() {
		FlowFigure outFlowFigure = figureStore.getFlowFigureByBeginUuid(uuid);
		if (outFlowFigure != null) {
			GeoDeleteFlowCommand command = new GeoDeleteFlowCommand(modelStore,
					figureStore, outFlowFigure);
			command.execute();
			deleteFlowCommands.add(command);
		}
		List<FlowFigure> inFlowFigures = figureStore
				.getFlowFiguresByContainsStr("#" + uuid + "#");
		for (FlowFigure inFlowFigure : inFlowFigures) {
			GeoDeleteFlowCommand command = new GeoDeleteFlowCommand(modelStore,
					figureStore, inFlowFigure);
			command.execute();
			deleteFlowCommands.add(command);
		}
		atomProcess = modelStore.getAtomProcessByUuid(uuid);
		modelStore.removeAtomProcess(atomProcess);
		figureStore.removeProcessFigure(processFigure);
		return true;
	}

	/**
	 * Redo GeoDeleteProcessCommand.
	 */
	@Override
	public void redo() {
		figureStore.removeProcessFigure(processFigure);
		modelStore.removeAtomProcess(atomProcess);
		for (GeoDeleteFlowCommand command : deleteFlowCommands) {
			command.redo();
		}
	}

	/**
	 * Undo GeoDeleteProcessCommand.
	 */
	@Override
	public void undo() {
		figureStore.addProcessFigure(processFigure);
		modelStore.addAtomProcess(atomProcess);
		int num = deleteFlowCommands.size();
		for (int i = num; i > 0; i--) {
			deleteFlowCommands.get(i - 1).undo();
		}
	}

}
