/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.commands;

import cn.edu.whu.lmars.drawpanel.ui.JDrawPanel;
import cn.edu.whu.lmars.figure.beans.AbstractFigure;
import cn.edu.whu.lmars.figure.beans.GeoNodeFigure;
import cn.edu.whu.lmars.figure.beans.GeoPointFigure;
import cn.edu.whu.lmars.figure.utils.FigureStore;
import cn.edu.whu.lmars.figure.utils.FigureUtility;
import conf.GlobeParam;

/**
 * This command is used to zoom out figures.
 * 
 * @author Mingda Zhang
 * 
 */
public class GeoZoomOutCommand extends GeoCommand {

	FigureUtility figureUtility = FigureUtility.getInstance();
	private JDrawPanel drawPanel;
	private double zoomSize;
	private int width,height;
	private double newWidth,newHeight;

	/**
	 * Instantiates a GeoZoomOutCommand.
	 * 
	 * @param drawPanel
	 *            target JDrawPanel
	 */
	public GeoZoomOutCommand(JDrawPanel drawPanel) {
		this.drawPanel = drawPanel;
		this.width = drawPanel.getWidth();
		this.height = drawPanel.getHeight();
		this.zoomSize = GlobeParam.ZOOM_SIZE+1;
		this.newHeight = this.height/this.zoomSize;
		this.newWidth = this.width/this.zoomSize;
	}

	/**
	 * Executes GeoZoomOutCommand.
	 */
	@Override
	public boolean execute() {
		FigureStore figureStore = drawPanel.getFigureStore();
		GeoNodeFigure nodeFigure ;
		double scaleX,scaleY;
		double newCenterX,newCenterY,figureX,figureY,figureWidth,figureHeight;
		for(AbstractFigure figure:figureStore.getAllFigures()){
			if (figure instanceof GeoNodeFigure) {
				nodeFigure = (GeoNodeFigure)figure;
				GeoPointFigure centerPoint = figureUtility.getFigureCenter(nodeFigure);
				scaleX = centerPoint.getX()*1.0/width;
				scaleY = centerPoint.getY()*1.0/height;
				newCenterX = this.newWidth *scaleX;
				newCenterY = this.newHeight*scaleY;
				scaleX = nodeFigure.getWidth()*1.0/width;
				scaleY = nodeFigure.getHeight()*1.0/height;
				figureWidth = newWidth*scaleX;
				figureHeight =newHeight*scaleY;
				figureX = newCenterX-figureWidth/2;
				figureY = newCenterY-figureHeight/2;
				nodeFigure.resize((int)(figureX+0.5), (int)(figureY+0.5), (int)(figureWidth+0.5), (int)(figureHeight+0.5));
			}
		}
		figureUtility.adjustFigures(figureStore.getAllFigures());
		drawPanel.repaint();
		return true;
	}

	/**
	 * Redo GeoZoomOutCommand.
	 */
	@Override
	public void redo() {
		execute();
	}

	/**
	 * Undo GeoZoomOutCommand.
	 */
	@Override
	public void undo() {

		FigureStore figureStore = drawPanel.getFigureStore();
		GeoNodeFigure nodeFigure ;
		double scaleX,scaleY;
		double newCenterX,newCenterY,figureX,figureY,figureWidth,figureHeight;
		for(AbstractFigure figure:figureStore.getAllFigures()){
			if (figure instanceof GeoNodeFigure) {
				nodeFigure = (GeoNodeFigure)figure;
				GeoPointFigure centerPoint = figureUtility.getFigureCenter(nodeFigure);
				scaleX = centerPoint.getX()*1.0/newWidth;
				scaleY = centerPoint.getY()*1.0/newHeight;
				newCenterX = this.width *scaleX;
				newCenterY = this.height*scaleY;
				
				scaleX = nodeFigure.getWidth()*1.0/newWidth;
				scaleY = nodeFigure.getHeight()*1.0/newHeight;
				figureWidth = this.width*scaleX;
				figureHeight =this.height*scaleY;
				
				figureX = newCenterX-figureWidth/2;
				figureY = newCenterY-figureHeight/2;
				nodeFigure.resize((int)(figureX+0.5), (int)(figureY+0.5), (int)(figureWidth+0.5), (int)(figureHeight+0.5));
			}
		}
		figureUtility.adjustFigures(figureStore.getAllFigures());
		drawPanel.repaint();
	}

}
