/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.commands;

import org.apache.log4j.Logger;

import cn.edu.whu.lmars.figure.beans.DataFigure;
import cn.edu.whu.lmars.figure.beans.FlowFigure;
import cn.edu.whu.lmars.figure.beans.LinkFigure;
import cn.edu.whu.lmars.figure.beans.ProcessFigure;
import cn.edu.whu.lmars.figure.utils.FigureStore;
import cn.edu.whu.lmars.model.AbstractParam;
import cn.edu.whu.lmars.model.AtomProcess;
import cn.edu.whu.lmars.model.ChainFlow;
import cn.edu.whu.lmars.model.util.PureModelStore;

/**
 * This command is used to merge result figure and data figure.
 * 
 * @author Mingda Zhang
 * 
 */
public class MergeCommand extends GeoCommand {

	private Logger logger = Logger.getLogger(GeoDeleteCommand.class);
	private FigureStore figureStore;
	private PureModelStore modelStore;
	private DataFigure resultFigure, dataFigure;
	private FlowFigure flowFigure;
	private LinkFigure linkFigure;
	private ChainFlow chainFlow;
	private ProcessFigure processFigure;
	private String state;
	private AbstractParam param;
	private String oldValue, newValue;

	/**
	 * Instantiates a MergeCommand.
	 * 
	 * @param modelStore
	 *            target ModelStore
	 * @param figureStore
	 *            target FigureStore
	 * @param resultFigure
	 *            target result figure
	 * @param dataFigure
	 *            target data figure
	 */
	public MergeCommand(PureModelStore modelStore, FigureStore figureStore,
			DataFigure resultFigure, DataFigure dataFigure) {
		this.modelStore = modelStore;
		this.figureStore = figureStore;
		this.resultFigure = resultFigure;
		this.dataFigure = dataFigure;
	}

	/**
	 * Executes MergeCommand.
	 */
	@Override
	public boolean execute() {
		String fromID = resultFigure.getUuid();
		String toID = dataFigure.getUuid();
		String fromUuid = fromID.split("#")[0];
		String toUuid = toID.split("#")[0];
		String paramName = toID.split("#")[1];

		AtomProcess atomProcess = modelStore.getAtomProcessByUuid(toUuid);
		param = atomProcess.getParamByName(paramName);
		newValue = fromID;
		oldValue = param.getValue();

		processFigure = figureStore.getProcessFigureByUuid(toUuid);
		flowFigure = new FlowFigure(resultFigure.pointR, processFigure.pointL,
				resultFigure, processFigure);
		flowFigure.setUuid(fromUuid + "#" + toID);
		figureStore.addFlowFigure(flowFigure);
		state = dataFigure.getState();
		dataFigure.setState("ref");
		linkFigure = processFigure.getLinkFigureByContainsUuid(toID);
		processFigure.removeLinkFigure(linkFigure);
		param.setValue(newValue);
		chainFlow = new ChainFlow();
		chainFlow.setUuid(fromUuid + "#" + toID);
		modelStore.addChainFlow(chainFlow);
		return true;
	}

	/**
	 * Redo MergeCommand.
	 */
	@Override
	public void redo() {
		figureStore.addFlowFigure(flowFigure);
		dataFigure.setState("ref");
		processFigure.removeLinkFigure(linkFigure);
		modelStore.addChainFlow(chainFlow);

		param.setValue(newValue);
		logger.info("redo MergeCommand");
	}

	/**
	 * Undo MergeCommand.
	 */
	@Override
	public void undo() {
		processFigure.addLinkFigure(linkFigure);
		dataFigure.setState(state);
		figureStore.removeFlowFigure(flowFigure);
		modelStore.removeChainFlow(chainFlow);

		param.setValue(oldValue);
		logger.info("undo MergeCommand");
	}

}
