/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.commands;

import java.util.ArrayList;

import org.apache.log4j.Logger;

import cn.edu.whu.lmars.drawpanel.ui.JDrawPanel;
import cn.edu.whu.lmars.figure.beans.AbstractFigure;
import cn.edu.whu.lmars.figure.beans.GeoPointFigure;
import cn.edu.whu.lmars.figure.utils.FigureUtility;

/**
 * This command is used to move figures
 * 
 * @author Mingda Zhang
 * 
 */

public class MoveCommand2 extends GeoCommand {
	private FigureUtility figureUtility = null;
	private ArrayList<AbstractFigure> figures;
	private GeoPointFigure startPoint, endPoint;
	private Logger logger = Logger.getLogger(MoveCommand2.class);
	private JDrawPanel drawPanel;

	/**
	 * Instantiates a MoveCommand2
	 * 
	 * @param figures
	 *            figures to be moved
	 * @param startPoint
	 *            start point
	 * @param endPoint
	 *            end point
	 * @param drawPanel
	 *            target JDrawPanel
	 */
	public MoveCommand2(ArrayList<AbstractFigure> figures,
			GeoPointFigure startPoint, GeoPointFigure endPoint,
			JDrawPanel drawPanel) {

		this.figures = figures;
		this.startPoint = startPoint;
		this.endPoint = endPoint;
		figureUtility = FigureUtility.getInstance();
		this.drawPanel = drawPanel;
		logger.info("Create a MoveCommand2");
	}

	/**
	 * Executes MoveCommand2.
	 */
	@Override
	public boolean execute() {
		return super.execute();
	}

	/**
	 * Redo MoveCommand2
	 */
	@Override
	public void redo() {
		figureUtility.moveFigure2(figures, startPoint, endPoint, drawPanel);
		logger.info("redo MoveCommand2");
	}

	/**
	 * Undo MoveCommand2.
	 */
	@Override
	public void undo() {
		figureUtility.moveFigure2(figures, endPoint, startPoint, drawPanel);
		logger.info("undo MoveCommand2");
	}

}
