/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.commands;

import java.util.HashMap;
import java.util.Map;

import cn.edu.whu.lmars.model.AbstractParam;
import cn.edu.whu.lmars.model.AtomProcess;
import cn.edu.whu.lmars.model.ComplexParam;

/**
 * This command is used to assign the parameters of a process.
 * 
 * @author Mingda Zhang
 *
 */
public class MultipleInputCommand extends GeoCommand {
	Map<String, String> newValueMap = new HashMap<String, String>();
	Map<String, String> oldValueMap = new HashMap<String, String>();
	AtomProcess atomProcess = null;
	String postFix = "_format";
	/**
	 * Instantiates a MultipleInputCommand
	 * @param atomProcess
	 * 			target AtomProcess
	 * @param newValueMap
	 * 			the new value map
	 * @param oldValueMap
	 * 			the old value map
	 */
	public MultipleInputCommand(AtomProcess atomProcess,Map< String, String> newValueMap,Map<String, String> oldValueMap){
		this.atomProcess = atomProcess;
		this.oldValueMap = oldValueMap;
		this.newValueMap = newValueMap;
	}
	
	/**
	 * Executes MultipleInputCommand.
	 */
	@Override
	public boolean  execute() {
		assigValue(oldValueMap, newValueMap);
		return true;
	}

	/**
	 * Redo MultipleInputCommand.
	 */
	@Override
	public void redo() {
		execute();
	}

	/**
	 * Undo MultipleInputCommand.
	 */
	@Override
	public void undo() {
		assigValue(newValueMap, oldValueMap);
	}

	private void assigValue(Map<String, String> oldMap,Map<String, String> newMap){
		String oldValue,newValue;
		for (String key:oldMap.keySet()) {
			oldValue = oldMap.get(key);
			newValue = newMap.get(key);
			if (newValue.equals(oldValue)) {
				continue;
			}
			if (key.equals("Outputformat")) {
				this.atomProcess.getOutPutParam().setFormat(newValue);
				continue;
			}
			ComplexParam complexParam;
			if (key.endsWith(postFix)) {
				String[] names = key.split("_");
				complexParam = atomProcess.getComplexParamByName(names[0]);
				complexParam.setFormat(newValue);
				continue;
			}
			
			AbstractParam param = atomProcess.getParamByName(key);
			param.setValue(newValue);
		}
	}
}
