/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.commands;

import conf.GlobeParam;
import cn.edu.whu.lmars.drawpanel.ui.JDrawPanel;
import cn.edu.whu.lmars.figure.beans.GeoNodeFigure;
import cn.edu.whu.lmars.figure.beans.geoenum.MarkDirection;
import cn.edu.whu.lmars.figure.utils.FigureUtility;

/**
 * This command is used to resize figures.
 * 
 * @author Mingda Zhang
 *
 */


public class ReSizeCommand extends GeoCommand {

	private FigureUtility figureUtility = null;
	private int x, y, width, height, pressX, pressY, dragX, dragY;
	private int oldX = 0, oldY = 0, oldWidth = 0, oldHeight = 0;
	private GeoNodeFigure node = null;
	private MarkDirection dir = null;
	private JDrawPanel drawPanel;
	
	/**
	 * Instantiate a ReSizeCommand
	 * @param node
	 * 			target figure to be resized
	 * @param dir
	 * 			resize direction
	 * @param drawPanel
	 * 			target JDrawPanel
	 */
	public ReSizeCommand(GeoNodeFigure node, MarkDirection dir,JDrawPanel drawPanel) {
		this.drawPanel =drawPanel;
		this.node = node;
		this.dir = dir;
		figureUtility = FigureUtility.getInstance();
		x = node.getX();
		y = node.getY();
		width = node.getWidth();
		height = node.getHeight();

		oldX = node.getX();
		oldY = node.getY();
		oldWidth = node.getWidth();
		oldHeight = node.getHeight();
		pressX = drawPanel.getLast_drag_x();
		pressY = drawPanel.getLast_drag_y();
		dragX = drawPanel.getDrag_x();
		dragY = drawPanel.getDrag_y();
	}

	/**
	 * Executes ReSizeCommand.
	 */
	@Override
	public boolean execute() {
		switch (dir) {
		case R:
			width = width + dragX - pressX;
			break;
		case U:
			y = dragY;
			height = height + pressY - dragY;
			break;
		case L:
			x = dragX;
			width = width + pressX - dragX;
			break;
		case D:
			height = height + dragY - pressY;
			break;
		case LU:
			x = dragX;
			width = width + pressX - dragX;
			y = dragY;
			height = height + pressY - dragY;
			break;
		case RU:
			y = dragY;
			width = width + dragX - pressX;
			height = height + pressY - dragY;
			break;
		case RD:
			width = width + dragX - pressX;
			height = height + dragY - pressY;
			break;
		case LD:
			x = dragX;
			width = width + pressX - dragX;
			height = height + dragY - pressY;
			break;
		default:
			break;
		}

		node.resize(x, y, width, height);
		figureUtility.reSizeDrawArea(drawPanel);
		return true;
	}

	/**
	 * Redo ReSizeCommand.
	 */
	@Override
	public void redo() {
		switch (dir) {
		case R:
			width = width + dragX - pressX;
			break;
		case U:
			y = dragY;
			height = height + pressY - dragY;
			break;
		case L:
			x = dragX;
			width = width + pressX - dragX;
			break;
		case D:
			height = height + dragY - pressY;
			break;
		case LU:
			x = dragX;
			width = width + pressX - dragX;
			y = dragY;
			height = height + pressY - dragY;
			break;
		case RU:
			y = dragY;
			width = width + dragX - pressX;
			height = height + pressY - dragY;
			break;
		case RD:
			width = width + dragX - pressX;
			height = height + dragY - pressY;
			break;
		case LD:
			x = dragX;
			width = width + pressX - dragX;
			height = height + dragY - pressY;
			break;
		default:
			break;
		}

		node.resize(x, y, width, height);
		figureUtility.reSizeDrawArea(drawPanel);
	}

	/**
	 * Undo ReSizeCommand.
	 */
	@Override
	public void undo() {
		node.resize(oldX, oldY, oldWidth, oldHeight);
		figureUtility.reSizeDrawArea(drawPanel);
		GlobeParam.getCurrentJDrawPanel().repaint();
	}

}
