/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.commands;


import org.apache.log4j.Logger;

import cn.edu.whu.lmars.drawpanel.ui.JDrawPanel;
import cn.edu.whu.lmars.figure.beans.GeoNodeFigure;
import cn.edu.whu.lmars.figure.beans.geoenum.MarkDirection;

/**
 * This command is used to save resize operation.
 * It is mainly used to support redo and undo.
 * 
 * @author Mingda Zhang
 *
 */
public class ReSizeCommand2 extends GeoCommand {

	private int x,y,width,height,pressX,pressY,dragX,dragY;
	private int oldX = 0,oldY = 0,oldWidth = 0,oldHeight = 0;
	private Logger logger = Logger.getLogger(ReSizeCommand2.class);
	private GeoNodeFigure nodeFigure;
	@SuppressWarnings("unused")
	private JDrawPanel drawPanel;
	
	/**
	 * Instantiate a ReSizeCommand2.
	 * @param node
	 * 			the figure to be resized.
	 * @param dir
	 * 			resize direction.
	 * @param drawPanel
	 * 			target JDrawPanel
	 */
	public ReSizeCommand2(GeoNodeFigure node,MarkDirection dir,JDrawPanel drawPanel){
		this.drawPanel = drawPanel;
		this.nodeFigure = node;
		x = node.getX();
		y = node.getY();
		width = node.getWidth();
		height = node.getHeight();
		 
	 
		oldX = node.getX();
		oldY = node.getY();
	    oldWidth = node.getWidth();
		oldHeight = node.getHeight();
		pressX = drawPanel.getPress_x();
		pressY = drawPanel.getPress_y();
		dragX = drawPanel.getDrag_x();
		dragY = drawPanel.getDrag_y();
		
		 switch (dir) {
			case R:
				width = width - dragX + pressX;
				break;
			case U:
				y = y+pressY - dragY;
				height = height - pressY +dragY;
				break;
			case L:
				x = x + pressX-dragX;
				width = width - pressX + dragX;
				break;
			case D:
				height = height - dragY + pressY;
				break;
			case LU:
				x = x+pressX-dragX;
				width = width - pressX +dragX;
				y = y+pressY-dragY;
				height = height - pressY + dragY;
				break;
			case RU:
				y = y+pressY - dragY;
				width = width - dragX + pressX;
				height = height - pressY + dragY;
				break;
			case RD:
				width = width - dragX +pressX;
				height = height - dragY + pressY;
				break;
			case LD:
				x = x+pressX-dragX;
				width = width - pressX + dragX;
				height = height - dragY + pressY;
				break;
			default:
				break;
			}
	}
	
	/**
	 * This method will not be executed.
	 */
	@Override
	public boolean execute() {
		logger.info("Execute ReSizeCommand2");
		return true;
	}

	/**
	 * Redo ReSizeCommand2.
	 */
	@Override
	public void redo() {
		nodeFigure.resize(oldX, oldY, oldWidth, oldHeight);
		logger.info("redo ReSizeCommand2");
	}

	/**
	 * Undo ReSizeCommand2.
	 */
	@Override
	public void undo() {
		nodeFigure.resize(x, y, width, height);
		logger.info("undo ReSizeCommand2");
	}

}
