/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.commands;

import cn.edu.whu.lmars.model.AbstractParam;


/**
 * This command is used to assign a single parameter.
 * 
 * @author Mingda Zhang
 *
 */
public class SingleInputCommand extends GeoCommand {

	private AbstractParam param;
	private String oldValue ="";
	private String newValue = "";
	/**
	 * Instantiate a SingleInputCommand.
	 * @param param
	 * 			target parameter to be assigned.
	 * @param newValue
	 * 			the new value of the parameter.
	 */
	public SingleInputCommand(AbstractParam param, String newValue){
		this.param = param;
		this.newValue = newValue;
		this.oldValue = param.getValue();
	}
	
	/**
	 * Assigns a new value to parameter.
	 */
	@Override
	public boolean  execute() {
		param.setValue(newValue);
		return true;
	}

	/**
	 * Redo SingleInputCommand.
	 */
	@Override
	public void redo() {
		execute();
	}

	/**
	 * Undo SingleInputCommand.
	 */
	@Override
	public void undo() {
		param.setValue(oldValue);
	}

}
