/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.drawpanel.listeners;

import java.awt.Cursor;
import java.awt.Dialog;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.List;
import java.util.ResourceBundle;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import conf.GlobeParam;

import cn.edu.whu.lmars.commands.SingleInputCommand;
import cn.edu.whu.lmars.drawpanel.ui.DrawPanelPopuMenu;
import cn.edu.whu.lmars.drawpanel.ui.JDrawPanel;
import cn.edu.whu.lmars.figure.beans.AbstractFigure;
import cn.edu.whu.lmars.figure.beans.DataFigure;
import cn.edu.whu.lmars.figure.beans.GeoMarkFigure;
import cn.edu.whu.lmars.figure.beans.GeoPointFigure;
import cn.edu.whu.lmars.figure.beans.ProcessFigure;
import cn.edu.whu.lmars.figure.beans.geoenum.ButtonType;
import cn.edu.whu.lmars.figure.beans.geoenum.DragType;
import cn.edu.whu.lmars.figure.utils.FigureUtility;
import cn.edu.whu.lmars.frame.ui.DataMetaDialog;
import cn.edu.whu.lmars.frame.ui.HandleInputDialog;
import cn.edu.whu.lmars.frame.ui.ProcessMetaDialog;
import cn.edu.whu.lmars.locale.LocaleKey;
import cn.edu.whu.lmars.locale.ResourceBundleFactory;
import cn.edu.whu.lmars.model.AbstractParam;
import cn.edu.whu.lmars.model.AtomProcess;
import cn.edu.whu.lmars.model.ComplexParam;
import cn.edu.whu.lmars.model.OutPutParam;
import cn.edu.whu.lmars.model.util.PureModelStore;
import cn.edu.whu.lmars.run.bind.ChoiceBindDialog;
import cn.edu.whu.lmars.run.ui.RunDetailDialog;
import cn.edu.whu.lmars.utils.HandleTool;

/**
 * The listener receives mouse events (press, release, click, enter, and exit)
 * on JDrawPanel.
 * 
 * @author Mingda Zhang
 * 
 */
public class DrawPanelMouseListener implements MouseListener {

	private FigureUtility utility = null;
	/** JPopuMenu for JDrawPanel. */
	private DrawPanelPopuMenu popuMenu = DrawPanelPopuMenu.getInstance();
	// private HandleTool handleTool = HandleTool.getInstance();
	/** The object that the listener listens for. */
	private JDrawPanel drawPanel = null;

	/**
	 * Instantiates a MouseListener.
	 * 
	 * @param drawPanel
	 *            The object that the listener listens for.
	 */
	public DrawPanelMouseListener(JDrawPanel drawPanel) {
		utility = FigureUtility.getInstance();
		this.drawPanel = drawPanel;
	}

	/**
	 * Take different actions depending on the button type.
	 * 
	 * @see java.awt.event.MouseListener#mouseClicked(MouseEvent)
	 */
	@Override
	public void mouseClicked(MouseEvent e) {
		List<AbstractFigure> figureList = drawPanel.getFigureStore()
				.getAllFigures();
		drawPanel.requestFocus();
		if (drawPanel.getBtnType() == ButtonType.Flow) {
			return;
		}

		AbstractFigure figure = null;
		drawPanel.setClick_x(e.getX());
		drawPanel.setClick_y(e.getY());
		figure = utility.getFigureByXY(e.getX(), e.getY(), figureList);
		if (figure == null) {
			utility.clearAllChoosen(figureList);
			return;
		}
		if (drawPanel.getBtnType() == ButtonType.Detail) {
			figure.setChoosen(true);
			detailClicked(figure, e);
			return;
		}

		if (drawPanel.getBtnType() == ButtonType.Bind) {
			figure.setChoosen(true);
			bindClicked(figure, e);
			return;
		}
		if (drawPanel.getBtnType() == ButtonType.DetailRun) {
			figure.setChoosen(true);
			detailRunClicked(figure, e);
			return;
		}
		if (drawPanel.getBtnType() == ButtonType.Select) {
			if (figure.isChoosen()) {
				doubleClicked(figure);
			}
			drawPanel.setCursor(new Cursor(Cursor.MOVE_CURSOR));
			figure.setChoosen(true);
			if (!drawPanel.isMutipable()) {
				utility.clearOtherChoosen(figure, figureList);
			}

		}
		HandleTool.refreshFigShow();
	}

	/**
	 * 
	 * Views running meta data.
	 * 
	 * @param figure
	 *            the selected figure
	 * @param e
	 */
	private void detailRunClicked(AbstractFigure figure, MouseEvent e) {
		JFrame client = GlobeParam.getClient();
		List<AbstractFigure> figureList = drawPanel.getFigureStore()
				.getAllFigures();
		Dialog dialog = null;
		ResourceBundle rbBundle = ResourceBundleFactory.getResourceBundle();
		utility.clearOtherChoosen(figure, figureList);
		PureModelStore modelStore = drawPanel.getModelStore();

		// pay attention to the sequence
		if (figure instanceof ProcessFigure) {
			String uuid = ((ProcessFigure) figure).getUuid();
			AtomProcess atomProcess = modelStore.getAtomProcessByUuid(uuid);
			if (atomProcess == null) {
				JOptionPane.showMessageDialog(client,
						rbBundle.getString(LocaleKey.Dialog_Msg_Detail_Fault));
				return;
			}
			dialog = new RunDetailDialog(client, (ProcessFigure) figure,
					atomProcess.getName());
			dialog.setVisible(true);
		} else {
			JOptionPane
					.showMessageDialog(client, rbBundle
							.getString(LocaleKey.Dialog_Msg_Detail_Wrong_Click));
		}
		if (dialog == null) {
			return;
		}
	}

	/**
	 * Binds a specific service to a process model.
	 * 
	 * @param figure
	 *            the selected figure.
	 * @param e
	 */
	private void bindClicked(AbstractFigure figure, MouseEvent e) {
		List<AbstractFigure> figureList = drawPanel.getFigureStore()
				.getAllFigures();
		utility.clearOtherChoosen(figure, figureList);
		ResourceBundle rbBundle = ResourceBundleFactory.getResourceBundle();
		PureModelStore modelStore = drawPanel.getModelStore();
		if (figure instanceof ProcessFigure) {
			String uuid = figure.getUuid();
			AtomProcess atomProcess = modelStore.getAtomProcessByUuid(uuid);
			if (atomProcess == null) {
				JOptionPane.showMessageDialog(null,
						rbBundle.getString(LocaleKey.Dialog_Msg_Detail_Fault));
				return;
			}

			ChoiceBindDialog dialog = new ChoiceBindDialog(
					GlobeParam.getClient(), atomProcess,
					drawPanel.getBindStore());
			dialog.setVisible(true);
		} else {
			JOptionPane
					.showMessageDialog(null, rbBundle
							.getString(LocaleKey.Dialog_Msg_Detail_Wrong_Click));
		}

		GlobeParam.getCurrentJDrawPanel().requestFocus();
	}

	/**
	 * Views meta data of a model.
	 * 
	 * @param figure
	 * @param e
	 */
	private void detailClicked(AbstractFigure figure, MouseEvent e) {

		JFrame client = GlobeParam.getClient();
		List<AbstractFigure> figureList = drawPanel.getFigureStore()
				.getAllFigures();
		Dialog dialog = null;
		utility.clearOtherChoosen(figure, figureList);
		PureModelStore modelStore = drawPanel.getModelStore();
		ResourceBundle rbBundle = ResourceBundleFactory.getResourceBundle();
		// pay attention to sequence because of inheritance relationship
		if (figure instanceof ProcessFigure) {
			String uuid = ((ProcessFigure) figure).getUuid();
			AtomProcess atomProcess = modelStore.getAtomProcessByUuid(uuid);
			if (atomProcess == null) {
				JOptionPane.showMessageDialog(client,
						rbBundle.getString(LocaleKey.Dialog_Msg_Detail_Fault));
				return;
			}
			dialog = new ProcessMetaDialog(client, atomProcess);
			dialog.setVisible(true);
		} else if (figure instanceof DataFigure) {
			DataFigure dataFigure = (DataFigure) figure;
			String uuid = dataFigure.getUuid();
			String[] uuids = uuid.split("#");
			if (uuids.length < 2) {
				JOptionPane.showMessageDialog(client, rbBundle
						.getString(LocaleKey.Dialog_Msg_Detail_Wrong_ID));
				return;
			}

			if (dataFigure.getState().equals("res")) {
				AtomProcess atomProcess = modelStore
						.getAtomProcessByUuid(uuids[0]);
				/*
				 * OutPutParam outPutParam = modelStore
				 * .getAtomProcessOutPutByUuid(uuids[0]);
				 */
				OutPutParam outPutParam = atomProcess.getOutPutParam();
				dialog = new DataMetaDialog(client, outPutParam);
				dialog.setVisible(true);
			} else {
				AtomProcess atomProcess = modelStore
						.getAtomProcessByUuid(uuids[0]);
				ComplexParam complexParam = atomProcess
						.getComplexParamByName(uuids[1]);
				dialog = new DataMetaDialog(client, complexParam);
				dialog.setVisible(true);
			}
		}
		if (dialog == null) {
			return;
		}

	}

	@Override
	public void mouseEntered(MouseEvent e) {
	}

	@Override
	public void mouseExited(MouseEvent e) {
	}

	/**
	 * 
	 * @see java.awt.event.MouseListener#mousePressed(MouseEvent)
	 */
	@Override
	public void mousePressed(MouseEvent e) {

		drawPanel.setPress_x(e.getX());
		drawPanel.setPress_y(e.getY());
		drawPanel.setLast_drag_x(e.getX());
		drawPanel.setLast_drag_y(e.getY());
		drawPanel.setDrag_x(0);
		drawPanel.setDrag_y(0);
		List<AbstractFigure> figureList = drawPanel.getFigureStore()
				.getAllFigures();
		if (e.isPopupTrigger())
			popuMenu.show(e.getComponent(), e.getX(), e.getY());

		if (drawPanel.getDragType() == DragType.DRAWLINE) {
			return;
		}
		GeoMarkFigure mark = utility.getMark(drawPanel.getPress_x(),
				drawPanel.getPress_y(), drawPanel);
		if (mark != null) {
			drawPanel.setDragType(DragType.RESIZE);
			drawPanel.setMark(mark);
			return;
		}
		if (utility.getChoosen(figureList).size() > 0) {
			drawPanel.setDragType(DragType.MOVE);
			if (utility.getFigureByXY(e.getX(), e.getY(),
					utility.getChoosen(figureList)) != null) {
				GlobeParam.getCurrentJDrawPanel().setCursor(
						new Cursor(Cursor.MOVE_CURSOR));
			}
			return;
		}
		drawPanel.setDragType(DragType.NONE);
	}

	/**
	 * Take different actions depending on the drag type.
	 * @see java.awt.event.MouseListener#mouseReleased(MouseEvent)
	 */
	@Override
	public void mouseReleased(MouseEvent e) {
		GlobeParam.TEMPORARY_FIGURES.clear();
		List<AbstractFigure> figureList = drawPanel.getFigureStore()
				.getAllFigures();
		drawPanel.setRelease_x(e.getX());
		drawPanel.setRealse_y(e.getY());
		if (e.isPopupTrigger())
			popuMenu.show(e.getComponent(), e.getX(), e.getY());

		if (drawPanel.getDragType() == DragType.RESIZE) {
			utility.resizeFigureEnd(drawPanel.getMark(), drawPanel);
			drawPanel.setMark(null);
			return;
		}
		utility.reSet();
		if (drawPanel.getDragType() == DragType.MOVE) {
			if (utility.getFigureByXY(e.getX(), e.getY(),
					utility.getChoosen(figureList)) != null) {
				drawPanel.setCursor(new Cursor(Cursor.MOVE_CURSOR));
			}

			utility.moveFigureEnd(
					utility.getChoosen(figureList),
					new GeoPointFigure(drawPanel.getPress_x(), drawPanel
							.getPress_y()),
					new GeoPointFigure(drawPanel.getDrag_x(), drawPanel
							.getDrag_y()));
			drawPanel.setMoved(false);
			FigureUtility.getInstance().reSizeDrawArea(drawPanel);
		}
		if (drawPanel.getDragType() == DragType.DRAWLINE) {

			if (drawPanel.getPress_x() == drawPanel.getRelease_x()
					&& drawPanel.getPress_y() == drawPanel.getRealse_y()) {
				return;
			}
			utility.createSpeFlowFigure(drawPanel);
			drawPanel.setChanged(true);
			drawPanel.setDragType(DragType.MOVE);
			drawPanel.setBtnType(ButtonType.Select);
			drawPanel.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}
		if (GlobeParam.TEMPORARY_FIGURES.size() > 0) {
			GlobeParam.TEMPORARY_FIGURES.clear();
		}
		drawPanel.repaint();
	}

	/**
	 * Handles the double click event
	 * 
	 * @param figure
	 */
	private void doubleClicked(AbstractFigure figure) {
		figure.setChoosen(true);
		List<AbstractFigure> figureList = drawPanel.getFigureStore()
				.getAllFigures();
		utility.clearOtherChoosen(figure, figureList);
		ResourceBundle rBundle = ResourceBundleFactory.getResourceBundle();
		if (figure instanceof ProcessFigure) {
			String uuid = figure.getUuid();
			PureModelStore modelStore = drawPanel.getModelStore();
			AtomProcess atomProcess = modelStore.getAtomProcessByUuid(uuid);

			Dialog dialog = new HandleInputDialog(GlobeParam.getClient(),
					atomProcess);
			dialog.setVisible(true);
			HandleTool.refreshFigShow();
			return;
		}
		DataFigure dataFigure;
		if (figure instanceof DataFigure) {
			dataFigure = (DataFigure) figure;
			if (dataFigure.getState().equals("res")) {
				JOptionPane.showMessageDialog(GlobeParam.getClient(), rBundle
						.getString(LocaleKey.Dialog_Msg_Input_Wrong_Click));
				return;
			}
			if (dataFigure.getState().equals("val")) {
				AbstractParam model = (AbstractParam) dataFigure.getModel();
				String input = JOptionPane.showInputDialog(
						GlobeParam.getClient(), model.getName(),
						model.getValue());
				if (input != null && !input.trim().equals(model.getValue())) {
					SingleInputCommand command = new SingleInputCommand(model,
							input.trim());
					drawPanel.getCommandStack().addCommand(command);
					command.execute();
					GlobeParam.getCurrentJDrawPanel().setChanged(true);
				}
			}
		}
	}

}
