/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.drawpanel.listeners;

import java.awt.Cursor;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import java.util.List;

import conf.GlobeParam;

import cn.edu.whu.lmars.drawpanel.ui.JDrawPanel;
import cn.edu.whu.lmars.drawpanel.ui.cursors.BindCursor;
import cn.edu.whu.lmars.drawpanel.ui.cursors.DetailCursor;
import cn.edu.whu.lmars.drawpanel.ui.cursors.DetailRunCursor;
import cn.edu.whu.lmars.figure.beans.AbstractFigure;
import cn.edu.whu.lmars.figure.beans.GeoMarkFigure;
import cn.edu.whu.lmars.figure.beans.GeoNodeFigure;
import cn.edu.whu.lmars.figure.beans.GeoPointFigure;
import cn.edu.whu.lmars.figure.beans.LinkFigure;
import cn.edu.whu.lmars.figure.beans.geoenum.ButtonType;
import cn.edu.whu.lmars.figure.beans.geoenum.DragType;
import cn.edu.whu.lmars.figure.utils.FigureUtility;
import cn.edu.whu.lmars.utils.Judger;

/**
 * The listener receives mouse motion events on JDrawPanel.
 * 
 * @author Mingda Zhang
 * 
 */
public class DrawPanelMouseMotionListener implements MouseMotionListener {

	private FigureUtility utility = null;
	private JDrawPanel drawPanel;

	/**
	 * Instantiates a DrawPanelMouseMotionListener.
	 * 
	 * @param drawPanel
	 *            the JDrawPanel to be listened for.
	 */
	public DrawPanelMouseMotionListener(JDrawPanel drawPanel) {
		utility = FigureUtility.getInstance();
		this.drawPanel = drawPanel;
	}

	/**
	 * Takes different actions depending on the drag type.
	 * @see java.awt.event.MouseMotionListener#mouseDragged(MouseEvent)
	 */
	@Override
	public void mouseDragged(MouseEvent e) {
		List<AbstractFigure> figureList = drawPanel.getFigureStore()
				.getAllFigures();
		if (drawPanel.getDrag_x() != 0 && drawPanel.getDrag_y() != 0) {
			drawPanel.setLast_drag_x(drawPanel.getDrag_x());
			drawPanel.setLast_drag_y(drawPanel.getDrag_y());
		}
		drawPanel.setDrag_x(e.getX());
		drawPanel.setDrag_y(e.getY());
		switch (drawPanel.getDragType()) {
		case MOVE:
			utility.moveFigure(
					utility.getChoosen(figureList),
					new GeoPointFigure(drawPanel.getLast_drag_x(), drawPanel
							.getLast_drag_y()),
					new GeoPointFigure(drawPanel.getDrag_x(), drawPanel
							.getDrag_y()), drawPanel);

			drawPanel.setCursor(new Cursor(Cursor.MOVE_CURSOR));
			break;
		case RESIZE:
			utility.resizeFigure(drawPanel.getMark());
			break;
		case DRAWLINE:
			LinkFigure line = new LinkFigure(new GeoPointFigure(
					drawPanel.getPress_x(), drawPanel.getPress_y()),
					new GeoPointFigure(drawPanel.getDrag_x(), drawPanel
							.getDrag_y()), null, null);
			GlobeParam.TEMPORARY_FIGURES.add(line);
			break;
		default:
			break;
		}

		if (GlobeParam.TEMPORARY_FIGURES.size() > 3) {
			for (int j = 0; j < 3; j++) {
				GlobeParam.TEMPORARY_FIGURES.remove(0);
			}
		}
		drawPanel.repaint();

	}

	/**
	 * Controls the shape of the mouse depending on button type and figure
	 * state.
	 * 
	 * @see java.awt.event.MouseMotionListener#mouseMoved(MouseEvent)
	 */
	@Override
	public void mouseMoved(MouseEvent e) {
		drawPanel.setMove_x(e.getX());
		drawPanel.setMove_y(e.getY());
		List<AbstractFigure> figureList = drawPanel.getFigureStore()
				.getAllFigures();
		if (drawPanel.getBtnType() == ButtonType.Detail) {
			drawPanel.setCursor(DetailCursor.getInstance().getDetailCursor());
		}

		if (drawPanel.getBtnType() == ButtonType.Bind) {
			drawPanel.setCursor(BindCursor.getInstance().getBindCursor());
		}
		if (drawPanel.getBtnType() == ButtonType.DetailRun) {
			drawPanel.setCursor(DetailRunCursor.getInstance()
					.getDetailRunCursor());
		}

		if (drawPanel.getBtnType() != ButtonType.Select) {
			return;
		}
		drawPanel.setDragType(DragType.NONE);
		ArrayList<AbstractFigure> chooses = utility.getChoosen(figureList);
		if (chooses.size() == 0)
			return;

		AbstractFigure figure = null;
		GeoMarkFigure[] marks = null;
		GeoNodeFigure nodeFigure;
		for (int i = 0; i < chooses.size(); i++) {
			figure = chooses.get(i);
			if (figure instanceof LinkFigure) {
				continue;
			}
			nodeFigure = (GeoNodeFigure) figure;
			marks = nodeFigure.getMarks();
			for (int j = 0; j < marks.length; j++) {
				if (Judger.getInstance().rectPoint(marks[j],
						drawPanel.getMove_x(), drawPanel.getMove_y())) {
					drawPanel.setCursor(new Cursor(Cursor.HAND_CURSOR));
					drawPanel.setDragType(DragType.RESIZE);
					return;
				}
			}
		}

		AbstractFigure movefigure = utility.getFigureByXY(
				drawPanel.getMove_x(), drawPanel.getMove_y(),
				utility.getChoosen(figureList));
		if (movefigure != null) {
			drawPanel.setCursor(new Cursor(Cursor.MOVE_CURSOR));
			drawPanel.setDragType(DragType.MOVE);
			return;
		}
		drawPanel.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	}

}
