/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.drawpanel.ui;

import java.awt.Font;
import java.awt.Frame;
import java.awt.Rectangle;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.WindowConstants;

import cn.edu.whu.lmars.utils.BaseTool;

/**
 * Views the model path.
 * @author zhangmingda
 *
 */
public class PathDialog extends JDialog {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JLabel jLabel = null;
	private JTextField pathjTextField = null;
	private JDrawPanel drawPanel = null;
	/**
	 * @param owner
	 */
	public PathDialog(Frame owner,JDrawPanel drawPanel) {
		super(owner);
		this.drawPanel = drawPanel;
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(433, 106);
		this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		this.setTitle("Model Path");
		this.setFont(new Font("Dialog", Font.PLAIN, 14));
		this.setContentPane(getJContentPane());
		String path = drawPanel.getModelPath();
		if (BaseTool.isNull(path)) {
			path = "Not saved.";
		}
		this.pathjTextField.setText(path);
		this.setLocationRelativeTo(null);
		this.setModal(true);
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jLabel = new JLabel();
			jLabel.setBounds(new Rectangle(9, 18, 87, 23));
			jLabel.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel.setText("Model Path:");
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(jLabel, null);
			jContentPane.add(getPathjTextField(), null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes pathjTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getPathjTextField() {
		if (pathjTextField == null) {
			pathjTextField = new JTextField();
			pathjTextField.setBounds(new Rectangle(108, 15, 290, 30));
			pathjTextField.setFont(new Font("Dialog", Font.PLAIN, 14));
		}
		return pathjTextField;
	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
