/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.figure.beans;

import java.awt.Graphics;

import cn.edu.whu.lmars.model.IModel;

/**
 * Defines the requirements for a figure. Figures play a view role in the
 * Model-View-Control pattern.
 * 
 * @author Mingda Zhang
 * 
 */

public abstract class AbstractFigure {

	/** Whether it is selected */
	protected boolean choosen = false;

	/** A reference to IModel */
	protected IModel model;

	/**
	 * Draws itself on the JDrawPanel.
	 * 
	 * @param g
	 */
	public void draw(Graphics g) {
	};

	/**
	 * Updates the attributes of the figure.
	 */
	public void refresh() {
	};

	/**
	 * Sets the reference IModel.
	 * 
	 * @param model
	 *            the reference model
	 */
	public void setModel(IModel model) {
		this.model = model;
	}

	/**
	 * Gets the reference IModel.
	 */
	public IModel getModel() {
		return model;
	}

	/**
	 * Gets the state of the figure whether it is selected.
	 */
	public boolean isChoosen() {
		return choosen;
	}

	/**
	 * Sets the state of the figure.
	 * 
	 * @param choosen
	 */
	public void setChoosen(boolean choosen) {
		this.choosen = choosen;
	}

	/**
	 * Gets the id of this figure.
	 */
	public String getUuid() {
		return "";
	}

	/**
	 * Sets the id of this figure.
	 * 
	 * @param id
	 */
	public void setUuid(String id) {
	}

}
