/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.figure.beans;

import java.awt.Color;
import java.awt.Graphics;

import cn.edu.whu.lmars.figure.utils.FigureUtility;
import cn.edu.whu.lmars.model.AbstractParam;
import cn.edu.whu.lmars.model.ComplexParam;

/**
 * The data figure stands for a complex input data or an output.
 * 
 * 
 * @author Mingda Zhang
 *
 */
public class DataFigure extends GeoNodeFigure{

    private String state;
    
    /** LinkFigures related to this figure. */
    private LinkFigure linkFigure;
    
    public DataFigure(int x,int y,int width,int height){
    	super(x, y, width, height);
    }
    

    /**
     * Gets the state of this figure.
     * @return
     */
    public String getState() {
        return state;
    }

    /**
     * Sets the state of this figure.
     */
    public void setState(String value) {
        this.state = value;
    }
    

    /**
     * Gets related LinkFigures.
     * @return
     */
    public LinkFigure getLinkFigure() {
		return linkFigure;
	}

    /**
     * Sets related LinkFigures.
     */
	public void setLinkFigure(LinkFigure linkFigure) {
		this.linkFigure = linkFigure;
	}


	@Override
	public void draw(Graphics g) {
    	if (this.state.equals("ref") || this.state.equals("nul")) {
			return;
		}
		Color c = g.getColor();
	//	Color color = new Color(255,248,220);
		Color color = new Color(72,118,255);

		g.setColor(color);
		g.fillOval(x, y, width, height);
		g.setColor(Color.black);
		g.drawOval(x, y, width, height);
	//	g.drawOval(x, y, width, height);
	//	g.drawImage(dataImage, x, y, width,height,null);  
		if (choosen) {
			for (int i = 0; i < 8; i++) {
				marks[i].draw(g);
			}
		}
		g.setColor(Color.BLACK);
		/*if (dataModel == null) {
			dataModel = (GeoDataModel)HandleTool.getInstance().getModeByID(getId());
		}*/
		String show = null;
		if (this.state.equals("res")) {
			show = "Result";
		}else {
			show = ((AbstractParam)model).getValue();
			if (show == null || show.trim().equals("")) {
				show = ((ComplexParam)model).getName();
			}else {
				int n = show.lastIndexOf("/");
				if (n>0) {
					show = show.substring(n+1, show.length());
				}else {
					n = show.lastIndexOf("//");
					if (n>0) {
						show = show.substring(n+1, show.length());;
					}else {
						n = show.lastIndexOf("\\");
						if (n>0) {
							show = show.substring(n+1, show.length());;
						}
					}
				}
			}
			if (show == null) {
				show = "wrong!";
			}
		}
		FigureUtility.getInstance().drawStringInMiddle(this, g, show);
		g.setColor(c);
	}

    @Override
	public String toString() {
		return "begin point :(" + x + "," + y + ")" + "heigh:" + height + ",width:" + width;
	}
}
