/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.figure.beans;

import java.awt.Color;
import java.awt.Graphics;

import conf.GlobeParam;

import cn.edu.whu.lmars.figure.beans.geoenum.MarkDirection;

/**
 * The mark figure which will display when its parent figure is selected. The
 * data figure and process figure own eight mark figures.
 * 
 * @author Mingda Zhang
 * 
 */
public class GeoMarkFigure {

	private int x, y, width = GlobeParam.MARK_WIDTH,
			height = GlobeParam.MARK_WIDTH;
	private int offest = height / 2;

	/** The direction of this Mark Figure. */
	private MarkDirection direction = null;

	/** This mark figure's owner */
	private AbstractFigure owner = null;

	public GeoMarkFigure(int x, int y, MarkDirection direction,
			AbstractFigure owner) {
		this.x = x;
		this.y = y;
		this.direction = direction;
		this.owner = owner;
	}

	public AbstractFigure getOwner() {
		return owner;
	}

	public void setX(int x) {
		this.x = x;
	}

	public void setY(int y) {
		this.y = y;
	}

	public MarkDirection getDirection() {
		return direction;
	}

	public int getX() {
		return x;
	}

	public int getY() {
		return y;
	}

	public int getWidth() {
		return width;
	}

	public int getOffest() {
		return offest;
	}

	public void draw(Graphics g) {
		g.setColor(Color.black);
		g.fillRect(x - offest, y - offest, width, height);

	}

	@Override
	public String toString() {
		String str = "";
		str = "Begin point:(" + x + "." + y + "),Width:" + width + ",Height:"
				+ height;
		return str;
	}

}
