/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.figure.beans;

import java.awt.Graphics;

import cn.edu.whu.lmars.figure.beans.geoenum.MarkDirection;
import cn.edu.whu.lmars.utils.HandleTool;

/**
 * The GeoNodeFigure is a parent class of DataFigure and ProcessFigure.
 * 
 * @author Mingda Zhang
 * 
 */
public class GeoNodeFigure extends AbstractFigure {

	protected String uuid;
	protected String locXY;
	protected int x, y, width, height;

	/** Mark figures will be drawed when this figure is selected */
	protected GeoMarkFigure[] marks;

	/** Start point or end point of a line. */
	public GeoPointFigure pointU, pointD, pointL, pointR;

	/**
	 * Instantiates a GeoNodeFigure.
	 * 
	 * @param x
	 *            the x coordinate value of this figure.
	 * @param y
	 *            the y coordinate value of this figure.
	 * @param width
	 *            the width of this figure.
	 * @param height
	 *            the height of this figure.
	 */
	public GeoNodeFigure(int x, int y, int width, int height) {
		this.init(x, y, width, height);
	}

	/**
	 * Initializes this GeoNodeFigure.
	 * 
	 * @param x
	 *            the x coordinate value of this figure.
	 * @param y
	 *            the y coordinate value of this figure.
	 * @param width
	 *            the width of this figure.
	 * @param height
	 *            the height of this figure.
	 */
	private void init(int x, int y, int width, int height) {
		marks = new GeoMarkFigure[8];
		this.x = x;
		this.y = y;
		this.width = width;
		this.height = height;
		pointU = new GeoPointFigure(x + width / 2, y);
		pointD = new GeoPointFigure(x + width / 2, y + height);
		pointL = new GeoPointFigure(x, y + height / 2);
		pointR = new GeoPointFigure(x + width, y + height / 2);

		marks[0] = new GeoMarkFigure(x, y, MarkDirection.LU, this);
		marks[1] = new GeoMarkFigure(x, y + height / 2, MarkDirection.L, this);
		marks[2] = new GeoMarkFigure(x, y + height, MarkDirection.LD, this);
		marks[3] = new GeoMarkFigure(x + width / 2, y, MarkDirection.U, this);
		marks[4] = new GeoMarkFigure(x + width / 2, y + height,
				MarkDirection.D, this);
		marks[5] = new GeoMarkFigure(x + width, y, MarkDirection.RU, this);
		marks[6] = new GeoMarkFigure(x + width, y + height / 2,
				MarkDirection.R, this);
		marks[7] = new GeoMarkFigure(x + width, y + height, MarkDirection.RD,
				this);
	}

	@Override
	public String getUuid() {
		return uuid;
	}

	@Override
	public void setUuid(String value) {
		this.uuid = value;
	}

	/**
	 * Gets the location of this figure.
	 * 
	 * @return
	 */
	public String getLocXY() {
		return x + ";" + y + ";" + width + ";" + height;
	}

	@Override
	public void refresh() {
		marks[0].setX(x);
		marks[0].setY(y);

		marks[1].setX(x);
		marks[1].setY(y + height / 2);

		marks[2].setX(x);
		marks[2].setY(y + height);

		marks[3].setX(x + width / 2);
		marks[3].setY(y);

		marks[4].setX(x + width / 2);
		marks[4].setY(y + height);

		marks[5].setX(x + width);
		marks[5].setY(y);

		marks[6].setX(x + width);
		marks[6].setY(y + height / 2);

		marks[7].setX(x + width);
		marks[7].setY(y + height);

		pointU.setX(x + width / 2);
		pointU.setY(y);

		pointD.setX(x + width / 2);
		pointD.setY(y + height);

		pointL.setX(x);
		pointL.setY(y + height / 2);

		pointR.setX(x + width);
		pointR.setY(y + height / 2);
	}

	/**
	 * Gets the x coordinate value.
	 */
	public int getX() {
		return x;
	}

	/**
	 * Gets the y coordinate value.
	 */
	public int getY() {
		return y;
	}

	/**
	 * Gets width of this figure.
	 */
	public int getWidth() {
		return width;
	}

	/**
	 * Gets height of this figure.
	 */
	public int getHeight() {
		return height;
	}

	/**
	 * Resizes the figure.
	 * 
	 * @param x
	 *            the new x coordinate value of this figure.
	 * @param y
	 *            the new y coordinate value of this figure.
	 * @param width
	 *            the new width of this figure.
	 * @param height
	 *            the new height of this figure.
	 */
	public void resize(int x, int y, int width, int height) {
		if (x < 0) {
			// FigureUtility.getInstance().moveAllRight(handleTool.getCurrentFigures(),
			// Math.abs(x));
			HandleTool.moveAllRight(Math.abs(x));
			x = 0;
		}
		if (y < 0) {
			// FigureUtility.getInstance().moveAllDown(handleTool.getCurrentFigures(),
			// Math.abs(y));
			HandleTool.moveAllDown(Math.abs(y));
			y = 0;
		}
		this.x = x;
		this.y = y;
		this.width = width;
		this.height = height;
		refresh();
	}

	/**
	 * Gets Mark Figures owned by this figure.
	 * 
	 * @return
	 */
	public GeoMarkFigure[] getMarks() {
		return marks;
	}

	@Override
	public void draw(Graphics g) {
	}

}
