/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.figure.beans;

import java.awt.Color;

import java.awt.Graphics;

import cn.edu.whu.lmars.figure.beans.geoenum.MarkDirection;
import cn.edu.whu.lmars.figure.utils.FigureUtility;

/**
 * The line figure.
 * 
 * @author Mingda Zhang
 * 
 */

public class LinkFigure extends AbstractFigure {

	/** start point */
	protected GeoPointFigure startP;
	/** end point */
	protected GeoPointFigure endP;
	/** start figure */
	protected GeoNodeFigure startFigure;

	/** end figure */
	protected GeoNodeFigure endFigure;
	/** id of the figure */
	private String uuid;
	protected GeoMarkFigure[] marks;

	/** arrow's location */
	protected int[] arrows = new int[4];
	protected int[] xPoints = new int[3];
	protected int[] yPoints = new int[3];

	protected int[] xFlows = new int[4];
	protected int[] yFlows = new int[4];

	/**
	 * Instantiates a LinkFigure.
	 * 
	 * @param startP
	 *            the start point of this figure.
	 * @param endP
	 *            the end point of this figure.
	 * @param startFigure
	 *            the start figure of this figure.
	 * @param endFigure
	 *            the end figure of this figure.
	 */
	public LinkFigure(GeoPointFigure startP, GeoPointFigure endP,
			GeoNodeFigure startFigure, GeoNodeFigure endFigure) {
		this.startP = startP;
		this.endP = endP;
		this.startFigure = startFigure;
		this.endFigure = endFigure;
		init(startP, endP, startFigure, endFigure);
	}

	/**
	 * Initializes the line figure.
	 * 
	 * @param startP
	 * @param endP
	 * @param startFigure
	 * @param endFigure
	 * @return
	 */
	public boolean init(GeoPointFigure startP, GeoPointFigure endP,
			GeoNodeFigure startFigure, GeoNodeFigure endFigure) {
		this.startP = startP;
		this.endP = endP;
		this.startFigure = startFigure;
		this.endFigure = endFigure;
		marks = new GeoMarkFigure[2];
		refresh();
		return true;
	}

	@Override
	public String getUuid() {
		return uuid;
	}

	@Override
	public void setUuid(String uuid) {
		this.uuid = uuid;
	}

	/**
	 * Sets start point of this figure.
	 * 
	 * @param startP
	 *            the start point.
	 */
	public void setStartP(GeoPointFigure startP) {
		this.startP = startP;
		refresh();
	}

	/**
	 * Sets end point of this figure.
	 * 
	 * @param endP
	 *            the end point.
	 */
	public void setEndP(GeoPointFigure endP) {
		this.endP = endP;
		refresh();
	}

	/**
	 * Gets start figure of this figure.
	 */
	public GeoNodeFigure getStartFigure() {
		return startFigure;
	}

	/**
	 * Gets end figure of this figure.
	 */
	public GeoNodeFigure getEndFigure() {
		return endFigure;
	}

	/**
	 * Sets start point.
	 */
	public GeoPointFigure getStartP() {
		return startP;
	}

	/**
	 * Sets start point.
	 */
	public GeoPointFigure getEndP() {
		return endP;
	}

	@Override
	public void draw(Graphics g) {
		Color color = new Color(72, 118, 255);
		g.setColor(color);
		drawFlow(g);
	}

	public void drawFlow(Graphics g) {
		refresh();
		// g.setFont(new Font("Dialog", Font.PLAIN, 16));
		Color oldColor = g.getColor();
		g.setColor(new Color(100, 149, 237));
		g.fillOval(startP.getX() - 5, startP.getY() - 5, 10, 10);
		g.drawPolygon(xFlows, yFlows, 4);
		g.fillPolygon(xFlows, yFlows, 4);

		// g.setColor(new Color(112, 128, 144));
		g.drawPolygon(xPoints, yPoints, 3);
		g.fillPolygon(xPoints, yPoints, 3);
		g.setColor(oldColor);
		// g.draw
		// g.drawLine(startP.getX(),startP.getY(),endP.getX(),endP.getY());
		if (choosen) {
			for (int i = 0; i < 2; i++) {
				marks[i].draw(g);
			}
		}

	}

	// GeoflowFigure can't be selected.
	@Override
	public void refresh() {
		// TODO Auto-generated method stub
		if (marks[0] == null) {
			marks[0] = new GeoMarkFigure(this.startP.getX(),
					this.startP.getY(), MarkDirection.L, this);
		} else {
			marks[0].setX(this.startP.getX());
			marks[0].setY(this.startP.getY());
		}
		if (marks[1] == null) {
			marks[1] = new GeoMarkFigure(this.endP.getX(), this.endP.getY(),
					MarkDirection.R, this);
		} else {
			marks[1].setX(this.endP.getX());
			marks[1].setY(this.endP.getY());
		}

		arrows = FigureUtility.getInstance().getArrows(startP.getX(),
				startP.getY(), endP.getX(), endP.getY());
		xPoints[0] = this.endP.getX();
		xPoints[1] = arrows[0];
		xPoints[2] = arrows[2];

		yPoints[0] = this.endP.getY();
		yPoints[1] = arrows[1];
		yPoints[2] = arrows[3];

		xFlows[0] = this.startP.getX() + (xPoints[1] - xPoints[2]) / 6;
		xFlows[3] = this.startP.getX() - (xPoints[1] - xPoints[2]) / 6;
		xFlows[1] = (xPoints[1] - xPoints[2]) * 2 / 3 + xPoints[2];
		xFlows[2] = (xPoints[1] - xPoints[2]) / 3 + xPoints[2];

		yFlows[0] = this.startP.getY() + (yPoints[1] - yPoints[2]) / 6;
		yFlows[3] = this.startP.getY() - (yPoints[1] - yPoints[2]) / 6;
		yFlows[1] = (yPoints[1] - yPoints[2]) * 2 / 3 + yPoints[2];
		yFlows[2] = (yPoints[1] - yPoints[2]) / 3 + yPoints[2];
	}

}
