/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.figure.beans;

import java.awt.Color;
import java.awt.Graphics;
import java.util.ArrayList;
import java.util.List;

import cn.edu.whu.lmars.figure.utils.FigureUtility;
import cn.edu.whu.lmars.model.AtomProcess;
import cn.edu.whu.lmars.run.beans.ExecutionState;
import cn.edu.whu.lmars.run.beans.ServiceExecutionInfo;

/**
 * The ProcessFigure stands for a processing model.
 * 
 * @author Mingda Zhang
 * 
 */
public class ProcessFigure extends GeoNodeFigure {

	/** Data figures standing for parameters contained in a model */
	protected List<DataFigure> dataFigure = new ArrayList<DataFigure>();
	/** Link figures contained in a processing figure */
	private List<LinkFigure> linkFigures = new ArrayList<LinkFigure>();
	/** Execution information. */
	private ServiceExecutionInfo executionInfo = null;

	public ProcessFigure(int x, int y, int width, int height) {
		super(x, y, width, height);
	}

	/** The execution state of this processing figure. */
	private ExecutionState state = ExecutionState.READY;

	/**
	 * Gets the data figures.
	 */
	public List<DataFigure> getDataFigures() {
		if (dataFigure == null) {
			dataFigure = new ArrayList<DataFigure>(5);
		}
		return this.dataFigure;
	}

	/**
	 * Gets the data figure by its id.
	 */
	public DataFigure getDataFigureByUuid(String uuid) {
		for (DataFigure figure : getDataFigures()) {
			if (figure.getUuid().equals(uuid)) {
				return figure;
			}
		}
		return null;
	}

	/**
	 * Adds a data figure to this Process Figure.
	 */
	public void addDataFigure(DataFigure figure) {
		if (!this.dataFigure.contains(figure)) {
			this.dataFigure.add(figure);
		}
	}

	/**
	 * Gets result figure in this Process Figure.
	 */
	public DataFigure getResultFigure() {
		for (DataFigure figure : getDataFigures()) {
			if (figure.getState().equals("res")) {
				return figure;
			}
		}
		return null;
	}

	/**
	 * Sets state of this ProcessFigure.
	 */
	public void setExecuteState(ExecutionState state) {
		this.state = state;
	}

	/**
	 * Gets state of this ProcessFigure.
	 */
	public ExecutionState getState() {
		return state;
	}

	/**
	 * Adds a link figure to this ProcessFigure.
	 */
	public void addLinkFigure(LinkFigure linkFigure) {
		if (!this.linkFigures.contains(linkFigure)) {
			this.linkFigures.add(linkFigure);
		}
	}

	/**
	 * Removes a link figure from this Process Figure.
	 */
	public void removeLinkFigure(LinkFigure linkFigure) {
		if (this.linkFigures.contains(linkFigure)) {
			this.linkFigures.remove(linkFigure);
		}
	}

	/**
	 * Gets link figures in this Process Figure.
	 */
	public List<LinkFigure> getLinkFigures() {
		return linkFigures;
	}

	@Override
	public void draw(Graphics g) {
		int arcWidth = width / 4;
		int arcHeigth = height / 4;
		Color color = null;
		switch (state) {
		case READY:
			color = new Color(244, 164, 96);
			break;
		case FINISHED:
			color = new Color(0, 205, 102);
			break;
		case FAILED:
			color = new Color(255, 48, 48);
			break;
		default:
			color = new Color(244, 164, 96);
			break;
		}
		g.setColor(color);
		g.fillRoundRect(x, y, width, height, arcWidth, arcHeigth);
		g.setColor(Color.black);
		g.drawRoundRect(x, y, width, height, arcWidth, arcHeigth);
		// g.drawImage(dataImage, x, y, width,height,null);
		if (choosen) {
			for (int i = 0; i < 8; i++) {
				marks[i].draw(g);
			}
		}
		String show = null;
		if (this.model == null) {
			show = "No Binding";
		} else {
			show = ((AtomProcess) model).getName();
		}
		FigureUtility.getInstance().drawStringInMiddle(this, g, show);
	}

	public LinkFigure getLinkFigureByContainsUuid(String uuid) {
		for (LinkFigure figure : this.getLinkFigures()) {
			if (figure.getUuid().contains(uuid)) {
				return figure;
			}
		}
		return null;
	}

	/**
	 * Tells whether all parameters are assigned.
	 */
	public boolean isNul() {
		for (DataFigure figure : this.getDataFigures()) {
			if (figure.getState().equals("nul")) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Gets ServiceExecutionInfo.
	 */
	public ServiceExecutionInfo getExecutionInfo() {
		return executionInfo;
	}

	/**
	 * Sets ServiceExecutionInfo.
	 */
	public void setExecutionInfo(ServiceExecutionInfo executionInfo) {
		this.executionInfo = executionInfo;
	}

}
