/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.figure.utils;

import java.io.FileWriter;
import java.io.IOException;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

import cn.edu.whu.lmars.figure.beans.DataFigure;
import cn.edu.whu.lmars.figure.beans.FlowFigure;
import cn.edu.whu.lmars.figure.beans.ProcessFigure;

/**
 * Saves the figures to configuration file.
 * The file extension is '.gur'.
 * 
 * @author Mingda Zhang
 * 
 */
public class FigureSave {
	
	/**
	 * Saves the figures to specified file.
	 * @param file
	 * 			the file to save figures.
	 * @param figureStore
	 * 			figure container.
	 * @return
	 */
	public boolean saveToFile(String file, FigureStore figureStore) {

		Document doc = DocumentHelper.createDocument();
		Element root = DocumentHelper.createElement("Figures");
		for (ProcessFigure processFigure : figureStore.getProcessFigures()) {
			root.add(createProcessElement(processFigure));
		}
		for (FlowFigure flowFigure : figureStore.getFlowFigures()) {
			root.add(createFlowElement(flowFigure));
		}
		doc.setRootElement(root);

		FileWriter output;
		try {
			output = new FileWriter(file);
			output.write(doc.asXML());
			output.flush();
			output.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}

		return true;
	}

	/**
	 * Creates process element according the process figure.
	 * @param processFigure
	 * 			process figure. 
	 * @return process element.
	 */
	private Element createProcessElement(ProcessFigure processFigure) {
		Element process = DocumentHelper.createElement("ProcessFigure");
		String uuid = processFigure.getUuid();
		String locXY = processFigure.getLocXY();
		process.addAttribute("uuid", uuid);
		process.addAttribute("locXY", locXY);
		for (DataFigure figure : processFigure.getDataFigures()) {
			process.add(createDataElement(figure));
		}
		return process;
	}

	/**
	 * Creates data element according the data figure.
	 * @param dataFigure 
	 * 			data figure.
	 * @return
	 * 			data element.
	 */
	private Element createDataElement(DataFigure dataFigure) {
		Element element = DocumentHelper.createElement("DataFigure");
		String uuid = dataFigure.getUuid();
		if (uuid == null) {
			uuid = "";
		}
		element.addAttribute("uuid", uuid);
		String locXY = dataFigure.getLocXY();
		if (locXY == null) {
			locXY = "";
		}
		element.addAttribute("locXY", locXY);
		String state = dataFigure.getState();
		if (state == null) {
			state = dataFigure.getState();
		}
		element.addAttribute("state", state);
		return element;
	}

	/**
	 * 
	 * @param flowFigure
	 * 			flow figure.
	 * @return
	 * 			flow element.
	 */
	private Element createFlowElement(FlowFigure flowFigure) {
		String uuid = flowFigure.getUuid();
		Element flow = DocumentHelper.createElement("FlowFigure");
		if (uuid == null) {
			uuid = "";
		}
		flow.addAttribute("uuid", uuid);
		return flow;
	}
}
