/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.figure.utils;

import java.awt.Graphics;
import java.util.ArrayList;
import java.util.List;

import cn.edu.whu.lmars.figure.beans.AbstractFigure;
import cn.edu.whu.lmars.figure.beans.DataFigure;
import cn.edu.whu.lmars.figure.beans.FlowFigure;
import cn.edu.whu.lmars.figure.beans.ProcessFigure;

/**
 * A figure container. All figures are stored in this container.
 * 
 * @author Mingda Zhang
 * 
 */
public class FigureStore {

	/** Process Figures */
	private List<ProcessFigure> processFigures = new ArrayList<ProcessFigure>();
	/** Flow Figures */
	private List<FlowFigure> flowFigures = new ArrayList<FlowFigure>();

	/**
	 * Gets ProcessFigures.
	 */
	public List<ProcessFigure> getProcessFigures() {
		return this.processFigures;
	}

	/**
	 * Gets FlowFigures.
	 */
	public List<FlowFigure> getFlowFigures() {
		return this.flowFigures;
	}

	/**
	 * Adds a FlowFigure to this container.
	 */
	public void addFlowFigure(FlowFigure flow) {
		if (!this.flowFigures.contains(flow)) {
			this.flowFigures.add(flow);
		}
	}

	/**
	 * Removes a FlowFigure from this container.
	 */
	public void removeFlowFigure(FlowFigure flow) {
		if (this.flowFigures.contains(flow)) {
			this.flowFigures.remove(flow);
		}
	}

	/**
	 * Adds a Process Figure to this container.
	 */
	public void addProcessFigure(ProcessFigure figure) {
		if (!this.processFigures.contains(figure)) {
			this.processFigures.add(figure);
		}
	}

	/**
	 * Removes a Process Figure from this container.
	 */
	public void removeProcessFigure(ProcessFigure figure) {
		if (this.processFigures.contains(figure)) {
			this.processFigures.remove(figure);
		}
	}

	/**
	 * Gets all figures in this container.
	 */
	public List<AbstractFigure> getAllFigures() {
		List<AbstractFigure> allFigures = new ArrayList<AbstractFigure>();
		ProcessFigure process;
		for (AbstractFigure figure : this.getFlowFigures()) {
			allFigures.add(figure);
		}

		for (AbstractFigure processFigure : this.getProcessFigures()) {
			allFigures.add(processFigure);
			process = (ProcessFigure) processFigure;
			for (DataFigure dataFigure : process.getDataFigures()) {
				if (dataFigure.getState().equals("res")
						|| dataFigure.getState().equals("val")) {
					allFigures.add(dataFigure);
				}
			}

			for (AbstractFigure linkFigure : process.getLinkFigures()) {
				allFigures.add(linkFigure);
			}
		}

		return allFigures;
	}

	/**
	 * Draws all the figure to Graphics.
	 * 
	 * @param g
	 */
	public void draw(Graphics g) {
		for (AbstractFigure figure : getAllFigures()) {
			figure.draw(g);
		}
	}

	/**
	 * Gets Process Figure by id from this container.
	 */
	public ProcessFigure getProcessFigureByUuid(String uuid) {
		for (ProcessFigure figure : getProcessFigures()) {
			if (figure.getUuid().equals(uuid)) {
				return figure;
			}
		}
		return null;
	}

	/**
	 * Clears this container.
	 */
	public void clear() {
		this.getFlowFigures().clear();
		this.getProcessFigures().clear();
	}

	/**
	 * Gets Flow Figure by its begin id from this container.
	 */
	public FlowFigure getFlowFigureByBeginUuid(String uuid) {
		for (FlowFigure figure : this.getFlowFigures()) {
			if (figure.getUuid().startsWith(uuid)) {
				return figure;
			}
		}
		return null;
	}

	/**
	 * Gets Flow Figure by end id from this container.
	 */
	public List<FlowFigure> getFlowFiguresByEndUuid(String uuid) {
		List<FlowFigure> flowFigures = new ArrayList<FlowFigure>(3);
		for (FlowFigure figure : this.getFlowFigures()) {
			if (figure.getUuid().endsWith(uuid)) {
				flowFigures.add(figure);
			}
		}
		return flowFigures;
	}

	/**
	 * Gets FlowFigure by contained string from this container.
	 */
	public List<FlowFigure> getFlowFiguresByContainsStr(String str) {
		List<FlowFigure> figures = new ArrayList<FlowFigure>(3);
		for (FlowFigure figure : this.getFlowFigures()) {
			if (figure.getUuid().contains(str)) {
				figures.add(figure);
			}
		}
		return figures;
	}

	/**
	 * Sets Process Figures.
	 */
	public void setProcessFigures(List<ProcessFigure> figures) {
		this.processFigures = figures;
	}

	/**
	 * Sets Flow Figures.
	 */
	public void setFlowFigures(List<FlowFigure> figures) {
		this.flowFigures = figures;
	}
}
