/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.figure.utils;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.font.LineMetrics;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileNameExtensionFilter;

import org.apache.log4j.Logger;

import conf.GlobeParam;

import cn.edu.whu.lmars.commands.FlowCommand;
import cn.edu.whu.lmars.commands.MergeCommand;
import cn.edu.whu.lmars.commands.MoveCommand2;
import cn.edu.whu.lmars.commands.ReSizeCommand;
import cn.edu.whu.lmars.commands.ReSizeCommand2;
import cn.edu.whu.lmars.drawpanel.ui.JDrawPanel;
import cn.edu.whu.lmars.figure.beans.AbstractFigure;
import cn.edu.whu.lmars.figure.beans.DataFigure;
import cn.edu.whu.lmars.figure.beans.GeoMarkFigure;
import cn.edu.whu.lmars.figure.beans.GeoNodeFigure;
import cn.edu.whu.lmars.figure.beans.GeoPointFigure;
import cn.edu.whu.lmars.figure.beans.LinkFigure;
import cn.edu.whu.lmars.figure.beans.ProcessFigure;
import cn.edu.whu.lmars.locale.LocaleKey;
import cn.edu.whu.lmars.locale.ResourceBundleFactory;
import cn.edu.whu.lmars.model.AbstractParam;
import cn.edu.whu.lmars.model.util.PureModelStore;
import cn.edu.whu.lmars.utils.HandleTool;
import cn.edu.whu.lmars.utils.Judger;

/**
 * This class offers methods to perform figure operation.
 * 
 * @author Mingda Zhang
 * 
 */
public class FigureUtility {
	private Logger logger = Logger.getLogger(FigureUtility.class);

	/** Zoom change size */
	private int changeX = 6, changeY = 4;
	private int changeZ = (new Double(Math.sqrt(changeX * changeX + changeY
			* changeY)).intValue()) * 2;
	/** judger used to select figure */
	Judger judger = Judger.getInstance();
	private static FigureUtility instance = null;

	/**
	 * Gets an instance of FigureUtility.
	 * 
	 * @return
	 */
	public static FigureUtility getInstance() {
		if (instance == null) {
			instance = new FigureUtility();
		}
		return instance;
	}

	private FigureUtility() {
	}

	/**
	 * Changes figures to be unselected.
	 * 
	 * @param figures
	 *            target figures
	 */
	public void clearAllChoosen(List<AbstractFigure> figures) {
		AbstractFigure figure = null;
		for (int i = 0; i < figures.size(); i++) {
			figure = figures.get(i);
			if (figure.isChoosen()) {
				figure.setChoosen(false);
			}
		}
	}

	/**
	 * Returns the center location of the Figure.
	 * @param nodeFigure
	 * @return
	 */
	public GeoPointFigure getFigureCenter(GeoNodeFigure nodeFigure){
		int x = nodeFigure.getX();
		int y = nodeFigure.getY();
		int width = nodeFigure.getWidth();
		int height = nodeFigure.getHeight();
		int centerX,centerY;
		centerX = x+width/2;
		centerY = y+height/2;
		return new GeoPointFigure(centerX, centerY);
		
	}
	/**
	 * Changes figures to be unselected besides specific figure.
	 * 
	 * @param figure
	 *            keeps selected figure.
	 * @param figures
	 *            figures that should change state to unselected.
	 */
	public void clearOtherChoosen(AbstractFigure figure,
			List<AbstractFigure> figures) {
		AbstractFigure figure1 = null;
		/*
		 * FigureStore figureStore = drawPanel.getFigureStore();
		 * List<AbstractFigure> figures = figureStore.getAllFigures();
		 */
		for (int i = 0; i < figures.size(); i++) {
			figure1 = figures.get(i);
			if (figure1 == figure) {
				continue;
			}
			figure1.setChoosen(false);
		}
	}

	/**
	 * Resizes the JDrawPanel so that it can show all figures.
	 */
	public void reSizeDrawArea(JDrawPanel drawPanel) {

		Dimension panelArea = new Dimension(drawPanel.getWidth(),
				drawPanel.getHeight());
		GeoNodeFigure nodeFigure = null;
		List<AbstractFigure> figures = drawPanel.getFigureStore()
				.getAllFigures();
		int x = 0, y = 0, width = 0, height = 0;
		int maxX = 0, maxY = 0;
		if (figures.size() == 0) {
			return;
		}
		for (AbstractFigure figure : figures) {
			if (figure instanceof LinkFigure) {
				continue;
			}
			nodeFigure = (GeoNodeFigure) figure;
			x = nodeFigure.getX();
			y = nodeFigure.getY();
			width = nodeFigure.getWidth();
			height = nodeFigure.getHeight();

			if (maxX < x + width) {
				maxX = x + width;
			}
			if (maxY < y + height) {
				maxY = y + height;
			}
		}

		if (maxX <= panelArea.getWidth() && maxY <= panelArea.getHeight()) {
			return;
		}
		if (maxX > panelArea.getWidth()) {
			panelArea.setSize(maxX, panelArea.getHeight());
		}
		if (maxY > panelArea.getHeight()) {
			panelArea.setSize(panelArea.getWidth(), maxY);
		}
		int width1 = (int) panelArea.getWidth();

		int height1 = (int) panelArea.getHeight();
		Rectangle rect = new Rectangle(width1 - 2, height1 - 2, width1, height1);
		drawPanel.scrollRectToVisible(rect);

		drawPanel.setPreferredSize(panelArea);
		drawPanel.revalidate();
		drawPanel.repaint();
	}

	/**
	 * Gets selected figures from target figure list.
	 * 
	 * @param figureList
	 *            target figure list
	 * @return
	 */
	public ArrayList<AbstractFigure> getChoosen(List<AbstractFigure> figureList) {
		ArrayList<AbstractFigure> figures = new ArrayList<AbstractFigure>();
		for (int i = 0; i < figureList.size(); i++) {
			if (figureList.get(i).isChoosen()) {
				figures.add(figureList.get(i));
			}
		}
		return figures;
	}

	/**
	 * Gets selected Mark Figure when mouse moves.
	 * 
	 * @param x
	 *            x coordinate value of the mouse moves
	 * @param y
	 *            y coordinate value of the mouse moves
	 * @param drawPanel
	 *            the current JDrawPanel
	 * @return
	 */
	public GeoMarkFigure getMark(int x, int y, JDrawPanel drawPanel) {

		List<AbstractFigure> figures = drawPanel.getFigureStore()
				.getAllFigures();
		boolean flag = false;
		GeoMarkFigure zSquare = null;
		AbstractFigure figure = null;
		GeoMarkFigure[] marks = null;
		for (int i = 0; i < figures.size(); i++) {
			figure = figures.get(i);
			if (!figure.isChoosen()) {
				continue;
			}
			if (figure instanceof LinkFigure) {
				continue;
			}
			marks = ((GeoNodeFigure) figure).getMarks();
			for (int j = 0; j < marks.length; j++) {
				if (judger.rectPoint(marks[j], x, y)) {
					drawPanel.setCursor(new Cursor(Cursor.HAND_CURSOR));
					flag = true;
					return marks[j];
				}
			}
		}
		if (!flag) {
			drawPanel.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}
		return zSquare;
	}

	/**
	 * Gets a specific figure by location.
	 * 
	 * @param x
	 *            x coordinate value.
	 * @param y
	 *            y coordinate value.
	 * @param figureList
	 *            figures to search from.
	 * @return
	 */
	public AbstractFigure getFigureByXY(int x, int y,
			List<AbstractFigure> figureList) {
		AbstractFigure figure = null;
		for (int i = 0; i < figureList.size(); i++) {
			figure = figureList.get(i);
			if (figure instanceof LinkFigure) {
				LinkFigure flowFigure = (LinkFigure) figure;
				if (judger.linePoint(flowFigure, x, y)) {
					return flowFigure;
				}
				continue;
			}
			if (figure instanceof GeoNodeFigure) {
				GeoNodeFigure nodeFigure = (GeoNodeFigure) figure;
				if (judger.rectPoint(nodeFigure, x, y)) {
					return nodeFigure;
				}
			}
		}
		return null;
	}

	/**
	 * Gets a selected figure by location from figures.
	 * 
	 * @param x
	 *            x coordinate value.
	 * @param y
	 *            y coordinate value.
	 * @param figures
	 *            figures to search from
	 * @return
	 */
	private AbstractFigure getChoosenFigureByXY(int x, int y,
			List<AbstractFigure> figures) {
		AbstractFigure figure = null;
		for (int i = 0; i < figures.size(); i++) {
			figure = figures.get(i);
			if (figure instanceof GeoNodeFigure) {
				GeoNodeFigure nodeFigure = (GeoNodeFigure) figure;
				if (judger.rectPoint(nodeFigure, x, y)
						&& nodeFigure.isChoosen()) {
					return nodeFigure;
				}
			}
		}
		return null;
	}

	/**
	 * Searches Process Figure by location from figures.
	 * @param press_x
	 * 			x coordinate value.
	 * @param press_y
	 * 			y coordinate value.
	 * @param figures
	 * 			 figures to search from.
	 * @return
	 */
	public ProcessFigure getHandleFigure(int press_x, int press_y,
			List<AbstractFigure> figures) {
		AbstractFigure figure = getFigureByXY(press_x, press_y, figures);
		if (figure == null) {
			return null;
		}
		if (figure instanceof ProcessFigure) {
			return (ProcessFigure) figure;
		}
		return null;
	}

	/**
	 * Searches Data Figure from figures by location.
	 * 
	 * @param drag_x
	 *            x coordinate value.
	 * @param drag_y
	 *            y coordinate value.
	 * @param figureList
	 *            figures to search from.
	 * @return
	 */
	public DataFigure getResultDataFigure(int drag_x, int drag_y,
			List<AbstractFigure> figureList) {
		AbstractFigure figure = getFigureByXY(drag_x, drag_y, figureList);
		if (figure == null) {
			return null;
		}
		if (figure instanceof DataFigure) {
			return (DataFigure) figure;
		}
		return null;
	}

	/**
	 * Creates Flow Figure.
	 * 
	 * @param drawPanel
	 * 			the current JDrawPanel.
	 */
	public void createSpeFlowFigure(JDrawPanel drawPanel) {
		ResourceBundle rBundle = ResourceBundleFactory.getResourceBundle();
		List<AbstractFigure> figureList = drawPanel.getFigureStore()
				.getAllFigures();
		String msg;
		ProcessFigure handleFigure = getHandleFigure(drawPanel.getRelease_x(),
				drawPanel.getRealse_y(), figureList);

		DataFigure dataFigure = getResultDataFigure(drawPanel.getPress_x(),
				drawPanel.getPress_y(), figureList);
		if (handleFigure == null || dataFigure == null
				|| !dataFigure.getState().equals("res")) {
			GlobeParam.TEMPORARY_FIGURES.clear();
			drawPanel.repaint();
			msg = rBundle
					.getString(LocaleKey.Dialog_Msg_Create_Flow_Wrong_Start);
			JOptionPane.showMessageDialog(GlobeParam.getClient(), msg);
			logger.info(msg);
			return;
		}

		String fromID = dataFigure.getUuid();
		String fromUuid = fromID.split("#")[0];

		String toUuid = handleFigure.getUuid();
		if (fromUuid.equals(toUuid)) {
			msg = rBundle.getString(LocaleKey.Dialog_Msg_Create_Flow_Wrong_End);
			JOptionPane.showMessageDialog(null, msg);
			return;
		}

		FigureStore figureStore = drawPanel.getFigureStore();
		PureModelStore modelStore = drawPanel.getModelStore();
		ProcessFigure toProcessFigure = figureStore
				.getProcessFigureByUuid(toUuid);
		List<String> parameters = getNulStateFigure(toProcessFigure);
		int num = parameters.size();
		if (num == 0) {
			msg = rBundle
					.getString(LocaleKey.Dialog_Msg_Create_Flow_Wrong_Merge);
			JOptionPane.showMessageDialog(GlobeParam.getClient(), msg);
			return;
		}
		// Select a parameter to bind
		String[] choices = new String[num];
		for (int i = 0; i < num; i++) {
			choices[i] = parameters.get(i);
		}
		String title = rBundle
				.getString(LocaleKey.Dialog_Input_Create_Flow_Title);
		msg = rBundle.getString(LocaleKey.Dialog_Input_Create_Flow_Msg);
		Object select = JOptionPane.showInputDialog(null, msg, title,
				JOptionPane.INFORMATION_MESSAGE, null, choices, choices[0]);
		if (select == null) {
			logger.info("Failed to create flow figure because of not binding a parameter.");
			return;
		}
		String identifier = select.toString().trim();
		logger.info("Binding parameter is " + select.toString());

		FlowCommand command = new FlowCommand(modelStore, figureStore,
				dataFigure, handleFigure, identifier);
		command.execute();
		GlobeParam.getCurrentJDrawPanel().setChanged(true);
		drawPanel.getCommandStack().addCommand(command);
	}

	/**
	 * Gets parameter names with 'nul' state from selected process figure.
	 * 
	 * @param toProcessFigure
	 * @return
	 */
	private List<String> getNulStateFigure(ProcessFigure toProcessFigure) {
		List<String> list = new ArrayList<String>(4);
		for (DataFigure dataFigure : toProcessFigure.getDataFigures()) {
			if (dataFigure.getState().equals("nul")) {
				list.add(((AbstractParam) dataFigure.getModel()).getName());
			}
		}
		return list;
	}

	/**
	 * Moves a single figure.
	 * 
	 * @param choosenFigure
	 *            the figure to be moved.
	 * @param startPoint
	 *            start point of motion.
	 * @param endPoint
	 *            end point of motion.
	 * @param drawPanel
	 *            the current JDrawPanel.
	 * @return
	 */
	public boolean moveFigure(AbstractFigure choosenFigure,
			GeoPointFigure startPoint, GeoPointFigure endPoint,
			JDrawPanel drawPanel) {
		boolean flag1 = false;
		GeoNodeFigure node = null;

		if (choosenFigure == null || choosenFigure instanceof LinkFigure) {
			return false;
		}

		List<AbstractFigure> figureList = drawPanel.getFigureStore()
				.getAllFigures();
		AbstractFigure targetFigure = getChoosenFigureByXY(GlobeParam
				.getCurrentJDrawPanel().getDrag_x(), GlobeParam
				.getCurrentJDrawPanel().getDrag_y(), figureList);

		if (!getChoosen(figureList).contains(targetFigure)) {
			return false;
		}

		node = (GeoNodeFigure) choosenFigure;
		int x = endPoint.getX() - startPoint.getX();
		int y = endPoint.getY() - startPoint.getY();

		int startX = 0, startY = 0;
		startX = node.getX() + x;
		startY = node.getY() + y;

		node.resize(startX, startY, node.getWidth(), node.getHeight());
		ArrayList<LinkFigure> flows = getRelatedFlows(node, figureList);
		adjustFlowFigures(flows);
		flag1 = true;
	//	reSizeDrawArea(drawPanel);
		return flag1;
	}

	public void moveFigure(ArrayList<AbstractFigure> figures,
			GeoPointFigure startPoint, GeoPointFigure endPoint,
			JDrawPanel drawPanel) {
		for (int i = 0; i < figures.size(); i++) {
			if (moveFigure(figures.get(i), startPoint, endPoint, drawPanel)) {
				GlobeParam.getCurrentJDrawPanel().setMoved(true);
			}
		}

	}

	/**
	 * This method is similar with 'moveFigure',but this method will create a
	 * command which supports redo and undo.
	 * 
	 * @param figures
	 * @param startPoint
	 * @param endPoint
	 * @param drawPanel
	 */
	public void moveFigure2(ArrayList<AbstractFigure> figures,
			GeoPointFigure startPoint, GeoPointFigure endPoint,
			JDrawPanel drawPanel) {
		List<AbstractFigure> figureList = drawPanel.getFigureStore()
				.getAllFigures();
		for (AbstractFigure figure : figures) {
			GeoNodeFigure node = null;

			if (figure == null || figure instanceof LinkFigure) {
				continue;
			}

			node = (GeoNodeFigure) figure;
			int x = endPoint.getX() - startPoint.getX();
			int y = endPoint.getY() - startPoint.getY();

			int startX = 0, startY = 0;
			startX = node.getX() + x;
			startY = node.getY() + y;

			node.resize(startX, startY, node.getWidth(), node.getHeight());
			ArrayList<LinkFigure> flows = getRelatedFlows(node, figureList);
			adjustFlowFigures(flows);
		}
		reSizeDrawArea(drawPanel);

	}

	/**
	 * Moves figures.
	 * 
	 * @param figures
	 *            figures to be moved.
	 * @param startPoint
	 *            start point of motion.
	 * @param endPoint
	 *            end point of motion.
	 */
	public void moveFigureEnd(ArrayList<AbstractFigure> figures,
			GeoPointFigure startPoint, GeoPointFigure endPoint) {
		if (!GlobeParam.getCurrentJDrawPanel().isMoved()) {
			return;
		}
		JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
		MoveCommand2 command2 = new MoveCommand2(figures, startPoint, endPoint,
				drawPanel);
		drawPanel.setChanged(true);
		drawPanel.getCommandStack().addCommand(command2);
	}

	/**
	 * Resizes figure,but the operation will not be saved.
	 * 
	 * @param mark
	 *            the selected mark figure.
	 * @return
	 */
	public boolean resizeFigure(GeoMarkFigure mark) {

		if (mark == null) {
			return false;
		}
		AbstractFigure choosenFigure = mark.getOwner();
		GeoNodeFigure node = null;

		if (choosenFigure instanceof LinkFigure) {
			return false;
		}
		node = (GeoNodeFigure) choosenFigure;
		JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
		ReSizeCommand command = new ReSizeCommand(node, mark.getDirection(),
				drawPanel);
		return command.execute();

	}

	/**
	 * Resizes a figure and create a ReSizeCommand2 which will be saved to command stack.
	 * 
	 * @param mark
	 *            the selected mark figure.
	 * @param drawPanel
	 *            the current JDrawPanel
	 * @return
	 */
	public boolean resizeFigureEnd(GeoMarkFigure mark, JDrawPanel drawPanel) {
		if (mark == null) {
			return false;
		}
		AbstractFigure choosenFigure = mark.getOwner();
		GeoNodeFigure node = null;

		if (choosenFigure instanceof LinkFigure) {
			return false;
		}
		node = (GeoNodeFigure) choosenFigure;

		ReSizeCommand2 command = new ReSizeCommand2(node, mark.getDirection(),
				drawPanel);
		drawPanel.getCommandStack().addCommand(command);
		drawPanel.setChanged(true);
		drawPanel.setChanged(true);
		return command.execute();

	}

	/**
	 * Zoom in figures.
	 * 
	 * @param drawPanel
	 *            the current JDrawPanel.
	 */
	public void zoomIn(JDrawPanel drawPanel) {
		ProcessFigure handleFigure = null;
		List<AbstractFigure> figureList = drawPanel.getFigureStore()
				.getAllFigures();
		for (AbstractFigure figure : figureList) {
			if (!(figure instanceof ProcessFigure)) {
				continue;
			}
			handleFigure = (ProcessFigure) figure;
			zoomHandleFigure(handleFigure, changeX, changeY, changeZ, drawPanel);
		}
		GlobeParam.getCurrentJDrawPanel().repaint();
		reSizeDrawArea(drawPanel);
	}

	/**
	 * Zooms in a process figure.
	 * @param handleFigure
	 * @param changeX
	 * @param changeY
	 * @param changeZ
	 * @param drawPanel
	 */
	private void zoomHandleFigure(ProcessFigure handleFigure, int changeX,
			int changeY, int changeZ, JDrawPanel drawPanel) {
		// GeoNodeFigure
		List<AbstractFigure> figureList = drawPanel.getFigureStore()
				.getAllFigures();
		int distance = 0, distanceX = 0, distanceY = 0;
		int anotherX, anotherY, handleX, handleY, anotherWidth, anotherHeight, handleWidth, handleHeigth;
		GeoNodeFigure anotherFigure;
		ArrayList<LinkFigure> flows = getRelatedFlows(handleFigure, figureList);
		GeoNodeFigure startFigure, endFigure;
		handleWidth = handleFigure.getWidth();
		handleHeigth = handleFigure.getHeight();
		handleX = handleFigure.getX() + handleWidth / 2;
		handleY = handleFigure.getY() + handleHeigth / 2;
		handleWidth += changeX;
		handleHeigth += changeY;
		handleFigure.resize(handleX - handleWidth / 2, handleY - handleHeigth
				/ 2, handleWidth, handleHeigth);
		for (LinkFigure flow : flows) {
			startFigure = flow.getStartFigure();
			endFigure = flow.getEndFigure();
			if (startFigure == handleFigure) {
				anotherFigure = endFigure;
			} else {
				anotherFigure = startFigure;
			}
			anotherWidth = anotherFigure.getWidth();
			anotherHeight = anotherFigure.getHeight();

			// get the central point
			anotherX = anotherFigure.getX() + anotherWidth / 2;
			anotherY = anotherFigure.getY() + anotherHeight / 2;

			distance = new Double(Math.sqrt((anotherX - handleX)
					* (anotherX - handleX) + (anotherY - handleY)
					* (anotherY - handleY))).intValue();
			distanceX = (anotherX - handleX) * (distance + changeZ) / distance;
			distanceY = (anotherY - handleY) * (distance + changeZ) / distance;
			// changeX = new Double(anotherFigure.getWidth()*innum).intValue();
			// changeY = new Double(anotherFigure.getHeight()*innum).intValue();

			anotherX = handleX + distanceX;
			anotherY = handleY + distanceY;
			anotherWidth = anotherWidth + changeX;
			anotherHeight = anotherHeight + changeY;

			anotherFigure.resize(anotherX - anotherWidth / 2, anotherY
					- anotherHeight / 2, anotherWidth, anotherHeight);
		}

	}

	/**
	 * Zoom out figures
	 * 
	 * @param drawPanel
	 *            the current JDrawPanel
	 */

	public void zoomOut(JDrawPanel drawPanel) {
		ProcessFigure handleFigure = null;
		List<AbstractFigure> figureList = drawPanel.getFigureStore()
				.getAllFigures();
		// int width = 0, height = 0;
		for (AbstractFigure figure : figureList) {
			if (!(figure instanceof ProcessFigure)) {
				continue;
			}
			handleFigure = (ProcessFigure) figure;
			zoomHandleFigure(handleFigure, -changeX, -changeY, -changeZ,
					drawPanel);
		}
		GlobeParam.getCurrentJDrawPanel().repaint();
		reSizeDrawArea(drawPanel);
	}

	/**
	 * Resets Temporary figure
	 */
	public void reSet() {
		GlobeParam.getCurrentJDrawPanel().setMark(null);
		if (GlobeParam.TEMPORARY_FIGURES.size() > 0) {
			GlobeParam.TEMPORARY_FIGURES.clear();
		}
	}

	/**
	 * Gets flow figures related to GeoNodeFigure from figure list
	 * 
	 * @param targetFigure
	 * @param figures
	 * @return
	 */
	public ArrayList<LinkFigure> getRelatedFlows(GeoNodeFigure targetFigure,
			List<AbstractFigure> figures) {

		ArrayList<LinkFigure> flows = new ArrayList<LinkFigure>();
		AbstractFigure figure;
		LinkFigure flowFigure;
		GeoNodeFigure startFigure, endFigure;
		for (int i = 0; i < figures.size(); i++) {
			figure = figures.get(i);
			if (figure instanceof LinkFigure) {
				flowFigure = (LinkFigure) figure;
				startFigure = flowFigure.getStartFigure();
				endFigure = flowFigure.getEndFigure();
				if (startFigure == targetFigure || endFigure == targetFigure) {
					flows.add(flowFigure);
				}
			}
		}
		return flows;
	}

	/**
	 * Adjusts flow figures' start point or end point.
	 * 
	 * @param flowFigures
	 */
	public void adjustFlowFigures(List<LinkFigure> flowFigures) {
		LinkFigure flowFigure;
		for (int i = 0; i < flowFigures.size(); i++) {
			flowFigure = flowFigures.get(i);
			adjustFlowFigure(flowFigure);
		}
	}

	/**
	 * Adjusts all the figures
	 * 
	 * @param figures
	 */
	public void adjustFigures(List<AbstractFigure> figures) {
		LinkFigure flowFigure;
		for (AbstractFigure figure : figures) {
			if (figure instanceof LinkFigure) {
				flowFigure = (LinkFigure) figure;
				adjustFlowFigure(flowFigure);
			}
		}
	}

	/**
	 * Adjust a single flow figure's start point or end point.
	 * 
	 * @param flowFigure
	 */
	public void adjustFlowFigure(LinkFigure flowFigure) {
		if (flowFigure == null) {
			logger.error("The flow figure is null");
			return;
		}

		GeoNodeFigure startFigure = flowFigure.getStartFigure();
		GeoNodeFigure endFigure = flowFigure.getEndFigure();

		int startX = startFigure.getX();
		int startY = startFigure.getY();
		int startWidth = startFigure.getWidth();
		int startHeight = startFigure.getHeight();
		int endX = endFigure.getX();
		int endY = endFigure.getY();
		int endWidth = endFigure.getWidth();
		int endHeight = endFigure.getHeight();

		if ((startX + startWidth) < endX) {
			flowFigure.setStartP(startFigure.pointR);
			flowFigure.setEndP(endFigure.pointL);
			return;
		}

		if ((endX + endWidth) < startX) {
			flowFigure.setStartP(startFigure.pointL);
			flowFigure.setEndP(endFigure.pointR);
			return;
		}

		if ((startY + startHeight) > endY) {
			flowFigure.setStartP(startFigure.pointU);
			flowFigure.setEndP(endFigure.pointD);
			return;
		}

		if ((endY + endHeight) > startY) {
			flowFigure.setStartP(startFigure.pointD);
			flowFigure.setEndP(endFigure.pointU);
			return;
		}
	}

	/**
	 * Exports the current JDrawPanel to an image.
	 * 
	 * @param panel
	 *            the JDrawPanel
	 * @return
	 */
	public boolean exprotImage(JDrawPanel panel) {

		boolean flag = true;
		JFileChooser fileChooser = new JFileChooser();
		FileNameExtensionFilter filter = new FileNameExtensionFilter(
				"JPG & PNG Images", "jpg", "png");
		fileChooser.setFileFilter(filter);
		fileChooser.setAcceptAllFileFilterUsed(false);
		boolean againflag = false;
		ResourceBundle rBundle = ResourceBundleFactory.getResourceBundle();
		String msg;
		int returnVal = fileChooser.showSaveDialog(GlobeParam.getClient());

		if (returnVal == JFileChooser.APPROVE_OPTION) {
			File file = fileChooser.getSelectedFile();
			String path = file.toString();
			if (!path.toLowerCase().endsWith(".jpg")
					&& !path.toLowerCase().endsWith(".png")) {
				msg = rBundle
						.getString(LocaleKey.Dialog_Msg_Export_Wrong_Format);
				JOptionPane.showMessageDialog(GlobeParam.getClient(), msg);
				againflag = true;
			}
			if (file.exists()) {
				msg = rBundle.getString(LocaleKey.Dialog_Msg_File_Exist_Msg);
				String title = rBundle
						.getString(LocaleKey.Dialog_Msg_File_Exist_Title);
				int flag2 = JOptionPane.showConfirmDialog(
						GlobeParam.getClient(), msg, title,
						JOptionPane.YES_NO_OPTION);
				if (flag2 == JOptionPane.NO_OPTION) {
					againflag = true;
				}
			}

			if (!againflag) {
				BufferedImage bi = (BufferedImage) panel.createImage(
						panel.getWidth(), panel.getHeight());
				panel.paint(bi.getGraphics());
				try {

					javax.imageio.ImageIO.write(bi, "jpg", file);
				} catch (IOException e1) {
					flag = false;
					e1.printStackTrace();
				}
			} else {
				flag = exprotImage(panel);
			}

		}
		return flag;
	}

	/**
	 * Merges a result figure and an input data figure. Creates a flow figure at
	 * the same time.
	 * 
	 * @param drawPanel
	 * 			the current JDrawPanel.
	 */
	public boolean mergeFigure(JDrawPanel drawPanel) {
		List<AbstractFigure> figureList = drawPanel.getFigureStore()
				.getAllFigures();
		ResourceBundle rBundle = ResourceBundleFactory.getResourceBundle();
		String msg;
		ArrayList<AbstractFigure> figures = getChoosen(figureList);
		if (figures.size() != 2) {
			msg = rBundle.getString(LocaleKey.Dialog_Msg_Merge_Wrong_Two);
			JOptionPane.showMessageDialog(GlobeParam.getClient(), msg);
			return false;
		}
		DataFigure resultFigure = getResultDataFigure(figures);

		if (resultFigure == null) {
			msg = rBundle.getString(LocaleKey.Dialog_Msg_Merge_Wrong_Result);
			JOptionPane.showMessageDialog(GlobeParam.getClient(), msg);
			return false;
		}
		DataFigure dataFigure = null;
		for (AbstractFigure figure2 : figures) {
			if (figure2 == resultFigure) {
				continue;
			}
			if (figure2 instanceof DataFigure) {
				dataFigure = (DataFigure) figure2;
			}
		}
		if (dataFigure == null) {
			msg = rBundle.getString(LocaleKey.Dialog_Msg_Merge_Wrong_Data);
			JOptionPane.showMessageDialog(GlobeParam.getClient(),msg);
			return false;
		}

		FigureStore figureStore = drawPanel.getFigureStore();
		PureModelStore modelStore = drawPanel.getModelStore();
		String fromID = resultFigure.getUuid();
		String toID = dataFigure.getUuid();
		String fromUuid = fromID.split("#")[0];
		String toUuid = toID.split("#")[0];

		if (fromUuid.equals(toUuid)) {
			msg = rBundle.getString(LocaleKey.Dialog_Msg_Merge_Wrong_Deffer);
			JOptionPane.showMessageDialog(null, msg);
		}

		MergeCommand command = new MergeCommand(modelStore, figureStore,
				resultFigure, dataFigure);
		command.execute();
		drawPanel.getCommandStack().addCommand(command);
		HandleTool.refreshFigShow();
		drawPanel.setChanged(true);
		return true;
	}

	/**
	 * Gets Result data figure from figure list.
	 * @param figures
	 */
	private DataFigure getResultDataFigure(ArrayList<AbstractFigure> figures) {
		DataFigure dataFigure = null;
		for (AbstractFigure figure : figures) {
			if (figure instanceof DataFigure) {
				dataFigure = (DataFigure) figure;
				if (dataFigure.getState().equals("res")) {
					return dataFigure;
				}
			}
		}
		return null;
	}
	
	/**
	 * Gets a arrow.
	 * @param x1
	 * @param y1
	 * @param x2
	 * @param y2
	 * @return
	 */
	public int[] getArrows(int x1, int y1, int x2, int y2) {
		int[] arrows = new int[4];
		double H = 10; // arrow height
		double L = 7; // 
		int x3 = 0;
		int y3 = 0;
		int x4 = 0;
		int y4 = 0;
		double awrad = Math.atan(L / H); // arrow angle
		double arraow_len = Math.sqrt(L * L + H * H); // arrow length
		double[] arrXY_1 = rotateVec(x2 - x1, y2 - y1, awrad, true, arraow_len);
		double[] arrXY_2 = rotateVec(x2 - x1, y2 - y1, -awrad, true, arraow_len);
		double x_3 = x2 - arrXY_1[0]; // (x3,y3) the first point
		double y_3 = y2 - arrXY_1[1];
		double x_4 = x2 - arrXY_2[0]; // (x4,y4) the second point
		double y_4 = y2 - arrXY_2[1];
		Double X3 = new Double(x_3);
		x3 = X3.intValue();
		Double Y3 = new Double(y_3);
		y3 = Y3.intValue();
		Double X4 = new Double(x_4);
		x4 = X4.intValue();
		Double Y4 = new Double(y_4);
		y4 = Y4.intValue();
		arrows[0] = x3;
		arrows[1] = y3;
		arrows[2] = x4;
		arrows[3] = y4;
		return arrows;
	}

	/**
	 * Gets the range of arrow.
	 */
	private double[] rotateVec(int px, int py, double ang, boolean isChLen,
			double newLen) {
		double mathstr[] = new double[2];
		double vx = px * Math.cos(ang) - py * Math.sin(ang);
		double vy = px * Math.sin(ang) + py * Math.cos(ang);
		if (isChLen) {
			double d = Math.sqrt(vx * vx + vy * vy);
			vx = vx / d * newLen;
			vy = vy / d * newLen;
			mathstr[0] = vx;
			mathstr[1] = vy;
		}
		return mathstr;
	}

	/**
	 * Moves all the figure right .
	 * @param figures
	 * 			figures to be moved.
	 * @param num
	 * 			motion size.
	 */
	public void moveAllRight(List<AbstractFigure> figures, int num) {
		GeoNodeFigure nodeFigure;
		for (AbstractFigure figure : figures) {
			if (figure instanceof LinkFigure) {
				continue;
			}
			nodeFigure = (GeoNodeFigure) figure;
			nodeFigure.resize(nodeFigure.getX() + num, nodeFigure.getY(),
					nodeFigure.getWidth(), nodeFigure.getHeight());
		}
	}

	/**
	 * Moves all the figure down .
	 * @param figures
	 * 			figures to be moved.
	 * @param num
	 * 			motion size.
	 */
	public void moveAllDown(List<AbstractFigure> figures, int num) {
		GeoNodeFigure nodeFigure;
		for (AbstractFigure figure : figures) {
			if (figure instanceof LinkFigure) {
				continue;
			}
			nodeFigure = (GeoNodeFigure) figure;
			nodeFigure.resize(nodeFigure.getX(), nodeFigure.getY() + num,
					nodeFigure.getWidth(), nodeFigure.getHeight());
		}
	}

	/**
	 * Paints string in a Node Figure in the middle.
	 * @param figure
	 * 			the GeoNodeFigure 
	 * @param g
	 * 			Graphics
	 * @param show
	 * 			string to be painted.
	 */
	public void drawStringInMiddle(GeoNodeFigure figure, Graphics g, String show) {
		g.setColor(Color.BLACK);
		FontMetrics fontMetrics = g.getFontMetrics();
		LineMetrics lineMetrics = fontMetrics.getLineMetrics(show, g);
		int length = 0, num = 0, startX = 0;
		int x = figure.getX();
		int y = figure.getY();
		int width = figure.getWidth();
		int height = figure.getHeight();
		char[] chars = show.toCharArray();
		int lineWidth = fontMetrics.charsWidth(chars, 0, chars.length);

		int lineHeight = (int) lineMetrics.getHeight();

		if (lineWidth < width) {
			startX = (width - lineWidth) / 2 + x;
			g.drawString(show, startX, y + height / 2 + 2);
		} else {
			// g.drawString(show, x + 6, y + height / 2);
			for (int i = 0; i < chars.length; i++) {
				length += fontMetrics.charWidth(chars[i]);
				if (length > (width - 6)) {
					num = i;
					break;
				}
			}
			// g.drawString(arg0, arg1, arg2)
			char[] chars1 = new char[num + 1];
			char[] chars2 = new char[chars.length - num - 1];
			System.arraycopy(chars, 0, chars1, 0, num + 1);
			System.arraycopy(chars, num + 1, chars2, 0, chars.length - num - 1);

			String str1 = new String(chars1);
			String str2 = new String(chars2);

			g.drawString(str1, x + 2, y + height / 2 + 2);
			lineWidth = fontMetrics.charsWidth(chars2, 0, chars2.length);

			if (lineWidth < width) {
				startX = (width - lineWidth) / 2 + x;
				g.drawString(str2, startX, y + height / 2 + lineHeight + 2 + 4);
			} else {
				g.drawString(str2, x, y + height / 2 + lineHeight + 2 + 4);
			}
		}
	}
}
