/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.frame.listeners;

import java.awt.Component;

import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JViewport;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import cn.edu.whu.lmars.drawpanel.ui.JDrawPanel;
import cn.edu.whu.lmars.frame.ui.ClientFrame;

/**
 * Defines a TabChangeListener which listens for JTabbedPane's ChangeEvents.
 * 
 * @author Mingda Zhang
 * 
 */
public class TabChangeListener implements ChangeListener {

	/** The last selected index of display JTabbedPanel. */
	private int foreTab = 1;
	/** ClientFrame this listener listens for. */
	private ClientFrame clientFrame;
	/** The JTabbedPanel to display WorldWindow and JDrawPanel. */
	private JTabbedPane displayJTabbedPanel;
	/** The JTabbedPanel Contains service list and data layer. */
	private JTabbedPane catalogJTabbedPanel;
	/** Instance of this listener. */
	private static TabChangeListener instance = null;
	/** Counter. */
	private static int tabNum = 1;

	private TabChangeListener(ClientFrame clientFrame) {
		this.clientFrame = clientFrame;
		this.displayJTabbedPanel = clientFrame.getDisplayJTabbedPanel();
		this.catalogJTabbedPanel = clientFrame.getCatalogJTabbedPanel();
	}

	/**
	 * Gets an instance of this listener.
	 * @param clientFrame
	 * 			Event source.
	 * @return
	 */
	public static TabChangeListener getInstance(ClientFrame clientFrame) {
		if (instance == null) {
			instance = new TabChangeListener(clientFrame);
		}
		return instance;
	}

	/**
	 * Keeps display JTabbedPanel and catalog JTabbedPanel in consistent.
	 * 
	 * @see javax.swing.event.ChangeListener#stateChanged(ChangeEvent)
	 */
	@Override
	public void stateChanged(ChangeEvent e) {
		Object scr = e.getSource();
		JTabbedPane tab;
		String name;

		if (scr instanceof JTabbedPane) {
			tab = (JTabbedPane) scr;
			name = tab.getName();
			int i = tab.getSelectedIndex();
			int num = tabNum % 2;
			if (num == 0) {
				tabNum++;
				return;
			}
			if (name.equals("left")) {
				if (i == 0) {
					foreTab = displayJTabbedPanel.getSelectedIndex();
					tabNum++;
					displayJTabbedPanel.setSelectedIndex(0);
				} else {
					if (foreTab < displayJTabbedPanel.getTabCount()
							&& foreTab > 1) {
						tabNum++;
						Component com = displayJTabbedPanel
								.getComponentAt(foreTab);
						JScrollPane scrollPane = (JScrollPane) com;
						JViewport viewport = scrollPane.getViewport();
						JDrawPanel drawPanel = (JDrawPanel) viewport
								.getComponent(0);
						clientFrame.setCurrentJDrawPanel(drawPanel);
						displayJTabbedPanel.setSelectedIndex(foreTab);
					} else {
						clientFrame.setCurrentJDrawPanel(clientFrame
								.getDefaultDrawPanel());
						tabNum++;
						displayJTabbedPanel.setSelectedIndex(1);
					}

				}
			} else {
				if (i == 0) {
					tabNum++;
					catalogJTabbedPanel.setSelectedIndex(0);
					// toWorldCom();
				} else {
					Component com = tab.getComponentAt(i);
					if (com instanceof JScrollPane) {
						JScrollPane scrollPane = (JScrollPane) com;
						JViewport viewport = scrollPane.getViewport();
						JDrawPanel drawPanel = (JDrawPanel) viewport
								.getComponent(0);
						clientFrame.setCurrentJDrawPanel(drawPanel);
					}
					if (catalogJTabbedPanel.getSelectedIndex() == 0) {
						tabNum++;
						catalogJTabbedPanel.setSelectedIndex(1);
					}
				}
			}
		}
	}

}
