/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.frame.ui;

import gov.nasa.worldwindx.examples.LayerPanel;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JToolBar;

import cn.edu.whu.lmars.drawpanel.ui.JDrawPanel;
import cn.edu.whu.lmars.figure.utils.FigureStore;
import cn.edu.whu.lmars.frame.listeners.GeoWindowListener;
import cn.edu.whu.lmars.frame.listeners.TabChangeListener;
import cn.edu.whu.lmars.frame.utils.LocTool;
import cn.edu.whu.lmars.frame.utils.TabsTool;
import cn.edu.whu.lmars.locale.LocaleKey;
import cn.edu.whu.lmars.locale.ResourceBundleFactory;
import cn.edu.whu.lmars.model.util.PureModelStore;
import cn.edu.whu.lmars.run.bind.beans.BindStore;
import cn.edu.whu.lmars.service.utils.AdapterInitThread;
import cn.edu.whu.lmars.starter.ui.WelcomeWin;
import cn.edu.whu.lmars.tree.ui.TreeScrollPanel;
import cn.edu.whu.lmars.utils.HandleTool;
import cn.edu.whu.lmars.worldwind.datatree.DataLayerPanel;
import cn.edu.whu.lmars.worldwind.ui.WWDPanel;
import conf.GlobeParam;
import conf.PathConf;

/**
 * An extended version of javax.swing.JFrame, the main user interface.
 * 
 * @author Mingda Zhang
 * 
 */
public class ClientFrame extends JFrame {
	private static final long serialVersionUID = 1L;
	private JPanel container = null;
	/** The JTabbedPanel to display WorldWindow and JDrawPanel. */
	private JTabbedPane displayJTabbedPanel = new JTabbedPane();
	/** The JTabbedPanel Contains service list and data layer. */
	private JTabbedPane catalogJTabbedPanel = new JTabbedPane();
	private Dimension canvasDim = new Dimension(600, 600);
	/** The default layer of the WorldWindow. */
	protected LayerPanel layerPanel;
	/** The JMenuBar. */
	private JMenuBar menuBar;
	/** The default JDrawPanel. */
	private JDrawPanel defaulJDrawPanel;
	/** The JToolBar. */
	private JToolBar toolBar;
	/** The current JDrawPanel. */
	private JDrawPanel currentJDrawPanel;
	/** The WWDPanel contains WorldWindow. */
	private WWDPanel appPanel;
	private ResourceBundle rBundle = null;
	/** The data layer panel contains data layers added by user. */
	private DataLayerPanel dataLayerPanel = null;
	/** The service container. */
	private TreeScrollPanel treeScrollPanel=null;
	
	/**
	 * Instantiate a JFrame.
	 * 
	 * @param title
	 *            the title of JFrame
	 */
	public ClientFrame(String title) {
		super(title);
		initialize();
	}

	/**
	 * initialize the jframe.
	 */
	public void initialize() {
		ImageIcon imageIcon = new ImageIcon(getClass().getResource(
				"/images/" + "icon_frame.png"));
		this.setIconImage(imageIcon.getImage());
		rBundle = ResourceBundleFactory.getResourceBundle();
		GlobeParam.setClient(this);
		WelcomeWin welcomeWin = WelcomeWin.getInstance();

		// this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setExtendedState(JFrame.MAXIMIZED_BOTH);

		// initialize MenuBar
		menuBar = new GeoMenuBar();
		this.setJMenuBar(menuBar);

		// initialize toolbar
		toolBar = new GeoToolBar();

		welcomeWin
				.setProMessage(rBundle.getString(LocaleKey.Starter_Init_Tree));
		welcomeWin.setProValue(30);

		// initialize tree
		treeScrollPanel = new TreeScrollPanel();
		
		//Initializes the adapters
		AdapterInitThread adapterInitThread = new AdapterInitThread(treeScrollPanel);
		adapterInitThread.start();
		
	//	treeScrollPanel.setPreferredSize(new Dimension(200, 600));
		welcomeWin.setProValue(50);
		welcomeWin.setProMessage(LocaleKey.Starter_Init_DrawPanel);

		// initialize the default drawpanel
		defaulJDrawPanel = new JDrawPanel();
		defaulJDrawPanel.setName(rBundle
				.getString(LocaleKey.JFrame_DrawPanel_Default_Name));
		defaulJDrawPanel.setUuid(GlobeParam.DEFAULT_UUID);
		defaulJDrawPanel.setFigureStore(new FigureStore());
		defaulJDrawPanel.setModelStore(new PureModelStore());
		defaulJDrawPanel.setPreferredSize(new Dimension(600, 600));

		JScrollPane rightJScrollPane = new JScrollPane();
		rightJScrollPane.setViewportView(defaulJDrawPanel);
		rightJScrollPane.setPreferredSize(new Dimension(600, 600));

		welcomeWin.setProValue(70);
		welcomeWin.setProMessage(rBundle
				.getString(LocaleKey.Starter_Init_WorldWind));
		try {
			appPanel = new WWDPanel(canvasDim, true);
			layerPanel = appPanel.getLayerPanel();
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null,"Error occured when load World Wind:\n"+e.getMessage());
			welcomeWin.stopPro();
			return;
		}
		
		

		JPanel wwdLayerPanel = new JPanel(new GridLayout(2, 1));
		wwdLayerPanel.add(layerPanel);
		dataLayerPanel = new DataLayerPanel();
		wwdLayerPanel.add(dataLayerPanel);

		catalogJTabbedPanel.add(
				rBundle.getString(LocaleKey.Jframe_jJabbedpanel_Worldwind),
				wwdLayerPanel);
		catalogJTabbedPanel.add(
				rBundle.getString(LocaleKey.JFrame_Jtabbedpanel_Service),
				treeScrollPanel);
		catalogJTabbedPanel.setName("left");
		catalogJTabbedPanel.setPreferredSize(new Dimension(300, 600));

		displayJTabbedPanel.add(
				rBundle.getString(LocaleKey.Jframe_jJabbedpanel_Worldwind),
				appPanel);
		displayJTabbedPanel.add(
				rBundle.getString(LocaleKey.JFrame_DrawPanel_Default_Name),
				rightJScrollPane);
		displayJTabbedPanel.setName("right");
		welcomeWin.setProMessage(rBundle
				.getString(LocaleKey.Starter_Init_Tabbed));
		welcomeWin.setProValue(90);

		// initialize tabs
		TabsTool tabsTool = new TabsTool(displayJTabbedPanel);
		tabsTool.initTabs();
		catalogJTabbedPanel.setSelectedIndex(1);
		displayJTabbedPanel.setSelectedIndex(1);
		setCurrentJDrawPanel(defaulJDrawPanel);
		catalogJTabbedPanel.addChangeListener(TabChangeListener
				.getInstance(this));
		displayJTabbedPanel.addChangeListener(TabChangeListener
				.getInstance(this));

		welcomeWin.setProMessage(rBundle
				.getString(LocaleKey.Starter_Init_Layout));
		welcomeWin.setProValue(95);

		

		LocTool locTool = LocTool.getInstance(PathConf.getLocConfDir());
		Dimension leftSize = locTool.getLeftSize();
		Dimension rightSize = locTool.getRightSize();
		if (leftSize == null || leftSize.getWidth()<50 || leftSize.getHeight()<50) {
			leftSize = new Dimension(200,600);
		}
		if (rightSize == null || rightSize.getWidth()<50 || rightSize.getHeight()<50) {
			rightSize = new Dimension(600,600);
		}
		catalogJTabbedPanel.setPreferredSize(leftSize);
		displayJTabbedPanel.setPreferredSize(rightSize);
		
		JSplitPane jSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
				catalogJTabbedPanel, displayJTabbedPanel);

		container = (JPanel) this.getContentPane();
		container.setLayout(new BorderLayout());
		container.add(jSplitPane, BorderLayout.CENTER);
		container.add(toolBar, BorderLayout.NORTH);
		this.pack();
//		this.setLocationRelativeTo(null);
		this.setLocale(new Locale("en", "US"));
		int index = locTool.getTabIndex();
		if (index!=-1 && index < displayJTabbedPanel.getTabCount()) {
			displayJTabbedPanel.setSelectedIndex(index);
		}
		Dimension frameLoc = locTool.getFrameLocation();
		if (frameLoc!=null) {
			this.setLocation((int)frameLoc.getWidth(),(int)frameLoc.getHeight());
		}else {
			this.setLocationRelativeTo(null);
		}
		
		this.addWindowListener(new GeoWindowListener());
		this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		
		//this.setSize(new Dimension(600, 800));
	}

	public static void main(String[] args) {
		ClientFrame client = new ClientFrame("GeoModelBuilder");
		client.setVisible(true);
	}

	/**
	 * Adds a new tab to the jframe. You can create a new process flow on it.
	 * 
	 * @param name
	 */
	public void addTab(String name) {
		int num = displayJTabbedPanel.getTabCount();
		if (name.trim().equals("")) {
			name = "Tab " + num;
		} else {
			name = name.trim();
		}
		JDrawPanel drawPanel = new JDrawPanel();
		drawPanel.setFigureStore(new FigureStore());
		drawPanel.setModelStore(new PureModelStore());
		drawPanel.setBindStore(new BindStore());
		JScrollPane scrollPane = new JScrollPane(drawPanel);

		displayJTabbedPanel.add(name, scrollPane);
		drawPanel.setName(name);
		drawPanel.setUuid(HandleTool.getUUID());
		TabButtonCom tabButtonCom = new TabButtonCom(displayJTabbedPanel,
				drawPanel);
		displayJTabbedPanel.setTabComponentAt(num, tabButtonCom);
		displayJTabbedPanel.setSelectedIndex(num);
		setCurrentJDrawPanel(drawPanel);
	}

	/**
	 * @return the default drawPanel
	 */
	public JDrawPanel getDefaultDrawPanel() {
		return this.defaulJDrawPanel;
	}

	/**
	 * @return the panel which contains the data layers added by user.
	 */
	public DataLayerPanel getDataLayerPanel() {
		return dataLayerPanel;
	}

	/*
	 * Control the components of the frame public void changeStateToWWD(){
	 * this.setJMenuBar(menuBar);
	 * this.getContentPane().add(toolBar,BorderLayout.NORTH); this.invalidate();
	 * this.validate(); } public void changeStateToModel(){
	 * this.remove(menuBar); this.setJMenuBar(null); this.remove(toolBar);
	 * this.invalidate(); this.validate(); }
	 */

	/**
	 * @return the current JDrawPanel.
	 */
	public JDrawPanel getCurrentJDrawPanel() {
		return currentJDrawPanel;
	}

	/**
	 * Sets the current JDrawPanel.
	 * 
	 * @param currentJDrawPanel
	 */
	public void setCurrentJDrawPanel(JDrawPanel currentJDrawPanel) {
		this.currentJDrawPanel = currentJDrawPanel;
	}

	/**
	 * 
	 * @return the right JTabbedPane.
	 */
	public JTabbedPane getDisplayJTabbedPanel() {
		return displayJTabbedPanel;
	}
	public TreeScrollPanel getTreeScrollPanel(){
		return treeScrollPanel;
	}
	/**
	 * 
	 * @return the left JTabbedPane.
	 */
	public JTabbedPane getCatalogJTabbedPanel() {
		return catalogJTabbedPanel;
	}

	/**
	 * 
	 * @return the WWDPanel which extends javax.swing.JPanel and contains
	 *         WorldWindow.
	 */
	public WWDPanel getWwPanel() {
		return appPanel;
	}

}
