/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.frame.ui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import conf.GlobeParam;

import cn.edu.whu.lmars.drawpanel.ui.JDrawPanel;
import cn.edu.whu.lmars.model.AbstractParam;
import cn.edu.whu.lmars.worldwind.utils.WWDUtility;

/**
 * Views the metadata of parameter.
 * 
 * @author Mingda Zhang
 * 
 */

public class DataMetaDialog extends JDialog implements ActionListener {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JLabel jLabel = null;
	private JLabel jLabel1 = null;
	private JTextField title1 = null;
	private JTextField abstract2 = null;
	private JLabel jLabel41 = null;
	private JTextField valueT = null;
	private JButton WWDbutton;

	/**
	 * Instantiates a DataMetaDialog.
	 * 
	 * @param owner
	 * @param complexParam
	 *            complex parameter.
	 */
	public DataMetaDialog(Frame owner, AbstractParam complexParam) {
		super(owner);
		initialize();
		this.setTitle(complexParam.getName() + " Detail Info");
		title1.setText(complexParam.getName());
		abstract2.setText(complexParam.getFormat());
		valueT.setText(complexParam.getValue());
		if (valueT.getText().trim().equals("")) {
			this.WWDbutton.setEnabled(false);
		}
	}

	/**
	 * Initializes the Dialog.
	 */
	private void initialize() {
		this.setContentPane(getJContentPane());
		this.setSize(344, 175);
		this.setResizable(false);
		this.setLocationRelativeTo(null);
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		this.setModal(true);
	}

	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jLabel41 = new JLabel();
			jLabel41.setBounds(new Rectangle(14, 70, 80, 25));
			jLabel41.setHorizontalAlignment(SwingConstants.RIGHT);
			jLabel41.setHorizontalTextPosition(SwingConstants.RIGHT);
			jLabel41.setText("Value:");
			jLabel41.setPreferredSize(new Dimension(80, 25));
			jLabel1 = new JLabel();
			jLabel1.setHorizontalTextPosition(SwingConstants.RIGHT);
			jLabel1.setText("Format:");
			jLabel1.setBounds(new Rectangle(14, 40, 80, 25));
			jLabel1.setHorizontalAlignment(SwingConstants.RIGHT);
			jLabel = new JLabel();
			jLabel.setHorizontalAlignment(SwingConstants.RIGHT);
			jLabel.setHorizontalTextPosition(SwingConstants.RIGHT);
			jLabel.setPreferredSize(new Dimension(80, 25));
			jLabel.setBounds(new Rectangle(14, 10, 80, 25));
			jLabel.setText("Name:");
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.setName("MetaData");
			jContentPane.add(jLabel, null);
			jContentPane.add(jLabel1, null);
			jContentPane.add(getTitle1(), null);
			jContentPane.add(getAbstract2(), null);
			jContentPane.add(jLabel41, null);
			jContentPane.add(getValueT(), null);
			jContentPane.add(getJButton(), null);
		}
		return jContentPane;
	}

	private JTextField getTitle1() {
		if (title1 == null) {
			title1 = new JTextField();
			title1.setBounds(new Rectangle(110, 10, 200, 25));
			title1.setEditable(false);
			title1.setBackground(Color.white);
			title1.setPreferredSize(new Dimension(200, 25));
		}
		return title1;
	}

	private JTextField getAbstract2() {
		if (abstract2 == null) {
			abstract2 = new JTextField();
			abstract2.setBounds(new Rectangle(110, 40, 200, 25));
			abstract2.setPreferredSize(new Dimension(200, 25));
			abstract2.setEditable(false);
			abstract2.setBackground(Color.white);
		}
		return abstract2;
	}

	private JTextField getValueT() {
		if (valueT == null) {
			valueT = new JTextField();
			valueT.setBounds(new Rectangle(110, 70, 200, 25));
			valueT.setPreferredSize(new Dimension(200, 25));
			valueT.setEditable(false);
			valueT.setBackground(Color.white);
		}
		return valueT;
	}

	private JButton getJButton() {
		if (WWDbutton == null) {
			WWDbutton = new JButton("Load to WorldWind");
			WWDbutton.setBounds(new Rectangle(110, 107, 200, 25));
			WWDbutton.addActionListener(this);
			WWDbutton.setActionCommand("addToWorldWind");
		}
		return WWDbutton;
	}

	/**
	 * Loads the data to world wind.
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		if (command.equals("addToWorldWind")) {
			this.dispose();
			new Thread(new Runnable() {

				@Override
				public void run() {
					JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
					String uuid = drawPanel.getUuid();
					WWDUtility.addToWorldWind(valueT.getText(), uuid);

				}
			}).start();
		}
	}
}
