/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.frame.ui;

import java.util.ResourceBundle;

import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

import cn.edu.whu.lmars.frame.listeners.MenuActionListener;
import cn.edu.whu.lmars.locale.LocaleKey;
import cn.edu.whu.lmars.locale.ResourceBundleFactory;

/**
 * Creates a new JMenu bar for ClientFrame.
 * 
 * @author Mingda Zhang
 * 
 */
@SuppressWarnings("serial")
public class GeoMenuBar extends JMenuBar {

	MenuActionListener listener = new MenuActionListener();
	private ResourceBundle rBundle = ResourceBundleFactory.getResourceBundle();

	public GeoMenuBar() {
		super();
		initilize();
	}

	/**
	 * Initializes the menu bar.
	 */
	private void initilize() {
		JMenu fileMenu = new JMenu(rBundle.getString(LocaleKey.Menu_File));
		fileMenu.add(createItem(rBundle.getString(LocaleKey.Menu_File_Open),
				"open"));
		fileMenu.add(createItem(rBundle.getString(LocaleKey.Menu_File_Save),
				"save"));
		fileMenu.addSeparator();
		fileMenu.add(createItem(rBundle.getString(LocaleKey.Menu_File_Export),
				"export"));
		fileMenu.addSeparator();
		/*fileMenu.add(createItem(rBundle.getString(LocaleKey.Menu_File_Switch),
				"workspace"));*/
	//	fileMenu.addSeparator();
		fileMenu.add(createItem(rBundle.getString(LocaleKey.Menu_File_Exit),
				"exit"));
		this.add(fileMenu);

		JMenu modelMenu = new JMenu(rBundle.getString(LocaleKey.Menu_Model));
		modelMenu.add(createItem(rBundle.getString(LocaleKey.Menu_Model_Add),
				"addModel"));
		this.add(modelMenu);

		JMenu editMenu = new JMenu(rBundle.getString(LocaleKey.Menu_Edit));
		editMenu.add(createItem(rBundle.getString(LocaleKey.Menu_Edit_Delete),
				"delete"));
		this.add(editMenu);

		JMenu projectJMenu = new JMenu(
				rBundle.getString(LocaleKey.Menu_Project));
		projectJMenu.add(createItem(
				rBundle.getString(LocaleKey.Menu_Project_NewTab), "tab"));
		this.add(projectJMenu);

		JMenu runMenu = new JMenu(rBundle.getString(LocaleKey.Menu_Run));
		runMenu.add(createItem(rBundle.getString(LocaleKey.Menu_Run_Run), "run"));
		runMenu.add(createItem(rBundle.getString(LocaleKey.Menu_Run_Info),
				"rundetail"));
		this.add(runMenu);

		JMenu loadFileJMenu = new JMenu(rBundle.getString(LocaleKey.Menu_Data));
		JMenu addDataJMenu = new JMenu("Add Data");
		addDataJMenu.add(createItem("Web...","addWebData"));
		addDataJMenu.add(createItem("Local...","addLocalData"));
		loadFileJMenu.add(addDataJMenu);
		/*JMenu shpMenu = new JMenu(rBundle.getString(LocaleKey.Menu_Data_Shp));
		shpMenu.add(createItem("Local", "addEsriShapefile"));
		shpMenu.add(createItem("web", "addWebData"));
		loadFileJMenu.add(shpMenu);

		JMenu tifMenu = new JMenu(rBundle.getString(LocaleKey.Menu_Data_Tif));
		tifMenu.add(createItem("Local", "addTifFile"));
		tifMenu.add(createItem("web", "addWebData"));
		loadFileJMenu.add(tifMenu);

		JMenu kmlMenu = new JMenu(rBundle.getString(LocaleKey.Menu_Data_Kml));
		kmlMenu.add(createItem("Local", "addKmlFile"));
		kmlMenu.add(createItem("web", "addWebData"));
		loadFileJMenu.add(kmlMenu);*/
		this.add(loadFileJMenu);

		JMenu sensorMenu = new JMenu(
				rBundle.getString(LocaleKey.Menu_sensorweb));
		sensorMenu
				.add(createItem(
						rBundle.getString(LocaleKey.Menu_sensorweb_Sos),
						"sosOperation"));
		sensorMenu
				.add(createItem(
						rBundle.getString(LocaleKey.Menu_sensorweb_Wns),
						"wnsOperation"));
		sensorMenu
				.add(createItem(
						rBundle.getString(LocaleKey.Menu_sensorweb_Ses),
						"sesOperation"));
		sensorMenu.add(createItem(
				rBundle.getString(LocaleKey.Menu_sensorweb_Subscribe),
				"subcribeAlert"));
		this.add(sensorMenu);
		
		JMenu helpMenu = new JMenu("Help");
		helpMenu.add(createItem("Help", "help"));
		helpMenu.add(createItem("Resource Center", "resource"));
		helpMenu.add(createItem("About...", "about"));
		this.add(helpMenu);
	}

	/**
	 * Creates a new item using the given name and action command.
	 * 
	 * @param name
	 *            the name of item
	 * @param actionCommand
	 *            the action command of the item.
	 */
	private JMenuItem createItem(String name, String actionCommand) {
		JMenuItem item = new JMenuItem(name);

		item.addActionListener(listener);
		item.setActionCommand(actionCommand);

		return item;
	}
}
