/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.frame.ui;

import java.awt.Dimension;
import java.util.ResourceBundle;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JToolBar;

import cn.edu.whu.lmars.frame.listeners.MenuActionListener;
import cn.edu.whu.lmars.locale.LocaleKey;
import cn.edu.whu.lmars.locale.ResourceBundleFactory;

/**
 * Creates a Tool bar for ClientFrame.
 * 
 * @author Mingda Zhang
 *
 */
@SuppressWarnings("serial")
public class GeoToolBar extends JToolBar {
	private ResourceBundle rBundle = ResourceBundleFactory.getResourceBundle();
	public GeoToolBar() {
		super();
		initialize();
	}

	/**
	 * Initializes the tool bar.
	 */
	private void initialize() {

		this.add(makeToolButton("open.png", "open", rBundle.getString(LocaleKey.Toolbar_Tip_Open)));
		this.add(makeToolButton("save.png", "save",  rBundle.getString(LocaleKey.Toolbar_Tip_Save)));
		this.add(makeToolButton("new.png", "tab",  rBundle.getString(LocaleKey.Toolbar_Tip_NewTab)));
		this.addSeparator();

		this.add(makeToolButton("add.png", "addModel",  rBundle.getString(LocaleKey.Toolbar_Tip_AddModel)));
		this.add(makeToolButton("delete.png", "delete", rBundle.getString(LocaleKey.Toolbar_Tip_Delete)));
		this.add(makeToolButton("clear.png", "clear",  rBundle.getString(LocaleKey.Toolbar_Tip_Clear)));
		this.addSeparator();

		this.add(makeToolButton("flow.png", "flow", rBundle.getString(LocaleKey.Toolbar_Tip_AddFlow)));
		this.addSeparator();

		this.add(makeToolButton("bind.png", "bind",  rBundle.getString(LocaleKey.Toolbar_Tip_Bind)));
		this.add(makeToolButton("run.png", "run",  rBundle.getString(LocaleKey.Toolbar_Tip_Run)));
		this.addSeparator();

		this.add(makeToolButton("zoomin.png", "zoomin",  rBundle.getString(LocaleKey.Toolbar_Tip_Zoomin)));
		this.add(makeToolButton("zoomout.png", "zoomout",  rBundle.getString(LocaleKey.Toolbar_Tip_Zoomout)));
		this.addSeparator();

		this.add(makeToolButton("undo.png", "undo",  rBundle.getString(LocaleKey.Toolbar_Tip_Undo)));
		this.add(makeToolButton("redo.png", "redo",  rBundle.getString(LocaleKey.Toolbar_Tip_Redo)));
		this.addSeparator();

		this.add(makeToolButton("select.png", "select",  rBundle.getString(LocaleKey.Toolbar_Tip_Select)));
		this.add(makeToolButton("message.png", "detail",  rBundle.getString(LocaleKey.Toolbar_Tip_Detail)));
	}

	/**
	 * create tool button using the given image, action command and the tool tip.
	 * @param filename
	 *			the name of image.
	 * @param command
	 * 			the action command.
	 * @param toolTip
	 * 			the tool tip.
	 */
	private JButton makeToolButton(String filename, String command,
			String toolTip) {

		JButton button = new JButton();

		button.setActionCommand(command);
		button.setToolTipText(toolTip);
		button.setPreferredSize(new Dimension(30, 30));
		button.setIcon(new ImageIcon(getClass().getResource(
				"/images/" + filename)));
		button.addActionListener(new MenuActionListener());
		return button;
	}
}
