/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.frame.ui;

import java.awt.Component;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.SwingConstants;
import javax.swing.tree.TreeNode;

import cn.edu.whu.lmars.commands.MultipleInputCommand;
import cn.edu.whu.lmars.drawpanel.ui.JDrawPanel;
import cn.edu.whu.lmars.figure.utils.FigureStore;
import cn.edu.whu.lmars.figure.utils.FigureUtility;
import cn.edu.whu.lmars.model.AtomProcess;
import cn.edu.whu.lmars.model.ComplexParam;
import cn.edu.whu.lmars.model.LiteralParam;
import cn.edu.whu.lmars.utils.BaseTool;
import cn.edu.whu.lmars.utils.HandleTool;
import cn.edu.whu.lmars.worldwind.datatree.DataLayerPanel;
import cn.edu.whu.lmars.worldwind.datatree.DataLayerInfo;
import cn.edu.whu.lmars.worldwind.datatree.DataTreeNode;
import conf.FormatProperty;
import conf.GlobeParam;

/**
 * This is an input dialog which user uses to assign parameters.
 * 
 * @author Mingda Zhang
 * 
 */

public class HandleInputDialog extends JDialog implements ActionListener {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private AtomProcess atomProcess = null;
	private String postFix = "_format";
	 //  @jve:decl-index=0:
	private int label_width = 110;
	// the old value
	private Map<String, String> oldValueMap = new HashMap<String, String>();  //  @jve:decl-index=0:

	public HandleInputDialog(Frame owner, AtomProcess atomprocess) {
		super(owner, true);
		this.atomProcess = atomprocess;
		initialize();

	}

	/**
	 * Initializes the dialog.
	 */
	private void initialize() {

		jContentPane = new JPanel();
		jContentPane.setLayout(null);
		int orgin = 15;
		String name, value, format;
		for (ComplexParam param : atomProcess.getComplexParam()) {
			name = param.getName();
			value = param.getValue();
			format = param.getFormat();
			jContentPane.add(createLabel(name, new Rectangle(0, orgin,
					label_width, 30)));
			/*
			 * JTextField jTextField = createField(name,value, new
			 * Rectangle(125, orgin, 350, 30)); if (value!=null &&
			 * value.endsWith("#result")) { jTextField.setEditable(false); }
			 * jContentPane.add(jTextField);
			 */
			JComboBox comboBox = createComplexJComboBox(name, value,
					new Rectangle(125, orgin, 350, 30));
			jContentPane.add(comboBox);
			if (value == null) {
				value = "";
			}
			oldValueMap.put(name, value);

			orgin += 35;
			jContentPane.add(createLabel("format", new Rectangle(0, orgin,
					label_width, 30)));
			jContentPane.add(createComboBox(name + postFix, format,
					new Rectangle(125, orgin, 350, 30)));
			if (format == null) {
				format = "";
			}
			oldValueMap.put(name + postFix, format);
			orgin += 35;
		}
		for (LiteralParam param : this.atomProcess.getLiteralParam()) {
			name = param.getName();
			value = param.getValue();
			jContentPane.add(createLabel(name, new Rectangle(0, orgin,
					label_width, 30)));
			if (name.trim().equalsIgnoreCase("outputformat")) {
				jContentPane.add(createComboBox(name, value, new Rectangle(125,
						orgin, 350, 30)));
			} else {
				jContentPane.add(createField(name, value, new Rectangle(125,
						orgin, 350, 30)));
			}
			if (value == null) {
				value = "";
			}
			oldValueMap.put(name, value);
			JLabel label = new JLabel(param.getFormat());
			label.setBounds(new Rectangle(480, orgin, 80, 30));
			label.setHorizontalAlignment(SwingConstants.LEFT);
			label.setHorizontalTextPosition(SwingConstants.LEFT);
			jContentPane.add(label);
			orgin += 35;
		}
		

		orgin += 10;
		jContentPane.add(createButton("OK", "confirm", new Rectangle(150,
				orgin, 100, 30)));
		jContentPane.add(createButton("Cancel", "concel", new Rectangle(310,
				orgin, 100, 30)));
		this.setContentPane(jContentPane);
		this.setSize(500, orgin + 75);
		this.setTitle(atomProcess.getName());
		this.setResizable(false);
		this.setLocationRelativeTo(null);
		this.setModal(true);
	}

	/**
	 * Create a button.
	 * 
	 * @param text
	 *            name of button.
	 * @param actionCommand
	 *            action command of button
	 * @param rectangle
	 *            range of button
	 * @return
	 */
	private JButton createButton(String text, String actionCommand,
			Rectangle rectangle) {
		JButton button = new JButton();
		button.setText(text);
		button.setActionCommand(actionCommand);
		button.setFont(new Font("Dialog", Font.BOLD, 14));
		button.setBounds(rectangle);
		button.addActionListener(this);
		return button;
	}

	/**
	 * Creates label.
	 * 
	 * @param name
	 *            name of label.
	 * @param rectangle
	 *            range of label.
	 * @return
	 */
	private JLabel createLabel(String name, Rectangle rectangle) {
		name = name + " :";
		JLabel label = new JLabel(name);
		label.setBounds(rectangle);
		label.setFont(new Font("Dialog", Font.BOLD, 14));
		label.setHorizontalAlignment(SwingConstants.RIGHT);
		label.setHorizontalTextPosition(SwingConstants.RIGHT);
		return label;
	}

	/**
	 * Creates JTextField.
	 * 
	 * @param name
	 *            name of text field.
	 * @param value
	 *            value of text field.
	 * @param rectangle
	 *            range of text field..
	 * @return
	 */
	private JTextField createField(String name, String value,
			Rectangle rectangle) {
		JTextField textField = new JTextField();
		textField.setBounds(rectangle);
		textField.setFont(new Font("Dialog", Font.BOLD, 14));
		textField.setName(name);
		textField.setText(value);
		return textField;
	}

	/**
	 * Creates JComboBox for complex parameter.
	 * 
	 * @param name
	 *            name of JComboBox.
	 * @param value
	 *            value of JComboBox.
	 * @param rectangle
	 *            range of JComboBox.
	 * @return
	 */
	@SuppressWarnings("rawtypes")
	private JComboBox createComplexJComboBox(String name, String value,
			Rectangle rectangle) {
		JComboBox comBox = new JComboBox();
		comBox.setEditable(true);
		comBox.setName(name);
		comBox.setBounds(rectangle);
		comBox.setFont(new Font("Dialog", Font.BOLD, 14));
		if (value != null && value.endsWith("#result")) {
			comBox.addItem(value);
			comBox.setEditable(false);
			comBox.setSelectedItem(value);
			return comBox;
			// comboBox.setEditable(false);
		}
		DataLayerPanel dataLayerPanel = GlobeParam.getDataLayerPanel();
		JTree tree = dataLayerPanel.getjTree();
		TreeNode root = (TreeNode) tree.getModel().getRoot();
		boolean flag = false;
		int num = root.getChildCount();
		if (num > 0) {
			for (Enumeration e = root.children(); e.hasMoreElements();) {
				TreeNode node = (TreeNode) e.nextElement();
				if (node instanceof DataTreeNode) {
					DataTreeNode dataNode = (DataTreeNode) node;
					DataLayerInfo nodeBean = dataNode.getDataNode();
					String path = nodeBean.getPath();
					// if (path!=null && (path.startsWith("http:")||
					// path.startsWith("ftp:"))) {
					if (path != null) {
						comBox.addItem(nodeBean);
						if (path.equals(value)) {
							comBox.setSelectedItem(nodeBean);
							flag = true;
						}
					}
				}
			}
		}
		if (!flag) {
			if (BaseTool.isNull(value)) {
				comBox.setSelectedItem("");
			} else {
				comBox.addItem(value);
				comBox.setSelectedItem(value);
			}
		}
		return comBox;
	}

	/**
	 * Creates JComboBox for complex parameter.
	 * 
	 * @param name
	 *            name of JComboBox.
	 * @param value
	 *            value of JComboBox.
	 * @param rectangle
	 *            range of JComboBox.
	 * @return
	 */
	private JComboBox createComboBox(String name, String value,
			Rectangle rectangle) {
		JComboBox comboBox = new JComboBox();
		comboBox.setEditable(false);
		Properties properties = FormatProperty.getInstance()
				.getPostFixProperty();
		for (Object key : properties.keySet()) {
			comboBox.addItem(key);
		}
		comboBox.setBounds(rectangle);
		comboBox.setFont(new Font("Dialog", Font.BOLD, 14));
		comboBox.setName(name);
		if (BaseTool.isNull(value)) {
			comboBox.setSelectedIndex(0);
			return comboBox;

		}
		for (int i = 0; i < comboBox.getItemCount(); i++) {
			if (comboBox.getItemAt(i).toString().equals(value)) {
				comboBox.setSelectedIndex(i);
				return comboBox;
			}
		}
		comboBox.setSelectedItem(0);
		return comboBox;
	}

	/**
	 * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		String str = e.getActionCommand();
		if (str.equals("confirm")) {
			new Thread(new Runnable() {
				@Override
				public void run() {
					confirmAction();
				}
			}).start();
			this.dispose();
		} else if (str.equals("concel")) {
			this.dispose();
		} else if (str.equals("choose")) {
		}
	}

	/**
	 * Assigns the value.
	 */
	private void confirmAction() {
		HashMap<String, String> newValueMap = new HashMap<String, String>();
		int count = jContentPane.getComponentCount();
		JTextField jText;
		JComboBox comboBox;
		String key = "", value = "";
		for (int i = 0; i < count; i++) {
			Component comp = jContentPane.getComponent(i);
			if (comp instanceof JTextField) {
				jText = (JTextField) comp;
				key = jText.getName();
				value = jText.getText().trim();
				newValueMap.put(key, value);
			} else if (comp instanceof JComboBox) {
				comboBox = (JComboBox) comp;
				Object object = comboBox.getSelectedItem();
				if (object instanceof DataLayerInfo) {
					DataLayerInfo dataNode = (DataLayerInfo) object;
					dataNode.setUuid(GlobeParam.getCurrentJDrawPanel()
							.getUuid());
				}
				key = comboBox.getName();
				value = object.toString();
				newValueMap.put(key, value);
			}
		}
		if (checkChange(newValueMap, oldValueMap)) {

			 HandleTool handleTool = HandleTool.getInstance(); 
			JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
			FigureStore figureStore = drawPanel.getFigureStore();
			MultipleInputCommand command = new MultipleInputCommand(
					atomProcess, newValueMap, oldValueMap);
			handleTool.changeNulToValue(figureStore
					.getProcessFigureByUuid(atomProcess.getUUID()));
			drawPanel.getCommandStack().addCommand(command);
			command.execute();
			FigureUtility.getInstance().adjustFigures(
					figureStore.getAllFigures());
			drawPanel.setChanged(true);
			HandleTool.refreshFigShow();
		}
	}

	/**
	 * Returns true if value is changed.
	 * 
	 * @param newMap
	 *            new value map.
	 * @param oldMap
	 *            old value map.
	 * @return
	 */
	private boolean checkChange(Map<String, String> newMap,
			Map<String, String> oldMap) {
		String oldValue, newValue;
		for (String key : newMap.keySet()) {
			newValue = newMap.get(key);
			oldValue = oldMap.get(key);
			if (!newValue.equals(oldValue)) {
				return true;
			}
		}
		return false;
	}
}
