/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.frame.ui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Rectangle;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import cn.edu.whu.lmars.model.AtomProcess;

/**
 * Views the metadata of a process model.
 * 
 * @author Mingda Zhang
 * 
 */

public class ProcessMetaDialog extends JDialog {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JLabel jLabel = null;
	private JLabel jLabel1 = null;
	private JTextField nameT = null;
	private JTextField abstractT = null;

	/**
	 * Instantiates a ProcessMetaDialog.
	 * 
	 * @param owner
	 * @param atomProcess
	 *            the model.
	 */
	public ProcessMetaDialog(Frame owner, AtomProcess atomProcess) {
		super(owner, true);
		initialize();
		nameT.setText(atomProcess.getName());
		abstractT.setText(atomProcess.getAbstract());
	}

	/**
	 * Initializes the dialog.
	 */
	private void initialize() {
		this.setContentPane(getJContentPane());
		this.setSize(472, 114);
		this.setTitle("ProcessMeta");
		this.setResizable(false);
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		this.setLocationRelativeTo(null);
	}

	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jLabel1 = new JLabel();
			jLabel1.setBounds(new Rectangle(14, 40, 94, 25));
			jLabel1.setHorizontalAlignment(SwingConstants.RIGHT);
			jLabel1.setHorizontalTextPosition(SwingConstants.RIGHT);
			jLabel1.setText("Abstract:");
			jLabel1.setPreferredSize(new Dimension(80, 25));
			jLabel = new JLabel();
			jLabel.setBounds(new Rectangle(14, 10, 94, 25));
			jLabel.setHorizontalAlignment(SwingConstants.RIGHT);
			jLabel.setHorizontalTextPosition(SwingConstants.RIGHT);
			jLabel.setPreferredSize(new Dimension(80, 25));
			jLabel.setText("Name:");
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(jLabel, null);
			jContentPane.add(jLabel1, null);
			jContentPane.add(getNameT(), null);
			jContentPane.add(getAbstractT(), null);
		}
		return jContentPane;
	}

	private JTextField getNameT() {
		if (nameT == null) {
			nameT = new JTextField();
			nameT.setBounds(new Rectangle(118, 10, 334, 25));
			nameT.setEditable(false);
			nameT.setBackground(Color.white);
			nameT.setPreferredSize(new Dimension(334, 25));
		}
		return nameT;
	}

	private JTextField getAbstractT() {
		if (abstractT == null) {
			abstractT = new JTextField();
			abstractT.setBounds(new Rectangle(118, 40, 334, 25));
			abstractT.setPreferredSize(new Dimension(334, 25));
			abstractT.setEditable(false);
			abstractT.setBackground(Color.white);
		}
		return abstractT;
	}

}
