/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.frame.utils;

import java.awt.Dimension;
import java.io.File;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;

import cn.edu.whu.lmars.utils.BaseTool;
import cn.edu.whu.lmars.utils.Dom4jXMLTool;


/**
 * This class will read and save location and layout information of the
 * ClientFrame.
 * 
 * @author Mingda Zhang
 * 
 */
public class LocTool {
	private String path = "";
	private static LocTool instance;
	private LocTool(String path){
		this.path = path;
	}
	private Document doc;
	public static LocTool getInstance(String path){
		if(instance == null){
			instance = new LocTool(path);
		}
		return instance;
	}
	
	/**
	 * Gets document from file.
	 * @return
	 */
	private Document getDoc(){
		SAXReader reader = new SAXReader();
		try {
			Document doc = reader.read(new File(path));
			return doc;
		} catch (DocumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
		
	}
	
	/**
	 * Gets size of the component from element.
	 * @param expression
	 * @return
	 */
	private Dimension getSize(String expression){
		
		if (this.doc == null) {
			doc = getDoc();
		}
		Element element = (Element)doc.selectSingleNode(expression);
		if (element == null) {
			return null;
		}
		String width = element.attributeValue("width");
		String height= element.attributeValue("height");
		if (BaseTool.isNull(width) || BaseTool.isNull(height)) {
			return null;
		}
		try {
			int w = Integer.valueOf(width);
			int h = Integer.valueOf(height);
			Dimension size = new Dimension(w, h);
			return size;
		} catch (Exception e) {
			// TODO: handle exception
			return null;
		}
	
	}
	
	/**
	 * Gets the size of catalog JTabbedPanel on the left.
	 */
	public Dimension getLeftSize(){
		return getSize("//LeftSize");
	}
	
	/**
	 * Returns the size of display JTabbedPanel on the right.
	 */
	public Dimension getRightSize(){
		return getSize("//RightSize");
	}
	
	/**
	 * Gets the location of the client frame.
	 */
	public Dimension getFrameLocation(){
		if (this.doc == null) {
			doc = getDoc();
		}
		Element element = (Element)doc.selectSingleNode("//Location");
		if (element == null) {
			return null;
		}
		String x = element.attributeValue("x");
		String y= element.attributeValue("y");
		if (BaseTool.isNull(x) || BaseTool.isNull(y)) {
			return null;
		}
		try {
			int xLoc = Integer.valueOf(x);
			int yLoc = Integer.valueOf(y);
			Dimension size = new Dimension(xLoc, yLoc);
			return size;
		} catch (Exception e) {
			// TODO: handle exception
			return null;
		}
	}
	/**
	 * Gets the index of selected tab in display JTabbedPanel.
	 * @return
	 */
	public int getTabIndex(){
		if (this.doc == null) {
			doc = getDoc();
		}
		Element element = (Element)doc.selectSingleNode("//TabNum");
		
		if (element == null) {
			return -1;
		}
		String value = element.getText();
		if(BaseTool.isNull(value)){
			return -1;
		}
		try {
			int index = Integer.parseInt(value);
			return index;
		} catch (Exception e) {
			return -1;
		}
	}
	public boolean saveInfo(Dimension left,Dimension right,Dimension frame,int index){
		if(this.doc==null){
			doc =  getDoc();
		}
		Element leftEle = (Element)doc.selectSingleNode("//LeftSize");
		leftEle.attribute("width").setText((int)left.getWidth()+"");
		leftEle.attribute("height").setText((int)left.getHeight()+"");
		
		Element rightEle = (Element)doc.selectSingleNode("//RightSize");
		rightEle.attribute("width").setText((int)right.getWidth()+"");
		rightEle.attribute("height").setText((int)right.getHeight()+"");
		
		Element frameEle = (Element)doc.selectSingleNode("//Location");
		frameEle.attribute("x").setText((int)frame.getWidth()+"");
		frameEle.attribute("y").setText((int)frame.getHeight()+"");
		
		Element indexEle = (Element)doc.selectSingleNode("//TabNum");
		indexEle.setText(index+"");
		
		Dom4jXMLTool.writeXml(doc,new File(this.path));
		return true;
	}
	
}
