/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.frame.utils;

import java.awt.Cursor;
import java.io.File;
import java.util.List;
import java.util.ResourceBundle;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileNameExtensionFilter;

import org.apache.log4j.Logger;

import cn.edu.whu.lmars.commands.GeoCommand;
import cn.edu.whu.lmars.commands.GeoDeleteAllCommand;
import cn.edu.whu.lmars.commands.GeoDeleteFlowCommand;
import cn.edu.whu.lmars.commands.GeoDeleteProcessCommand;
import cn.edu.whu.lmars.commands.GeoZoomInCommand;
import cn.edu.whu.lmars.commands.GeoZoomOutCommand;
import cn.edu.whu.lmars.drawpanel.ui.JDrawPanel;
import cn.edu.whu.lmars.drawpanel.ui.PathDialog;
import cn.edu.whu.lmars.drawpanel.ui.cursors.BindCursor;
import cn.edu.whu.lmars.drawpanel.ui.cursors.DetailCursor;
import cn.edu.whu.lmars.drawpanel.ui.cursors.DetailRunCursor;
import cn.edu.whu.lmars.figure.beans.AbstractFigure;
import cn.edu.whu.lmars.figure.beans.FlowFigure;
import cn.edu.whu.lmars.figure.beans.ProcessFigure;
import cn.edu.whu.lmars.figure.beans.geoenum.ButtonType;
import cn.edu.whu.lmars.figure.beans.geoenum.DragType;
import cn.edu.whu.lmars.figure.utils.FigureStore;
import cn.edu.whu.lmars.figure.utils.FigureUtility;
import cn.edu.whu.lmars.frame.ui.AboutDialog;
import cn.edu.whu.lmars.frame.ui.ClientFrame;
import cn.edu.whu.lmars.locale.LocaleKey;
import cn.edu.whu.lmars.locale.ResourceBundleFactory;
import cn.edu.whu.lmars.model.ui.AddModelDialog;
import cn.edu.whu.lmars.model.util.PureModelStore;
import cn.edu.whu.lmars.run.ui.ExecuteMonitorDialog;
import cn.edu.whu.lmars.sensorweb.ses.ui.SESOperaDialog;
import cn.edu.whu.lmars.sensorweb.sos.SosOperaDialog;
import cn.edu.whu.lmars.sensorweb.ui.SubscribeNotifyDialog;
import cn.edu.whu.lmars.sensorweb.wns.ui.WNSDialog;
import cn.edu.whu.lmars.service.utils.NetUtil;
import cn.edu.whu.lmars.starter.Starter;
import cn.edu.whu.lmars.utils.BaseTool;
import cn.edu.whu.lmars.utils.HandleTool;
import cn.edu.whu.lmars.worldwind.balloon.SensorBalloon;
import cn.edu.whu.lmars.worldwind.ui.WWDPanel;
import cn.edu.whu.lmars.worldwind.utils.WWDUtility;
import conf.GlobeParam;
import conf.PathConf;

/**
 * This MenuActionHandle class handles the operation of the processing tool bar
 * and menu bar. A most important parameter is JDrawPanel in the methods,because
 * you should decide which JDrawPanel to be changed.
 * 
 * @author Mingda Zhang
 * 
 */
public class MenuActionHandle {
	private FigureUtility figureUtility = null;
	private HandleTool handleTool = HandleTool.getInstance();
	private JDrawPanel drawPanel;
	private Logger logger = Logger.getLogger(MenuActionHandle.class);
	public MenuActionHandle() {
		figureUtility = FigureUtility.getInstance();
	}

	/**
	 * Handles the runnint operation.
	 */
	public void runAction() {
		drawPanel = GlobeParam.getCurrentJDrawPanel();
		ResourceBundle rbBundle = ResourceBundleFactory.getResourceBundle();
		int num = drawPanel.getFigureStore().getAllFigures().size();
		if (num == 0) {
			JOptionPane.showMessageDialog(GlobeParam.getClient(),
					rbBundle.getString(LocaleKey.Dialog_Msg_NoModel));
			return;
		} else {
			ExecuteMonitorDialog monitorDialog = ExecuteMonitorDialog
					.getInstance(GlobeParam.getClient());
			monitorDialog.setVisible(true);
		}
	}

	/**
	 * Handles the opening operation.
	 */
	public void openAction() {
		JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
		handleTool.saveModel(drawPanel);
		handleTool.openModel(drawPanel);
	}

	/**
	 * Handles the saving operation.
	 */
	public void saveAction() {
		JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
		handleTool.saveModel(drawPanel);
	//	drawPanel.setChanged(false);
	}

	/**
	 * Handles the exiting operation.
	 */
	public void exitAction() {
		ResourceBundle rBundle = ResourceBundleFactory.getResourceBundle();
		int flag = JOptionPane.showConfirmDialog(GlobeParam.getClient(),
				rBundle.getString(LocaleKey.Dialog_Msg_Exit_Title),
				rBundle.getString(LocaleKey.Dialog_Msg_Exit_Msg),
				JOptionPane.OK_CANCEL_OPTION);
		if (flag == JOptionPane.OK_OPTION) {
			ClientFrame client = GlobeParam.getClient();
			WorkSpaceUtil.saveWorkSpace(client);
			GlobeParam.getClient().dispose();
			System.exit(0);
		}
	}

	/**
	 * Handles the deleting operation.
	 */
	public void deleteAction() {
		JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
		ResourceBundle rBundle = ResourceBundleFactory.getResourceBundle();
		FigureStore figureStore = drawPanel.getFigureStore();
		PureModelStore modelStore = drawPanel.getModelStore();
		List<AbstractFigure> choosens = figureUtility.getChoosen(figureStore
				.getAllFigures());

		if (choosens.size() == 0) {
			return;
		}
		int conf = JOptionPane.showConfirmDialog(GlobeParam.getClient(),
				rBundle.getString(LocaleKey.Dialog_Msg_Delete_Msg),
				rBundle.getString(LocaleKey.Dialog_Msg_Delete_Title),
				JOptionPane.OK_CANCEL_OPTION);
		if (conf == JOptionPane.OK_OPTION) {
			GeoCommand command = null;
			if (choosens.get(0) instanceof FlowFigure) {
				command = new GeoDeleteFlowCommand(modelStore, figureStore,
						(FlowFigure) choosens.get(0));
			} else {
				String uuid = choosens.get(0).getUuid().split("#")[0];
				ProcessFigure processFigure = figureStore
						.getProcessFigureByUuid(uuid);
				command = new GeoDeleteProcessCommand(modelStore, figureStore,
						processFigure);
			}
			command.execute();
			drawPanel.getCommandStack().addCommand(command);
			drawPanel.setChanged(true);

			if (drawPanel.getFigureStore().getAllFigures().size() == 0) {
				drawPanel.getModelStore().clear();
			}
		} else {
			return;
		}
		HandleTool.refreshFigShow();
	}

	public void addAction() {
		JOptionPane.showMessageDialog(null, "add a process");
	}

	/**
	 * Handles the creating flow operation.
	 */
	public void flowAction() {
		JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
		drawPanel.setDragType(DragType.DRAWLINE);
		drawPanel.setBtnType(ButtonType.Flow);
		drawPanel.setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
		figureUtility.clearAllChoosen(drawPanel.getFigureStore()
				.getAllFigures());
	}

	/**
	 * Handles the selecting operation.
	 */
	public void selectAction() {
		JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
		figureUtility.clearAllChoosen(drawPanel.getFigureStore()
				.getAllFigures());
		drawPanel.setDragType(DragType.MOVE);
		drawPanel.setBtnType(ButtonType.Select);
		drawPanel.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	}

	/**
	 * Handles the querying details operation.
	 */
	public void detailAction() {
		JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
		figureUtility.clearAllChoosen(drawPanel.getFigureStore()
				.getAllFigures());
		drawPanel.setBtnType(ButtonType.Detail);
		drawPanel.requestFocus();
		drawPanel.setCursor(DetailCursor.getInstance().getDetailCursor());
	}

	/**
	 * Handles the zoom in operation.
	 */
	public void zoomInAction() {
		JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
		GeoZoomInCommand zoomInCommand = new GeoZoomInCommand(drawPanel);
		zoomInCommand.execute();
		drawPanel.setChanged(true);
		drawPanel.getCommandStack().addCommand(zoomInCommand);
	}

	/**
	 * Handle the zoom out operation
	 */
	public void zoomOutAction() {
		JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
//		GeoZoomOutCommand zoomOutCommand = new GeoZoomOutCommand(drawPanel);
		GeoZoomOutCommand zoomOutCommand = new GeoZoomOutCommand(drawPanel);
		zoomOutCommand.execute();
		drawPanel.setChanged(true);
		drawPanel.getCommandStack().addCommand(zoomOutCommand);
	}

	/**
	 * Handles the exporting image operation.
	 */
	public void exportAction() {
		JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
		boolean flag = figureUtility.exprotImage(drawPanel);
		if (flag) {
			JOptionPane.showMessageDialog(GlobeParam.getClient(), "Export image successfully.");
		}
	}

	/**
	 * Handles the merging operation.
	 */
	public void mergeAction() {
		JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
		figureUtility.mergeFigure(drawPanel);
	}

	/**
	 * Handles the clearing operation.
	 */
	public void clearAction() {
		JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
		FigureStore figureStore = drawPanel.getFigureStore();
		ResourceBundle rBundle = ResourceBundleFactory.getResourceBundle();
		PureModelStore modelStore = drawPanel.getModelStore();
		if (figureStore.getAllFigures().size() == 0) {
			return;
		}
		int conf = JOptionPane.showConfirmDialog(GlobeParam.getClient(),
				rBundle.getString(LocaleKey.Dialog_Msg_Clear_Msg),
				rBundle.getString(LocaleKey.Dialog_Msg_Delete_Title),
				JOptionPane.OK_CANCEL_OPTION);
		if (conf == JOptionPane.OK_OPTION) {
			GeoDeleteAllCommand command = new GeoDeleteAllCommand(figureStore,
					modelStore);
			command.execute();
			drawPanel.setChanged(true);
			drawPanel.getCommandStack().addCommand(command);
		} else {
			return;
		}
		HandleTool.refreshFigShow();
	}

	/**
	 * Handles the undo operation.
	 */
	public void undoAction() {
		JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
		drawPanel.getCommandStack().undo();
	}

	/**
	 * Handles the redo operation.
	 */
	public void redoAction() {
		JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
		drawPanel.getCommandStack().redo();
	}

	public void newAction() {
		JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
		handleTool.saveModel(drawPanel);
		drawPanel.getFigureStore().clear();
		drawPanel.getModelStore().clear();
		drawPanel.setChanged(false);
		drawPanel.setModelPath("");
		drawPanel.repaint();
	}

	/**
	 * Handles adding tab operation.
	 */
	public void addTab() {
		ResourceBundle rBundle = ResourceBundleFactory.getResourceBundle();
		String input = JOptionPane.showInputDialog(GlobeParam.getClient(),
				rBundle.getString(LocaleKey.Dialog_Input_Tab_Msg));
		if (input == null)
			return;
		ClientFrame client = GlobeParam.getClient();
		client.addTab(input);
	}

	/**
	 * Handles switching workspace operation.
	 */
	public void switchWorkSpace() {
		new Starter().reStart();
	}

	/**
	 * Loads Esri Shapefile to WorldWind.
	 */
	public void addEsriShapefile() {
		WWDPanel wwPanel = GlobeParam.getWwPanel();
		JFileChooser fc = new JFileChooser(PathConf.getDataDir());
		fc.addChoosableFileFilter(new FileNameExtensionFilter("ESRI Shapefile",
				"shp"));
		int retVal = fc.showOpenDialog(wwPanel);
		if (retVal != JFileChooser.APPROVE_OPTION)
			return;
		String value = fc.getSelectedFile().getPath();
		WWDUtility.addToWorldWind(value, null);
		/*
		 * IFileLoad fileLoad = FileLoadFactory.create(fc.getSelectedFile());
		 * fileLoad.setWWDPanel(wwPanel); fileLoad.execute(); ((Component)
		 * wwPanel.getWwd()) .setCursor(new Cursor(Cursor.WAIT_CURSOR));
		 */
	}

	/**
	 * Loads Tif file to WorldWind.
	 */
	public void addTifFile() {
		WWDPanel wwPanel = GlobeParam.getWwPanel();
		JFileChooser fc = new JFileChooser(PathConf.getDataDir());
		fc.addChoosableFileFilter(new FileNameExtensionFilter(
				"TIFF GeoTIFF(.tif)", "tif"));
		int retVal = fc.showOpenDialog(wwPanel);
		if (retVal != JFileChooser.APPROVE_OPTION)
			return;
		String value = fc.getSelectedFile().getPath();
		WWDUtility.addToWorldWind(value, null);
		/*
		 * IFileLoad fileLoad = FileLoadFactory.create(fc.getSelectedFile());
		 * fileLoad.setWWDPanel(wwPanel); fileLoad.execute();
		 */
	}

	/**
	 * Loads Kml file to WorldWind.
	 */
	public void addKmlFile() {
		WWDPanel wwPanel = GlobeParam.getWwPanel();
		JFileChooser fc = new JFileChooser(PathConf.getDataDir());
		fc.addChoosableFileFilter(new FileNameExtensionFilter(".kml", "kml"));
		int retVal = fc.showOpenDialog(wwPanel);
		if (retVal != JFileChooser.APPROVE_OPTION)
			return;
		String value = fc.getSelectedFile().getPath();
		WWDUtility.addToWorldWind(value, null);
		/*
		 * IFileLoad fileLoad = FileLoadFactory.create(fc.getSelectedFile());
		 * fileLoad.setWWDPanel(wwPanel); fileLoad.execute();
		 */
	}

	/**
	 * Loads web data to worldwind.
	 */
	public void addWebData() {
		String path = JOptionPane.showInputDialog(GlobeParam.getClient(),"Url");
		if (BaseTool.isNull(path)) {
			return;
		}
		boolean flag = NetUtil.validUrl(path);
		if (!flag) {
			ResourceBundle rBundle = ResourceBundleFactory.getResourceBundle();
			String msg = rBundle.getString(LocaleKey.Dialog_Msg_Net_Wrong_Url);
			JOptionPane.showMessageDialog(GlobeParam.getClient(),msg);
			return ;
		}
		WWDUtility.addToWorldWind(path,null);
	}

	/**
	 * Handles WNS operation.
	 */
	public void wnsOpera() {
		WNSDialog dialog = new WNSDialog(GlobeParam.getClient());
		dialog.setVisible(true);
	}

	/**
	 * Handles SOS operation.
	 */
	public void sosOpera() {
		SosOperaDialog dialog = new SosOperaDialog(GlobeParam.getClient());
		dialog.setVisible(true);
	}

	/**
	 * Handles SES operation.
	 */
	public void sesOpera() {
		SESOperaDialog dialog = new SESOperaDialog(GlobeParam.getClient());
		dialog.setVisible(true);
	}

	/**
	 * Handles Subscription operation.
	 */
	public void subcribeAlert() {
		SubscribeNotifyDialog dialog = new SubscribeNotifyDialog(
				GlobeParam.getClient());
		dialog.setVisible(true);
	}

	public void grassRun() {
		JOptionPane.showMessageDialog(GlobeParam.getClient(),
				"Not Implemention");
	}

	/**
	 * Handles adding model operation.
	 */
	public void addModel() {
		AddModelDialog dialog = new AddModelDialog(null);
		dialog.setVisible(true);
	}

	/**
	 * Handles binding model operation.
	 */
	public void bindModel() {
		JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
		figureUtility.clearAllChoosen(drawPanel.getFigureStore()
				.getAllFigures());
		drawPanel.setBtnType(ButtonType.Bind);
		drawPanel.requestFocus();
		drawPanel.setCursor(BindCursor.getInstance().getBindCursor());
	}

	/**
	 * Has a detail of running information.
	 */
	public void detailRun() {
		JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
		figureUtility.clearAllChoosen(drawPanel.getFigureStore()
				.getAllFigures());
		drawPanel.setBtnType(ButtonType.DetailRun);
		drawPanel.requestFocus();
		drawPanel.setCursor(DetailRunCursor.getInstance().getDetailRunCursor());
	}

	/**
	 * Adds sensor balloon to WorldWindow.
	 */
	public void addBalloon() {
		WWDPanel wwdPanel = GlobeParam.getWwPanel();
		SensorBalloon sensorBalloon = new SensorBalloon(wwdPanel);
		sensorBalloon.add(null);
	}

	/**
	 * Adds local data to the world wind.
	 */
	public void addLocalData() {

		WWDPanel wwPanel = GlobeParam.getWwPanel();
		JFileChooser fc = new JFileChooser(PathConf.getDataDir());
		fc.addChoosableFileFilter(new FileNameExtensionFilter("KML", "kml"));
		fc.addChoosableFileFilter(new FileNameExtensionFilter("ESRI Shapefile",
		"shp"));
		fc.addChoosableFileFilter(new FileNameExtensionFilter(
				"TIFF GeoTIFF(.tif)", "tif"));
		int retVal = fc.showOpenDialog(wwPanel);
		if (retVal != JFileChooser.APPROVE_OPTION)
			return;
		String value = fc.getSelectedFile().getPath();
		WWDUtility.addToWorldWind(value, null);
		/*
		 * IFileLoad fileLoad = FileLoadFactory.create(fc.getSelectedFile());
		 * fileLoad.setWWDPanel(wwPanel); fileLoad.execute();
		 */
	
		
	}

	/**
	 * Shows the user manual.
	 */
	public void helpAction() {
		
		String path = PathConf.getCurrentWorkingDirectory()+File.separator+"help.pdf";
		Runtime runtime = Runtime.getRuntime();
		try {
			runtime.exec("cmd.exe /c " + path);
		} catch (Exception e) {
			logger.error(e.getMessage());
			JOptionPane.showMessageDialog(GlobeParam.getClient(), "Failed to open the file:\n"+path);
		}
	}

	/**
	 * Goes to resource center.
	 */
	public void resourceAction() {
		String uri = "https://sourceforge.net/projects/geopw/";
		NetUtil.viewToWeb(uri);
      
	}

	/**
	 * Views the information of the tool.
	 */
	public void aboutAction() {
		AboutDialog dialog = new AboutDialog(GlobeParam.getClient());
		dialog.setVisible(true);
	}

	/**
	 * Views the path of the model of the current JDrawPanel.
	 */
	public void modelPath() {
		JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
		PathDialog dialog = new PathDialog(GlobeParam.getClient(), drawPanel);
		dialog.setVisible(true);
	}

}
