/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.frame.utils;

import java.awt.Component;
import java.io.File;
import java.util.List;

import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JViewport;

import org.apache.log4j.Logger;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;

import cn.edu.whu.lmars.drawpanel.ui.JDrawPanel;
import cn.edu.whu.lmars.figure.utils.FigureStore;
import cn.edu.whu.lmars.frame.ui.TabButtonCom;
import cn.edu.whu.lmars.model.util.PureModelStore;
import cn.edu.whu.lmars.run.bind.beans.BindStore;
import cn.edu.whu.lmars.utils.HandleTool;

import conf.GlobeParam;
import conf.PathConf;

/**
 * This class will read the tabs' configuration file and initialize the
 * JTabbedPane.
 * 
 * @author Mingda Zhang
 * 
 */
public class TabsTool {
	private JTabbedPane tabbedPane;
	private HandleTool handleTool = HandleTool.getInstance();
	private Logger logger = Logger.getLogger(TabsTool.class);

	public TabsTool(JTabbedPane tabbedPane) {
		this.tabbedPane = tabbedPane;
	}

	/**
	 * Initializes the JTabbedPanel according to the configuration file.
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public boolean initTabs() {
		Document document;
		SAXReader reader = new SAXReader();
		try {
			document = reader.read(new File(PathConf.getTabConfDir()));
		} catch (DocumentException e) {
			e.printStackTrace();
			return false;
		}

		if (document == null) {
			logger.info("There is nothing from the file");
			return false;
		}

		Element root = document.getRootElement();
		List<Element> tabs = root.selectNodes("//Tab");
		String title, modelPath, uuid;
		for (Element tab : tabs) {
			title = tab.attributeValue("title");
			modelPath = tab.attributeValue("modelPath").trim();
			uuid = tab.attributeValue("uuid");
			JDrawPanel drawPanel;
			if (uuid.equals(GlobeParam.DEFAULT_UUID)) {
				Component com = tabbedPane.getComponentAt(1);

				JScrollPane scrollPane = (JScrollPane) com;
				JViewport viewport = scrollPane.getViewport();
				drawPanel = (JDrawPanel) viewport.getComponent(0);
				tabbedPane.setSelectedIndex(0);
			} else {
				int num = tabbedPane.getTabCount();
				drawPanel = new JDrawPanel();
				JScrollPane scrollPane = new JScrollPane(drawPanel);
				tabbedPane.add(title, scrollPane);
				drawPanel.setName(title);
				drawPanel.setUuid(uuid);
				drawPanel.setFigureStore(new FigureStore());
				drawPanel.setModelStore(new PureModelStore());
				drawPanel.setBindStore(new BindStore());
				TabButtonCom tabButtonCom = new TabButtonCom(tabbedPane,
						drawPanel);
				tabbedPane.setTabComponentAt(num, tabButtonCom);
				tabbedPane.setSelectedIndex(num);
			}

			GlobeParam.getClient().setCurrentJDrawPanel(drawPanel);

			if (modelPath.equals("")) {
				logger.info(title + "There are no related models");
				continue;
			}

			File modelFile = new File(modelPath);
			if (modelFile.exists() && modelFile.isFile()) {
				boolean flag = handleTool.addModelToPanel(modelPath, drawPanel);
				if (!flag) {
					logger.error("Failed to open the models related to "
							+ title + " tab\n" + "Model Path:" + modelPath);
					drawPanel.setFigureStore(new FigureStore());
					drawPanel.setModelStore(new PureModelStore());
					continue;
				}
				drawPanel.setModelPath(modelPath);
			} else {
				logger.error("There are no models related to " + title + " tab");
			}
		}
		return true;

	}

	public boolean saveTabs() {
		return true;
	}
}
