/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.frame.utils;

import java.awt.Component;
import java.awt.Dimension;
import java.io.FileWriter;
import java.io.IOException;

import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JViewport;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.XMLWriter;

import cn.edu.whu.lmars.drawpanel.ui.JDrawPanel;
import cn.edu.whu.lmars.frame.ui.ClientFrame;
import cn.edu.whu.lmars.utils.HandleTool;
import conf.PathConf;

/**
 * This class is used to save and read the state of the application information.
 * 
 * @author Mingda Zhang
 *
 */
public class WorkSpaceUtil {
	
	/**
	 * Saves JTabbedPane's information to configuration file.
	 */
	private static void saveJTabbedPaneConf(ClientFrame client){
		HandleTool handleTool = HandleTool.getInstance();
		JTabbedPane tabbedPane = client.getDisplayJTabbedPanel();
		Document doc  =DocumentHelper.createDocument();
		Element root = DocumentHelper.createElement("Tabs");
		doc.setRootElement(root);
		for (int i = 0; i < tabbedPane.getTabCount(); i++) {
			Component com = tabbedPane.getComponentAt(i);
			if (com instanceof JScrollPane) {
				JScrollPane scrollPane = (JScrollPane)com;
				JViewport viewport = scrollPane.getViewport();
				JDrawPanel drawPanel = (JDrawPanel)viewport.getComponent(0);
				handleTool.saveModel(drawPanel);
				root.add(createTabEle(drawPanel.getUuid(),drawPanel.getName(), drawPanel.getModelPath()));
			}
		}
		try {
			XMLWriter output = new XMLWriter(new FileWriter(PathConf.getTabConfDir()));
			output.write(doc);
			output.close();
		} catch (IOException e1) {
			System.out.println(e1.getMessage());
			return;
		}
	}
	
	/**
	 * Creates a tab element.
	 * @param uuid
	 * 			id of tab.
	 * @param title
	 * 			title of tab.
	 * @param modelPath
	 * 			the configuration file path of models contained in the tab.
	 * @return
	 */
	private static Element createTabEle(String uuid,String title,String modelPath){
		Element tab = DocumentHelper.createElement("Tab");
		tab.addAttribute("uuid", uuid);
		tab.addAttribute("title", title);
		tab.addAttribute("modelPath", modelPath);
		return tab;
	}
	
	/**
	 * Saves JFrames's location and layout information to configuration file.
	 */
	private static boolean saveLoc(ClientFrame client){
		Dimension clientLoc = new Dimension((int)client.getLocation().getX(),(int)client.getLocation().getY());
		Dimension frameSize = client.getSize();
		JTabbedPane lefTabbedPane= client.getCatalogJTabbedPanel();
		int leftWidth= (int)lefTabbedPane.getSize().getWidth();
		int leftHeight = (int)frameSize.getHeight() - 110;
		Dimension leftSize = new Dimension(leftWidth,leftHeight);
		JTabbedPane rightJTabbedPane = client.getDisplayJTabbedPanel();
		Dimension rightSize = new Dimension((int)(frameSize.getWidth()-leftWidth-30),leftHeight);
		int index = rightJTabbedPane.getSelectedIndex();
		LocTool locTool = LocTool.getInstance(PathConf.getLocConfDir());
		return locTool.saveInfo(leftSize, rightSize, clientLoc, index);
	}
	
	public static void saveWorkSpace(ClientFrame client){
		saveLoc(client);
		saveJTabbedPaneConf(client);
	}
}
