/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.locale;

import java.util.Locale;
import java.util.ResourceBundle;

/**
 * This factory will create different Resource Bundles according to the setting.
 * Now, this class supports switching between Chinese and English. Makes the
 * application easily localized, or translated, into other languages.
 * 
 * @author Mingda Zhang
 * 
 */
public class ResourceBundleFactory {
	private static ResourceBundle enrb, zhrb;

	/**
	 * English version resource.
	 * 
	 * @return java.util.ResourceBundle
	 */
	public static ResourceBundle get_en_ResourceBundle() {
		if (enrb == null) {
			Locale enLocale = new Locale("en", "US");
			enrb = ResourceBundle.getBundle(LocaleKey.Res_location, enLocale);
		}
		return enrb;
	}

	/**
	 * Chinese version resource.
	 * 
	 * @return java.util.ResourceBundle
	 */
	public static ResourceBundle get_zh_ResourceBundle() {
		if (zhrb == null) {
			Locale cnLocale = new Locale("zh", "CN");
			zhrb = ResourceBundle.getBundle(LocaleKey.Res_location, cnLocale);
		}
		return zhrb;
	}

	/**
	 * Default version resource.
	 * 
	 * @return java.util.ResourceBundle
	 */
	public static ResourceBundle getResourceBundle() {
		return get_en_ResourceBundle();
	}
}
