/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.model;

import java.util.ArrayList;
import java.util.List;

/**
 * An implementor of IModel for process model.
 * 
 * @author Mingda Zhang
 * 
 */
public class AtomProcess implements IModel {

	/** Id of process model. */
	protected String uuid;
	/** Name of process model. */
	protected String name;
	/** Abstract of process model. */
	protected String _abstract;
	/** Complex Parameters */
	protected List<ComplexParam> complexParam = new ArrayList<ComplexParam>(4);
	/** Literal Parameters */
	protected List<LiteralParam> literalParam = new ArrayList<LiteralParam>(3);
	/** OutPut Parameter */
	protected OutPutParam outPutParam;

	/**
	 * Gets id.
	 * 
	 * @return
	 */
	public String getUUID() {
		return uuid;
	}

	/**
	 * Sets id.
	 * 
	 * @param value
	 */
	public void setUUID(String value) {
		this.uuid = value;
	}

	/**
	 * Gets name.
	 */
	public String getName() {
		return name;
	}

	/**
	 * Sets name.
	 * 
	 * @param value
	 */
	public void setName(String value) {
		this.name = value;
	}

	/**
	 * Gets abstract.
	 */
	public String getAbstract() {
		return _abstract;
	}

	/**
	 * Sets abstract.
	 */
	public void setAbstract(String value) {
		this._abstract = value;
	}

	/**
	 * Gets complex parameters
	 */
	public List<ComplexParam> getComplexParam() {
		return this.complexParam;
	}

	/**
	 * Gets literal parameters.
	 */
	public List<LiteralParam> getLiteralParam() {
		return this.literalParam;
	}

	/**
	 * Gets output parameter.
	 */
	public OutPutParam getOutPutParam() {
		return outPutParam;
	}

	/**
	 * Sets output parameter.
	 * 
	 * @param value
	 */
	public void setOutPutParam(OutPutParam value) {
		this.outPutParam = value;
	}

	/**
	 * Adds complex parameter to the this process model.
	 * 
	 * @param param
	 *            complex parameter.
	 */
	public void addComplexParam(ComplexParam param) {
		if (param == null) {
			return;
		}
		if (!complexParam.contains(param)) {
			this.complexParam.add(param);
		}
	}

	/**
	 * Adds literal parameter to the process model.
	 * 
	 * @param param
	 *            literal parameter.
	 */
	public void addLiteralParam(LiteralParam param) {
		if (param == null) {
			return;
		}
		if (!literalParam.contains(param)) {
			this.literalParam.add(param);
		}
	}

	/**
	 * Gets complex parameter by name.
	 * 
	 * @param paramName
	 *            parameter name.
	 */
	public ComplexParam getComplexParamByName(String paramName) {
		for (ComplexParam param : getComplexParam()) {
			if (param.getName().equals(paramName)) {
				return param;
			}
		}
		return null;
	}

	/**
	 * Gets literal parameter by name.
	 * 
	 * @param name
	 *            parameter name.
	 */
	public LiteralParam getLiteralParamByName(String name) {
		for (LiteralParam param : getLiteralParam()) {
			if (param.getName().equals(name)) {
				return param;
			}
		}
		return null;
	}

	public AbstractParam getParamByName(String name) {
		AbstractParam param = getComplexParamByName(name);
		if (param != null) {
			return param;
		}
		return getLiteralParamByName(name);

	}
}
