/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.model.ui;

import java.awt.Dialog;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.WindowConstants;
import javax.swing.table.DefaultTableModel;

import conf.GlobeParam;

import cn.edu.whu.lmars.drawpanel.ui.JDrawPanel;
import cn.edu.whu.lmars.figure.beans.DataFigure;
import cn.edu.whu.lmars.figure.beans.LinkFigure;
import cn.edu.whu.lmars.figure.beans.ProcessFigure;
import cn.edu.whu.lmars.figure.utils.FigureStore;
import cn.edu.whu.lmars.locale.LocaleKey;
import cn.edu.whu.lmars.locale.ResourceBundleFactory;
import cn.edu.whu.lmars.model.AtomProcess;
import cn.edu.whu.lmars.model.ComplexParam;
import cn.edu.whu.lmars.model.LiteralParam;
import cn.edu.whu.lmars.model.OutPutParam;
import cn.edu.whu.lmars.model.util.PureModelStore;
import cn.edu.whu.lmars.utils.HandleTool;

/**
 * This dialog is used to add a process model to the current JDrawPanel.
 * 
 * @author Mingda Zhang
 * 
 */
public class AddModelDialog extends JDialog implements ActionListener {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JLabel jLabel = null;
	private JLabel jLabel1 = null;
	private JTextField nameTextField = null;
	private JTextField abstractTextField = null;
	private JLabel jLabel2 = null;
	private JScrollPane jScrollPane = null;
	private JTable paramTable = null;
	private JButton addButton = null;
	private JButton deleteButton = null;
	private JButton confirmButton = null;
	private JButton concelButton = null;
	private ResourceBundle rBundle = null;
	private String msg;

	public AddModelDialog(Frame owner) {
		super(owner);
		initialize();
	}

	/**
	 * Initializes AddModelDialog.
	 */
	private void initialize() {
		rBundle = ResourceBundleFactory.getResourceBundle();
		this.setSize(436, 352);
		this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		this.setFont(new Font("Dialog", Font.PLAIN, 14));
		msg = rBundle.getString(LocaleKey.dialog_Model_Title);
		this.setTitle(msg);
		this.setContentPane(getJContentPane());
		this.setLocationRelativeTo(null);
		this.setModal(true);
	}

	/**
	 * This method initializes nameTextField.
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getNameTextField() {
		if (nameTextField == null) {
			nameTextField = new JTextField();
			nameTextField.setBounds(new Rectangle(90, 15, 290, 30));
			nameTextField.setFont(new Font("Dialog", Font.PLAIN, 14));
		}
		return nameTextField;
	}

	/**
	 * This method initializes abstractTextField.
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getAbstractTextField() {
		if (abstractTextField == null) {
			abstractTextField = new JTextField();
			abstractTextField.setBounds(new Rectangle(90, 50, 290, 30));
		}
		return abstractTextField;
	}

	/**
	 * This method initializes jScrollPane.
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setBounds(new Rectangle(28, 124, 274, 135));
			jScrollPane.setFont(new Font("Dialog", Font.PLAIN, 14));
			jScrollPane.setViewportView(getParamTable());
		}
		return jScrollPane;
	}

	/**
	 * This method initializes ParamTable.
	 * 
	 * @return javax.swing.JTable
	 */
	private JTable getParamTable() {
		if (paramTable == null) {
			String name = rBundle
					.getString(LocaleKey.dialog_Model_Table_Column_Name);
			String type = rBundle
					.getString(LocaleKey.dialog_Model_Table_Column_Type);
			String[] columnTitles = { name, type };
			DefaultTableModel tableModel = new DefaultTableModel(null,
					columnTitles);
			paramTable = new JTable(tableModel);
			paramTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			paramTable.setRowHeight(25);
			paramTable.setFont(new Font("Dialog", Font.PLAIN, 14));
		}
		return paramTable;
	}

	/**
	 * This method initializes addButton.
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getAddButton() {
		if (addButton == null) {
			addButton = new JButton();
			addButton.setBounds(new Rectangle(317, 147, 87, 25));
			msg = rBundle.getString(LocaleKey.button_Add);
			addButton.setText(msg);
			addButton.setActionCommand("add");
			addButton.addActionListener(this);
			addButton.setFont(new Font("Dialog", Font.BOLD, 14));
		}
		return addButton;
	}

	/**
	 * This method initializes deleteButton.
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getDeleteButton() {
		if (deleteButton == null) {
			deleteButton = new JButton();
			deleteButton.setBounds(new Rectangle(317, 195, 87, 25));
			msg = rBundle.getString(LocaleKey.button_Delete);
			deleteButton.setText(msg);
			deleteButton.setActionCommand("delete");
			deleteButton.addActionListener(this);
			deleteButton.setFont(new Font("Dialog", Font.BOLD, 14));
		}
		return deleteButton;
	}

	/**
	 * This method initializes confirmButton.
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getConfirmButton() {
		if (confirmButton == null) {
			confirmButton = new JButton();
			confirmButton.setBounds(new Rectangle(46, 268, 91, 30));
			msg = rBundle.getString(LocaleKey.Button_OK);
			confirmButton.setText(msg);
			confirmButton.setActionCommand("confirm");
			confirmButton.addActionListener(this);
			confirmButton.setFont(new Font("Dialog", Font.BOLD, 14));
		}
		return confirmButton;
	}

	/**
	 * This method initializes concelButton.
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getConcelButton() {
		if (concelButton == null) {
			concelButton = new JButton();
			concelButton.setBounds(new Rectangle(155, 268, 91, 30));
			msg = rBundle.getString(LocaleKey.Button_Concel);
			concelButton.setText(msg);
			concelButton.setActionCommand("concel");
			concelButton.addActionListener(this);
			concelButton.setFont(new Font("Dialog", Font.BOLD, 14));
		}
		return concelButton;
	}

	/**
	 * This method initializes jContentPane.
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jLabel2 = new JLabel();
			jLabel2.setBounds(new Rectangle(10, 85, 105, 30));
			jLabel2.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel2.setHorizontalAlignment(SwingConstants.RIGHT);
			msg = rBundle.getString(LocaleKey.dialog_Model_Label_Param);
			jLabel2.setText(msg);
			jLabel1 = new JLabel();
			jLabel1.setBounds(new Rectangle(10, 50, 70, 30));
			jLabel1.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel1.setHorizontalAlignment(SwingConstants.RIGHT);
			msg = rBundle.getString(LocaleKey.dialog_Model_Label_Abstract);
			jLabel1.setText(msg);
			jLabel = new JLabel();
			jLabel.setBounds(new Rectangle(10, 15, 70, 30));
			jLabel.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel.setHorizontalAlignment(SwingConstants.RIGHT);
			msg = rBundle.getString(LocaleKey.dialog_Model_Label_Name);
			jLabel.setText(msg);
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.setFont(new Font("Dialog", Font.PLAIN, 14));
			jContentPane.add(jLabel, null);
			jContentPane.add(jLabel1, null);
			jContentPane.add(getNameTextField(), null);
			jContentPane.add(getAbstractTextField(), null);
			jContentPane.add(jLabel2, null);
			jContentPane.add(getJScrollPane(), null);
			jContentPane.add(getAddButton(), null);
			jContentPane.add(getDeleteButton(), null);
			jContentPane.add(getConfirmButton(), null);
			jContentPane.add(getConcelButton(), null);
		}
		return jContentPane;
	}

	public static void main(String[] args) {
		Dialog dialog = new AddModelDialog(null);
		dialog.setVisible(true);
	}

	/**
	 * Creates figures and models and adds them to containers.
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		String str = e.getActionCommand();
		DefaultTableModel model = (DefaultTableModel) paramTable.getModel();
		if (str.equals("add")) {
			AddParamDialog dialog = new AddParamDialog(null, paramTable);
			dialog.setVisible(true);
		} else if (str.equals("delete")) {
			int num = this.paramTable.getSelectedRow();
			if (num == -1) {
				return;
			}
			model.removeRow(num);
		} else if (str.equals("confirm")) {
			String name = nameTextField.getText();
			String _abstract = abstractTextField.getText();
			if (name.trim().equals("")) {
				msg = rBundle
						.getString(LocaleKey.Dialog_Msg_Model_Wrong_NulName);
				JOptionPane.showMessageDialog(null, msg);
				return;
			}

			ProcessFigure processFigure = new ProcessFigure(10, 10, 90, 60);
			String uuid = HandleTool.getUUID();
			AtomProcess atomProcess = new AtomProcess();
			atomProcess.setUUID(uuid);
			atomProcess.setName(name);
			atomProcess.setAbstract(_abstract);
			processFigure.setUuid(uuid);
			processFigure.setModel(atomProcess);
			String paramName, paramType;
			for (int i = 0; i < model.getRowCount(); i++) {
				paramName = model.getValueAt(i, 0).toString();
				paramType = model.getValueAt(i, 1).toString();
				if (paramType.equals("Complex Parameter")) {
					ComplexParam complexParam = new ComplexParam();
					complexParam.setName(paramName);
					atomProcess.addComplexParam(complexParam);
					DataFigure dataFigure = new DataFigure(0, 0, 90, 60);
					dataFigure.setState("nul");
					dataFigure.setUuid(uuid + "#" + paramName);
					processFigure.addDataFigure(dataFigure);
					LinkFigure linkFigure = new LinkFigure(dataFigure.pointR,
							processFigure.pointL, dataFigure, processFigure);
					linkFigure.setUuid(dataFigure.getUuid());
					dataFigure.setLinkFigure(linkFigure);
					dataFigure.setModel(complexParam);
				} else if (paramType.equals("Literal Parameter")) {
					LiteralParam literalParam = new LiteralParam();
					literalParam.setName(paramName);
					atomProcess.addLiteralParam(literalParam);
				}

			}

			OutPutParam outPutParam = new OutPutParam();
			atomProcess.setOutPutParam(outPutParam);
			DataFigure dataFigure = new DataFigure(120, 10, 90, 60);
			dataFigure.setState("res");
			dataFigure.setModel(outPutParam);
			dataFigure.setUuid(uuid + "#result");
			processFigure.addDataFigure(dataFigure);

			LinkFigure linkFigure = new LinkFigure(processFigure.pointR,
					dataFigure.pointL, processFigure, dataFigure);
			linkFigure.setUuid(dataFigure.getUuid());
			dataFigure.setLinkFigure(linkFigure);
			processFigure.addLinkFigure(linkFigure);
			JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
			PureModelStore modelStore = drawPanel.getModelStore();
			FigureStore figureStore = drawPanel.getFigureStore();
			modelStore.addAtomProcess(atomProcess);
			figureStore.addProcessFigure(processFigure);
			HandleTool.refreshFigShow();
			this.dispose();
		} else if (str.equals("concel")) {
			this.dispose();
		}
	}
}
