/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.model.ui;

import java.awt.Font;
import java.awt.Frame;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.WindowConstants;
import javax.swing.table.DefaultTableModel;

import cn.edu.whu.lmars.locale.LocaleKey;
import cn.edu.whu.lmars.locale.ResourceBundleFactory;

/**
 * Adds parameters to a process model.
 * 
 * @author Mingda Zhang
 *
 */
public class AddParamDialog extends JDialog implements ActionListener{

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JLabel jLabel = null;
	private JTextField nameTextField = null;
	private JLabel jLabel1 = null;
	private JComboBox typeComboBox = null;
	private JButton confirmButton = null;
	private JButton concelButton = null;
	private JTable table = null;
	private ResourceBundle rBundle = null;
	private String msg;
	/**
	 * @param owner
	 */
	public AddParamDialog(Frame owner,JTable table) {
		super(owner);
		rBundle = ResourceBundleFactory.getResourceBundle();
		this.table = table;
		initialize();
	}
	private void initialize() {
		this.setSize(377, 182);
		msg = rBundle.getString(LocaleKey.dialog_Model_Add_Column_Title);
		this.setTitle(msg);
		this.setFont(new Font("Dialog", Font.PLAIN, 14));
		this.setContentPane(getJContentPane());
		this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		initComp();
		this.setLocationRelativeTo(null);
		this.setModal(true);
		
	}
	private void initComp(){
		this.typeComboBox.addItem("Complex Parameter");
		this.typeComboBox.addItem("Literal Parameter");
		this.typeComboBox.setSelectedIndex(0);
	}
	private JTextField getNameTextField() {
		if (nameTextField == null) {
			nameTextField = new JTextField();
			nameTextField.setBounds(new Rectangle(90, 15, 240, 30));
		}
		return nameTextField;
	}

	/**
	 * This method initializes typeComboBox	.
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getTypeComboBox() {
		if (typeComboBox == null) {
			typeComboBox = new JComboBox();
			typeComboBox.setBounds(new Rectangle(90, 50, 240, 30));
		}
		return typeComboBox;
	}

	/**
	 * This method initializes confirmButton.	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getConfirmButton() {
		if (confirmButton == null) {
			confirmButton = new JButton();
			confirmButton.setBounds(new Rectangle(45, 96, 86, 28));
			msg = rBundle.getString(LocaleKey.Button_OK);
			confirmButton.setText(msg);
			confirmButton.setActionCommand("confirm");
			confirmButton.addActionListener(this);
			confirmButton.setFont(new Font("Dialog", Font.BOLD, 14));
		}
		return confirmButton;
	}

	/**
	 * This method initializes concelButton.	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getConcelButton() {
		if (concelButton == null) {
			concelButton = new JButton();
			concelButton.setBounds(new Rectangle(175, 96, 97, 28));
			msg = rBundle.getString(LocaleKey.Button_Concel);
			concelButton.setText(msg);
			concelButton.addActionListener(this);
			concelButton.setActionCommand("concel");
			concelButton.setFont(new Font("Dialog", Font.BOLD, 14));
		}
		return concelButton;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub

	}

	

	/**
	 * This method initializes jContentPane.
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jLabel1 = new JLabel();
			jLabel1.setBounds(new Rectangle(10, 50, 65, 30));
			jLabel1.setHorizontalAlignment(SwingConstants.RIGHT);
			jLabel1.setFont(new Font("Dialog", Font.BOLD, 14));
			msg = rBundle.getString(LocaleKey.dialog_Model_Table_Column_Type);
			jLabel1.setText(msg+":");
			jLabel = new JLabel();
			jLabel.setBounds(new Rectangle(10, 15, 65, 30));
			jLabel.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel.setHorizontalAlignment(SwingConstants.RIGHT);
			msg = rBundle.getString(LocaleKey.dialog_Model_Table_Column_Name);
			jLabel.setText(msg+":");
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(jLabel, null);
			jContentPane.add(getNameTextField(), null);
			jContentPane.add(jLabel1, null);
			jContentPane.add(getTypeComboBox(), null);
			jContentPane.add(getConfirmButton(), null);
			jContentPane.add(getConcelButton(), null);
		}
		return jContentPane;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		// TODO Auto-generated method stub
		String str = e.getActionCommand();
		if (str.equals("confirm")) {
			String name = nameTextField.getText();
			String type = typeComboBox.getSelectedItem().toString();
			if (name.trim().equals("")) {
				msg = rBundle.getString(LocaleKey.Dialog_Msg_Model_Wrong_NulName);
				JOptionPane.showMessageDialog(null, msg);
				return;
			}
			DefaultTableModel model = (DefaultTableModel)this.table.getModel();
			model.addRow(new String[]{name,type});
			this.dispose();
		}else if (str.equals("concel")) {
			this.dispose();
		}
	}

}  
