/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.run.beans;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import cn.edu.whu.lmars.model.AbstractParam;
import cn.edu.whu.lmars.model.AtomProcess;
import cn.edu.whu.lmars.model.ComplexParam;
import cn.edu.whu.lmars.run.bind.beans.BindAtom;
import cn.edu.whu.lmars.run.bind.beans.BindAtom.ParamMapInfo;
import cn.edu.whu.lmars.run.ui.ExecuteMonitorDialog;
import cn.edu.whu.lmars.service.adapter.AdapterFactory;
import cn.edu.whu.lmars.service.adapter.base.IAdapter;
import cn.edu.whu.lmars.service.adapter.base.IOperationInfo;
import cn.edu.whu.lmars.service.adapter.base.IParameterInfo;
import cn.edu.whu.lmars.service.meta.ParameterMeta;
import cn.edu.whu.lmars.service.utils.NetUtil;
import cn.edu.whu.lmars.utils.BaseTool;
import cn.edu.whu.lmars.utils.Dom4jXMLTool;
import conf.GlobeParam;

/**
 * 
 * The class is an execution activity which is responsible for the
 * implementation of a single service.
 * 
 * @author Mingda Zhang
 * 
 */

public class FlowActivity {
	/** model. */
	private AtomProcess model;
	/** the map of parameter value. */
	private Map<String, String> paramMap = new HashMap<String, String>();
	/** the map of result value. */
	private Map<String, String> outValues;
	/** logger. */
	private Logger logger = Logger.getLogger(FlowActivity.class);
	/** service address. */
	private String service;
	/** operation name. */
	private String operationName;
	/** service type. */
	private String serviceType;
	/** service adapter. */
	private IAdapter adapter = null;
	/** binding information. */
	private BindAtom bindAtom;
	/** execution monitor dialog. */
	private ExecuteMonitorDialog monitorDialog = ExecuteMonitorDialog
			.getInstance(GlobeParam.getClient());
	/** execution information. */
	private ServiceExecutionInfo sei = new ServiceExecutionInfo();

	/**
	 * Instantiates a FlowActivity.
	 * 
	 * @param handleModel
	 *            process model.
	 * @param bindAtom
	 *            binding information.
	 */
	public FlowActivity(AtomProcess handleModel, BindAtom bindAtom) {
		this.model = handleModel;
		this.bindAtom = bindAtom;
	}

	/**
	 * Checks whether all parameters are assigned and binding information is
	 * available.
	 * 
	 * @return true if this activity meets the requirements.
	 */
	public boolean isReady() {
		
		if (paramMap.size()>0) {
			paramMap.clear();
		}
		if (bindAtom == null) {
			monitorDialog.appendMsg("Failed to get " + model.getName()
					+ "'s binding information.\n");
			return false;
		}
		monitorDialog.appendMsg("Checking the integrity of parameters of " + model.getName()
				+ ".\n");
		service = bindAtom.getUrl();
		if (!NetUtil.validUrl(service)) {
			monitorDialog.appendMsg(model.getName()+"'s binding service is unavailable. \n Service address:"+service
					+ ".\n");
			return false;
		}
		serviceType = bindAtom.getServiceType().toString();
		operationName = bindAtom.getOperationName();
		sei.setServiceUrl(service);
		sei.setType(serviceType);
		sei.setOperation(operationName);
		adapter = AdapterFactory.getInstance().getAdapter(bindAtom);
		if (adapter == null) {
			monitorDialog.appendMsg("Failed to get the adapter.");
			return false;
		}

		//Extracts parameters.
		assignValue();
		/*AbstractParam abstractParam;
		String fromParam, toParam;
		for (ParamMapInfo paramMapInfo : bindAtom.getParamMapInfos()) {
			fromParam = paramMapInfo.getFrom();
			toParam = paramMapInfo.getTo();
			if (fromParam.equals("result_outputformat")) {
				abstractParam = model.getOutPutParam();
				paramMap.put(toParam, abstractParam.getFormat());
				continue;
			}

			abstractParam = model.getParamByName(fromParam);
			paramMap.put(toParam, abstractParam.getValue());
			if (abstractParam instanceof ComplexParam) {
				paramMap.put(toParam + "_datatype", abstractParam.getFormat());
			}
		}*/
		
		IOperationInfo operationInfo = adapter.getOperationInfo(operationName);
		if (operationInfo == null) {
			monitorDialog.appendMsg("Failed to get the operation information of " + service + "@"
					+ operationName + "\n");
			return false;
		}
		List<IParameterInfo> paramInfos = operationInfo.getInputParameterInfos();
		String paramName,minOccur,value;
		ParameterMeta paramMeta;
		for(IParameterInfo paramInfo:paramInfos){
			paramName = paramInfo.getName();
			value = paramMap.get(paramName);
			if (BaseTool.isNull(value)) {
				paramMeta = paramInfo.getMeta();
				if (paramMeta == null) {
					monitorDialog.appendMsg("The parameter named " + paramName +" in "+operationName + " is null\n");
					return false;
				}
				minOccur = paramMeta.get_minoccurs();
				try {
					int min = Integer.parseInt(minOccur);
					if (min!=0) {
						monitorDialog.appendMsg("The parameter named " + paramName +" in "+operationName + " is null\n");
						return false;
					}
				} catch (Exception e) {
					return false;
				}
			}
		}
		return true;
	}

	/**
	 * Extracts the value of parameter.
	 */
	private void assignValue(){
		AbstractParam abstractParam;
		String fromParam, toParam;
		for (ParamMapInfo paramMapInfo : bindAtom.getParamMapInfos()) {
			fromParam = paramMapInfo.getFrom();
			toParam = paramMapInfo.getTo();
			if (fromParam.equals("result_outputformat")) {
				abstractParam = model.getOutPutParam();
				paramMap.put(toParam, abstractParam.getFormat());
				continue;
			}

			abstractParam = model.getParamByName(fromParam);
			paramMap.put(toParam, abstractParam.getValue());
			if (abstractParam instanceof ComplexParam) {
				paramMap.put(toParam + "_datatype", abstractParam.getFormat());
			}
		}
	}
	/**
	 * Executes this service activity.
	 * @return
	 */
	public boolean execute() {
		assignValue();
		monitorDialog.appendMsg("Number:" + sei.getExecuteSequence()+"\n");
		monitorDialog.appendMsg("Execute " + service + "@" + operationName
				+ "\n");
		logger.info("Execute " + service + "@" + operationName);
		logger.info("Parameters: " + paramMap);
		sei.setInputMap(paramMap);
		String request = adapter.createRequest(operationName, paramMap);

		if (request == null) {
			monitorDialog.appendMsg("Failed to create request.\n");
			logger.error("Failed to create request.");
			return false;
		}
		String formatRequest = Dom4jXMLTool.format(request);
		if (!BaseTool.isNull(formatRequest)) {
			sei.setRequestXML(formatRequest);
			logger.info("Request Message : \n" + formatRequest);
		} else {
			sei.setRequestXML(request);
			logger.info("Request Message : \n" + request + "\n");
		}
		long begintime = System.currentTimeMillis();

		String resp = adapter.sendRequest(request);
		long endtime = System.currentTimeMillis();
		if (resp == null) {
			// errBuf.append(operationName+"没有返回结果");
			monitorDialog.appendMsg("Failed to get response.\n");
			return false;
		}
		String respFormat = Dom4jXMLTool.format(resp);

		if (!BaseTool.isNull(respFormat)) {
			sei.setResponseXML(respFormat);
			logger.info("Response Message :\n" + respFormat);
		} else {
			sei.setResponseXML(resp);
			logger.info("Response Message :\n" + resp + "\n");
		}
		sei.setExecutionBeginTime(begintime);
		sei.setExecutionEndTime(endtime);

		outValues = adapter.parseResponse(resp);
		monitorDialog.appendMsg("The reslut:\n" + outValues + "\n");
		monitorDialog.appendMsg("beginTime:"
				+ BaseTool.getTimeStringByLong(begintime) + "\n");
		monitorDialog.appendMsg("endTime:"
				+ BaseTool.getTimeStringByLong(endtime) + "\n");
		logger.info("Result:"+outValues);
		if (outValues == null) {
			logger.error("No result");
			monitorDialog.appendMsg("No result.\n");
			return false;
		}
		sei.setOutputMap(outValues);
		return true;
	}

	/**
	 * Gets ServiceExecutionInfo.
	 * 
	 * @return ServiceExecutionInfo
	 */
	public ServiceExecutionInfo getServiceExecutionInfo() {
		return sei;
	}
}
