/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.run.beans;

import java.util.ArrayList;

import cn.edu.whu.lmars.model.AtomProcess;
import cn.edu.whu.lmars.run.bind.beans.BindAtom;

/**
 * This tree node represents an atomic service in the service chain.
 * 
 * @author Mingda Zhang
 * 
 */
public class ModelTreeNode {
	/** process model. */
	private AtomProcess model = null;
	/** parent of this node. */
	private ModelTreeNode parent = null;
	/** children of this node. */
	private ArrayList<ModelTreeNode> children = new ArrayList<ModelTreeNode>();
	/** execution activity. */
	private FlowActivity flowActivity;

	public ModelTreeNode(AtomProcess self, BindAtom bindAtom) {
		this.model = self;
		flowActivity = new FlowActivity(model, bindAtom);
	}

	/**
	 * Gets process model.
	 */
	public AtomProcess getSelf() {
		return model;
	}

	/**
	 * Sets process model.
	 * 
	 * @param self
	 *            process model.
	 */
	public void setSelf(AtomProcess self) {
		this.model = self;
	}

	/**
	 * Gets children nodes.
	 */
	public ArrayList<ModelTreeNode> getChildren() {
		return children;
	}

	/**
	 * @return parent tree node.
	 */
	public ModelTreeNode getParent() {
		return parent;
	}

	/**
	 * Sets parent tree node.
	 * 
	 * @param parent
	 */
	public void setParent(ModelTreeNode parent) {
		this.parent = parent;
	}

	/**
	 * Adds child tree node.
	 * 
	 * @param child
	 */
	public void addChild(ModelTreeNode child) {
		if (!children.contains(child)) {
			children.add(child);
		}
	}

	/**
	 * Removed child tree node.
	 * 
	 * @param child
	 */
	public void removeChild(ModelTreeNode child) {
		if (children.contains(child)) {
			children.remove(child);
		}
	}

	/**
	 * Gets ServiceExecutionInfo.
	 */
	public ServiceExecutionInfo getExecutionInfo() {
		return flowActivity.getServiceExecutionInfo();
	}

	/**
	 * Executes this node.
	 * 
	 * @return
	 */
	public boolean execute() {

		/*if (flowActivity.isReady()) {
			flag = flowActivity.execute();
		}*/
		return flowActivity.execute();
		
	}

	/**
	 * Checks if getting ready to execute this node.
	 * @return
	 */
	public boolean isReady(){
		return flowActivity.isReady();
	}
	/**
	 * Sets execution sequence of this node.
	 * 
	 * @param num
	 */
	public void setSequence(int num) {
		flowActivity.getServiceExecutionInfo().setExecuteSeqnum(num);
	}
}
