/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.run.beans;

import java.util.Map;

/**
 * This class stores service execution information.
 * 
 * @author Ziheng Sun, Mingda Zhang
 * 
 */

public class ServiceExecutionInfo {

	/** Input parameter Map. */
	Map<String, String> inputMap;
	/** Output parameter Map. */
	Map<String, String> outputMap;
	/** Request. */
	String requestXML;
	/** Response. */
	String responseXML;
	long executionBeginTime, executionEndTime;
	/** ExecutionState. */
	ExecutionState es;

	/** Execution sequence. */
	int executeseqnum;
	/** service address. */
	private String serviceUrl;
	/** service type. */
	private String type;
	/** operation name. */
	private String operation;

	/**
	 * Gets ExecutionState.
	 */
	public ExecutionState getEs() {
		ExecutionState es = ExecutionState.READY;
		if (getRequestXML() != null && getResponseXML() == null) {
			es = ExecutionState.RUNNING;
		} else if (getRequestXML() != null && getResponseXML() != null) {
			es = ExecutionState.FINISHED;
		}
		return es;
	}

	/**
	 * Sets ExecutionState.
	 * 
	 * @param es
	 */
	public void setEs(ExecutionState es) {
		this.es = es;
	}

	/**
	 * Gets input parameter map.
	 */
	public Map<String, String> getInputMap() {
		return inputMap;
	}

	/**
	 * Sets input parameter map.
	 * 
	 * @param inputMap
	 *            input parameter map.
	 */
	public void setInputMap(Map<String, String> inputMap) {
		this.inputMap = inputMap;
	}

	/**
	 * Gets output parameter map.
	 * 
	 * @return output parameter map.
	 */
	public Map<String, String> getOutputMap() {
		return outputMap;
	}

	/**
	 * Sets output parameter map.
	 * 
	 * @param outputMap
	 *            output parameter map.
	 */
	public void setOutputMap(Map<String, String> outputMap) {
		this.outputMap = outputMap;
	}

	/**
	 * Gets request xml.
	 * 
	 * @return request xml.
	 */
	public String getRequestXML() {
		return requestXML;
	}

	/**
	 * Sets request xml.
	 * 
	 * @param requestXML
	 *            request xml.
	 */
	public void setRequestXML(String requestXML) {
		this.requestXML = requestXML;
	}

	/**
	 * Gets response xml.
	 * 
	 * @return response xml.
	 */
	public String getResponseXML() {
		return responseXML;
	}

	/**
	 * Sets response xml.
	 * 
	 * @param responseXML
	 */
	public void setResponseXML(String responseXML) {
		this.responseXML = responseXML;
	}

	/**
	 * Gets begin time of execution .
	 */
	public long getExecutionBeginTime() {
		return executionBeginTime;
	}

	/**
	 * Sets begin time of execution .
	 * 
	 * @param executionBeginTime
	 */
	public void setExecutionBeginTime(long executionBeginTime) {
		this.executionBeginTime = executionBeginTime;
	}

	/**
	 * Gets end time of execution.
	 * 
	 * @return end time of execution.
	 */
	public long getExecutionEndTime() {
		return executionEndTime;
	}

	/**
	 * Sets end time of execution.
	 * 
	 * @param executionEndTime
	 */
	public void setExecutionEndTime(long executionEndTime) {
		this.executionEndTime = executionEndTime;
	}

	/**
	 * Sets execution sequence.
	 * 
	 * @param num
	 *            execution sequence.
	 */
	public void setExecuteSeqnum(int num) {
		executeseqnum = num;
	}

	/**
	 * Gets execution sequence.
	 * 
	 * @return execution sequence.
	 */
	public int getExecuteSequence() {

		return executeseqnum;
	}

	/**
	 * Gets service address.
	 * 
	 * @return service address.
	 */
	public String getServiceUrl() {
		return serviceUrl;
	}

	/**
	 * Sets service address.
	 * 
	 * @param serviceUrl
	 *            service address.
	 */
	public void setServiceUrl(String serviceUrl) {
		this.serviceUrl = serviceUrl;
	}

	/**
	 * Gets service type.
	 * 
	 * @return service type.
	 */
	public String getType() {
		return type;
	}

	/**
	 * Sets service type.
	 * 
	 * @param type
	 *            service type.
	 */
	public void setType(String type) {
		this.type = type;
	}

	/**
	 * Gets operation name.
	 * 
	 * @return operation name.
	 */
	public String getOperation() {
		return operation;
	}

	/**
	 * Sets operation name.
	 * 
	 * @param operation
	 *            operation name.
	 */
	public void setOperation(String operation) {
		this.operation = operation;
	}

}
