/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.run.bind;

import java.awt.Font;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Enumeration;
import java.util.List;
import java.util.ResourceBundle;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.WindowConstants;
import javax.swing.table.DefaultTableModel;

import org.apache.log4j.Logger;

import cn.edu.whu.lmars.locale.LocaleKey;
import cn.edu.whu.lmars.locale.ResourceBundleFactory;
import cn.edu.whu.lmars.model.AtomProcess;
import cn.edu.whu.lmars.model.ComplexParam;
import cn.edu.whu.lmars.model.LiteralParam;
import cn.edu.whu.lmars.run.bind.beans.BindAtom;
import cn.edu.whu.lmars.run.bind.beans.BindProcess;
import cn.edu.whu.lmars.run.bind.beans.BindStore;
import cn.edu.whu.lmars.service.adapter.AdapterFactory;
import cn.edu.whu.lmars.service.adapter.base.IAdapter;
import cn.edu.whu.lmars.service.adapter.base.IOperationInfo;
import cn.edu.whu.lmars.service.adapter.base.IParameterInfo;
import cn.edu.whu.lmars.tree.nodes.ServiceTreeNode;
import cn.edu.whu.lmars.tree.nodes.TreeNodeImpl;
import cn.edu.whu.lmars.tree.ui.TreeScrollPanel;
import cn.edu.whu.lmars.utils.HandleTool;
import cn.edu.whu.lmars.utils.KMPMatch;
import conf.GlobeParam;

/**
 * This dialog is used to add a binding to the selected process model.
 * 
 * @author Mingda Zhang
 * 
 */
public class AddBindDialog extends JDialog implements ActionListener,
		ItemListener {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JLabel jLabel = null;
	private JComboBox serviceVbox = null;
	private JLabel jLabel1 = null;
	private JComboBox processCBox = null;
	private JLabel jLabel2 = null;
	private JLabel jLabel3 = null;
	private AtomProcess atomProcess;
	private JLabel jLabel4 = null;
	private JTextField nameTextField1 = null;
	private JComboBox originCombox = null;
	private JComboBox bindcombox = null;
	private JButton bindbtn = null;
	private JScrollPane jScrollPane = null;
	private JTable jTable = null;
	private JButton confirmBtn = null;
	private JButton concelBtn = null;
	private JButton deleteBtn = null;
	private BindStore bindStore;
	private ResourceBundle rBundle = null;
	private String msg; 
	private Logger logger = Logger.getLogger(AddBindDialog.class);  //  @jve:decl-index=0:
	/**
	 * Instantiates a AddBindDialog
	 * 
	 * @param owner
	 * @param atomProcess
	 * @param bindStore
	 */
	public AddBindDialog(JDialog owner, AtomProcess atomProcess,
			BindStore bindStore) {
		super(owner);
		this.rBundle = ResourceBundleFactory.getResourceBundle();
		this.atomProcess = atomProcess;
		this.bindStore = bindStore;
		initialize();
	}

	/**
	 * Initializes the AddBindDialog.
	 */
	private void initialize() {
		this.setFont(new Font("Dialog", Font.PLAIN, 14));
		this.setBounds(new Rectangle(0, 0, 539, 493));
		this.setContentPane(getJContentPane());
		this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		this.setResizable(false);
		initComp();
	//	msg = rBundle.getString(LocaleKey.Dialog_Bind_Add_Title);
		this.setTitle("Add Binding to "+atomProcess.getName());
		this.setLocationRelativeTo(null);
		this.setModal(true);
	}

	/**
	 * Initializes the components.
	 */
	private void initComp() {
		TreeScrollPanel treePanel = GlobeParam.getClient().getTreeScrollPanel();
		List<ServiceTreeNode> sosTreeNodes = treePanel.getSOSServiceNode();
		List<ServiceTreeNode> wpsTreeNodes = treePanel.getWPSServiceNode();
		
		for(ServiceTreeNode service:wpsTreeNodes){
			this.serviceVbox.addItem(service);
		}
		for(ServiceTreeNode service:sosTreeNodes){
			this.serviceVbox.addItem(service);
		}
		if (this.serviceVbox.getItemCount()>0) {
			this.serviceVbox.setSelectedIndex(0);
		}
		for (ComplexParam complexParam : atomProcess.getComplexParam()) {
			this.originCombox.addItem(complexParam.getName());
		}
		for (LiteralParam literalParam : atomProcess.getLiteralParam()) {
			this.originCombox.addItem(literalParam.getName());
		}
	}

	private JComboBox getServiceVbox() {
		if (serviceVbox == null) {
			serviceVbox = new JComboBox();
			serviceVbox.setBounds(new Rectangle(123, 55, 352, 30));
			serviceVbox.setFont(new Font("Dialog", Font.BOLD, 14));
			serviceVbox.setName("service");
			serviceVbox.addItemListener(this);
		}
		return serviceVbox;
	}

	int num = 0;

	private JComboBox getProcessCBox() {
		if (processCBox == null) {
			processCBox = new JComboBox();
			processCBox.setBounds(new Rectangle(123, 95, 353, 30));
			processCBox.setFont(new Font("Dialog", Font.BOLD, 14));
			processCBox.setName("process");
			processCBox.addItemListener(this);
			processCBox.setEditable(false);
		}
		/*processCBox.getEditor().getEditorComponent()
				.addKeyListener(new KeyAdapter() {

					@Override
					public void keyReleased(KeyEvent evt) {
						if (evt.getKeyCode() == KeyEvent.VK_ENTER) {
							String strItem = processCBox.getEditor().getItem()
									.toString();
							processCBox.setPopupVisible(true);
							match(strItem);
						}

					}
				});*/
		return processCBox;
	}

	/**
	 * Matches process.
	 * 
	 * @param mathStr
	 */
	@SuppressWarnings("unused")
	private void match(String mathStr) {
		String source, target;
		Object object;
		processCBox.removeAllItems();
		/*for (String str : currentOperations) {
			processCBox.addItem(str);
		}*/
		for (int i = 0; i < processCBox.getItemCount(); i++) {
			object = processCBox.getItemAt(i);
			source = object.toString().toLowerCase().trim();
			target = mathStr.trim().toLowerCase();
			if (KMPMatch.searchKMP(source.toCharArray(), target.toCharArray()) == -1) {
				processCBox.removeItem(object);
			}
		}
		processCBox.updateUI();
		((JTextField) processCBox.getEditor().getEditorComponent())
				.setText(mathStr);
	}

	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jLabel4 = new JLabel();
			jLabel4.setBounds(new Rectangle(16, 15, 90, 30));
			jLabel4.setHorizontalAlignment(SwingConstants.RIGHT);
			jLabel4.setFont(new Font("Dialog", Font.BOLD, 14));
			msg = rBundle.getString(LocaleKey.Dialog_Bind_Add_Label_Name);
			jLabel4.setText(msg);
			jLabel3 = new JLabel();
			jLabel3.setBounds(new Rectangle(11, 175, 149, 30));
			jLabel3.setFont(new Font("Dialog", Font.BOLD, 14));
			msg = rBundle.getString(LocaleKey.Dialog_Bind_Add_Label_Bind_Param);
			jLabel3.setText(msg);
			jLabel2 = new JLabel();
			jLabel2.setBounds(new Rectangle(11, 135, 149, 30));
			jLabel2.setFont(new Font("Dialog", Font.BOLD, 14));
			msg = rBundle
					.getString(LocaleKey.Dialog_Bind_Add_Label_Model_Param);
			jLabel2.setText(msg);
			jLabel1 = new JLabel();
			jLabel1.setBounds(new Rectangle(16, 95, 90, 30));
			jLabel1.setHorizontalAlignment(SwingConstants.RIGHT);
			jLabel1.setFont(new Font("Dialog", Font.BOLD, 14));
			msg = rBundle.getString(LocaleKey.Dialog_Bind_Add_Label_Process);
			jLabel1.setText(msg);
			jLabel = new JLabel();
			jLabel.setBounds(new Rectangle(16, 55, 90, 30));
			jLabel.setHorizontalAlignment(SwingConstants.RIGHT);
			jLabel.setFont(new Font("Dialog", Font.BOLD, 14));
			msg = rBundle.getString(LocaleKey.Dialog_Bind_Add_Label_ProcessSet);
			jLabel.setText(msg);
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.setFont(new Font("Dialog", Font.PLAIN, 14));
			jContentPane.add(jLabel, null);
			jContentPane.add(getServiceVbox(), null);
			jContentPane.add(jLabel1, null);
			jContentPane.add(getProcessCBox(), null);
			jContentPane.add(jLabel2, null);
			jContentPane.add(jLabel3, null);
			jContentPane.add(jLabel4, null);
			jContentPane.add(getNameTextField1(), null);
			jContentPane.add(getOriginCombox(), null);
			jContentPane.add(getBindcombox(), null);
			jContentPane.add(getBindbtn(), null);
			jContentPane.add(getJScrollPane(), null);
			jContentPane.add(getConfirmBtn(), null);
			jContentPane.add(getConcelBtn(), null);
			jContentPane.add(getDeleteBtn(), null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes nameTextField1.
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getNameTextField1() {
		if (nameTextField1 == null) {
			nameTextField1 = new JTextField();
			nameTextField1.setBounds(new Rectangle(124, 15, 350, 30));
		}
		return nameTextField1;
	}

	/**
	 * This method initializes originCombox.
	 * 
	 * @return javax.swing.JComboBox
	 */
	private JComboBox getOriginCombox() {
		if (originCombox == null) {
			originCombox = new JComboBox();
			originCombox.setBounds(new Rectangle(169, 135, 308, 30));
			originCombox.setFont(new Font("Dialog", Font.BOLD, 14));
		}
		return originCombox;
	}

	/**
	 * This method initializes bindcombox.
	 * 
	 * @return javax.swing.JComboBox
	 */
	private JComboBox getBindcombox() {
		if (bindcombox == null) {
			bindcombox = new JComboBox();
			bindcombox.setBounds(new Rectangle(170, 175, 306, 30));
			bindcombox.setFont(new Font("Dialog", Font.BOLD, 14));
		}
		return bindcombox;
	}

	/**
	 * This method initializes bindbtn.
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getBindbtn() {
		if (bindbtn == null) {
			bindbtn = new JButton();
			bindbtn.setBounds(new Rectangle(430, 235, 84, 30));
			msg = rBundle.getString(LocaleKey.Button_Bind);
			bindbtn.setText(msg);
			bindbtn.setFont(new Font("Dialog", Font.BOLD, 14));
			bindbtn.addActionListener(this);
			bindbtn.setActionCommand("bind");
		}
		return bindbtn;
	}

	/**
	 * This method initializes jScrollPane.
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setBounds(new Rectangle(13, 215, 396, 187));
			jScrollPane.setViewportView(getJTable());
		}
		return jScrollPane;
	}

	/**
	 * This method initializes jTable.
	 * 
	 * @return javax.swing.JTable
	 */
	private JTable getJTable() {
		if (jTable == null) {
			String modelParam = rBundle
					.getString(LocaleKey.Dialog_Bind_Add_Column_Model);
			String bindParam = rBundle
					.getString(LocaleKey.Dialog_Bind_Add_Column_Bind);
			String[] columnTitles = { modelParam, bindParam };
			// String[][] value = {{"","",""}};
			DefaultTableModel model = new DefaultTableModel(null, columnTitles);
			jTable = new JTable(model);
			jTable.setFont(new Font("Dialog", Font.PLAIN, 14));
		}
		return jTable;
	}

	/**
	 * This method initializes confirmBtn.
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getConfirmBtn() {
		if (confirmBtn == null) {
			confirmBtn = new JButton();
			confirmBtn.setBounds(new Rectangle(106, 413, 101, 30));
			confirmBtn.setFont(new Font("Dialog", Font.BOLD, 14));
			msg = rBundle.getString(LocaleKey.Button_OK);
			confirmBtn.setText(msg);
			confirmBtn.addActionListener(this);
			confirmBtn.setActionCommand("confirm");
		}
		return confirmBtn;
	}

	private JButton getConcelBtn() {
		if (concelBtn == null) {
			concelBtn = new JButton();
			concelBtn.setBounds(new Rectangle(319, 413, 101, 30));
			msg = rBundle.getString(LocaleKey.Button_Concel);
			concelBtn.setText(msg);
			concelBtn.setFont(new Font("Dialog", Font.BOLD, 14));
			concelBtn.addActionListener(this);
			concelBtn.setActionCommand("concel");
		}
		return concelBtn;
	}

	/**
	 * This method initializes deleteBtn.
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getDeleteBtn() {
		if (deleteBtn == null) {
			deleteBtn = new JButton();
			deleteBtn.setBounds(new Rectangle(430, 320, 84, 30));
			msg = rBundle.getString(LocaleKey.button_Delete);
			deleteBtn.setText(msg);
			deleteBtn.setFont(new Font("Dialog", Font.BOLD, 14));
			deleteBtn.addActionListener(this);
			deleteBtn.setActionCommand("delete");
		}
		return deleteBtn;
	}

	@SuppressWarnings("rawtypes")
	@Override
	public void itemStateChanged(ItemEvent e) {
		JComboBox comboBox = (JComboBox) e.getSource();
		String name = comboBox.getName();
		if (name.equals("service")) {
			Object serviceObject = comboBox.getSelectedItem();
			if (serviceObject == null) {
				return;
			}
			ServiceTreeNode serviceTreeNode = (ServiceTreeNode)serviceObject;
			this.processCBox.removeAllItems();
			for (Enumeration node = serviceTreeNode.children(); node.hasMoreElements();) {
				TreeNodeImpl n = (TreeNodeImpl) node.nextElement();
				this.processCBox.addItem(n.getName());
			}
		} else if (name.equals("process")) {
			Object serviceObject = this.serviceVbox.getSelectedItem();
			if (serviceObject == null) {
				return;
			}

			
			Object processObject = this.processCBox.getSelectedItem();
			if (processObject == null) {
				return;
			}
			this.bindcombox.removeAllItems();
			ServiceTreeNode serviceNode = (ServiceTreeNode) serviceObject;
			String process = processObject.toString();
			IAdapter adapter = AdapterFactory.getInstance().getAdapter(serviceNode);
			if (adapter == null) {
				logger.error("Failed to get adapter for "+serviceNode.getUrl());
				return;
			}
			IOperationInfo info = adapter.getOperationInfo(process);
			if (info == null) {
				logger.error("Failed to get information for "+process+"@"+serviceNode.getUrl());
				return;
			}
			for (IParameterInfo iParameterInfo : info.getInputParameterInfos()) {
				this.bindcombox.addItem(iParameterInfo.getName());
			}
		}
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		String str = e.getActionCommand();
		if (str.equals("bind")) {
			String orign = this.originCombox.getSelectedItem().toString();
			String bind = this.bindcombox.getSelectedItem().toString();
			String value;
			for (int i = 0; i < jTable.getRowCount(); i++) {
				value = jTable.getValueAt(i, 1).toString();
				if (value.equals(bind)) {
					msg = rBundle
							.getString(LocaleKey.Dialog_Bind_Add_Msg_Repeat);
					JOptionPane.showMessageDialog(null, value + " " + msg);
					return;
				}
			}
			for (int i = 0; i < jTable.getRowCount(); i++) {
				value = jTable.getValueAt(i, 0).toString();
				if (value.equals(orign)) {
					msg = rBundle.getString(LocaleKey.Dialog_Bind_Add_Msg_Redo);
					int flag = JOptionPane.showConfirmDialog(null, value + " "
							+ msg);
					if (flag == JOptionPane.YES_OPTION) {
						jTable.setValueAt(bind, i, 1);
					}
					return;
				}
			}
			// int n = jTable.getRowCount();
			DefaultTableModel model = (DefaultTableModel) jTable.getModel();
			model.addRow(new String[] { orign, bind });
		} else if (str.equals("confirm")) {
			int num = atomProcess.getComplexParam().size()
					+ atomProcess.getLiteralParam().size();
			if (num != jTable.getRowCount()) {
				msg = rBundle.getString(LocaleKey.Dialog_Bind_Add_Msg_All);
				JOptionPane.showMessageDialog(null, msg);
				return;
			}
			String name = this.nameTextField1.getText();
			if (name.trim().equals("")) {
				msg = rBundle.getString(LocaleKey.Dialog_Bind_Add_Msg_Name);
				JOptionPane.showMessageDialog(null, msg);
				return;
			}

			ServiceTreeNode serviceModel = (ServiceTreeNode) this.serviceVbox
					.getSelectedItem();
			BindProcess bindProcess = bindStore.getBindProcess(atomProcess
					.getUUID());
			BindAtom bindAtom = new BindAtom();
			bindAtom.setId(HandleTool.getUUID());
			bindAtom.setUrl(serviceModel.getUrl());
			bindAtom.setServiceType(serviceModel.getServiceType());
			bindAtom.setVersion(serviceModel.getVersion());
			bindAtom.setName(name);

			String process = this.processCBox.getSelectedItem().toString();
			bindProcess.addMapInfo(bindAtom);
			bindAtom.setOperationName(process);
			// serviceMap.put("process", process);
			String key, value;
			for (int i = 0; i < jTable.getRowCount(); i++) {
				key = jTable.getValueAt(i, 0).toString();
				value = jTable.getValueAt(i, 1).toString();
				bindAtom.addParamMapInfo(bindAtom.new ParamMapInfo(key, value));
				// paramMap.put(key, value);
			}
			this.dispose();
		} else if (str.equals("concel")) {
			// serviceMap.clear();
			// paramMap.clear();
			this.dispose();
		} else if (str.equals("delete")) {
			int num = jTable.getSelectedRow();
			if (num == -1) {
				return;
			}
			DefaultTableModel model = (DefaultTableModel) jTable.getModel();
			model.removeRow(num);
		}
	}

}
