/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.run.bind;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Rectangle;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.WindowConstants;

import cn.edu.whu.lmars.run.bind.beans.BindAtom;

/**
 * Views the binding information.
 * 
 * @author Mingda Zhang
 * 
 */
public class BindingInfoDialog extends JDialog {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JLabel jLabel = null;
	private JLabel jLabel1 = null;
	private JLabel jLabel11 = null;
	private JLabel jLabel111 = null;
	private JTextField operationNameT = null;
	private JTextField serviceTypeT = null;
	private JTextField serviceAddressT = null;
	private JTextField serviceVersionT = null;
	private BindAtom bindAtom = null;
	/**
	 * @param owner
	 */
	public BindingInfoDialog(JDialog owner,BindAtom bindAtom) {
		super(owner);
		this.bindAtom = bindAtom;
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(515, 205);
		this.setFont(new Font("Dialog", Font.PLAIN, 14));
		this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		this.setTitle("Binding Info");
		this.setContentPane(getJContentPane());
		this.setLocationRelativeTo(null);
		this.setModal(true);
		initComp();
	}
	
	private void initComp(){
		this.operationNameT.setText(bindAtom.getOperationName());
		this.serviceTypeT.setText(bindAtom.getServiceType().toString());
		this.serviceVersionT.setText(bindAtom.getVersion());
		this.serviceAddressT.setText(bindAtom.getUrl());
	}
	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jLabel111 = new JLabel();
			jLabel111.setBounds(new Rectangle(5, 115, 130, 30));
			jLabel111.setPreferredSize(new Dimension(130, 30));
			jLabel111.setHorizontalAlignment(SwingConstants.RIGHT);
			jLabel111.setText("Service Address:");
			jLabel111.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel11 = new JLabel();
			jLabel11.setBounds(new Rectangle(5, 80, 130, 30));
			jLabel11.setPreferredSize(new Dimension(130, 30));
			jLabel11.setHorizontalAlignment(SwingConstants.RIGHT);
			jLabel11.setText("Service Version:");
			jLabel11.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel1 = new JLabel();
			jLabel1.setBounds(new Rectangle(5, 45, 130, 30));
			jLabel1.setPreferredSize(new Dimension(130, 30));
			jLabel1.setHorizontalAlignment(SwingConstants.RIGHT);
			jLabel1.setText("Service Type:");
			jLabel1.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel = new JLabel();
			jLabel.setBounds(new Rectangle(5, 10, 130, 30));
			jLabel.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel.setPreferredSize(new Dimension(130, 30));
			jLabel.setHorizontalAlignment(SwingConstants.RIGHT);
			jLabel.setText("Operation Name:");
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.setFont(new Font("Dialog", Font.PLAIN, 14));
			jContentPane.add(jLabel, null);
			jContentPane.add(jLabel1, null);
			jContentPane.add(jLabel11, null);
			jContentPane.add(jLabel111, null);
			jContentPane.add(getOperationNameT(), null);
			jContentPane.add(getServiceTypeT(), null);
			jContentPane.add(getServiceAddressT(), null);
			jContentPane.add(getServiceVersionT(), null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes operationNameT	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getOperationNameT() {
		if (operationNameT == null) {
			operationNameT = new JTextField();
			operationNameT.setBounds(new Rectangle(142, 10, 340, 30));
			operationNameT.setFont(new Font("Dialog", Font.PLAIN, 14));
			operationNameT.setPreferredSize(new Dimension(340, 30));
		}
		return operationNameT;
	}

	/**
	 * This method initializes serviceTypeT	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getServiceTypeT() {
		if (serviceTypeT == null) {
			serviceTypeT = new JTextField();
			serviceTypeT.setBounds(new Rectangle(142, 45, 340, 30));
			serviceTypeT.setPreferredSize(new Dimension(340, 30));
			serviceTypeT.setFont(new Font("Dialog", Font.PLAIN, 14));
		}
		return serviceTypeT;
	}

	/**
	 * This method initializes serviceAddressT	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getServiceAddressT() {
		if (serviceAddressT == null) {
			serviceAddressT = new JTextField();
			serviceAddressT.setBounds(new Rectangle(142, 115, 340, 30));
			serviceAddressT.setPreferredSize(new Dimension(340, 30));
			serviceAddressT.setFont(new Font("Dialog", Font.PLAIN, 14));
		}
		return serviceAddressT;
	}

	/**
	 * This method initializes serviceVersionT	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getServiceVersionT() {
		if (serviceVersionT == null) {
			serviceVersionT = new JTextField();
			serviceVersionT.setBounds(new Rectangle(142, 80, 340, 30));
			serviceVersionT.setPreferredSize(new Dimension(340, 30));
			serviceVersionT.setFont(new Font("Dialog", Font.PLAIN, 14));
		}
		return serviceVersionT;
	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
