/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.run.bind;

import java.awt.Font;
import java.awt.Frame;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.List;
import java.util.ResourceBundle;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.apache.log4j.Logger;

import cn.edu.whu.lmars.locale.LocaleKey;
import cn.edu.whu.lmars.locale.ResourceBundleFactory;
import cn.edu.whu.lmars.model.AtomProcess;
import cn.edu.whu.lmars.run.bind.beans.BindAtom;
import cn.edu.whu.lmars.run.bind.beans.BindProcess;
import cn.edu.whu.lmars.run.bind.beans.BindStore;

/**
 * Chooses a binding to the selected process model.
 * 
 * @author Mingda Zhang
 * 
 */
public class ChoiceBindDialog extends JDialog implements ActionListener {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JLabel jLabel = null;
	private JComboBox jComboBox = null;
	private JButton confirmBtn = null;
	private JButton addBtn = null;
	private JButton concleBtn = null;
	private AtomProcess atomProcess = null;
	private BindProcess bindProcess = null;  //  @jve:decl-index=0:
	private BindStore bindStore = null;
	private Logger logger = Logger.getLogger(ChoiceBindDialog.class); // @jve:decl-index=0:
	private ResourceBundle rBundle = null;
	private String msg;  //  @jve:decl-index=0:
	private JButton detailBtn = null;

	/**
	 * Instantiates a ChoiceBindDialog.
	 * 
	 * @param owner
	 * @param atomProcess
	 * @param bindStore
	 */
	public ChoiceBindDialog(Frame owner, AtomProcess atomProcess,
			BindStore bindStore) {
		super(owner);
		rBundle = ResourceBundleFactory.getResourceBundle();
		this.atomProcess = atomProcess;
		this.bindStore = bindStore;
		initialize();
		initComp();
	}

	/**
	 * This method initializes this dialog.
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(519, 143);
		this.setResizable(false);
		msg = rBundle.getString(LocaleKey.Dialog_Bind_Choice_Title);
		this.setTitle(msg);
		this.setFont(new Font("Dialog", Font.PLAIN, 14));
		this.setContentPane(getJContentPane());

		this.setLocationRelativeTo(null);
		this.setModal(true);
	}

	private void initComp() {
		this.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
			}
		});
		String id = atomProcess.getUUID();
		bindProcess = bindStore.getBindProcess(id);
		if (bindProcess == null) {
			bindProcess = new BindProcess();
			bindProcess.setModelID(atomProcess.getUUID());
			bindStore.addBindProcess(bindProcess);
		}
		List<BindAtom> bindAtoms = bindProcess.getMapInfos();
		this.jComboBox.removeAllItems();
		for (BindAtom atom : bindAtoms) {
			this.jComboBox.addItem(atom);
		}
		if (bindProcess.getCurBindAtom()!=null) {
			this.jComboBox.setSelectedItem(bindProcess.getCurBindAtom());
		}

	}

	/**
	 * This method initializes jComboBox.
	 * 
	 * @return javax.swing.JComboBox
	 */
	private JComboBox getJComboBox() {
		if (jComboBox == null) {
			jComboBox = new JComboBox();
			jComboBox.setBounds(new Rectangle(113, 14, 362, 30));
			jComboBox.setFont(new Font("Dialog", Font.BOLD, 14));
		}
		return jComboBox;
	}

	/**
	 * This method initializes confirmBtn.
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getConfirmBtn() {
		if (confirmBtn == null) {
			confirmBtn = new JButton();
			confirmBtn.setBounds(new Rectangle(41, 66, 90, 30));
			msg = rBundle.getString(LocaleKey.Button_OK);
			confirmBtn.setText(msg);
			confirmBtn.setFont(new Font("Dialog", Font.BOLD, 14));
			confirmBtn.addActionListener(this);
			confirmBtn.setActionCommand("confirm");
		}
		return confirmBtn;
	}

	/**
	 * This method initializes addBtn
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getAddBtn() {
		if (addBtn == null) {
			addBtn = new JButton();
			addBtn.setBounds(new Rectangle(156, 66, 90, 30));
			msg = rBundle.getString(LocaleKey.button_Add);
			addBtn.setText(msg);
			addBtn.setFont(new Font("Dialog", Font.BOLD, 14));
			addBtn.addActionListener(this);
			addBtn.setActionCommand("add");
		}
		return addBtn;
	}

	/**
	 * This method initializes concleBtn
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getConcleBtn() {
		if (concleBtn == null) {
			concleBtn = new JButton();
			concleBtn.setBounds(new Rectangle(403, 66, 90, 30));
			msg = rBundle.getString(LocaleKey.Button_Concel);
			concleBtn.setText(msg);
			concleBtn.setFont(new Font("Dialog", Font.BOLD, 14));
			concleBtn.setActionCommand("concel");
			concleBtn.addActionListener(this);
		}
		return concleBtn;
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jLabel = new JLabel();
			jLabel.setBounds(new Rectangle(10, 14, 87, 30));
			jLabel.setFont(new Font("Dialog", Font.BOLD, 14));
			msg = rBundle.getString(LocaleKey.Dialog_Bind_Choice_Label_Bind);
			jLabel.setText(msg);
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(jLabel, null);
			jContentPane.add(getJComboBox(), null);
			jContentPane.add(getConfirmBtn(), null);
			jContentPane.add(getAddBtn(), null);
			jContentPane.add(getConcleBtn(), null);
			jContentPane.add(getDetailBtn(), null);
		}
		return jContentPane;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		// TODO Auto-generated method stub
		String str = e.getActionCommand();
		if (str.equals("confirm")) {
			Object object = this.jComboBox.getSelectedItem();
			if (object != null) {
				BindAtom bindAtom = (BindAtom)object;
				String newID = bindAtom.getId();
				String oldID = bindProcess.getCurMapID();
				if (oldID == null || !oldID.equals(newID)) {
					// map.put(atomProcess.getUUID(),model.getId());
					bindProcess.setCurMapID(newID);
					logger.info("Add new binding:" + atomProcess.getUUID()
							+ "---->" + newID);
					msg = rBundle
							.getString(LocaleKey.Dialog_Bind_Choice_Msg_Success);
					JOptionPane.showMessageDialog(null, msg);
				}

			}
			this.dispose();

		} else if (str.equals("concel")) {

			this.dispose();
		} else if (str.equals("add")) {

			AddBindDialog dialog = new AddBindDialog(this, atomProcess,
					bindStore);
			dialog.setVisible(true);
			initComp();
		}else if (str.equals("detail")) {
			Object object = this.jComboBox.getSelectedItem();
			if (object != null) {
				BindAtom bindAtom = (BindAtom)object;
				BindingInfoDialog dialog = new BindingInfoDialog(this,bindAtom);
				dialog.setVisible(true);
			}
		}
	}

	/**
	 * This method initializes detailBtn	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getDetailBtn() {
		if (detailBtn == null) {
			detailBtn = new JButton();
			detailBtn.setBounds(new Rectangle(276, 66, 90, 30));
			detailBtn.setText("Detail");
			detailBtn.setFont(new Font("Dialog", Font.BOLD, 14));
			detailBtn.setActionCommand("detail");
			detailBtn.addActionListener(this);
		}
		return detailBtn;
	}

}