/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/

package cn.edu.whu.lmars.run.bind.beans;

import java.util.ArrayList;
import java.util.List;

import cn.edu.whu.lmars.service.beans.IService;
import cn.edu.whu.lmars.service.beans.ServiceType;

/**
 * Binding information.
 * 
 * @author Mingda Zhang
 * 
 */
public class BindAtom implements IService {

	/** Binding Name */
	private String name;
	/** Binding Id */
	private String id;
	/** Binding Service Url */
	private String serviceUrl;
	/** Binding Service version */
	private String version;
	/** Binding operation name */
	private String operationName;
	/** Binding service type */
	private ServiceType serviceType;
	/** Binding parameter map */
	private List<ParamMapInfo> paramMapInfos;

	/**
	 * Gets binding name.
	 * 
	 * @return
	 */
	@Override
	public String getName() {
		return name;
	}

	/**
	 * Sets binding name.
	 * 
	 * @param name
	 *            binding name.
	 */
	@Override
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Gets binding id.
	 * 
	 * @return
	 */
	@Override
	public String getId() {
		return id;
	}

	/**
	 * Sets binding id.
	 * 
	 * @param id
	 */
	@Override
	public void setId(String id) {
		this.id = id;
	}

	/**
	 * Gets binding service address.
	 * 
	 * @return
	 */
	@Override
	public String getUrl() {
		return serviceUrl;
	}

	/**
	 * Sets binding service address.
	 * 
	 * @param serviceUrl
	 */
	@Override
	public void setUrl(String url) {
		this.serviceUrl = url;
	}

	/**
	 * Gets binding service version.
	 * 
	 * @return
	 */
	@Override
	public String getVersion() {
		return version;
	}

	/**
	 * Sets binding service version.
	 * 
	 * @param version
	 */
	@Override
	public void setVersion(String version) {
		this.version = version;
	}

	/**
	 * Gets binding operation name.
	 * 
	 * @return
	 */
	public String getOperationName() {
		return operationName;
	}

	/**
	 * Sets binding operation name.
	 * 
	 * @param operationName
	 */
	public void setOperationName(String operationName) {
		this.operationName = operationName;
	}

	/**
	 * Gets service type.
	 * 
	 * @return
	 */
	@Override
	public ServiceType getServiceType() {
		return serviceType;
	}

	/**
	 * Sets service type.
	 * 
	 * @param serviceType
	 */
	public void setServiceType(ServiceType serviceType) {
		this.serviceType = serviceType;
	}

	/**
	 * Gets parameter map info.
	 * 
	 * @return
	 */
	public List<ParamMapInfo> getParamMapInfos() {
		return paramMapInfos;
	}

	/**
	 * Sets parameter map info.
	 * 
	 * @param paramMapInfos
	 */
	public void setParamMapInfos(List<ParamMapInfo> paramMapInfos) {
		this.paramMapInfos = paramMapInfos;
	}

	/**
	 * Adds parameter map.
	 * 
	 * @param mapInfo
	 */
	public void addParamMapInfo(ParamMapInfo mapInfo) {
		if (paramMapInfos == null) {
			paramMapInfos = new ArrayList<BindAtom.ParamMapInfo>(4);
		}
		this.paramMapInfos.add(mapInfo);
	}

	@Override
	public String toString() {
		// TODO Auto-generated method stub
		return getName();
	}
	
	/**
	 * Parameter map information.
	 * 
	 * @author Mingda Zhang
	 * 
	 */
	public class ParamMapInfo {
		/** Model parameter */
		private String from;
		/** Binding operation parameter */
		private String to;

		public ParamMapInfo() {
		}

		public ParamMapInfo(String from, String to) {
			this.from = from;
			this.to = to;
		}

		/**
		 * Gets model's parameter.
		 * 
		 * @return
		 */
		public String getFrom() {
			return from;
		}

		/**
		 * Sets model's parameter.
		 * 
		 * @param from
		 */
		public void setFrom(String from) {
			this.from = from;
		}

		/**
		 * Gets binding operation's parameter.
		 * 
		 * @return
		 */
		public String getTo() {
			return to;
		}

		/**
		 * Sets binding operation's parameter.
		 * 
		 * @param to
		 */
		public void setTo(String to) {
			this.to = to;
		}
	}

}
