/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.run.bind.beans;

import java.util.ArrayList;
import java.util.List;

import cn.edu.whu.lmars.utils.BaseTool;

/**
 * Binding process information. Binds a process model to this class. Every
 * process model may have more than one binding.
 * 
 * @author Mingda Zhang
 * 
 */
public class BindProcess {

	/** The id of model */
	private String modelID;
	/** All BindAtoms of the model */
	private List<BindAtom> mapInfos = new ArrayList<BindAtom>(3);
	/** The id of current BindAtom */
	private String curMapID;

	/**
	 * Returns the id of model.
	 */
	public String getModelID() {
		return modelID;
	}

	/**
	 * Sets the id of model.
	 * 
	 * @param modelID
	 */
	public void setModelID(String modelID) {
		this.modelID = modelID;
	}

	/**
	 * Gets all bindings.
	 */
	public List<BindAtom> getMapInfos() {
		return mapInfos;
	}

	/**
	 * Gets the id of current BindAtom.
	 * 
	 */
	public String getCurMapID() {
		return curMapID;
	}

	/**
	 * Sets the if of current BindAtom.
	 * 
	 * @param curMapID
	 */
	public void setCurMapID(String curMapID) {
		this.curMapID = curMapID;
	}

	/**
	 * Sets bindings.
	 * 
	 * @param mapInfos
	 */
	public void setMapInfos(List<BindAtom> mapInfos) {
		this.mapInfos = mapInfos;
	}

	/**
	 * Adds a new binding to the process model.
	 * 
	 * @param mapInfo
	 */
	public void addMapInfo(BindAtom mapInfo) {
		/*
		 * if(mapInfos == null){ mapInfos = new ArrayList<BindAtom>(3); }
		 */
		if (!mapInfos.contains(mapInfo)) {
			mapInfos.add(mapInfo);
		}
	}

	/**
	 * Gets current binding.
	 */
	public BindAtom getCurBindAtom() {
		String id = getCurMapID();
		if (mapInfos == null || mapInfos.size() == 0) {
			return null;
		}

		if (BaseTool.isNull(id)) {
			return mapInfos.get(0);
		}
		return getMapInfo(id);
	}

	/**
	 * Gets binding by id.
	 * 
	 * @param mapID
	 * @return
	 */
	public BindAtom getMapInfo(String mapID) {
		for (BindAtom mapInfo : mapInfos) {
			if (mapInfo.getId().equals(mapID)) {
				return mapInfo;
			}
		}
		return null;
	}

	/**
	 * Gets all bindings' id.
	 * 
	 * @return
	 */
	public List<String> getMapInfosID() {
		if (mapInfos == null) {
			return null;
		}
		int num = mapInfos.size();
		if (num == 0) {
			return null;
		}
		List<String> mapIdList = new ArrayList<String>(num);
		for (BindAtom mapInfo : this.mapInfos) {
			mapIdList.add(mapInfo.getId());
		}
		return mapIdList;
	}
}
