/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.run.bind.beans;

import java.util.ArrayList;
import java.util.List;

/**
 * Binding information container.
 * 
 * @author Mingda Zhang
 * 
 */
public class BindStore {
	/** All process bindings. */
	private List<BindProcess> bindProcessList = null;

	/**
	 * Gets process binding.
	 */
	public List<BindProcess> getBindProcessList() {
		return bindProcessList;
	}

	/**
	 * Sets process bindings.
	 * 
	 * @param processMapInfos
	 *            process binding list.
	 */
	public void setBindProcesses(List<BindProcess> processMapInfos) {
		this.bindProcessList = processMapInfos;
	}

	/**
	 * Adds process binding to the container.
	 * 
	 * @param bindProcess
	 *            process binding.
	 */
	public void addBindProcess(BindProcess bindProcess) {
		if (this.bindProcessList == null) {
			this.bindProcessList = new ArrayList<BindProcess>();
		}
		this.bindProcessList.add(bindProcess);
	}

	/**
	 * Get all ids of process bindings.
	 */
	public List<String> getBindProcessIDs() {
		if (this.bindProcessList == null) {
			return null;
		}
		int num = this.bindProcessList.size();
		if (num == 0) {
			return null;
		}
		List<String> processMapIDs = new ArrayList<String>(num);
		for (BindProcess mapInfo : this.bindProcessList) {
			processMapIDs.add(mapInfo.getModelID());
		}
		return processMapIDs;
	}

	/**
	 * Gets process binding by id.
	 * 
	 * @param id
	 *            the id of process binding.
	 */
	public BindProcess getBindProcess(String id) {
		if (this.bindProcessList == null) {
			return null;
		}
		for (BindProcess bindProcess : this.bindProcessList) {
			if (bindProcess.getModelID().equals(id)) {
				return bindProcess;
			}
		}
		return null;
	}

	/**
	 * Gets the size of the container.
	 * 
	 * @return
	 */
	public int getSize() {
		if (this.bindProcessList == null) {
			return 0;
		}
		return this.bindProcessList.size();
	}
}
