/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.run.ui;

import java.awt.Font;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import conf.GlobeParam;

import cn.edu.whu.lmars.drawpanel.ui.JDrawPanel;
import cn.edu.whu.lmars.locale.LocaleKey;
import cn.edu.whu.lmars.locale.ResourceBundleFactory;
import cn.edu.whu.lmars.run.beans.RunModelTree;
import cn.edu.whu.lmars.utils.HandleTool;

/**
 * This dialog monitors the execution process.
 * 
 * @author Mingda Zhang
 * 
 */
public class ExecuteMonitorDialog extends JDialog implements ActionListener {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JLabel jLabel = null;
	private JButton jButton = null;
	private JScrollPane jScrollPane = null;
	private JTextArea jTextArea = null;
	private JButton jButton1 = null;
	private JButton jButton2 = null;
	private static ExecuteMonitorDialog instance;
	private static ResourceBundle rBundle = null;

	private ExecuteMonitorDialog(JFrame owner) {
		super(owner);
		initialize();
	}

	/**
	 * Gets an instance of ExecuteMonitorDialog. Singleton Pattern.
	 * 
	 * @param owner
	 * @return
	 */
	public static ExecuteMonitorDialog getInstance(JFrame owner) {
		if (instance == null) {
			rBundle = ResourceBundleFactory.getResourceBundle();
			instance = new ExecuteMonitorDialog(owner);
		}
		return instance;
	}

	/**
	 * Initializes the dialog.
	 */
	private void initialize() {
		this.setName("");
		this.setTitle(rBundle.getString(LocaleKey.Dialog_Execute_Title));
		this.setSize(501, 250);
		this.setContentPane(getJContentPane());
		this.setModal(true);
		this.setResizable(false);
		this.setLocationRelativeTo(null);
		this.setVisible(false);
	}

	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jLabel = new JLabel();
			jLabel.setBounds(new Rectangle(16, 7, 459, 33));
			jLabel.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel.setText(rBundle.getString(LocaleKey.Dialog_Execute_Label));
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.setName("");
			jContentPane.add(jLabel, null);
			jContentPane.add(getJButton(), null);
			jContentPane.add(getJScrollPane(), null);
			jContentPane.setVisible(true);
			jContentPane.add(getJButton1(), null);
			jContentPane.add(getJButton2(), null);
		}
		return jContentPane;
	}

	private JButton getJButton() {
		if (jButton == null) {
			jButton = new JButton();
			jButton.setBounds(new Rectangle(166, 177, 100, 30));
			jButton.setText(rBundle.getString(LocaleKey.Button_OK));
			jButton.setFont(new Font("Dialog", Font.BOLD, 14));
			jButton.addActionListener(this);
			jButton.setActionCommand("confirm");
		}
		return jButton;
	}

	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setBounds(new Rectangle(15, 47, 458, 121));
			jScrollPane
					.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
			jScrollPane
					.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
			jScrollPane.setViewportView(getJTextArea());
			jScrollPane.setFont(new Font("Dialog", Font.PLAIN, 14));
		}
		return jScrollPane;
	}

	public JTextArea getJTextArea() {
		if (jTextArea == null) {
			jTextArea = new JTextArea();
			jTextArea.setLineWrap(true);
			jTextArea.setFont(new Font("Dialog", Font.PLAIN, 14));
		}
		return jTextArea;
	}

	private JButton getJButton1() {
		if (jButton1 == null) {
			jButton1 = new JButton();
			jButton1.setBounds(new Rectangle(297, 177, 163, 30));
			jButton1.setText(rBundle
					.getString(LocaleKey.Dialog_Execute_Button_Copy));
			jButton1.setFont(new Font("Dialog", Font.BOLD, 14));
			jButton1.setActionCommand("copy");
			jButton1.addActionListener(this);
		}
		return jButton1;
	}

	private JButton getJButton2() {
		if (jButton2 == null) {
			jButton2 = new JButton();
			jButton2.setBounds(new Rectangle(39, 177, 100, 30));
			jButton2.setActionCommand("do");
			jButton2.setText(rBundle
					.getString(LocaleKey.Dialog_Execute_Button_Exe));
			jButton2.setFont(new Font("Dialog", Font.BOLD, 14));
			jButton2.addActionListener(this);
		}
		return jButton2;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		if (command.equals("confirm")) {
			jTextArea.setText("");
			this.dispose();
		} else if (command.equals("copy")) {
			HandleTool.getInstance().setSysClipboardText(jTextArea.getText());
			JOptionPane.showMessageDialog(this,
					rBundle.getString(LocaleKey.Dialog_Execute_Dialog_CopyOk));
		} else if (command.equals("do")) {
			new Thread(new Runnable() {
				@Override
				public void run() {
					execute();
				}
			}).start();
		}

	}

	/**
	 * Starts the process of execution.
	 */
	private void execute() {
		this.jTextArea.setText("");
		JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
		RunModelTree run = new RunModelTree(drawPanel.getModelStore(),
				drawPanel.getFigureStore(), drawPanel.getBindStore());
		boolean flag = run.execute();
		if (!flag) {
			this.appendMsg(rBundle.getString(LocaleKey.Dialog_Execute_Error)
					+ "\n");
			this.appendMsg(run.getErr() + "\n");
			JOptionPane.showMessageDialog(this,
					rBundle.getString(LocaleKey.Dialog_Execute_Error));
			// JOptionPane.showMessageDialog(null,"Error occured "+run.getErr()
			// );
		} else {
			JOptionPane.showMessageDialog(this,
					rBundle.getString(LocaleKey.Dialog_Execute_Success));
			this.appendMsg(rBundle.getString(LocaleKey.Dialog_Execute_Success)
					+ "\n");
		}
	}

	/**
	 * Appends execution message to the JTextArea.
	 * @param msg
	 */
	public void appendMsg(String msg) {
		jTextArea.append(msg);
		jTextArea.setCaretPosition(jTextArea.getText().length());
		// dialog.repaint();
		jTextArea.paintImmediately(jTextArea.getBounds());
		jTextArea.requestFocus();
		// dialog.validate();
		jTextArea.requestFocus();
	}
}
