/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.run.ui;

import java.awt.Font;
import java.awt.Frame;
import java.awt.Rectangle;
import java.util.ResourceBundle;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.WindowConstants;

import cn.edu.whu.lmars.figure.beans.ProcessFigure;
import cn.edu.whu.lmars.locale.LocaleKey;
import cn.edu.whu.lmars.locale.ResourceBundleFactory;
import cn.edu.whu.lmars.run.beans.ExecutionState;
import cn.edu.whu.lmars.run.beans.ServiceExecutionInfo;
import cn.edu.whu.lmars.utils.BaseTool;

/**
 * Views detail information of execution.
 * 
 * @author Mingda Zhang
 * 
 */
public class RunDetailDialog extends JDialog {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JLabel jLabel = null;
	private JTextField modelNameTextField = null;
	private JTextField serviceUrlTextField = null;
	private JLabel jLabel1 = null;
	private JLabel jLabel11 = null;
	private JLabel jLabel12 = null;
	private JLabel jLabel13 = null;
	private JTextField typeTextField = null;
	private JTextField operationTextField1 = null;
	private JTextField stateTextField1 = null;
	private JLabel jLabel2 = null;
	private JLabel jLabel3 = null;
	private JLabel jLabel4 = null;
	private JTextField beginTextField1 = null;
	private JTextField endTextField1 = null;
	private JTextField sequeTextField1 = null;
	private ProcessFigure processFigure = null;
	private String name = null;
	private ResourceBundle rBundle = null;
	private String msg;

	/**
	 * @param owner
	 */
	public RunDetailDialog(Frame owner, ProcessFigure processFigure,
			String modelName) {
		super(owner);
		this.rBundle = ResourceBundleFactory.getResourceBundle();
		this.processFigure = processFigure;
		this.name = modelName;
		initialize();
		initComp();
	}

	private void initialize() {
		this.setSize(381, 381);
		msg = rBundle.getString(LocaleKey.Dialog_Run_Detail_Title);
		this.setTitle(msg);
		this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		this.setFont(new Font("Dialog", Font.PLAIN, 14));
		this.setContentPane(getJContentPane());
		this.setLocationRelativeTo(null);
		this.setModal(true);
	}

	private void initComp() {
		this.modelNameTextField.setText(name);
		ServiceExecutionInfo sei = processFigure.getExecutionInfo();
		ExecutionState state = processFigure.getState();
		if (state == ExecutionState.READY) {
			msg = rBundle
					.getString(LocaleKey.Dialog_Run_Detail_Text_State_Ready);
			this.stateTextField1.setText(msg);
			return;
		} else if (state == ExecutionState.FAILED) {
			msg = rBundle
					.getString(LocaleKey.dialog_Run_Detail_Text_State_Fail);
			this.stateTextField1.setText(msg);
		} else if (state == ExecutionState.FINISHED) {
			msg = rBundle
					.getString(LocaleKey.dialog_Run_Detail_Text_State_Success);
			this.stateTextField1.setText(msg);
		}
		String value = sei.getServiceUrl();
		if (!BaseTool.isNull(value)) {
			this.serviceUrlTextField.setText(value);
		}

		value = sei.getType();
		if (!BaseTool.isNull(value)) {
			this.typeTextField.setText(value);
		}

		value = sei.getOperation();
		if (!BaseTool.isNull(value)) {
			this.operationTextField1.setText(value);
		}

		Long begin = sei.getExecutionBeginTime();
		if (begin != null) {
			value = BaseTool.getTimeStringByLong(begin);
			this.beginTextField1.setText(value);
		}

		Long end = sei.getExecutionEndTime();
		if (end != null) {
			value = BaseTool.getTimeStringByLong(end);
			this.endTextField1.setText(value);
		}

		int num = sei.getExecuteSequence();
		this.sequeTextField1.setText(num + "");
	}

	/**
	 * This method initializes modelNameTextField
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getModelNameTextField() {
		if (modelNameTextField == null) {
			modelNameTextField = new JTextField();
			modelNameTextField.setBounds(new Rectangle(146, 14, 200, 30));
			modelNameTextField.setFont(new Font("Dialog", Font.PLAIN, 14));
			modelNameTextField.setEditable(false);
		}
		return modelNameTextField;
	}

	/**
	 * This method initializes serviceUrlTextField
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getServiceUrlTextField() {
		if (serviceUrlTextField == null) {
			serviceUrlTextField = new JTextField();
			serviceUrlTextField.setBounds(new Rectangle(146, 55, 200, 30));
			serviceUrlTextField.setEditable(false);
			serviceUrlTextField.setFont(new Font("Dialog", Font.PLAIN, 14));
		}
		return serviceUrlTextField;
	}

	/**
	 * This method initializes typeTextField
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getTypeTextField() {
		if (typeTextField == null) {
			typeTextField = new JTextField();
			typeTextField.setBounds(new Rectangle(146, 95, 200, 30));
			typeTextField.setEditable(false);
			typeTextField.setFont(new Font("Dialog", Font.PLAIN, 14));
		}
		return typeTextField;
	}

	/**
	 * This method initializes operationTextField1
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getOperationTextField1() {
		if (operationTextField1 == null) {
			operationTextField1 = new JTextField();
			operationTextField1.setBounds(new Rectangle(146, 135, 200, 30));
			operationTextField1.setEditable(false);
			operationTextField1.setFont(new Font("Dialog", Font.PLAIN, 14));
		}
		return operationTextField1;
	}

	/**
	 * This method initializes stateTextField1
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getStateTextField1() {
		if (stateTextField1 == null) {
			stateTextField1 = new JTextField();
			stateTextField1.setBounds(new Rectangle(146, 175, 200, 30));
			stateTextField1.setEditable(false);
			stateTextField1.setFont(new Font("Dialog", Font.PLAIN, 14));
		}
		return stateTextField1;
	}

	/**
	 * This method initializes beginTextField1
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getBeginTextField1() {
		if (beginTextField1 == null) {
			beginTextField1 = new JTextField();
			beginTextField1.setBounds(new Rectangle(146, 215, 200, 30));
			beginTextField1.setEditable(false);
			beginTextField1.setFont(new Font("Dialog", Font.PLAIN, 14));
		}
		return beginTextField1;
	}

	/**
	 * This method initializes endTextField1
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getEndTextField1() {
		if (endTextField1 == null) {
			endTextField1 = new JTextField();
			endTextField1.setBounds(new Rectangle(146, 255, 200, 30));
			endTextField1.setEditable(false);
			endTextField1.setFont(new Font("Dialog", Font.PLAIN, 14));
		}
		return endTextField1;
	}

	/**
	 * This method initializes sequeTextField1
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getSequeTextField1() {
		if (sequeTextField1 == null) {
			sequeTextField1 = new JTextField();
			sequeTextField1.setBounds(new Rectangle(146, 295, 200, 30));
			sequeTextField1.setEditable(false);
			sequeTextField1.setFont(new Font("Dialog", Font.PLAIN, 14));
		}
		return sequeTextField1;
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jLabel4 = new JLabel();
			jLabel4.setBounds(new Rectangle(15, 295, 118, 30));
			jLabel4.setFont(new Font("Dialog", Font.BOLD, 14));
			msg = rBundle.getString(LocaleKey.dialog_Run_Detail_Label_Seq);
			jLabel4.setText(msg);
			jLabel3 = new JLabel();
			jLabel3.setBounds(new Rectangle(15, 255, 118, 30));
			jLabel3.setFont(new Font("Dialog", Font.BOLD, 14));
			msg = rBundle.getString(LocaleKey.dialog_Run_Detail_Label_EndTime);
			jLabel3.setText(msg);
			jLabel2 = new JLabel();
			jLabel2.setBounds(new Rectangle(15, 215, 118, 30));
			jLabel2.setFont(new Font("Dialog", Font.BOLD, 14));
			msg = rBundle
					.getString(LocaleKey.dialog_Run_Detail_Label_StartTime);
			jLabel2.setText(msg);
			jLabel13 = new JLabel();
			jLabel13.setBounds(new Rectangle(15, 175, 118, 30));
			msg = rBundle.getString(LocaleKey.dialog_Run_Detail_Label_State);
			jLabel13.setText(msg);
			jLabel13.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel12 = new JLabel();
			jLabel12.setBounds(new Rectangle(15, 135, 118, 30));
			msg = rBundle
					.getString(LocaleKey.dialog_Run_Detail_Label_Bind_Name);
			jLabel12.setText(msg);
			jLabel12.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel11 = new JLabel();
			jLabel11.setBounds(new Rectangle(15, 95, 118, 30));
			msg = rBundle
					.getString(LocaleKey.dialog_Run_Detail_Label_Bind_Type);
			jLabel11.setText(msg);
			jLabel11.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel1 = new JLabel();
			jLabel1.setBounds(new Rectangle(15, 55, 118, 30));
			msg = rBundle.getString(LocaleKey.dialog_Run_Detail_Label_Bind_Url);
			jLabel1.setText(msg);
			jLabel1.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel = new JLabel();
			jLabel.setBounds(new Rectangle(15, 15, 118, 30));
			jLabel.setFont(new Font("Dialog", Font.BOLD, 14));
			msg = rBundle
					.getString(LocaleKey.dialog_Run_Detail_Label_Bind_Model);
			jLabel.setText(msg);
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.setFont(new Font("Dialog", Font.PLAIN, 14));
			jContentPane.add(jLabel, null);
			jContentPane.add(getModelNameTextField(), null);
			jContentPane.add(getServiceUrlTextField(), null);
			jContentPane.add(jLabel1, null);
			jContentPane.add(jLabel11, null);
			jContentPane.add(jLabel12, null);
			jContentPane.add(jLabel13, null);
			jContentPane.add(getTypeTextField(), null);
			jContentPane.add(getOperationTextField1(), null);
			jContentPane.add(getStateTextField1(), null);
			jContentPane.add(jLabel2, null);
			jContentPane.add(jLabel3, null);
			jContentPane.add(jLabel4, null);
			jContentPane.add(getBeginTextField1(), null);
			jContentPane.add(getEndTextField1(), null);
			jContentPane.add(getSequeTextField1(), null);
		}
		return jContentPane;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		RunDetailDialog dialog = new RunDetailDialog(null, null, null);
		dialog.setVisible(true);
	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
