/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.sensorweb.ses.ui;

import java.awt.Font;
import java.awt.Frame;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Map;
import java.util.Properties;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.WindowConstants;

import org.dom4j.Document;

import cn.edu.whu.lmars.service.utils.RequestMethodStore;
import cn.edu.whu.lmars.utils.BaseTool;
import cn.edu.whu.lmars.utils.FileTool;
import conf.PathConf;
import conf.SensorWebProperty;

/**
 * SES Operation Dialog.
 * 
 * @author Mingda Zhang
 * 
 */
public class SESOperaDialog extends JDialog implements ActionListener,ItemListener{

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JLabel jLabel = null;
	private JLabel jLabel1 = null;
	private JTextField urlTextField = null;
	private JComboBox operaCombox = null;
	private JButton jButton = null;
	private JScrollPane jScrollPane = null;
	private JTextArea requestTextArea = null;
	private JScrollPane jScrollPane1 = null;
	private JTextArea responseTextArea = null;
	/**
	 * @param owner
	 */
	public SESOperaDialog(Frame owner) {
		super(owner);
		initialize();
	}

	/**
	 * This method initializes this dialog.
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(642, 598);
		this.setFont(new Font("Dialog", Font.PLAIN, 14));
		this.setTitle("SES Dialog");
		this.setResizable(false);
		this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		this.setContentPane(getJContentPane());
		
		initComp();
		
		this.setLocationRelativeTo(null);
		this.setModal(true);
	}

	/**
	 * Initializes the components.
	 */
	private void initComp(){
		/*File sosFile = new File(PathConf.SES_Template);
		File[] files = sosFile.listFiles();
		for(File file:files){
			if (file.isFile()) {
				this.operaCombox.addItem(file.getName());
			}
		}*/
		Map<String,String> SesMap = PathConf.SESTempMap;
		for(String key:SesMap.keySet()){
			this.operaCombox.addItem(key);
		}
		Properties properties = SensorWebProperty.getInstance().getServiceProperty();
		String sesUrl = properties.getProperty("ses");
		this.urlTextField.setText(sesUrl);
		this.urlTextField.setEditable(false);
	}
	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jLabel1 = new JLabel();
			jLabel1.setBounds(new Rectangle(16, 46, 79, 27));
			jLabel1.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel1.setHorizontalAlignment(SwingConstants.RIGHT);
			jLabel1.setText("Examples:");
			jLabel = new JLabel();
			jLabel.setBounds(new Rectangle(9, 11, 88, 27));
			jLabel.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel.setHorizontalAlignment(SwingConstants.RIGHT);
			jLabel.setText("Requset Url:");
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(jLabel, null);
			jContentPane.add(jLabel1, null);
			jContentPane.add(getUrlTextField(), null);
			jContentPane.add(getOperaCombox(), null);
			jContentPane.add(getJButton(), null);
			jContentPane.add(getJScrollPane(), null);
			jContentPane.add(getJScrollPane1(), null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes urlTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getUrlTextField() {
		if (urlTextField == null) {
			urlTextField = new JTextField();
			urlTextField.setBounds(new Rectangle(111, 10, 513, 26));
			urlTextField.setFont(new Font("Dialog", Font.PLAIN, 14));
		}
		return urlTextField;
	}

	/**
	 * This method initializes operaCombox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getOperaCombox() {
		if (operaCombox == null) {
			operaCombox = new JComboBox();
			operaCombox.setBounds(new Rectangle(111, 46, 513, 24));
			operaCombox.setFont(new Font("Dialog", Font.BOLD, 14));
			operaCombox.addItemListener(this);
		}
		return operaCombox;
	}

	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButton() {
		if (jButton == null) {
			jButton = new JButton();
			jButton.setBounds(new Rectangle(14, 355, 107, 28));
			jButton.setFont(new Font("Dialog", Font.BOLD, 14));
			jButton.setText("Send");
			jButton.addActionListener(this);
			jButton.setActionCommand("sendRequest");
		}
		return jButton;
	}

	
	
	/**
	 * This method initializes jScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setBounds(new Rectangle(12, 86, 611, 257));
			jScrollPane.setViewportView(getRequestTextArea());
		}
		return jScrollPane;
	}

	/**
	 * This method initializes requestTextArea	
	 * 	
	 * @return javax.swing.JTextArea	
	 */
	private JTextArea getRequestTextArea() {
		if (requestTextArea == null) {
			requestTextArea = new JTextArea();
		}
		return requestTextArea;
	}

	/**
	 * This method initializes jScrollPane1	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getJScrollPane1() {
		if (jScrollPane1 == null) {
			jScrollPane1 = new JScrollPane();
			jScrollPane1.setBounds(new Rectangle(11, 389, 611, 172));
			jScrollPane1.setViewportView(getResponseTextArea());
		}
		return jScrollPane1;
	}

	/**
	 * This method initializes responseTextArea	
	 * 	
	 * @return javax.swing.JTextArea	
	 */
	private JTextArea getResponseTextArea() {
		if (responseTextArea == null) {
			responseTextArea = new JTextArea();
		}
		return responseTextArea;
	}

	/**
	 * When JComboBox changes, change the content of JTextArea.
	 */
	@Override
	public void itemStateChanged(ItemEvent e) {
		// TODO Auto-generated method stub
		JComboBox jcb = (JComboBox) e.getSource();
		String fileName = jcb.getSelectedItem().toString();
		String path = PathConf.SESTempMap.get(fileName);
		if (BaseTool.isNull(path)) {
			this.requestTextArea.setText("Failed to read the file.");
		}else {
			Document doc = FileTool.getDocFromResource(path);
			if (doc!=null) {
				this.requestTextArea.setText(doc.asXML());
			}
		}
	}

	/**
	 * Sends request.
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		String str = e.getActionCommand();
		if(str.equals("sendRequest")){
			String url = this.urlTextField.getText();
			String request = this.requestTextArea.getText();
			String response = RequestMethodStore.POST(request, url);
			this.responseTextArea.setText(response);
		}
	}
	
	public static void main(String[] args){
		SESOperaDialog dialog = new SESOperaDialog(null);
		dialog.setVisible(true);
	}
} 
