/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.sensorweb.ses.util;

import java.util.HashMap;
import java.util.Map;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.XPath;

/**
 * SES operation.
 * 
 * @author Mingda Zhang
 * 
 */
public class SESUtility {
	private SESUtility() {
	}

	private static Map<String, String> map = new HashMap<String, String>();

	static {
		map.put("soap", "http://www.w3.org/2003/05/soap-envelope");
		map.put("wsnt", "http://docs.oasis-open.org/wsn/b-2");
		map.put("muse-wsa", "http://ws.apache.org/muse/addressing");
		map.put("wsa", "http://www.w3.org/2005/08/addressing");
	}

	/**
	 * Gets NotificationMessage from the document.
	 * 
	 * @param doc
	 * 
	 * @return
	 */
	public static Element getNotification(Document doc) {
		Element root = doc.getRootElement();
		XPath path = DocumentHelper
				.createXPath("soap:Body/wsnt:Notify/wsnt:NotificationMessage");
		path.setNamespaceURIs(map);
		Element element = (Element) path.selectSingleNode(root);
		return element;
	}

	/**
	 * Gets subscription id from response of subscription to ses.
	 * 
	 * @param doc
	 * @return
	 */
	public static String getSESIDFromSubscriptionResp(Document doc) {
		Element root = doc.getRootElement();
		XPath path = DocumentHelper
				.createXPath("soap:Body/wsnt:SubscribeResponse/wsnt:SubscriptionReference/wsa:ReferenceParameters/muse-wsa:ResourceId");
		System.out.println(root.asXML());
		path.setNamespaceURIs(map);
		Element element = (Element) path.selectSingleNode(root);
		if (element == null) {
			return null;
		}
		return element.getText();
	}

	/**
	 * Gets subscription id from response of notification to ses.
	 * 
	 * @param doc
	 * @return
	 */
	public static String getSESIDFromNotifyResp(Document doc) {
		Element root = doc.getRootElement();
		XPath path = DocumentHelper
				.createXPath("soap:Body/wsnt:Notify/wsnt:NotificationMessage/wsnt:SubscriptionReference/wsa:ReferenceParameters/muse-wsa:ResourceId");
		path.setNamespaceURIs(map);
		Element element = (Element) path.selectSingleNode(root);
		if (element == null) {
			return null;
		}
		return element.getText();
	}

	/**
	 * Gets SES address from response of notification.
	 * 
	 * @param doc
	 * @return
	 */
	public static String getSESURLFromNotifyResp(Document doc) {
		Element root = doc.getRootElement();
		XPath path = DocumentHelper
				.createXPath("soap:Header/wsa:From/wsa:Address");
		path.setNamespaceURIs(map);
		Element element = (Element) path.selectSingleNode(root);
		if (element == null) {
			return null;
		}
		return element.getText();
	}

}
