/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.sensorweb.sos.adapter;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Namespace;
import org.dom4j.Node;
import org.dom4j.QName;
import org.dom4j.XPath;
import org.dom4j.io.SAXReader;

import cn.edu.whu.lmars.sensorweb.sos.SOSConf;
import cn.edu.whu.lmars.service.adapter.base.IAdapter;
import cn.edu.whu.lmars.service.adapter.base.IOperationInfo;
import cn.edu.whu.lmars.service.adapter.base.IParameterInfo;
import cn.edu.whu.lmars.service.adapter.base.IServiceInfo;
import cn.edu.whu.lmars.service.adapter.base.impl.OperationInfoImpl;
import cn.edu.whu.lmars.service.adapter.base.impl.ParameterInfoImpl;
import cn.edu.whu.lmars.service.adapter.base.impl.ServiceInfoImpl;
import cn.edu.whu.lmars.service.beans.IService;
import cn.edu.whu.lmars.service.meta.ParameterMeta;
import cn.edu.whu.lmars.service.meta.ServiceMeta;
import cn.edu.whu.lmars.service.meta.ServiceProvider;
import cn.edu.whu.lmars.service.meta.ServiceProvider.ServiceContact;
import cn.edu.whu.lmars.service.meta.ServiceProvider.ServiceContact.Address;
import cn.edu.whu.lmars.service.utils.RequestMethodStore;
import cn.edu.whu.lmars.utils.BaseTool;
import cn.edu.whu.lmars.utils.FileTool;
import conf.PathConf;

/**
 * Sensor Observation Service adapter.
 * 
 * @author Mingda Zhang
 * 
 */
public class SOSAdapter implements IAdapter {
	private Logger logger = Logger.getLogger(SOSAdapter.class);
	private String address = null;
	private IServiceInfo serviceInfo;
	private IService service;
	private String cacheDir;
	private Document decribeDoc;
	private String outputFormat;
	private boolean fresh = false;
	private Namespace sosNS = new Namespace("sos",
			"http://www.opengis.net/sos/1.0"), owsNs = new Namespace("ows",
			"http://www.opengis.net/ows/1.1"), xlink = new Namespace("xlink",
			"http://www.w3.org/1999/xlink"), gmlNS = new Namespace("gml",
			"http://www.opengis.net/gml"), omns = new Namespace("om",
	"http://www.opengis.net/om/1.0"),swens = new Namespace("swe",
	"http://www.opengis.net/swe/1.0.1"),smlNs= new Namespace("sml", "http://www.opengis.net/sensorML/1.0.1");
	private QName hrefQN = new QName("href", xlink), cityQN = new QName("City",
			owsNs), coutryQN = new QName("Country", owsNs),
			emailQN = new QName("ElectronicMailAddress", owsNs),
			individualQN = new QName("IndividualName", owsNs),
			positionQN = new QName("PositionName", owsNs),
			serviceIdentifyQN = new QName("ServiceIdentification", owsNs),
			titleQN = new QName("Title", owsNs), abstractQN = new QName(
					"Abstract", owsNs), keywordsQN = new QName("Keywords",
					owsNs), keywordQN = new QName("Keyword", owsNs),
			serviceTypQN = new QName("ServiceTyp", owsNs),
			serviceVersionQN = new QName("ServiceTypeVersion", owsNs),
			offeringIDQN = new QName("id", gmlNS), procedureQN = new QName(
					"procedure", sosNS),
					propertyQN = new QName("observedProperty",sosNS),
					foiQN = new QName("featureOfInterest",sosNS);

	Map<String, String> map = new HashMap<String, String>();

	public SOSAdapter(IService service,boolean fresh) {
		this.service = service;
		this.address = service.getUrl();
		this.fresh = fresh;
	}

	@SuppressWarnings("unchecked")
	@Override
	public IServiceInfo parseService() {
		if (serviceInfo != null) {
			return serviceInfo;
		}
		boolean flag = true;
		Document doc = null;
		if (fresh) {
			doc = RequestMethodStore.GetDoc(getAddress() + getCapParameter());
			if (doc == null) {
				return null;
			}
		}else {
			if (BaseTool.isNull(cacheDir)) {
				cacheDir = PathConf.getDataDir()+File.separator+PathConf.Cache_Name+File.separator+FileTool.getUniqueDirFromService(service);
			}
			File cacheFile = new File(cacheDir);
			if (!cacheFile.exists()) {
				flag =cacheFile.mkdirs();
			}
			String wsPath = cacheDir+File.separator+"GetCapabilities.xml";
				File wsFile = new File(wsPath);
				if (flag && wsFile.exists()) {
					SAXReader reader = new SAXReader();
					try {
						doc = reader.read(wsFile);
					} catch (DocumentException e) {
						e.printStackTrace();
						doc = null;
					}
				}
			
			if (doc == null) {
				doc = RequestMethodStore.GetDoc(getAddress() + getCapParameter());
				if (doc == null) {
					return null;
				}

			}
		}
		
		Element capele = doc.getRootElement();
		if (capele == null) {
			logger.error("errors occured when get the capabilities of "
					+ getAddress());
			logger.error("the request url is " + getAddress()
					+ getCapParameter());
			return null;
		}

		serviceInfo = new ServiceInfoImpl();
		map.put(sosNS.getPrefix(), sosNS.getURI());
		map.put(owsNs.getPrefix(), owsNs.getURI());
		map.put(omns.getPrefix(), omns.getURI());
		map.put(swens.getPrefix(), swens.getURI());
		map.put(smlNs.getPrefix(),smlNs.getURI());
		map.put(gmlNS.getPrefix(),gmlNS.getURI());
		// parse DescribeSensor url
		XPath describeXpath = DocumentHelper
				.createXPath("ows:OperationsMetadata/ows:Operation[@name='DescribeSensor']/ows:DCP/ows:HTTP/ows:Post");
		describeXpath.setNamespaceURIs(map);
		Node describeEle = describeXpath.selectSingleNode(capele);
		if (describeEle == null) {
			serviceInfo.setDescribeURI(getAddress());
		} else {
			String describeURl = ((Element) describeEle).attributeValue(hrefQN);
			if (BaseTool.isNull(describeURl)) {
				serviceInfo.setDescribeURI(getAddress());
			} else {
				serviceInfo.setDescribeURI(describeURl);
			}
		}

		// parse GetObservation url
		XPath executeXpath = DocumentHelper
				.createXPath("ows:OperationsMetadata/ows:Operation[@name='GetObservation']/ows:DCP/ows:HTTP/ows:Post");
		executeXpath.setNamespaceURIs(map);
		Node executeEle = executeXpath.selectSingleNode(capele);
		if (executeEle == null) {
			serviceInfo.setExcuteURI(getAddress());
		} else {
			String executeURl = ((Element) executeEle).attributeValue(hrefQN);
			if (!BaseTool.isNull(executeURl)) {
				serviceInfo.setExcuteURI(getAddress());
			} else {
				serviceInfo.setExcuteURI(executeURl);
			}
		}

		// parse service metadata
		ServiceMeta serviceMeta = new ServiceMeta();
		ServiceProvider provider = new ServiceProvider();
		ServiceContact contact = provider.new ServiceContact();
		Address address = contact.new Address();

		// parse address
		XPath addressXPath = DocumentHelper
				.createXPath("ows:ServiceProvider/ows:ServiceContact/ows:ContactInfo/ows:Address");
		addressXPath.setNamespaceURIs(map);
		Node addressNode = addressXPath.selectSingleNode(capele);
		if (addressNode != null) {
			Element addressEle = (Element) addressNode;
			String city = addressEle.elementText(cityQN);
			if (!BaseTool.isNull(city)) {
				address.set_city(city);
			}
			String coutry = addressEle.elementText(coutryQN);
			if (!BaseTool.isNull(coutry)) {
				address.set_country(coutry);
			}
			String email = addressEle.elementText(emailQN);
			if (!BaseTool.isNull(email)) {
				address.set_electronicmailaddress(email);
			}
		}

		contact.setAd(address);

		// parse contact
		XPath contactXPath = DocumentHelper
				.createXPath("ows:ServiceProvider/ows:ServiceContact");
		contactXPath.setNamespaceURIs(map);
		Node contactNode = contactXPath.selectSingleNode(capele);
		if (contactNode != null) {
			Element contactEle = (Element) contactNode;
			String individual = contactEle.elementText(individualQN);
			if (!BaseTool.isNull(individual)) {
				contact.setIndividualname(individual);
			}
			String position = contactEle.elementText(positionQN);
			if (!BaseTool.isNull(position)) {
				contact.setPositionname(position);
			}
		}
		provider.setSc(contact);

		// parse provider
		XPath providerXPath = DocumentHelper
				.createXPath("ows:ServiceProvider/ows:ProviderName");
		providerXPath.setNamespaceURIs(map);
		Node providerNameNode = providerXPath.selectSingleNode(capele);
		if (providerNameNode != null) {
			String providerName = providerNameNode.getText();
			if (!BaseTool.isNull(providerName)) {
				provider.setProvidername(providerName);
			}
		}
		serviceMeta.setSp(provider);

		// parse the service meta
		Node serviceMetaNode = capele.element(serviceIdentifyQN);
		if (serviceMetaNode != null) {
			Element serviceMetaEle = (Element) serviceMetaNode;
			Node titleNode = serviceMetaEle.element(titleQN);
			if (titleNode != null) {
				serviceMeta.set_title(titleNode.getText());
			}
			Node abstractNode = serviceMetaEle.element(abstractQN);
			if (abstractNode != null) {
				serviceMeta.set_abstract(abstractNode.getText());
			}
			Node keyWordsNode = serviceMetaEle.element(keywordsQN);
			if (keyWordsNode != null) {
				List<Node> keyWordNodes = ((Element) keyWordsNode)
						.elements(keywordQN);
				String keyword = "";
				for (Node node : keyWordNodes) {
					keyword += node.getText() + ";";
				}
				serviceMeta.set_keywords(keyword);
			}
			Node serviceTypeNode = serviceMetaEle.element(serviceTypQN);
			if (serviceTypeNode != null) {
				serviceMeta.set_type(serviceTypeNode.getText());
			}
			Node serviceVersionNode = serviceMetaEle.element(serviceVersionQN);
			if (serviceVersionNode != null) {
				serviceMeta.set_version(serviceVersionNode.getText());
			}
		}

		serviceInfo.SetServiceMeta(serviceMeta);

		// parse the procedure that the sos offers
		List<String> operationList = new ArrayList<String>();
		XPath procedureXPath = DocumentHelper
				.createXPath("sos:Contents/sos:ObservationOfferingList/sos:ObservationOffering/sos:procedure");
		procedureXPath.setNamespaceURIs(map);
		List<Element> procedureEles = procedureXPath.selectNodes(capele);
		String procedureName = "";
		for (Element element : procedureEles) {
			procedureName = element.attributeValue(hrefQN);
			if (!BaseTool.isNull(procedureName)) {
				operationList.add(procedureName);
			}
		}

		serviceInfo.setOperationList(operationList);

		// parse all the opration information
		XPath offerXpath = DocumentHelper
				.createXPath("sos:Contents/sos:ObservationOfferingList/sos:ObservationOffering");
		offerXpath.setNamespaceURIs(map);
		List<Element> offerEles = offerXpath.selectNodes(capele);
		String offering;
		for (Element offerEle : offerEles) {
			offering = offerEle.attributeValue(offeringIDQN);
			if (BaseTool.isNull(offering)) {
				offering = "";
			}
			String properties="";
			List<Element> propertyEles = offerEle.elements(propertyQN);
			for(int i=0;i<propertyEles.size();i++){
				if (i==0) {
					properties+=propertyEles.get(i).attributeValue(hrefQN);
				}else {
					properties+=";"+propertyEles.get(i).attributeValue(hrefQN);
				}
			}
			
			List<Element> foiEles = offerEle.elements(foiQN);
			String foiStr="";
			for(int i=0;i<foiEles.size();i++){
				if (i==0) {
					foiStr+=foiEles.get(i).attributeValue(hrefQN);
				}else {
					foiStr+=";"+foiEles.get(i).attributeValue(hrefQN);
				}
			}
			// List<Element> propertyEles =
			// offerEle.elements(observedPropertyQN);
			List<Element> proceEles = offerEle.elements(procedureQN);
			for (Element proce : proceEles) {
				String procedure = proce.attributeValue(hrefQN);
				if (BaseTool.isNull(procedure)) {
					continue;
				}
				IOperationInfo operationInfo = getSOSOperationInfo();
				operationInfo.setName(procedure);
				IParameterInfo procedureParam = operationInfo
						.getIParameterInfo(SOSConf.Procedure);
				procedureParam.getMeta().set_allowValue(procedure);
				IParameterInfo offeringParam = operationInfo
						.getIParameterInfo(SOSConf.Offering);
				offeringParam.getMeta().set_allowValue(offering);
				IParameterInfo propertyParam = operationInfo.getIParameterInfo(SOSConf.ObservedProperty);
				propertyParam.getMeta().set_allowValue(properties);
				
				IParameterInfo foiParam = operationInfo.getIParameterInfo(SOSConf.FeatureOfInterest);
				foiParam.getMeta().set_allowValue(foiStr);
				serviceInfo.addOperationInfo(operationInfo);
				
			}
		}

		return serviceInfo;

	}

	@SuppressWarnings("unchecked")
	@Override
	public Map<String, String> parseResponse(String response) {
		Map<String, String> respMap = new HashMap<String, String>(2);
		try {
			Element rootele = DocumentHelper.parseText(response.trim())
					.getRootElement();
			
			
			// nsMap.put(swens.getPrefix(),swens.getURI());
			XPath valueXPath = DocumentHelper
					.createXPath("om:member/om:Observation/om:result/swe:DataArray/swe:values");
			valueXPath.setNamespaceURIs(map);
			
			String sepRecord = ";", sepValue = ",";
			XPath sepratorXPath = DocumentHelper
					.createXPath("om:member/om:Observation/om:result/swe:DataArray/swe:encoding/swe:TextBlock");
			sepratorXPath.setNamespaceURIs(map);
			Node sepNode = sepratorXPath.selectSingleNode(rootele);
			if (sepNode != null) {
				Element sepEle = (Element) sepNode;
				String sepStr = sepEle.attributeValue("blockSeparator");
				if (!BaseTool.isNull(sepStr)) {
					sepRecord = sepStr.trim();
				}
				sepStr = sepEle.attributeValue("tokenSeparator");
				if (!BaseTool.isNull(sepStr)) {
					sepValue = sepStr;
				}
			}
			
			if (!BaseTool.isNull(outputFormat) && outputFormat.equalsIgnoreCase("text")) {
				List<Node> valueNodes = valueXPath.selectNodes(rootele);
				List<String> recordList = new ArrayList<String>();
				if (valueNodes==null) {
					return null;
				}
				for(Node node:valueNodes){
					String value =node.getText();
					String[] records = value.split(sepRecord);
					for (String record:records) {
						if (record.trim().equals("")) {
							continue;
						}
						recordList.add(record);
					}
				}
				String path = FileTool.saveToText(recordList);
				if (path==null) {
					return null;
				}
				if (path!=null) {
					String url = RequestMethodStore.uploadFile(path);
					if (BaseTool.isNull(url)) {
						return null;
					}
					respMap.put("result_value", url);
					respMap.put("result_datatype", outputFormat);
				}
			}else {
				Node valueNode = valueXPath.selectSingleNode(rootele);
				if (valueNode == null) {
					return respMap;
				}
				String value = valueNode.getText();
				String[] records = value.split(sepRecord);
				if (records.length > 0) {
					String[] values = records[0].split(sepValue);
					if (values.length > 0) {
						respMap.put("result_value", values[values.length - 1]);
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		return respMap;
	}

	@Override
	public String createRequest(String operationname,
			Map<String, String> parameternameandvalue) {
		StringBuffer req = new StringBuffer();
		String eventTime = parameternameandvalue.get(SOSConf.Event_Time);
		String obsProperty = parameternameandvalue
				.get(SOSConf.ObservedProperty);
		String offering = parameternameandvalue.get(SOSConf.Offering);
		String foi = parameternameandvalue.get(SOSConf.FeatureOfInterest);
		String procedure = parameternameandvalue.get(SOSConf.Procedure);
		outputFormat = parameternameandvalue.get(SOSConf.OutputFormat);
		/*if (eventTime == null || obsProperty == null || offering == null
				|| foi == null) {
			logger.error("Please input all parameters.");
			return null;
		}*/
		if (offering == null || obsProperty==null) {
			logger.error("Offering or observedProperty is null");
			return null;
		}
		
	//	req.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?><GetObservation xmlns=\"http://www.opengis.net/sos/1.0\"  xmlns:ows=\"http://www.opengis.net/ows/1.1\"  xmlns:gml=\"http://www.opengis.net/gml\"  xmlns:ogc=\"http://www.opengis.net/ogc\"  xmlns:om=\"http://www.opengis.net/om/1.0\"  xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"  xsi:schemaLocation=\"http://www.opengis.net/sos/1.0  http://schemas.opengis.net/sos/1.0.0/sosGetObservation.xsd\"  service=\"SOS\" version=\"1.0.0\" srsName=\"urn:ogc:def:crs:EPSG:4326\">  <offering>");
		req.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?><GetObservation xmlns=\"http://www.opengis.net/sos/1.0\"  xmlns:ows=\"http://www.opengis.net/ows/1.1\"  xmlns:gml=\"http://www.opengis.net/gml\"  xmlns:ogc=\"http://www.opengis.net/ogc\"  xmlns:om=\"http://www.opengis.net/om/1.0\"  xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"  xsi:schemaLocation=\"http://www.opengis.net/sos/1.0  http://schemas.opengis.net/sos/1.0.0/sosGetObservation.xsd\"  service=\"SOS\" version=\"1.0.0\" >  <offering>");
		req.append(offering);
		req.append("</offering>");
		
		if (!BaseTool.isNull(eventTime)) {
			String[] times = eventTime.split(";");
			if (times.length < 2) {
				logger.error("Time parameter is wrong, the correct format is 'begintime;endtime'");
			}else {
				req.append("<eventTime>    <ogc:TM_During>    <ogc:PropertyName>om:samplingTime</ogc:PropertyName>  <gml:TimePeriod> <gml:beginPosition>");
				req.append(times[0].trim());
				req.append("</gml:beginPosition><gml:endPosition>");
				req.append(times[1].trim());
				req.append("</gml:endPosition>   </gml:TimePeriod>  </ogc:TM_During></eventTime>");
			}
		}
		
		if (!BaseTool.isNull(procedure)) {
			String[] procedures = procedure.split(";");
			for(String str:procedures){
				if (!BaseTool.isNull(str)) {
					req.append("<procedure>");
					req.append(str);
					req.append("</procedure>");
				}
			}
		}
		
		String[] obsProperties = obsProperty.split(";");
		for(String property:obsProperties){
			if (!BaseTool.isNull(property)) {
				req.append("<observedProperty>");
				req.append(property);
				req.append("</observedProperty>");
			}
		}
		
		
		if (!BaseTool.isNull(foi)) {
			
			String[] extents = foi.split(";");
			if (extents.length < 4) {
				if (extents.length>0) {
					req.append(" <featureOfInterest><ObjectID>");
					req.append(extents[0]);
					req.append("</ObjectID></featureOfInterest>");
				}else {
					logger.error("Extent parameter is wrong.");
				}
			//	return null;
			}else {
				req.append("<featureOfInterest>  <ogc:BBOX>  <ogc:PropertyName>urn:ogc:data:location</ogc:PropertyName>      <gml:Envelope srsName=\"urn:ogc:def:crs:EPSG:4326\">    <gml:lowerCorner>");
				req.append(extents[0].trim()).append(" ").append(extents[1].trim())
						.append("</gml:lowerCorner>");
				req.append("<gml:upperCorner>").append(extents[2].trim()).append(" ")
						.append(extents[3].trim());
				req.append("</gml:upperCorner>   </gml:Envelope> </ogc:BBOX> </featureOfInterest>");
			}
		}
		req.append("<responseFormat>text/xml;subtype=&quot;om/1.0.0&quot;</responseFormat></GetObservation>");
		/*req.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?><GetObservation xmlns=\"http://www.opengis.net/sos/1.0\"  xmlns:ows=\"http://www.opengis.net/ows/1.1\"  xmlns:gml=\"http://www.opengis.net/gml\"  xmlns:ogc=\"http://www.opengis.net/ogc\"  xmlns:om=\"http://www.opengis.net/om/1.0\"  xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"  xsi:schemaLocation=\"http://www.opengis.net/sos/1.0  http://schemas.opengis.net/sos/1.0.0/sosGetObservation.xsd\"  service=\"SOS\" version=\"1.0.0\" srsName=\"urn:ogc:def:crs:EPSG::4326\">  <offering>");
		req.append(offering);
		req.append("</offering>  <eventTime>    <ogc:TM_During>    <ogc:PropertyName>om:samplingTime</ogc:PropertyName>  <gml:TimePeriod> <gml:beginPosition>");
		req.append(times[0].trim());
		req.append("</gml:beginPosition><gml:endPosition>");
		req.append(times[1].trim());
		req.append("</gml:endPosition>   </gml:TimePeriod>  </ogc:TM_During></eventTime> <observedProperty>");
		req.append(obsProperty);
		req.append("</observedProperty>  <featureOfInterest>  <ogc:BBOX>  <ogc:PropertyName>urn:ogc:data:location</ogc:PropertyName>      <gml:Envelope srsName=\"urn:ogc:def:crs:EPSG:4326\">    <gml:lowerCorner>");
		req.append(extents[0].trim()).append(" ").append(extents[1].trim())
				.append("</gml:lowerCorner>");
		req.append("<gml:upperCorner>").append(extents[2].trim()).append(" ")
				.append(extents[3].trim());
		req.append("</gml:upperCorner>   </gml:Envelope> </ogc:BBOX> </featureOfInterest><responseFormat>text/xml;subtype=&quot;om/1.0.0&quot;</responseFormat></GetObservation>");*/
		return req.toString().trim();
	}

	@Override
	public String sendRequest(String request) {
		return RequestMethodStore.POST(request, serviceInfo.getExecuteURI());
	}

	@SuppressWarnings("unchecked")
	@Override
	public IOperationInfo getOperationInfo(String operationName) {
		if (serviceInfo == null) {
			if (parseService() == null) {
				return null;
			}
		}
		IOperationInfo operationInfo = serviceInfo
				.getOperationInfo(operationName);
		if (operationInfo != null) {
			return operationInfo;
		}
		Document doc = null;
		if (BaseTool.isNull(cacheDir)) {
			cacheDir = PathConf.getDataDir()+File.separator+PathConf.Cache_Name+File.separator+FileTool.getUniqueDirFromService(service);
		}
		
			String fileName = operationName.replace(":","_");
			String path = cacheDir+ File.separator + fileName + ".xml";
			File file = new File(path);
			if (file.exists()) {
				SAXReader reader = new SAXReader();
				try {
					doc = reader.read(file);
				} catch (DocumentException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					doc = null;
				}
			}
		
		if (doc == null) {
			doc = getDescribeDocFromUrl(operationName);
			if (doc == null) {
				return null;
			}
		}
		Element describeEle = doc.getRootElement();
		XPath outputXPath = DocumentHelper.createXPath("//sml:outputs/sml:OutputList/sml:output");
		outputXPath.setNamespaceURIs(map);
		List<Element> outPutEles = outputXPath.selectNodes(describeEle);
		operationInfo = getSOSOperationInfo(); 
		//List<String> offerList = new ArrayList<String>(2);
		List<String> propertyList = new ArrayList<String>(3);
		
		for(Element outputEle:outPutEles){
			Element propertyEle = (Element)outputEle.elements().get(0);
			String propertyID = propertyEle.attributeValue("definition");
			propertyList.add(propertyID);
			//	XPath offerXPath = DocumentHelper.createXPath("//sos:offering/sos:id");
			/*XPath offerXPath = DocumentHelper.createXPath("//sos:offering/sos:id");
			offerXPath.setNamespaceURIs(map);
			Element offerEle = (Element)offerXPath.selectSingleNode(propertyEle);
			String offerStr = offerEle.getText();
			if (!offerList.contains(offerStr)) {
				offerList.add(offerStr);
			}*/
		}
		IParameterInfo procedureParam = operationInfo.getIParameterInfo(SOSConf.Procedure);
		procedureParam.getMeta().set_allowValue(operationName);
		String propertys = "";
		for(int i=0;i<propertyList.size();i++){
			if (i==0) {
				propertys+=propertyList.get(i);
			}else {
				propertys+=";"+propertyList.get(i);
			}
		}
		IParameterInfo propertyParam = operationInfo
		.getIParameterInfo(SOSConf.ObservedProperty);
		propertyParam.getMeta().set_allowValue(propertys);
		/*
		 * if (operationInfo==null) {
		 * 
		 * }
		 */
		serviceInfo.addOperationInfo(operationInfo);
		return operationInfo;
	}

	public Document getDescribeDocFromUrl(String operationName){
		if (serviceInfo==null) {
			if (parseService()==null) {
				return null;
			}
		}
		if (decribeDoc==null) {
			decribeDoc = FileTool.getDocFromResource(PathConf.SOS_Describe_Sensor);
		}
		XPath xPath = DocumentHelper.createXPath("//sos:procedure");
		xPath.setNamespaceURIs(map);
		Element sensorEle = (Element)xPath.selectSingleNode(decribeDoc);
		if (sensorEle == null) {
			return null;
		}
		sensorEle.setText(operationName);
		return RequestMethodStore.POSTDoc(decribeDoc.asXML(), serviceInfo.getDescribeURI());
	}
	private IOperationInfo getSOSOperationInfo() {
		IOperationInfo operationInfo = new OperationInfoImpl();
		List<IParameterInfo> inputParameters = new ArrayList<IParameterInfo>();
		List<IParameterInfo> outputParameters = new ArrayList<IParameterInfo>();

		ParameterMeta eventTimeMeta = new ParameterMeta();
		eventTimeMeta.set_title("EventTime");
		eventTimeMeta.set_abstract("time during which the feature is observed");
		eventTimeMeta.set_type("xs:string");
		eventTimeMeta.set_minoccurs("0");
		IParameterInfo eventTimeParamInfo = new ParameterInfoImpl();
		eventTimeParamInfo.setMeta(eventTimeMeta);
		eventTimeParamInfo.setName(SOSConf.Event_Time);

		inputParameters.add(eventTimeParamInfo);

		ParameterMeta observedPropertyMeta = new ParameterMeta();
		observedPropertyMeta.set_title("ObservedProperty");
		observedPropertyMeta.set_abstract("ObservedProperty");
		observedPropertyMeta.set_type("xs:string");
		IParameterInfo observedPropertyParamInfo = new ParameterInfoImpl();
		observedPropertyParamInfo.setMeta(observedPropertyMeta);
		observedPropertyParamInfo.setName(SOSConf.ObservedProperty);
		inputParameters.add(observedPropertyParamInfo);

		ParameterMeta offerMeta = new ParameterMeta();
		offerMeta.set_title("GetObserv_offering");
		offerMeta.set_abstract("GetObserv_offering");
		offerMeta.set_type("xs:string");
		IParameterInfo offerParamInfo = new ParameterInfoImpl();
		offerParamInfo.setMeta(offerMeta);
		offerParamInfo.setName(SOSConf.Offering);
		inputParameters.add(offerParamInfo);

		ParameterMeta interestMeta = new ParameterMeta();
		interestMeta.set_title("FeatureOfInterest");
		interestMeta.set_minoccurs("0");
		interestMeta.set_abstract("FeatureOfInterest");
		interestMeta.set_type("xs:string");
		IParameterInfo interestParamInfo = new ParameterInfoImpl();
		interestParamInfo.setMeta(interestMeta);
		interestParamInfo.setName(SOSConf.FeatureOfInterest);
		inputParameters.add(interestParamInfo);

		ParameterMeta procedureMeta = new ParameterMeta();
		procedureMeta.set_title("procedure");
		procedureMeta.set_abstract("procedure");
		procedureMeta.set_minoccurs("0");
		procedureMeta.set_type("xs:string");
		IParameterInfo procedureParamInfo = new ParameterInfoImpl();
		procedureParamInfo.setMeta(procedureMeta);
		procedureParamInfo.setName(SOSConf.Procedure);
		inputParameters.add(procedureParamInfo);

		ParameterMeta outputFormatMeta = new ParameterMeta();
		outputFormatMeta.set_title("OutputFormat");
		outputFormatMeta.set_abstract("OutputFormat");
		outputFormatMeta.set_minoccurs("1");
		outputFormatMeta.set_type("xs:string");
		IParameterInfo outputformatParam = new ParameterInfoImpl();
		outputformatParam.setMeta(procedureMeta);
		outputformatParam.setName(SOSConf.OutputFormat);
		inputParameters.add(outputformatParam);
		
		operationInfo.setInputParameters(inputParameters);

		// output message
		IParameterInfo outputparam = new ParameterInfoImpl();
		ParameterMeta outputMeta = new ParameterMeta();
		// meta7.set_type("");
		outputMeta.set_abstract("return the dataURL");
		outputMeta.set_title("outputdata");
		outputparam.setName("OutPutData");
		outputparam.setMeta(outputMeta);
		outputParameters.add(outputparam);

		operationInfo.setOutputParameters(outputParameters);
		return operationInfo;
	}

	@Override
	public String getCapParameter() {
		// TODO Auto-generated method stub
		return "?Request=GetCapabilities&Service=SOS";
	}

	@Override
	public IServiceInfo getServiceInfo() {
		// TODO Auto-generated method stub
		/*
		 * if (serviceInfo == null) { serviceInfo = parseService(); }
		 */
		return serviceInfo;
	}

	@Override
	public String getAddress() {
		// TODO Auto-generated method stub
		return address;
	}

	public Document getDescribeDoc(String sensorName){
		if (serviceInfo == null) {
			if (parseService() == null) {
				return null;
			}
		}
		
		Document doc = null;
		if (BaseTool.isNull(cacheDir)) {
			cacheDir = PathConf.getDataDir()+File.separator+PathConf.Cache_Name+File.separator+FileTool.getUniqueDirFromService(service);
		}
			String fileName = sensorName.replace(":","_");
			String path = cacheDir+File.separator + fileName + ".xml";
			File file = new File(path);
			if (file.exists()) {
				SAXReader reader = new SAXReader();
				try {
					doc = reader.read(file);
				} catch (DocumentException e) {
					e.printStackTrace();
					doc = null;
				}
			}
		
		if (doc == null) {
			doc = getDescribeDocFromUrl(sensorName);
		}
		return doc;
	}
	@Override
	public String getDesParameter(String operationName) {
		// TODO Auto-generated method stub
		 return "?Request=DescribeSensor&Service=SOS&Version=1.0&Language=en&Identifier="
		+ operationName;
	}
}
