/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.sensorweb.sos.adapter;

import java.util.ArrayList;
import java.util.List;

import cn.edu.whu.lmars.service.adapter.base.IOperationInfo;
import cn.edu.whu.lmars.service.adapter.base.IParameterInfo;
import cn.edu.whu.lmars.service.meta.IOperationMeta;

/**
 * An implementor of IOperationInfo, stores SOS operation information.
 * 
 * @author Mingda Zhang
 * 
 */
public class SOSOperationInfo implements IOperationInfo {

	/** procedure of sos */
	private String name;
	/** metadata of operation */
	private IOperationMeta operationMeta;
	/** offering of procedure */
	private String offering;
	/** event time */
	private String eventTime;
	/** feature of interest */
	private String featureOfInterest;
	/** observed properties */
	private List<String> observedProperties;

	@Override
	public String getName() {
		return name;
	}

	@Override
	public void setName(String name) {
		this.name = name;
	}

	@Override
	public IOperationMeta getMeta() {
		return operationMeta;
	}

	@Override
	public void setMeta(IOperationMeta meta) {
		this.operationMeta = meta;
	}

	/**
	 * Gets offering.
	 */
	public String getOffering() {
		return offering;
	}

	/**
	 * Sets offering.
	 * 
	 * @param offering
	 */
	public void setOffering(String offering) {
		this.offering = offering;
	}

	/**
	 * Gets event time.
	 * 
	 * @return
	 */
	public String getEventTime() {
		return eventTime;
	}

	/**
	 * Sets event time.
	 * 
	 * @param eventTime
	 */
	public void setEventTime(String eventTime) {
		this.eventTime = eventTime;
	}

	/**
	 * Gets observed properties.
	 */
	public List<String> getObservedProperties() {
		return observedProperties;
	}

	/**
	 * Gets feature of interest.
	 */
	public String getFeatureOfInterest() {
		return featureOfInterest;
	}

	/**
	 * Sets feature of interest.
	 * 
	 * @param featureOfInterest
	 */
	public void setFeatureOfInterest(String featureOfInterest) {
		this.featureOfInterest = featureOfInterest;
	}

	/**
	 * Adds a observed property.
	 * 
	 * @param observedProperty
	 */
	public void addObservedProperty(String observedProperty) {
		if (this.observedProperties == null) {
			this.observedProperties = new ArrayList<String>(3);
		}
		if (!this.observedProperties.contains(observedProperty)) {
			this.observedProperties.add(observedProperty);
		}
	}

	/*
	 * Sets observed properties.
	 */
	public void setObservedProperties(List<String> observedProperties) {
		this.observedProperties = observedProperties;
	}

	@Override
	public void setInputParameters(List<IParameterInfo> inputParamInfos) {
	}

	@Override
	public List<IParameterInfo> getInputParameterInfos() {
		return null;
	}

	@Override
	public IParameterInfo getIParameterInfo(String parameterName) {
		return null;
	}

	@Override
	public void setOutputParameters(List<IParameterInfo> outputParameterInfos) {
	}

	@Override
	public List<IParameterInfo> getOutputParameters() {
		return null;
	}

}
