/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.sensorweb.sos.beans;

import java.util.ArrayList;
import java.util.List;

/**
 * The SOS sensor class.
 * 
 * @author Mingda Zhang
 * 
 */
public class SOSSensor {
	/** SOS address */
	private String url;
	/** SOS version */
	private String version;
	/** SOS procedure */
	private String procedure;
	/** SOS feature of interset */
	private String featureOfInterest;
	/** sensor event time */
	private String eventTime;
	/** sensor offering */
	private String offer;
	/** observed properties */
	private List<String> observedProperties;
	/** sensor position */
	private SensorPosition sensorPosition;
	/** sensor type,mobile or not */
	boolean mobile;

	/**
	 * Returns true if the sensor is mobile.
	 * 
	 * @return
	 */
	public boolean isMobile() {
		return mobile;
	}

	/**
	 * Sets mobile of this sensor.
	 * 
	 * @param mobile
	 */
	public void setMobile(boolean mobile) {
		this.mobile = mobile;
	}

	/**
	 * Sets SOS address.
	 * 
	 * @return
	 */
	public String getUrl() {
		return url;
	}

	/**
	 * Gets SOS address.
	 * 
	 * @param url
	 */
	public void setUrl(String url) {
		this.url = url;
	}

	/**
	 * Sets SOS version.
	 * 
	 * @return
	 */
	public String getVersion() {
		return version;
	}

	/**
	 * Sets SOS version.
	 * 
	 * @param version
	 */
	public void setVersion(String version) {
		this.version = version;
	}

	/**
	 * Gets procedure.
	 * 
	 * @return
	 */
	public String getProcedure() {
		return procedure;
	}

	/**
	 * Sets procedure.
	 * 
	 * @param procedure
	 */
	public void setProcedure(String procedure) {
		this.procedure = procedure;
	}

	/**
	 * Gets feature of interest.
	 * 
	 * @return
	 */
	public String getFeatureOfInterest() {
		return featureOfInterest;
	}

	/**
	 * Sets feature of interest.
	 * 
	 * @param featureOfInterest
	 */
	public void setFeatureOfInterest(String featureOfInterest) {
		this.featureOfInterest = featureOfInterest;
	}

	/**
	 * Gets event time.
	 * 
	 * @return
	 */
	public String getEventTime() {
		return eventTime;
	}

	/**
	 * Sets event time.
	 * 
	 * @param eventTime
	 */
	public void setEventTime(String eventTime) {
		this.eventTime = eventTime;
	}

	/**
	 * Gets offering.
	 * 
	 * @return
	 */
	public String getOffer() {
		return offer;
	}

	/**
	 * Sets offering.
	 * 
	 * @param offer
	 */
	public void setOffer(String offer) {
		this.offer = offer;
	}

	/**
	 * Gets observed properties.
	 * 
	 * @return
	 */
	public List<String> getObservedProperties() {
		return observedProperties;
	}

	/**
	 * Sets observed properties.
	 * 
	 * @param observedProperties
	 */
	public void setObservedProperties(List<String> observedProperties) {
		this.observedProperties = observedProperties;
	}

	/**
	 * Adds observed property.
	 * 
	 * @param observedProperty
	 */
	public void addObservedProperty(String observedProperty) {
		if (this.observedProperties == null) {
			this.observedProperties = new ArrayList<String>(4);
		}
		if (!this.observedProperties.contains(observedProperty)) {
			this.observedProperties.add(observedProperty);
		}
	}

	/**
	 * Gets sensor position.
	 * 
	 * @return
	 */
	public SensorPosition getSensorPosition() {
		return sensorPosition;
	}

	/**
	 * Sets sensor position.
	 * 
	 * @param sensorPosition
	 */
	public void setSensorPosition(SensorPosition sensorPosition) {
		this.sensorPosition = sensorPosition;
	}

}
