/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.sensorweb.ui;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Enumeration;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.WindowConstants;

import org.apache.log4j.Logger;
import org.dom4j.Document;

import cn.edu.whu.lmars.frame.ui.ClientFrame;
import cn.edu.whu.lmars.sensorweb.ses.util.SESUtility;
import cn.edu.whu.lmars.sensorweb.sos.SOSConf;
import cn.edu.whu.lmars.sensorweb.wns.beans.WNSUser;
import cn.edu.whu.lmars.sensorweb.wns.util.WNSUtil;
import cn.edu.whu.lmars.service.adapter.AdapterFactory;
import cn.edu.whu.lmars.service.adapter.base.IAdapter;
import cn.edu.whu.lmars.service.adapter.base.IOperationInfo;
import cn.edu.whu.lmars.service.adapter.base.IParameterInfo;
import cn.edu.whu.lmars.service.utils.NetUtil;
import cn.edu.whu.lmars.service.utils.RequestMethodStore;
import cn.edu.whu.lmars.starter.InitParam;
import cn.edu.whu.lmars.tree.nodes.ServiceTreeNode;
import cn.edu.whu.lmars.tree.nodes.TreeNodeImpl;
import cn.edu.whu.lmars.tree.ui.TreeScrollPanel;
import cn.edu.whu.lmars.utils.BaseTool;
import cn.edu.whu.lmars.utils.FileTool;
import conf.GlobeParam;
import conf.PathConf;
import conf.SensorWebProperty;

/**
 * This dialog is used to subscribe warning for a sensor.
 * 
 * @author Mingda Zhang
 * 
 */
public class SubscribeNotifyDialog extends JDialog implements ActionListener,
		ItemListener {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JLabel jLabel = null;
	private JComboBox sosJComboBox = null;
	private JLabel jLabel1 = null;
	private JComboBox offerCombox = null;
	private JLabel jLabel2 = null;
	private JComboBox procedureComboBox = null;
	private JLabel jLabel3 = null;
	private JComboBox propertyComboBox = null;
	private JButton confirmButton = null;
	private JButton concelButton = null;
	private JLabel jLabel4 = null;
	private JTextField wnsTextField = null;
	private JLabel jLabel5 = null;
	private JTextField wnsUserTextField = null;
	private JLabel jLabel6 = null;
	private JComboBox sesComboBox = null;
	private JLabel jLabel7 = null;
	private JTextField sesurlTextField = null;
	private JScrollPane jScrollPane = null;
	private JTextArea sesRequestTextArea = null;
	private Logger logger = Logger.getLogger(SubscribeNotifyDialog.class);
	private JButton sesbtn = null;
	private JTextField sesIDTextField = null;
	private String offerName = "offer";
	private String sosName = "sos";
	private String propertyName = "property";
	private String procedureName = "procedure";
	/**
	 * @param owner
	 */
	public SubscribeNotifyDialog(Frame owner) {
		super(owner);
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {

		this.setSize(522, 787);
		this.setResizable(false);
		this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		this.setTitle("Subscription Dialog");
		this.setContentPane(getJContentPane());
		this.initComp();
		this.setLocationRelativeTo(null);
		this.setModal(true);

	}

	private void initComp() {
		ClientFrame frame = GlobeParam.getClient();
		TreeScrollPanel treePanel = frame.getTreeScrollPanel();
		List<ServiceTreeNode> sosService = treePanel.getSOSServiceNode();
	//	this.sosJComboBox.setText("http://localhost:8081/52nSOSv3/sos");
		this.sosJComboBox.removeAllItems();
		for(ServiceTreeNode treeNode:sosService){
			this.sosJComboBox.addItem(treeNode);
		}
		Map<String, String> SesMap = PathConf.SESTempMap;
		for(String key:SesMap.keySet()){
			this.sesComboBox.addItem(key);
		}
		Properties properties = SensorWebProperty.getInstance()
				.getServiceProperty();
		String sesUrl = properties.getProperty("ses");
		this.sesurlTextField.setText(sesUrl);
		this.sesurlTextField.setEditable(true);

		String wnsUrl = properties.getProperty("wns");
		this.wnsTextField.setText(wnsUrl);
		this.wnsTextField.setEditable(true);
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jLabel7 = new JLabel();
			jLabel7.setBounds(new Rectangle(15, 330, 85, 30));
			jLabel7.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel7.setHorizontalAlignment(SwingConstants.RIGHT);
			jLabel7.setText("Examples:");
			jLabel6 = new JLabel();
			jLabel6.setBounds(new Rectangle(15, 285, 85, 30));
			jLabel6.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel6.setHorizontalAlignment(SwingConstants.RIGHT);
			jLabel6.setText("SES URL:");
			jLabel5 = new JLabel();
			jLabel5.setBounds(new Rectangle(15, 240, 85, 30));
			jLabel5.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel5.setHorizontalAlignment(SwingConstants.RIGHT);
			jLabel5.setText("Email:");
			jLabel4 = new JLabel();
			jLabel4.setBounds(new Rectangle(15, 195, 85, 30));
			jLabel4.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel4.setHorizontalAlignment(SwingConstants.RIGHT);
			jLabel4.setText("WNS URL:");
			jLabel3 = new JLabel();
			jLabel3.setBounds(new Rectangle(15, 150, 85, 30));
			jLabel3.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel3.setHorizontalAlignment(SwingConstants.RIGHT);
			jLabel3.setText("Property:");
			jLabel2 = new JLabel();
			jLabel2.setBounds(new Rectangle(8, 105, 92, 30));
			jLabel2.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel2.setHorizontalAlignment(SwingConstants.RIGHT);
			jLabel2.setText("Offering:");
			jLabel1 = new JLabel();
			jLabel1.setBounds(new Rectangle(15, 60, 85, 30));
			jLabel1.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel1.setHorizontalAlignment(SwingConstants.RIGHT);
			jLabel1.setText("Procedure:");
			jLabel = new JLabel();
			jLabel.setBounds(new Rectangle(15, 15, 85, 30));
			jLabel.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel.setHorizontalAlignment(SwingConstants.RIGHT);
			jLabel.setText("SOS:");
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(jLabel, null);
			jContentPane.add(getSosJComboBox(), null);
			jContentPane.add(jLabel1, null);
			jContentPane.add(getOfferCombox(), null);
			jContentPane.add(jLabel2, null);
			jContentPane.add(getProcedureComboBox(), null);
			jContentPane.add(jLabel3, null);
			jContentPane.add(getPropertyComboBox(), null);
			jContentPane.add(getConfirmButton(), null);
			jContentPane.add(getConcelButton(), null);
			jContentPane.add(jLabel4, null);
			jContentPane.add(getWnsTextField(), null);
			jContentPane.add(jLabel5, null);
			jContentPane.add(getWnsUserComboBox(), null);
			jContentPane.add(jLabel6, null);
			jContentPane.add(getSesComboBox(), null);
			jContentPane.add(jLabel7, null);
			jContentPane.add(getSesurlTextField(), null);
			jContentPane.add(getJScrollPane(), null);
			jContentPane.add(getSesbtn(), null);
			jContentPane.add(getSesIDTextField(), null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes sosUrlTextField
	 * 
	 * @return javax.swing.JTextField
	 */
	private JComboBox getSosJComboBox() {
		if (sosJComboBox == null) {
			sosJComboBox = new JComboBox();
			sosJComboBox.setBounds(new Rectangle(119, 15, 369, 28));
			sosJComboBox.setFont(new Font("Dialog", Font.PLAIN, 14));
			sosJComboBox.setName(this.sosName);
			sosJComboBox.addItemListener(this);
			sosJComboBox.setEditable(false);
		}
		return sosJComboBox;
	}

	/**
	 * This method initializes offerCombox
	 * 
	 * @return javax.swing.JComboBox
	 */
	private JComboBox getOfferCombox() {
		if (offerCombox == null) {
			offerCombox = new JComboBox();
			offerCombox.setBounds(new Rectangle(119, 105, 368, 28));
			offerCombox.setFont(new Font("Dialog", Font.BOLD, 14));
			offerCombox.addItemListener(this);
			offerCombox.setName(this.offerName);
		}
		return offerCombox;
	}
	
	/**
	 * This method initializes procedureComboBox
	 * 
	 * @return javax.swing.JComboBox
	 */
	private JComboBox getProcedureComboBox() {
		if (procedureComboBox == null) {
			procedureComboBox = new JComboBox();
			procedureComboBox.setBounds(new Rectangle(119, 60, 370, 28));
			procedureComboBox.setFont(new Font("Dialog", Font.BOLD, 14));
			procedureComboBox.setName(this.procedureName);
			procedureComboBox.addItemListener(this);
		}
		return procedureComboBox;
	}

	/**
	 * This method initializes propertyComboBox
	 * 
	 * @return javax.swing.JComboBox
	 */
	private JComboBox getPropertyComboBox() {
		if (propertyComboBox == null) {
			propertyComboBox = new JComboBox();
			propertyComboBox.setBounds(new Rectangle(119, 150, 367, 28));
			propertyComboBox.setFont(new Font("Dialog", Font.BOLD, 14));
			propertyComboBox.setName(this.propertyName);
		}
		return propertyComboBox;
	}

	/**
	 * This method initializes confirmButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getConfirmButton() {
		if (confirmButton == null) {
			confirmButton = new JButton();
			confirmButton.setBounds(new Rectangle(69, 707, 112, 30));
			confirmButton.setText("Subscribe");
			confirmButton.setFont(new Font("Dialog", Font.BOLD, 14));
			confirmButton.setActionCommand("confirm");
			confirmButton.addActionListener(this);
		}
		return confirmButton;
	}

	/**
	 * This method initializes concelButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getConcelButton() {
		if (concelButton == null) {
			concelButton = new JButton();
			concelButton.setBounds(new Rectangle(304, 710, 101, 30));
			concelButton.setText("Cancel");
			concelButton.setFont(new Font("Dialog", Font.BOLD, 14));
			concelButton.addActionListener(this);
			concelButton.setActionCommand("concel");
		}
		return concelButton;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		String str = e.getActionCommand();

		if (str.equals("confirm")) {
			Properties properties = SensorWebProperty.getInstance()
					.getServiceProperty();
			String saveServlet = properties.getProperty("seswnsfeeder_save");
			ServiceTreeNode treeNode = (ServiceTreeNode)this.sosJComboBox.getSelectedItem();
			String sosUrl = treeNode.getUrl();
			String sosOffer = this.offerCombox.getSelectedItem().toString();
			String sosSensor = this.procedureComboBox.getSelectedItem()
					.toString();
			String sosProperty = this.propertyComboBox.getSelectedItem()
					.toString();
			String wnsUrl = this.wnsTextField.getText();
			String wnsUser = this.wnsUserTextField.getText();
			String sesUrl = this.sesurlTextField.getText();
			String wnsID = null;
			String sesID = this.sesIDTextField.getText();
			if (!NetUtil.validUrl(wnsUrl)) {
				JOptionPane.showMessageDialog(GlobeParam.getClient(), "The address of wns is not availiable. Please check the network or the address.");
				return;
			}
			List<WNSUser> wnsUsers = WNSUtil.getUserList(wnsUrl);
			for(WNSUser user:wnsUsers){
				if (user.getEmail().equals(wnsUser)) {
					wnsID = user.getId();
					break;
				}
			}
			if (BaseTool.isNull(wnsID)) {
				JOptionPane.showMessageDialog(GlobeParam.getClient(), "The Email is not registed.");
				return;
			}
			StringBuffer buffer = new StringBuffer();
			buffer.append(saveServlet + "?");
			buffer.append("wnsID=" + wnsID);
			buffer.append("&sesID=" + sesID);
			buffer.append("&wnsURL=" + wnsUrl);
			buffer.append("&sesURL=" + sesUrl);
			buffer.append("&sosSensor=" + sosSensor);
			buffer.append("&sosOffer=" + sosOffer);
			buffer.append("&sosProperty=" + sosProperty);
			buffer.append("&sosURL=" + sosUrl);
			logger.info("Request: \n" + buffer.toString());
			String saveResp = RequestMethodStore.GETStr(buffer.toString());
			JOptionPane.showMessageDialog(null, saveResp);
		} else if (str.equals("concel")) {
			this.dispose();
		} else if (str.equals("parse")) {
			/*String sosURL = this.sosJComboBox.getText();
			sosOffers = SOSUtility.getInstance().getOffers(sosURL);
			for (SOSOffer offer : sosOffers) {
				this.offerCombox.addItem(offer.getName());
			}
			this.offerCombox.setSelectedIndex(0);*/
		}else if (str.equals("sesid")) {
			String sesReq = this.sesRequestTextArea.getText();
			String sesUrl = this.sesurlTextField.getText();
			logger.info("SES Request:\n" + sesReq);
			if (!NetUtil.validUrl(sesUrl)) {
				JOptionPane.showMessageDialog(this,"SES address is not available.");
				return;
			}
			Document sesResp = RequestMethodStore.POSTDoc(sesReq, sesUrl);
			logger.info("SES Response\n" + sesResp.asXML());
			String sesID = SESUtility.getSESIDFromSubscriptionResp(sesResp);
			if (BaseTool.isNull(sesID)) {
				JOptionPane.showMessageDialog(null,
						"Failed to parse subscription id from SES response.");
				return;
			}
			this.sesIDTextField.setText(sesID);
		}

	}

	@SuppressWarnings("rawtypes")
	@Override
	public void itemStateChanged(ItemEvent e) {
		JComboBox jcb = (JComboBox) e.getSource();
		Object object = jcb.getSelectedItem();
		if (object == null) {
			return;
		}
		String item = object.toString();
		String comboName = jcb.getName();
		if (comboName.equals(this.sosName)) {
			if (!(object instanceof ServiceTreeNode)) {
				return;
			}
			ServiceTreeNode treeNode = (ServiceTreeNode)object;
		//	IAdapter adapter = AdapterFactory.getInstance().getAdapter(treeNode);
		//	SOSOffer selectedOffer = null;
			/*for (SOSOffer offer : sosOffers) {
				if (offer.getName().equals(item)) {
					selectedOffer = offer;
					break;
				}
			}
			if (selectedOffer == null) {
				return;
			}*/
			this.procedureComboBox.removeAllItems();
		//	this.propertyComboBox.removeAllItems();
			for (Enumeration node = treeNode.children(); node.hasMoreElements();) {
				TreeNodeImpl n = (TreeNodeImpl) node.nextElement();
				this.procedureComboBox.addItem(n.getName());
			}
			/*for (String property : selectedOffer.getProperties()) {
				this.propertyComboBox.addItem(property);
			}*/
		} else if (comboName.equals(this.procedureName)) {
			Object sosObject  = this.sosJComboBox.getSelectedItem();
			this.propertyComboBox.removeAllItems();
			this.offerCombox.removeAllItems();
			if (sosObject==null) {
				return;
			}
			ServiceTreeNode serviceTreeNode = (ServiceTreeNode)sosObject;
			Object procedureObj = this.procedureComboBox.getSelectedItem();
			if (procedureObj==null) {
				return;
			}
			String procedure = procedureObj.toString();
			IAdapter adapter = AdapterFactory.getInstance().getAdapter(serviceTreeNode);
			IOperationInfo operationInfo = adapter.getOperationInfo(procedure);
			IParameterInfo offerParam = operationInfo.getIParameterInfo(SOSConf.Offering);
			String offerStr= offerParam.getMeta().get_allowValue();
			if (!BaseTool.isNull(offerStr)) {
				for(String offer:offerStr.split(";")){
					if (!BaseTool.isNull(offer)) {
						this.offerCombox.addItem(offer);
					}
				}
			}
			IParameterInfo propertyParam = operationInfo.getIParameterInfo(SOSConf.ObservedProperty);
			String propertyStr = propertyParam.getMeta().get_allowValue();
			if (!BaseTool.isNull(propertyStr)) {
				for(String property:propertyStr.split(";")){
					if (!BaseTool.isNull(property)) {
						this.propertyComboBox.addItem(property);
					}
				}
			}
		}else if (comboName.equals("sescombo")) {
			String fileName = item;
			String path = PathConf.SESTempMap.get(fileName);
			if (BaseTool.isNull(path)) {
				this.sesRequestTextArea.setText("Failed to read file.");
				return;
			}
			Document doc = FileTool.getDocFromResource(path);
			if (doc == null) {
				this.sesRequestTextArea.setText("Failed to read file.");
				return;
			}
			this.sesRequestTextArea.setText(doc.asXML());
		}

	}

	/**
	 * This method initializes wnsTextField
	 * 
	 * @return javax.swing.JTextArea
	 */
	private JTextField getWnsTextField() {
		if (wnsTextField == null) {
			wnsTextField = new JTextField();
			wnsTextField.setBounds(new Rectangle(119, 195, 368, 28));
			wnsTextField.setFont(new Font("Dialog", Font.PLAIN, 14));
		}
		return wnsTextField;
	}

	/**
	 * This method initializes wnsUserComboBox
	 * 
	 * @return javax.swing.JComboBox
	 */
	private JTextField getWnsUserComboBox() {
		if (wnsUserTextField == null) {
			wnsUserTextField = new JTextField();
			wnsUserTextField.setBounds(new Rectangle(119, 240, 370, 28));
			wnsUserTextField.setFont(new Font("Dialog", Font.BOLD, 14));
		}
		return wnsUserTextField;
	}

	/**
	 * This method initializes sesComboBox
	 * 
	 * @return javax.swing.JComboBox
	 */
	private JComboBox getSesComboBox() {
		if (sesComboBox == null) {
			sesComboBox = new JComboBox();
			sesComboBox.setBounds(new Rectangle(119, 330, 370, 28));
			sesComboBox.setFont(new Font("Dialog", Font.BOLD, 14));
			sesComboBox.setName("sescombo");
			sesComboBox.addItemListener(this);
		}
		return sesComboBox;
	}

	/**
	 * This method initializes sesurlTextField
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getSesurlTextField() {
		if (sesurlTextField == null) {
			sesurlTextField = new JTextField();
			sesurlTextField.setBounds(new Rectangle(119, 285, 366, 30));
			sesurlTextField.setFont(new Font("Dialog", Font.PLAIN, 14));
		}
		return sesurlTextField;
	}

	/**
	 * This method initializes jScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setBounds(new Rectangle(20, 378, 468, 265));
			jScrollPane.setViewportView(getSesRequestTextArea());
		}
		return jScrollPane;
	}

	/**
	 * This method initializes sesRequestTextArea
	 * 
	 * @return javax.swing.JTextArea
	 */
	private JTextArea getSesRequestTextArea() {
		if (sesRequestTextArea == null) {
			sesRequestTextArea = new JTextArea();
		}
		return sesRequestTextArea;
	}

	/**
	 * This method initializes sesbtn
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getSesbtn() {
		if (sesbtn == null) {
			sesbtn = new JButton();
			sesbtn.setText("Get Subscription ID");
			sesbtn.setSize(new Dimension(171, 30));
			sesbtn.setFont(new Font("Dialog", Font.BOLD, 14));
			sesbtn.setLocation(new Point(18, 655));
			sesbtn.setActionCommand("sesid");
			sesbtn.addActionListener(this);
		}
		return sesbtn;
	}

	/**
	 * This method initializes sesIDTextField
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getSesIDTextField() {
		if (sesIDTextField == null) {
			sesIDTextField = new JTextField();
			sesIDTextField.setBounds(new Rectangle(197, 657, 292, 30));
			sesIDTextField.setEditable(true);
		}
		return sesIDTextField;
	}

	public static void main(String[] args) {
		new InitParam().execute();
		SubscribeNotifyDialog dialog = new SubscribeNotifyDialog(null);
		dialog.setVisible(true);
	}
}
