/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.sensorweb.wns.ui;

import java.awt.Font;
import java.awt.Frame;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.table.DefaultTableModel;

import cn.edu.whu.lmars.sensorweb.wns.beans.WNSUser;
import cn.edu.whu.lmars.sensorweb.wns.util.WNSUtil;
import cn.edu.whu.lmars.starter.InitParam;

import conf.SensorWebProperty;

/**
 * WNS operation dialog.
 * 
 * @author Mingda Zhang
 * 
 */
@Deprecated
public class WNSOperaDialog extends JDialog implements ActionListener {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JLabel jLabel = null;
	private JTextField urlTextField = null;
	private JButton getUsersBtn = null;
	private JButton addbtn = null;
	private JButton deleteBtn = null;
	private JScrollPane jScrollPane = null;
	private JTable userTable = null;

	/**
	 * @param owner
	 */
	public WNSOperaDialog(Frame owner) {
		super(owner);
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(520, 389);
		this.setFont(new Font("Dialog", Font.PLAIN, 14));
		this.setTitle("WNS Dialog");
		this.setContentPane(getJContentPane());

		initComp();

		this.setModal(true);
		this.setLocationRelativeTo(null);
	}

	private void initComp() {
		Properties properties = SensorWebProperty.getInstance()
				.getServiceProperty();
		String wnsUrl = properties.getProperty("wns");
		this.urlTextField.setText(wnsUrl);
	}

	/**
	 * This method initializes urlTextField
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getUrlTextField() {
		if (urlTextField == null) {
			urlTextField = new JTextField();
			urlTextField.setBounds(new Rectangle(105, 16, 374, 27));
			urlTextField.setEditable(true);
		}
		return urlTextField;
	}

	/**
	 * This method initializes getUsersBtn
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getGetUsersBtn() {
		if (getUsersBtn == null) {
			getUsersBtn = new JButton();
			getUsersBtn.setBounds(new Rectangle(17, 58, 134, 24));
			getUsersBtn.setFont(new Font("Dialog", Font.BOLD, 14));
			getUsersBtn.addActionListener(this);
			getUsersBtn.setActionCommand("get");
			getUsersBtn.setText("Get Users");
		}
		return getUsersBtn;
	}

	/**
	 * This method initializes addbtn
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getAddbtn() {
		if (addbtn == null) {
			addbtn = new JButton();
			addbtn.setBounds(new Rectangle(273, 295, 90, 27));
			addbtn.setFont(new Font("Dialog", Font.BOLD, 14));
			addbtn.setActionCommand("add");
			addbtn.addActionListener(this);
			addbtn.setText("Add");
		}
		return addbtn;
	}

	/**
	 * This method initializes deleteBtn
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getDeleteBtn() {
		if (deleteBtn == null) {
			deleteBtn = new JButton();
			deleteBtn.setBounds(new Rectangle(107, 293, 88, 27));
			deleteBtn.setText("Delete");
			deleteBtn.setActionCommand("delete");
			deleteBtn.addActionListener(this);
			deleteBtn.setFont(new Font("Dialog", Font.BOLD, 14));
		}
		return deleteBtn;
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jLabel = new JLabel();
			jLabel.setBounds(new Rectangle(16, 17, 79, 26));
			jLabel.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel.setHorizontalAlignment(SwingConstants.RIGHT);
			jLabel.setText("WNS URL:");
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(jLabel, null);
			jContentPane.add(getUrlTextField(), null);
			jContentPane.add(getGetUsersBtn(), null);
			jContentPane.add(getAddbtn(), null);
			jContentPane.add(getDeleteBtn(), null);
			jContentPane.add(getJScrollPane(), null);
		}
		return jContentPane;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		// TODO Auto-generated method stub
		String str = e.getActionCommand();
		String url = urlTextField.getText();
		DefaultTableModel model = (DefaultTableModel) this.userTable.getModel();
		if (str.equals("get")) {
			List<WNSUser> userList = WNSUtil.getUserList(url);
			for (WNSUser user : userList) {
				model.addRow(new String[] { user.getId(), user.getName(),
						user.getEmail() });
			}
		} else if (str.equals("add")) {
			SwingUtilities.invokeLater(new Runnable() {

				@Override
				public void run() {
					// TODO Auto-generated method stub
					String url = urlTextField.getText();
					DefaultTableModel model = (DefaultTableModel) userTable
							.getModel();
					List<WNSUser> userList = new ArrayList<WNSUser>();
					AddUserDialog dialog = new AddUserDialog(null, userList,
							url);
					dialog.setVisible(true);
					if (userList.size() > 0) {
						for (WNSUser user : userList) {
							model.addRow(new String[] { user.getId(),
									user.getName(), user.getEmail() });
						}
					}
				}
			});

		} else if (str.equals("delete")) {
			int num = this.userTable.getSelectedRow();
			if (num == -1) {
				return;
			} else {
				String id = this.userTable.getValueAt(num, 0).toString();
				// userTable.remove(num);
				boolean flag = WNSUtil.unRegister(id, url);
				if (flag) {
					model.removeRow(num);
					JOptionPane.showMessageDialog(null, "Delete the WNS User by id:" + id + "successfully");
				}else {
					JOptionPane.showMessageDialog(null, "Failed to delete.");
				}
			}
		} else if(str.equals("detail")) {

		}
	}

	/**
	 * This method initializes jScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setBounds(new Rectangle(18, 92, 463, 186));
			jScrollPane.setFont(new Font("Dialog", Font.PLAIN, 14));
			jScrollPane.setViewportView(getUserTable());
		}
		return jScrollPane;
	}

	/**
	 * This method initializes userTable
	 * 
	 * @return javax.swing.JTable
	 */
	private JTable getUserTable() {
		if (userTable == null) {
			// List<WNSUser> userList = WNSUtility.getUserList(url)
			String[] columnTitles = { "ID", "Name", "Email" };
			DefaultTableModel model = new DefaultTableModel(null, columnTitles);
			userTable = new JTable(model);
			userTable.remove(0);
			userTable.setFont(new Font("Dialog", Font.PLAIN, 14));
			userTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		}
		return userTable;
	}

	public static void main(String[] args) {
		new InitParam().execute();
		WNSOperaDialog dialog = new WNSOperaDialog(null);
		dialog.setVisible(true);
	}

}
