/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.sensorweb.wns.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.XPath;

import cn.edu.whu.lmars.sensorweb.wns.beans.WNSUser;
import cn.edu.whu.lmars.service.utils.RequestMethodStore;
import cn.edu.whu.lmars.utils.FileTool;
import conf.PathConf;

/**
 * Handles WNS operation.
 * 
 * @author Mingda Zhang
 * 
 */
public class WNSUtil {

	private static Map<String, String> map = new HashMap<String, String>();
	private static Logger logger = Logger.getLogger(WNSUtil.class);
	static {
		map.put("wns", "http://www.opengis.net/wns");
	}

	/**
	 * Registers a new user to WNS.
	 * 
	 * @param userName
	 *            user name.
	 * @param userEmail
	 *            user email.
	 * @param url
	 *            WNS address.
	 * @return the user id.
	 */
	public static String register(String userName, String userEmail, String url) {
		String templateFile = PathConf.WNS_Register;
		try {
			Document requestDoc = FileTool.getDocFromResource(templateFile);
			Element rootEle = requestDoc.getRootElement();

			XPath namePath = DocumentHelper
					.createXPath("wns:SingleUser/wns:Name");
			namePath.setNamespaceURIs(map);
			Element nameEle = (Element) namePath.selectSingleNode(rootEle);
			nameEle.setText(userName);
			XPath emailPath = DocumentHelper
					.createXPath("wns:SingleUser/wns:CommunicationProtocol/wns:Email");
			emailPath.setNamespaceURIs(map);
			Element emailEle = (Element) emailPath.selectSingleNode(rootEle);
			emailEle.setText(userEmail);
			Document responseDoc = RequestMethodStore.POSTDoc(
					requestDoc.asXML(), url);
			Element respRoot = responseDoc.getRootElement();
			logger.info("request:");
			logger.info(requestDoc.asXML());
			logger.info("response:");
			logger.info(responseDoc.asXML());
			XPath idPath = DocumentHelper.createXPath("wns:UserID");
			idPath.setNamespaceURIs(map);
			Element idEle = (Element) idPath.selectSingleNode(respRoot);
			if (idEle == null) {
				logger.error("Failed to register.");
				return null;
			}
			String id = idEle.getText();
			return id;

		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * Unregisters a WNS user.
	 * 
	 * @param userID
	 *            user id.
	 * @param url
	 *            WNS address.
	 * @return
	 */
	public static boolean unRegister(String userID, String url) {
		String templateFile = PathConf.WNS_Unregister;
		try {
			Document requestDoc = FileTool.getDocFromResource(templateFile);
			Element rootEle = requestDoc.getRootElement();

			XPath idPath = DocumentHelper.createXPath("wns:ID");
			idPath.setNamespaceURIs(map);
			Element idEle = (Element) idPath.selectSingleNode(rootEle);
			idEle.setText(userID);
			Document responseDoc = RequestMethodStore.POSTDoc(
					requestDoc.asXML(), url);
			Element respEle = responseDoc.getRootElement();
			logger.info("request:");
			logger.info(requestDoc.asXML());
			logger.info("response:");
			logger.info(responseDoc.asXML());
			XPath statusXpath = DocumentHelper.createXPath("wns:status");
			statusXpath.setNamespaceURIs(map);
			Element statusEle = (Element) statusXpath.selectSingleNode(respEle);

			if (statusEle == null) {
				logger.error("Failed to delete.");
				return false;
			}

			String flag = statusEle.getText();
			if (flag.trim().equalsIgnoreCase("success")) {
				return true;
			}
			return false;
		} catch (Exception e) {
			e.printStackTrace();
			logger.info(e.getMessage());
			return false;
		}
	}

	/**
	 * Gets all users from WNS.
	 * 
	 * @param url
	 *            WNS address.
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static List<WNSUser> getUserList(String url) {
		Map<String, String> map52 = new HashMap<String, String>();
		map52.put("wns", "http://www.52north.org/wns");
		List<WNSUser> userList = new ArrayList<WNSUser>();
		String param = "request=ListWNSUser";
		Document doc = RequestMethodStore.GETDoc(url, param);
		logger.info(doc.asXML());
		Element root = doc.getRootElement();
		XPath usersXPath = DocumentHelper
				.createXPath("wns:SingleUser/wns:User");
		usersXPath.setNamespaceURIs(map52);
		List<Element> userEleList = usersXPath.selectNodes(root);
		XPath idXPath = DocumentHelper.createXPath("wns:ID");
		idXPath.setNamespaceURIs(map52);
		XPath nameXpath = DocumentHelper.createXPath("wns:Name");
		nameXpath.setNamespaceURIs(map52);
		XPath protocalXPath = DocumentHelper.createXPath("wns:Protocol");
		protocalXPath.setNamespaceURIs(map52);
		XPath chanelXPath = DocumentHelper.createXPath("wns:Channel");
		chanelXPath.setNamespaceURIs(map52);
		XPath emailXpath = DocumentHelper.createXPath("wns:Target");
		emailXpath.setNamespaceURIs(map52);
		Element idEle, nameEle, protocalEle, emailEle;
		List<Element> channels;
		for (Element element : userEleList) {
			WNSUser user = new WNSUser();
			idEle = (Element) idXPath.selectSingleNode(element);
			user.setId(idEle.getText());
			nameEle = (Element) nameXpath.selectSingleNode(element);
			user.setName(nameEle.getText());
			channels = chanelXPath.selectNodes(element);
			for (Element channel : channels) {
				protocalEle = (Element) protocalXPath.selectSingleNode(channel);
				if (protocalEle.getText().trim().equalsIgnoreCase("email")) {
					emailEle = (Element) emailXpath.selectSingleNode(channel);
					user.setEmail(emailEle.getText());
					userList.add(user);
					break;
				}
			}
		}
		return userList;
	}

	/**
	 * DoNotification.
	 * 
	 * @param userID
	 *            user id.
	 * @param element
	 *            content of notification.
	 * @param url
	 *            WNS address.
	 * @return
	 */
	public static boolean doNotification(String userID, Element element,
			String url) {
		String templateFile = PathConf.WNS_DoNotification;
		try {
			Document requestDoc = FileTool.getDocFromResource(templateFile);
			Element rootEle = requestDoc.getRootElement();

			XPath idPath = DocumentHelper.createXPath("wns:UserID");
			idPath.setNamespaceURIs(map);
			Element idEle = (Element) idPath.selectSingleNode(rootEle);
			idEle.setText(userID);

			XPath contentXPath = DocumentHelper
					.createXPath("wns:Message/wns:NotificationMessage/wns:Payload");
			contentXPath.setNamespaceURIs(map);
			Element contentEle = (Element) contentXPath
					.selectSingleNode(rootEle);
			// contentEle.setText(content);
			contentEle.add(element);
			Document responseDoc = RequestMethodStore.POSTDoc(
					requestDoc.asXML(), url);
			Element respEle = responseDoc.getRootElement();
			logger.info("request:");
			logger.info(requestDoc.asXML());
			logger.info("response:");
			logger.info(responseDoc.asXML());
			XPath statusXpath = DocumentHelper.createXPath("wns:status");
			statusXpath.setNamespaceURIs(map);
			Element statusEle = (Element) statusXpath.selectSingleNode(respEle);

			if (statusEle == null) {
				logger.error("Failed to delete.");
				return false;
			}

			String flag = statusEle.getText();
			if (flag.trim().equalsIgnoreCase("success")) {
				return true;
			}
			return false;
		} catch (Exception e) {
			e.printStackTrace();
			logger.info(e.getMessage());
			return false;
		}
	}

}
