/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.service.adapter;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import cn.edu.whu.lmars.sensorweb.sos.adapter.SOSAdapter;
import cn.edu.whu.lmars.service.adapter.base.IAdapter;
import cn.edu.whu.lmars.service.beans.IService;
import cn.edu.whu.lmars.service.wfs.WFSAdapterData;
import cn.edu.whu.lmars.service.wps.WPSAdapter040;

/**
 * Creates adapter for different services.
 * 
 * @author Mingda Zhang
 *
 */
public class AdapterFactory {
	/** Adapter container. */
	private List<IAdapter> adapterStore = new ArrayList<IAdapter>();
	private AdapterFactory(){}
	/** Logger. */
	private Logger logger = Logger.getLogger(AdapterFactory.class);
	/** Adapter instance. */
	private static AdapterFactory instance =null;
	/**
	 * Gets an instance of the adapter factory.
	 * @return
	 */
	public static AdapterFactory getInstance(){
		if (instance == null) {
			instance = new AdapterFactory();
		}
		return instance;
	}
	
	/**
	 * Gets a adapter from adapter container.
	 * @param address
	 * @return
	 */
	private IAdapter getAdapter(String address){
		for(IAdapter adapter:adapterStore){
			if (adapter.getAddress().equals(address)) {
				return adapter;
			}
		}
		return null;
	}
	
	//public IAdapter createAdapter()
	/**
	 * Returns a adapter that meets the service requirements.
	 * @param service
	 * 			service information.
	 * @return
	 */
	public IAdapter getAdapter(IService service){
		IAdapter adapter = getAdapter(service.getUrl());
		if (adapter==null) {
			adapter = createAdapter(service,false);
			if (adapter!=null) {
				addAdapter(adapter);
			}
		}
		return adapter;
	}
	/**
	 * Creates a new adapter that meets the service requirements.
	 * @param service
	 * 			service information.
	 * @param fresh
	 * 			Whether to use cache when created.
	 * @return
	 */
	public IAdapter createAdapter(IService service,boolean fresh) {
		String version = service.getVersion();
		IAdapter a = null;
		switch (service.getServiceType()) {
		case WPS:
			if (version.equals("0.4.0")) {
				a = new WPSAdapter040(service,fresh);
			}
			break;
		case WFS:
			a = new WFSAdapterData(service,fresh);
			break;
		case SOS:
			a = new SOSAdapter(service,fresh);
			break;
		default:
			break;
		}
		if (a == null) {
			logger.error("Failed to create a adapter.");
			return null;
		}else {
			a.parseService();
		}
		//addAdapter(a);
		return a;
	}
	
	/**
	 * Removes an adapter from the container.
	 * @param adapter
	 */
	public void removeAdapter(IAdapter adapter){
		if (adapterStore.contains(adapter)) {
			adapterStore.remove(adapter);
		}
	}
	
	/**
	 * Adds an adapter to adapter container.
	 * @param adapter
	 */
	public void addAdapter(IAdapter adapter){
		if (getAdapter(adapter.getAddress())==null) {
			adapterStore.add(adapter);
		}
	}
}
