/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.service.adapter.base;

import java.util.Map;

/**
 * Defines the requirements for an adapter that parses service document.
 * 
 * @author Mingda Zhang
 * 
 */

public interface IAdapter {
	/**
	 * Parses the GetCapability document. Gets service meta such as provider,
	 * key words, description address and so no.
	 * 
	 * @return
	 */
	public IServiceInfo parseService();

	/**
	 * Gets IServiceInfo.
	 * 
	 * @return
	 */
	public IServiceInfo getServiceInfo();

	/**
	 * Creates request for execution.
	 * 
	 * @param operationname
	 *            operation name.
	 * @param parameternameandvalue
	 *            parameters.
	 * @return
	 */
	public String createRequest(String operationname,
			Map<String, String> parameternameandvalue);

	/**
	 * Parses the response to get the result information.
	 * 
	 * @param response
	 *            response xml.
	 * @return
	 */
	public Map<String, String> parseResponse(String response);

	/**
	 * Sends request to the service.
	 * 
	 * @param request
	 *            request xml.
	 * @return
	 */
	public String sendRequest(String request);

	/**
	 * Gets address of the service.
	 * 
	 * @return
	 */

	public String getAddress();

	/**
	 * Gets the parameters needed to send getcapabilities request.
	 * 
	 * @return
	 */
	public String getCapParameter();

	/**
	 * Gets parameters needed to send description request.
	 * 
	 * @param operationName
	 *            operation name.
	 * @return
	 */
	public String getDesParameter(String operationName);

	/**
	 * Gets operation information.
	 * 
	 * @param operationName
	 *            operation name.
	 * @return
	 */
	public IOperationInfo getOperationInfo(String operationName);
}
