/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.service.adapter.base;

import java.util.List;

import cn.edu.whu.lmars.service.meta.IOperationMeta;

/**
 * Defines the requirements for operation information.
 * 
 * @author Mingda Zhang
 * 
 */
public interface IOperationInfo {

	/**
	 * Gets the name of operation.
	 * 
	 * @return the name of operation.
	 */
	public String getName();

	/**
	 * Sets name of operation.
	 * 
	 * @param name
	 */
	public void setName(String name);

	/**
	 * Gets operation metadata.
	 * 
	 * @return operation metadata
	 */
	public IOperationMeta getMeta();

	/**
	 * Sets operation metadata.
	 * 
	 * @param meta
	 *            operation metadata.
	 */
	public void setMeta(IOperationMeta meta);

	/**
	 * Sets input parameters of the operation.
	 * 
	 * @param inputParamInfos
	 *            input parameter information.
	 */
	public void setInputParameters(List<IParameterInfo> inputParamInfos);

	/**
	 * Gets input parameters of the operation.
	 * 
	 * @return input parameter information.
	 */
	public List<IParameterInfo> getInputParameterInfos();

	/**
	 * Gets parameter information by its name.
	 * 
	 * @param parameterName
	 *            name of parameter.
	 * @return
	 */
	public IParameterInfo getIParameterInfo(String parameterName);

	/**
	 * Sets output parameters.
	 * 
	 * @param outputParameterInfos
	 *            output parameter information.
	 */
	public void setOutputParameters(List<IParameterInfo> outputParameterInfos);

	/**
	 * Gets output parameters.
	 * 
	 * @return
	 */
	public List<IParameterInfo> getOutputParameters();
}
