/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.service.adapter.base;

import java.util.List;

import cn.edu.whu.lmars.service.meta.ServiceMeta;

/**
 * Defines the requirements for an object that describes a service.
 * 
 * @author Mingda Zhang
 * 
 */

public interface IServiceInfo {
	/**
	 * Returns service metadata.
	 * 
	 * @return service metadata.
	 */
	public ServiceMeta getMetadata();

	/**
	 * Sets the service metadata.
	 * 
	 * @param serviceMeta
	 *            service metadata.
	 */
	public void SetServiceMeta(ServiceMeta serviceMeta);

	/**
	 * Returns the operation information by its name.
	 * 
	 * @param operationName
	 *            operation name.
	 */
	public IOperationInfo getOperationInfo(String operationName);

	/**
	 * Adds operation information to the service info.
	 * 
	 * @param operationInfo
	 */
	public void addOperationInfo(IOperationInfo operationInfo);

	/**
	 * Returns description address of the service.
	 */
	public String getDescribeURI();

	/**
	 * Sets the description address.
	 * 
	 * @param describe_uri
	 *            description address.
	 */
	public void setDescribeURI(String describe_uri);

	/**
	 * Returns execution address of the service.
	 */
	public String getExecuteURI();

	/**
	 * Sets execution address of the service.
	 * 
	 * @param execute_uri
	 *            execution address of the service
	 */
	public void setExcuteURI(String execute_uri);

	/**
	 * Gets all operation information contained in the service.
	 */
	public List<String> getOperationList();

	/**
	 * Sets the operation info that the service supports.
	 * 
	 * @param operationList
	 *            operation information list.
	 */
	public void setOperationList(List<String> operationList);
}
