/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.service.adapter.base.impl;

import java.util.Map;

import org.apache.log4j.Logger;

import cn.edu.whu.lmars.service.adapter.base.IAdapter;
import cn.edu.whu.lmars.service.adapter.base.IOperationInfo;
import cn.edu.whu.lmars.service.adapter.base.IServiceInfo;

/**
 * Implementation of the interface IAdapter.
 * 
 * @author Mingda Zhang
 * 
 */

public abstract class AdapterImpl implements IAdapter {
	private Logger logger = Logger.getLogger(AdapterImpl.class);
	protected String address;
	protected IServiceInfo serviceInfo;

	@Override
	public abstract IServiceInfo parseService();

	@Override
	public abstract String createRequest(String operationname,
			Map<String, String> parameternameandvalue);

	@Override
	public abstract Map<String, String> parseResponse(String response);

	@Override
	public abstract String sendRequest(String request);

	@Override
	public abstract String getCapParameter();

	public abstract IOperationInfo parseOperationInfo(String operationName);

	@Override
	public IServiceInfo getServiceInfo() {
		return serviceInfo;
	}

	@Override
	public IOperationInfo getOperationInfo(String operationName) {
		IOperationInfo operationInfo = serviceInfo
				.getOperationInfo(operationName);
		if (operationInfo == null) {
			operationInfo = parseOperationInfo(operationName);
			if (operationInfo == null) {
				logger.error("errors occured when get the information of "
						+ operationName);
			} else {
				serviceInfo.addOperationInfo(operationInfo);
			}
		}
		return operationInfo;
	}
}
