/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.service.utils;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;

import org.dom4j.Document;
import org.dom4j.io.SAXReader;

import conf.SensorWebProperty;

/**
 * Sends a request with a post or get method.
 * 
 * @author Mingda Zhang
 * 
 */
public class RequestMethodStore {

	/**
	 * Sends a request by post method and returns response in string format.
	 * 
	 * @param param
	 *            content to send
	 * @param input_url
	 *            service address.
	 * 
	 */
	public static String POST(String param, String input_url) {

		try {
			URL url = new URL(input_url);
			HttpURLConnection con = (HttpURLConnection) url.openConnection();
			con.setDoOutput(true);
			con.setRequestMethod("POST");
			con.setRequestProperty("Content-Type", "application/xml");
			con.setDoOutput(true);
			con.setDoInput(true);
			con.setUseCaches(false);

			PrintWriter xmlOut = new PrintWriter(con.getOutputStream());
			xmlOut.write(param);
			xmlOut.flush();
			BufferedReader response = new BufferedReader(new InputStreamReader(
					con.getInputStream()));
			String result = "";
			String line;
			while ((line = response.readLine()) != null) {
				result += "\n" + line;
			}
			return result.toString();
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * Sends a request by post method and returns response in document format.
	 * 
	 * @param param
	 *            content to send
	 * @param input_url
	 *            service address.
	 * 
	 */
	public static Document POSTDoc(String param, String input_url) {

		try {
			URL url = new URL(input_url);
			HttpURLConnection con = (HttpURLConnection) url.openConnection();
			con.setDoOutput(true);
			con.setRequestMethod("POST");
			con.setRequestProperty("Content-Type", "application/xml");
			con.setDoOutput(true);
			con.setDoInput(true);
			con.setUseCaches(false);

			PrintWriter xmlOut = new PrintWriter(con.getOutputStream());
			xmlOut.write(param);
			xmlOut.flush();
			SAXReader reader = new SAXReader();
			Document doc = reader.read(con.getInputStream());
			return doc;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * Sends a request by get method and returns response in document format.
	 * 
	 * @param param
	 *            content to send.
	 * @param url
	 *            service address.
	 * 
	 */
	public static Document GETDoc(String url, String param) {
		String realUrl = null;
		if (url.endsWith("?")) {
			realUrl = url + param;
		} else {
			realUrl = url + "?" + param;
		}
		return GetDoc(realUrl);
	}

	/**
	 * Sends a request by post method and returns response in string format.
	 * 
	 * @param url
	 *            service address and parameters.
	 * 
	 */
	public static Document GetDoc(String url) {
		try {
			URL realUrl = new URL(url);
			URLConnection conn = realUrl.openConnection();
			conn.setRequestProperty("accept", "*/*");
			conn.setRequestProperty("connection", "Keep-Alive");
			conn.setRequestProperty("user-agent",
					"Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)");
			SAXReader reader = new SAXReader();
			Document document = reader.read(conn.getInputStream());
			return document;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * Sends a request by get method and returns response in string format.
	 * 
	 * @param url
	 *            service address containing parameters.
	 * 
	 */
	public static String GETStr(String url) {

		try {

			URL realUrl = new URL(url);
			URLConnection conn = realUrl.openConnection();
			conn.setRequestProperty("accept", "text/xml");
			conn.setRequestProperty("connection", "Keep-Alive");
			conn.setRequestProperty("user-agent",
					"Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)");

			BufferedReader response = new BufferedReader(new InputStreamReader(
					conn.getInputStream()));
			StringBuffer result = new StringBuffer();
			String line;
			while ((line = response.readLine()) != null) {
				result.append(line.trim() + "\n");
			}
			return result.toString();
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}

	}

	/**
	 * Uploads file.
	 * @param fname
	 * 			the path of file.
	 * @return
	 */
	public static String uploadFile(String fname) {
		try {
			String BOUNDARY = "---------7d4a6d158c9"; 
			String urlStr = SensorWebProperty.getInstance().getServiceProperty().getProperty("upload");
			URL url = new URL(urlStr);
			HttpURLConnection conn = (HttpURLConnection) url.openConnection();
			conn.setDoOutput(true);
			conn.setDoInput(true);
			conn.setUseCaches(false);
			conn.setRequestMethod("POST");
			conn.setRequestProperty("connection", "Keep-Alive");
			conn.setRequestProperty("user-agent",
					"Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)");
			conn.setRequestProperty("Charsert", "UTF-8");
			conn.setRequestProperty("Content-Type",
					"multipart/form-data; boundary=" + BOUNDARY);

			OutputStream out = new DataOutputStream(conn.getOutputStream());
			byte[] end_data = ("\r\n--" + BOUNDARY + "--\r\n").getBytes();
			File file = new File(fname);
			StringBuilder sb = new StringBuilder();
			sb.append("--");
			sb.append(BOUNDARY);
			sb.append("\r\n");
			sb.append("Content-Disposition: form-data;name=\"file" + 0
					+ "\";filename=\"" + file.getName() + "\"\r\n");
			sb.append("Content-Type:application/octet-stream\r\n\r\n");

			byte[] data = sb.toString().getBytes();
			out.write(data);
			DataInputStream in = new DataInputStream(new FileInputStream(file));
			int bytes = 0;
			byte[] bufferOut = new byte[1024];
			while ((bytes = in.read(bufferOut)) != -1) {
				out.write(bufferOut, 0, bytes);
			}
			out.write("\r\n".getBytes()); 
			in.close();

			out.write(end_data);
			out.flush();
			out.close();

			BufferedReader reader = new BufferedReader(new InputStreamReader(
					conn.getInputStream()));
			String line = null;
			String content="";
			while ((line = reader.readLine()) != null) {
				content+=line;
			}
			String[] results = content.split(";");
			if (results.length<2) {
				return null;
			}
			boolean flag = Boolean.parseBoolean(results[0]);
			if (flag) {
				return results[1];
			}else {
				return null;
			}
		} catch (Exception e) {
			System.out.println("Error to send post request." + e);
			e.printStackTrace();
		}
		return null;
	}
	
	public static void main(String[] args){
		String filePath = "D:/loc1.xml";
		String url = uploadFile(filePath);
		System.out.println(url);
	}
}
