/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.service.utils;

import java.io.File;
import java.io.IOException;

import org.apache.log4j.Logger;
import org.dom4j.Document;

import cn.edu.whu.lmars.sensorweb.sos.adapter.SOSAdapter;
import cn.edu.whu.lmars.service.adapter.AdapterFactory;
import cn.edu.whu.lmars.service.adapter.base.IAdapter;
import cn.edu.whu.lmars.service.adapter.base.IServiceInfo;
import cn.edu.whu.lmars.service.beans.IService;
import cn.edu.whu.lmars.utils.BaseTool;
import cn.edu.whu.lmars.utils.Dom4jXMLTool;
import cn.edu.whu.lmars.utils.FileTool;
import conf.PathConf;

/**
 * This thread manages cache file of service. When user adds a new service to
 * the application, this thread will start.
 * 
 * @author Mingda Zhang
 * 
 */
public class ServiceCacheThread extends Thread {
	/** Service information. */
	private IService service;
	private boolean refresh;
	private String wsPath;
//	private Map<String,String> nsMap = null;
	/** logger. */
	private Logger logger = Logger.getLogger(ServiceCacheThread.class);

	public ServiceCacheThread(IService service, boolean refresh) {
		this.service = service;
		this.refresh = refresh;
	}

	/**
	 * Starts the thread. Begins to down load files related to the service.
	 */
	@Override
	public void run() {
		/*if (BaseTool.isNull(service.getId())) {
			logger.error("There is no service id, service cache thread terminates");
			return;
		}*/
		if (BaseTool.isNull(wsPath)) {
			wsPath = PathConf.getDataDir()+File.separator+PathConf.Cache_Name+File.separator+FileTool.getUniqueDirFromService(service);
		}
		 /*wsPath = PathConf.getDataDir() + File.separator
				+ service.getId();*/
		File dir = new File(wsPath);
		boolean flag = true;
		if (!dir.exists()) {
			flag = dir.mkdirs();
		}
		if (!flag) {
			logger.error("Error occured when creating folders.Service cache thread terminates");
		}

		IAdapter adapter = AdapterFactory.getInstance().getAdapter(service);
		if (adapter == null) {
			logger.error("Error occured when getting the right adapter");
			return;
		}
		IServiceInfo serviceInfo = adapter.getServiceInfo();
		if (serviceInfo == null) {
			serviceInfo = adapter.parseService();
			if (serviceInfo == null) {
				logger.error("Error occured when parse service");
				return;
			}
		}

		String capPath = wsPath + File.separator + "GetCapabilities.xml";
		File capFile = new File(capPath);
		flag = true;
		Document doc;
		if (!capFile.exists()) {
			try {
				flag = capFile.createNewFile();
			} catch (IOException e) {
				e.printStackTrace();
				logger.error("Error occured when create GetCapabilities.xml file");
			}
			if (flag) {
				doc = RequestMethodStore.GetDoc(adapter.getAddress()
						+ adapter.getCapParameter());
				if (doc != null) {
					Dom4jXMLTool.writeXml(doc, capFile);
				}
			}

			// if you want to refresh the cache.
		}else {
			if (this.refresh) {
				doc = RequestMethodStore.GetDoc(adapter.getAddress()
						+ adapter.getCapParameter());
				if (doc != null) {
					Dom4jXMLTool.writeXml(doc, capFile);
				}
			}
		}

		/*if (service.getServiceType() != ServiceType.WPS) {
			return;
		}*/
		String fileName;
		Document contentDoc;
		for (String operationName : serviceInfo.getOperationList()) {
			
			switch (service.getServiceType()) {
			case WPS:
				fileName = operationName;
				contentDoc =  RequestMethodStore.GetDoc(serviceInfo
						.getDescribeURI()
						+ adapter.getDesParameter(operationName));
				break;
			case SOS:
				fileName = operationName.replace(":","_");
				contentDoc = ((SOSAdapter)adapter).getDescribeDocFromUrl(operationName);
				break;
			default:
				return;
			}
			File operaFile = new File(wsPath + File.separator + fileName
					+ ".xml");
			flag = true;
			if (!operaFile.exists()) {
				try {
					flag = operaFile.createNewFile();
				} catch (IOException e) {
					flag = false;
					e.printStackTrace();
				}
				if (flag) {
					/*doc = RequestMethodStore.GetDoc(serviceInfo
							.getDescribeURI()
							+ adapter.getDesParameter(operationName));*/
					if (contentDoc != null) {
						Dom4jXMLTool.writeXml(contentDoc, operaFile);
					}
				}
			}else {
				if (this.refresh) {
					if (contentDoc != null) {
						Dom4jXMLTool.writeXml(contentDoc, operaFile);
					}
				}
			}
		}
	}

}
