/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.service.wfs;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Namespace;
import org.dom4j.Node;
import org.dom4j.QName;
import org.dom4j.XPath;
import org.dom4j.io.SAXReader;

import cn.edu.whu.lmars.service.adapter.base.IAdapter;
import cn.edu.whu.lmars.service.adapter.base.IOperationInfo;
import cn.edu.whu.lmars.service.adapter.base.IParameterInfo;
import cn.edu.whu.lmars.service.adapter.base.IServiceInfo;
import cn.edu.whu.lmars.service.adapter.base.impl.OperationInfoImpl;
import cn.edu.whu.lmars.service.adapter.base.impl.ParameterInfoImpl;
import cn.edu.whu.lmars.service.adapter.base.impl.ServiceInfoImpl;
import cn.edu.whu.lmars.service.beans.IService;
import cn.edu.whu.lmars.service.meta.ParameterMeta;
import cn.edu.whu.lmars.service.meta.ServiceMeta;
import cn.edu.whu.lmars.service.meta.ServiceProvider;
import cn.edu.whu.lmars.service.meta.ServiceProvider.ServiceContact;
import cn.edu.whu.lmars.service.meta.ServiceProvider.ServiceContact.Address;
import cn.edu.whu.lmars.service.utils.RequestMethodStore;
import cn.edu.whu.lmars.service.wfs.meta.WFSDataMeta;
import cn.edu.whu.lmars.utils.BaseTool;
import cn.edu.whu.lmars.utils.FileTool;
import conf.PathConf;

/**
 * WFS adapter for version 1.1.0. This class parses WFS data from the service.
 * 
 * @author Mingda Zhang
 * 
 */

public class WFSAdapterData implements IAdapter {

	private String address;
	private String outputformat = "";
	private String version;
	private String request_get;
	private IService service;
	private String cacheDir;
	private boolean fresh = false;
	private IServiceInfo serviceInfo = null;
	private final Logger log = Logger.getLogger(this.getClass());
	private final Namespace ows = new Namespace("ows",
			"http://www.opengis.net/ows"), wfs = new Namespace("wfs",
			"http://www.opengis.net/wfs"), xlink = new Namespace("xlink",
			"http://www.w3.org/1999/xlink");
	private final QName operationMetaq = new QName("OperationsMetadata", ows),
			hrefq = new QName("href", xlink), serviceIdenQ = new QName(
					"ServiceIdentification", ows), titleQ = new QName("Title",
					ows), abstractQ = new QName("Abstract", ows),
			keywordsQ = new QName("Keywords", ows), keywordQ = new QName(
					"Keyword", ows), typeQ = new QName("ServiceType", ows),
			versionQ = new QName("ServiceTypeVersion", ows),
			onlineResourceQ = new QName("OnlineResource", ows),
			providerQ = new QName("ServiceProvider", ows),
			providerNameQ = new QName("ProviderName", ows);

	public WFSAdapterData(IService service,boolean fresh) {
		this.service = service;
		this.address = service.getUrl();
		this.version = service.getVersion();
		this.fresh = fresh;
	}

	@SuppressWarnings("unchecked")
	@Override
	public IServiceInfo parseService() {
		if (serviceInfo != null) {
			return serviceInfo;
		}
		Document doc = null;
		
		if (fresh) {
			doc = RequestMethodStore.GetDoc(getAddress() + getCapParameter());
			if (doc == null) {
				return null;
			}
		}else {
			boolean flag = true;
			if (BaseTool.isNull(cacheDir)) {
				cacheDir = PathConf.getDataDir() + File.separator
						+ PathConf.Cache_Name + File.separator
						+ FileTool.getUniqueDirFromService(service);
			}
			File cacheFile = new File(cacheDir);
			if (!cacheFile.exists()) {
				flag = cacheFile.mkdirs();
			}
			String wsPath = cacheDir + File.separator + "GetCapabilities.xml";
			File wsFile = new File(wsPath);
				if (flag && wsFile.exists()) {
					SAXReader reader = new SAXReader();
					try {
						doc = reader.read(wsFile);
					} catch (DocumentException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
						doc = null;
					}
				}
			
			if (doc == null) {
				doc = RequestMethodStore.GetDoc(getAddress() + getCapParameter());
				if (doc == null) {
					return null;
				}

			}
		}
		
		Element capele = doc.getRootElement();
		if (capele == null) {
			log.error("errors occured when get the capabilities of "
					+ getAddress());
			log.error("the request url is " + getAddress() + getCapParameter());
			return null;
		}

		serviceInfo = new ServiceInfoImpl();

		Element omdele = capele.element(operationMetaq);
		Map<String, String> map = new HashMap<String, String>();
		map.put("ows", ows.getURI());
		map.put("wfs", wfs.getURI());

		XPath xpath = DocumentHelper
				.createXPath("ows:Operation[@name='DescribeFeatureType']");
		xpath.setNamespaceURIs(map);

		Element dfele = (Element) xpath.selectSingleNode(omdele);
		XPath xPath2 = DocumentHelper.createXPath("ows:DCP/ows:HTTP/ows:Get");
		xPath2.setNamespaceURIs(map);
		Element dfuriele = (Element) xPath2.selectSingleNode(dfele);
		String dfuriStr = dfuriele.attributeValue(hrefq);
		serviceInfo.setDescribeURI(dfuriStr);
		Element gfele = (Element) omdele
				.selectSingleNode("ows:Operation[@name='GetFeature']");
		Element gfuriele = (Element) xPath2.selectSingleNode(gfele);
		String gfuriStr = gfuriele.attributeValue(hrefq);
		serviceInfo.setExcuteURI(gfuriStr);
		ServiceMeta serviceMeta = new ServiceMeta();
		Element serviceEle = capele.element(serviceIdenQ);
		Element titleEle = serviceEle.element(titleQ);
		if (!BaseTool.isNull(titleEle)) {
			serviceMeta.set_title(titleEle.getTextTrim());
		}
		Element abstractEle = serviceEle.element(abstractQ);
		if (!BaseTool.isNull(abstractEle)) {
			serviceMeta.set_abstract(abstractEle.getTextTrim());
		}
		Element typeEle = serviceEle.element(typeQ);
		if (!BaseTool.isNull(typeEle)) {
			serviceMeta.set_type(typeEle.getTextTrim());
		}
		Element versionEle = serviceEle.element(versionQ);
		if (!BaseTool.isNull(versionEle)) {
			serviceMeta.set_version(versionEle.getTextTrim());
		}
		Element onlineEle = serviceEle.element(onlineResourceQ);
		if (!BaseTool.isNull(onlineEle)) {
			serviceMeta.set_onlineresource(onlineEle.getTextTrim());
		}
		Element keywordsEle = serviceEle.element(keywordsQ);
		List<Element> kweles = keywordsEle.elements(keywordQ);
		if (kweles.size() > 0) {
			StringBuffer ks = new StringBuffer();
			ks.append(kweles.get(0).getTextTrim());
			for (int i = 1, n = kweles.size(); i < n; i++) {
				ks.append(" ").append(kweles.get(i).getTextTrim());
			}
			serviceMeta.set_keywords(ks.toString());
		}
		// get the service provider information
		ServiceProvider provider = new ServiceProvider();
		Element pdEle = capele.element(providerQ);
		Element pdNameEle = pdEle.element(providerNameQ);
		provider.setProvidername(pdNameEle.getTextTrim());
		ServiceContact sc = provider.new ServiceContact();
		XPath xPath3 = DocumentHelper
				.createXPath("ows:ServiceContact/ows:IndividualName");
		xPath3.setNamespaceURIs(map);
		Node indName = xPath3.selectSingleNode(pdEle);
		if (!BaseTool.isNull(indName)) {
			sc.setIndividualname(((Element) indName).getTextTrim());
		}
		XPath xPath4 = DocumentHelper
				.createXPath("ows:ServiceContact/ows:PositionName");
		xPath4.setNamespaceURIs(map);
		Node posName = xPath4.selectSingleNode(pdEle);
		if (!BaseTool.isNull(posName)) {
			sc.setPositionname(((Element) posName).getTextTrim());
		}

		Address address = sc.new Address();
		XPath xPath5 = DocumentHelper
				.createXPath("ows:ServiceContact/ows:ContactInfo/ows:Address/ows:City");
		xPath5.setNamespaceURIs(map);
		Node node = xPath5.selectSingleNode(pdEle);
		if (!BaseTool.isNull(node)) {
			address.set_city(((Element) node).getTextTrim());
		}

		XPath xPath6 = DocumentHelper
				.createXPath("ows:ServiceContact/ows:ContactInfo/ows:Address/ows:Country");
		xPath5.setNamespaceURIs(map);
		Node node1 = xPath6.selectSingleNode(pdEle);
		if (!BaseTool.isNull(node1)) {
			address.set_country(((Element) node1).getTextTrim());
		}
		sc.setAd(address);
		provider.setSc(sc);
		serviceMeta.setSp(provider);
		serviceInfo.SetServiceMeta(serviceMeta);
		// si.setServices(services);

		// gets all data offered by this service
		// List operationList = new ArrayList();
		XPath xPath7 = DocumentHelper
				.createXPath("wfs:FeatureTypeList/wfs:FeatureType");
		xPath7.setNamespaceURIs(map);
		List<Element> nodes = xPath7.selectNodes(capele);
		XPath nameXPath = DocumentHelper.createXPath("wfs:Name");
		nameXPath.setNamespaceURIs(map);

		XPath titleXPath = DocumentHelper.createXPath("wfs:Title");
		titleXPath.setNamespaceURIs(map);

		XPath abstractXPath = DocumentHelper.createXPath("wfs:Abstract");
		abstractXPath.setNamespaceURIs(map);

		XPath keywordXPath = DocumentHelper
				.createXPath("ows:Keywords/ows:Keyword");
		keywordXPath.setNamespaceURIs(map);

		XPath defaultSRSXPath = DocumentHelper.createXPath("wfs:DefaultSRS");
		defaultSRSXPath.setNamespaceURIs(map);

		XPath lowerCornerXPath = DocumentHelper
				.createXPath("ows:WGS84BoundingBox/ows:LowerCorner");
		lowerCornerXPath.setNamespaceURIs(map);

		XPath upCornerXPath = DocumentHelper
				.createXPath("ows:WGS84BoundingBox/ows:UpperCorner");
		upCornerXPath.setNamespaceURIs(map);

		String opeName = "";
		List<String> operationList2 = new ArrayList<String>();

		for (Element node3 : nodes) {
			OperationInfoImpl operationInfo = new OperationInfoImpl();
			WFSDataMeta wfsDataMeta = new WFSDataMeta();
			List<Namespace> spacesList = node3.declaredNamespaces();
			Node nameEle = nameXPath.selectSingleNode(node3);
			if (!BaseTool.isNull(nameEle)) {
				opeName = ((Element) nameEle).getTextTrim();
				operationInfo.setName(opeName);
				operationList2.add(opeName);
			}
			for (Namespace namespace : spacesList) {
				if (opeName.startsWith(namespace.getPrefix())) {
					wfsDataMeta.setWorkspace(namespace.getURI());
					break;
				}
			}
			Node titleNode = titleXPath.selectSingleNode(node3);
			if (!BaseTool.isNull(titleNode)) {
				wfsDataMeta.set_title(titleNode.getText());
			}

			Node abstractNode = abstractXPath.selectSingleNode(node3);
			if (!BaseTool.isNull(abstractNode)) {
				wfsDataMeta.set_abstract(abstractNode.getText());
			}

			List<Node> keywordNode = keywordXPath.selectNodes(node3);
			String keyWord = "";
			for (Node keynode : keywordNode) {
				keyWord += keynode.getText() + ",";
			}
			wfsDataMeta.setKeyWord(keyWord);

			Node crsNode = defaultSRSXPath.selectSingleNode(node3);
			if (!BaseTool.isNull(crsNode)) {
				wfsDataMeta.setDefaultSRS(crsNode.getText());
			}

			String[] corners = new String[4];

			Node lowerNode = lowerCornerXPath.selectSingleNode(node3);
			if (!BaseTool.isNull(lowerNode)) {
				// String lowerStr = lowerNode.getText();
				String[] lowers = lowerNode.getText().split(" ");
				corners[0] = lowers[0];
				corners[1] = lowers[1];
			}

			Node upNode = upCornerXPath.selectSingleNode(node3);
			if (!BaseTool.isNull(upNode)) {
				String[] ups = upNode.getText().split(" ");
				corners[2] = ups[0];
				corners[3] = ups[1];
			}
			wfsDataMeta.setBoudingBox(corners);

			operationInfo.setMeta(wfsDataMeta);

			List<IParameterInfo> inputParameters = new ArrayList<IParameterInfo>();
			List<IParameterInfo> outputParameters = new ArrayList<IParameterInfo>();

			ParameterInfoImpl parameter1 = new ParameterInfoImpl();
			parameter1.setName("outputFormat");
			ParameterMeta inMeta = new ParameterMeta();
			inMeta.set_type("xs:string");
			parameter1.setMeta(inMeta);
			inputParameters.add(parameter1);

			ParameterInfoImpl outParameter = new ParameterInfoImpl();
			outParameter.setName("OutPutData");
			ParameterMeta outMeta = new ParameterMeta();
			outMeta.set_type("ComplexData");
			outParameter.setMeta(outMeta);
			outputParameters.add(outParameter);
			operationInfo.setInputParameters(inputParameters);

			operationInfo.setOutputParameters(outputParameters);
			serviceInfo.addOperationInfo(operationInfo);
		}

		serviceInfo.setOperationList(operationList2);
		return serviceInfo;
	}

	private String getParameterValueBySimpleKey(String simplekey,
			Map<String, String> parameternameandvalue) {
		@SuppressWarnings("rawtypes")
		Iterator it = parameternameandvalue.keySet().iterator();
		String value = null;
		while (it.hasNext()) {
			String key = (String) it.next();
			// the old is endwith
			if (key.toLowerCase().endsWith(simplekey.toLowerCase())) {
				value = parameternameandvalue.get(key);
				break;
			}
		}
		return value;
	}

	@Override
	public String createRequest(String operationname,
			Map<String, String> parameternameandvalue) {
		request_get = "";
		outputformat = getParameterValueBySimpleKey("outputformat",
				parameternameandvalue);
		String format = "";
		if (BaseTool.isNull(outputformat)) {
			format = "SHAPE-ZIP";
			outputformat = "ESRI Shapefile";
		} else if (outputformat.equals("ESRI Shapefile")) {
			format = "SHAPE-ZIP";
		} else if (outputformat.equalsIgnoreCase("gml")) {
			format = "GML2";
		} else {
			outputformat = "ESRI Shapefile";
			format = "SHAPE-ZIP";
		}
		request_get += serviceInfo.getExecuteURI()
				+ "?request=GetFeature&version=" + this.version;
		request_get += "&outputFormat=" + format + "&typeName=" + operationname;
		return request_get;
	}

	@Override
	public Map<String, String> parseResponse(String response) {
		Map<String, String> respmap = new HashMap<String, String>();
		// respmap.put("FeatureCollection", response);
		respmap.put("result_datatype", outputformat);
		respmap.put("result_value", request_get);
		return respmap;
	}

	@Override
	public String sendRequest(String request) {
		return "request";
	}

	@Override
	public String getAddress() {
		return address;
	}

	@Override
	public String getCapParameter() {
		return "?service=WFS&request=GetCapabilities&version=" + this.version;
	}

	@Override
	public IOperationInfo getOperationInfo(String operationName) {
		IOperationInfo operationInfo = serviceInfo
				.getOperationInfo(operationName);
		return operationInfo;
	}

	@Override
	public IServiceInfo getServiceInfo() {
		return serviceInfo;
	}

	public String getVersion() {
		return version;
	}

	public void setVersion(String version) {
		this.version = version;
	}

	@Override
	public String getDesParameter(String operationName) {
		return null;
	}

}
