/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.service.wps;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Namespace;
import org.dom4j.QName;
import org.dom4j.XPath;
import org.dom4j.io.SAXReader;

import cn.edu.whu.lmars.service.adapter.base.IAdapter;
import cn.edu.whu.lmars.service.adapter.base.IOperationInfo;
import cn.edu.whu.lmars.service.adapter.base.IParameterInfo;
import cn.edu.whu.lmars.service.adapter.base.IServiceInfo;
import cn.edu.whu.lmars.service.adapter.base.impl.OperationInfoImpl;
import cn.edu.whu.lmars.service.adapter.base.impl.ParameterInfoImpl;
import cn.edu.whu.lmars.service.adapter.base.impl.ServiceInfoImpl;
import cn.edu.whu.lmars.service.beans.IService;
import cn.edu.whu.lmars.service.meta.OperationMeta;
import cn.edu.whu.lmars.service.meta.ParameterMeta;
import cn.edu.whu.lmars.service.meta.ServiceMeta;
import cn.edu.whu.lmars.service.meta.ServiceProvider;
import cn.edu.whu.lmars.service.meta.ServiceProvider.ServiceContact;
import cn.edu.whu.lmars.service.meta.ServiceProvider.ServiceContact.Address;
import cn.edu.whu.lmars.service.utils.RequestMethodStore;
import cn.edu.whu.lmars.utils.BaseTool;
import cn.edu.whu.lmars.utils.Dom4jXMLTool;
import cn.edu.whu.lmars.utils.FileTool;
import conf.PathConf;

/**
 * Web Processing Service adapter for version 0.4.0.
 * 
 * @author Mingda Zhang
 * 
 */

public class WPSAdapter040 implements IAdapter {
	private IServiceInfo serviceInfo = null;
	private IService service;
	private Logger log = Logger.getLogger(this.getClass());
	private String cacheDir = null;
	private boolean fresh = false;
	private Namespace ows = new Namespace("ows",
			"http://www.opengeospatial.net/ows"), outputows = new Namespace(
			"ows2", "http://www.opengis.net/ows"), xmlns = new Namespace(
			"xmlns", "http://www.opengeospatial.net/wps"),
			xlink = new Namespace("xlink", "http://www.w3.org/1999/xlink"),
			ws = new Namespace("ioxsd", "http://www.lmars.whu.edu.cn/ws"),
			wps = new Namespace("wps", "http://www.opengeospatial.net/wps");
	private QName processofferingsq = new QName("ProcessOfferings", xmlns),
			processq = new QName("Process", xmlns), identifierq = new QName(
					"Identifier", ows), serviceidentificationq = new QName(
					"ServiceIdentification", ows), titleq = new QName("Title",
					ows), abstractq = new QName("Abstract", ows),
			keywordsq = new QName("Keywords", ows), keywordq = new QName(
					"Keyword", ows), onlineresourceq = new QName(
					"OnlineResource", ows), serviceproviderq = new QName(
					"ServiceProvider", ows), providernameq = new QName(
					"ProviderName", ows), servicecontactq = new QName(
					"ServiceContact", ows), contactinfoq = new QName(
					"ContactInfo", ows), addressq = new QName("Address", ows),
			cityq = new QName("City", ows),
			countryq = new QName("Country", ows),
			processdescriptionq = new QName("ProcessDescription", wps),
			datainputsq = new QName("DataInputs", wps),
			processoutsq = new QName("ProcessOutputs", wps),
			outputdefinitionsq = new QName("OutputDefinitions", wps),
			outputq = new QName("Output", wps), complexoutputq = new QName(
					"ComplexOutput", wps), inputq = new QName("Input", wps),
			minimumq = new QName("MinimumOccurs", wps),
			complexdataq = new QName("ComplexData", wps),
			literaldataq = new QName("LiteralData", wps),
			datatypeq = new QName("DataType", ows), literalvalueq = new QName(
					"LiteralValue", wps), complexdatavalueq = new QName(
					"ComplexDataValue", ws), complexvalueq = new QName(
					"ComplexValue", wps), referenceq = new QName("reference",
					ows), executereferenceq = new QName("Reference", ws),
			formatq = new QName("Format", ows), outputformatq = new QName(
					"Format", outputows), hrefq = new QName("href", xlink),
			maxinumq = new QName("MaximumOccurs", wps),
			serviceTypeq = new QName("ServiceType", ows),
			serviceVersionq = new QName("ServiceTypeVersion", ows);
	Map<String, String> map = new HashMap<String, String>();

	public WPSAdapter040(IService service, boolean fresh) {
		this.service = service;
		this.fresh = fresh;
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	@Override
	public IServiceInfo parseService() {
		if (serviceInfo != null) {
			return serviceInfo;
		}
		Document doc = null;
		/*
		 * String wsPath = PathConf.getDataDir()+File.separator +
		 * service.getId() + File.separator + "GetCapabilities.xml";
		 */
		if (fresh) {
			doc = RequestMethodStore.GetDoc(getAddress() + getCapParameter());
			if (doc == null) {
				return null;
			}
		} else {
			boolean flag = true;
			if (BaseTool.isNull(cacheDir)) {
				cacheDir = PathConf.getDataDir() + File.separator
						+ PathConf.Cache_Name + File.separator
						+ FileTool.getUniqueDirFromService(service);
			}
			File cacheFile = new File(cacheDir);
			if (!cacheFile.exists()) {
				flag = cacheFile.mkdirs();
			}
			String wsPath = cacheDir + File.separator + "GetCapabilities.xml";
			File wsFile = new File(wsPath);
			if (flag && wsFile.exists()) {
				SAXReader reader = new SAXReader();
				try {
					doc = reader.read(wsFile);
				} catch (DocumentException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					doc = null;
				}
			}

			if (doc == null) {
				doc = RequestMethodStore.GetDoc(getAddress()
						+ getCapParameter());
				if (doc == null) {
					return null;
				}

			}
		}

		Element capele = doc.getRootElement();
		if (capele == null) {
			log.error("errors occured when get the capabilities of "
					+ getAddress());
			log.error("the request url is " + getAddress() + getCapParameter());
			return null;
		}
		map.put("ows", ows.getURI());
		map.put("wps", wps.getURI());
		serviceInfo = new ServiceInfoImpl();
		String str;

		// description url
		XPath xPath = DocumentHelper
				.createXPath("ows:OperationsMetadata/ows:Operation[@name='DescribeProcess']/ows:DCP/ows:HTTP/ows:Post");
		xPath.setNamespaceURIs(map);
		Element element = (Element) xPath.selectSingleNode(capele);
		if (element == null) {
			log.error("errors occured,when parse the description adrress");
		} else {
			str = element.attributeValue(hrefq);
			serviceInfo.setDescribeURI(str);
		}

		// execution url
		xPath = DocumentHelper
				.createXPath("ows:OperationsMetadata/ows:Operation[@name='Execute']/ows:DCP/ows:HTTP/ows:Post");
		xPath.setNamespaceURIs(map);
		element = (Element) xPath.selectSingleNode(capele);
		if (element == null) {
			log.error("errors occured,when parse the execute adrress");
		} else {
			str = element.attributeValue(hrefq);
			serviceInfo.setExcuteURI(str);
		}

		// Get service metadata.
		ServiceMeta serviceMeta = new ServiceMeta();
		Element serviceidentificationele = capele
				.element(serviceidentificationq);
		Element titleele = serviceidentificationele.element(titleq);
		serviceMeta.set_title(titleele.getTextTrim());
		// services.add(smd.get_title());
		Element absele = serviceidentificationele.element(abstractq);
		serviceMeta.set_abstract(absele.getTextTrim());
		Element serviceTypeele = serviceidentificationele.element(serviceTypeq);
		if (serviceTypeele != null) {
			serviceMeta.set_type(serviceTypeele.getTextTrim());
		}
		Element serviceVersionEle = serviceidentificationele
				.element(serviceVersionq);
		if (serviceVersionEle != null) {
			serviceMeta.set_version(serviceVersionEle.getTextTrim());
		}
		Element kwsele = serviceidentificationele.element(keywordsq);
		List<Element> kwele = kwsele.elements(keywordq);
		if (kwele.size() > 0) {
			StringBuffer ks = new StringBuffer();
			ks.append(kwele.get(0).getTextTrim());
			for (int i = 1, n = kwele.size(); i < n; i++) {
				ks.append(";").append(kwele.get(i).getTextTrim());
			}
			serviceMeta.set_keywords(ks.toString());
		}
		Element orele = serviceidentificationele.element(onlineresourceq);
		if (!BaseTool.isNull(orele)) {
			serviceMeta.set_onlineresource(orele.getTextTrim());
		}
		Element spele = capele.element(serviceproviderq);
		ServiceProvider sp = new ServiceProvider();
		Element providername = spele.element(providernameq);
		sp.setProvidername(providername.getTextTrim());
		Element scele = spele.element(servicecontactq);
		Element ciele = scele.element(contactinfoq);
		Element adrele = ciele.element(addressq);
		Element cityele = adrele.element(cityq);
		Element countryele = adrele.element(countryq);
		ServiceContact sct = sp.new ServiceContact();
		Address ad = sct.new Address();
		ad.set_city(cityele.getTextTrim());
		ad.set_country(countryele.getTextTrim());
		sct.setAd(ad);
		sp.setSc(sct);
		serviceMeta.setSp(sp);
		serviceInfo.SetServiceMeta(serviceMeta);

		// Get operation list.
		Element processofferingsele = capele.element(processofferingsq);
		Iterator processit = processofferingsele.elementIterator(processq);
		List<String> operationlList = new ArrayList<String>();
		String proname;
		while (processit.hasNext()) {
			Element proele = (Element) processit.next();
			Element identierele = proele.element(identifierq);
			proname = identierele.getTextTrim();
			operationlList.add(proname);
		}
		serviceInfo.setOperationList(operationlList);
		return serviceInfo;
	}

	@Override
	public String createRequest(String operationname,
			Map<String, String> parameternameandvalue) {
		IOperationInfo operationInfo = getOperationInfo(operationname);
		if (operationInfo == null) {
			return null;
		}
		Document doc = FileTool
				.getDocFromResource(PathConf.WPS_Execute_Template);
		/*
		 * Element executeele = Dom4jXMLTool .readElementFromFile(new
		 * StringBuffer().append(PathConf.BASE)
		 * .append("datas/").append("ExecuteRequestTemplate.xml") .toString());
		 */
		Element executeele = doc.getRootElement();
		Element processidentifierele = executeele.element(identifierq);
		processidentifierele.setText(operationname);
		Element datainputsele = executeele.element(datainputsq);
		Element dataoutputsele = executeele.element(outputdefinitionsq);

		List<IParameterInfo> inputParameters = operationInfo
				.getInputParameterInfos();
		for (int j = 0, m = inputParameters.size(); j < m; j++) {
			IParameterInfo pi = inputParameters.get(j);
			ParameterMeta meta = pi.getMeta();
			Element inputele = datainputsele.addElement(inputq);
			Element inputnameele = inputele.addElement(identifierq);
			inputnameele.setText(pi.getName());
			if (meta.get_type().equals("ComplexData")) {
				Element complexvalueele = inputele.addElement("ComplexValue");
				complexvalueele.addAttribute("schema",
						"http://www.lmars.whu.edu.cn/ws/iodata.xsd");
				Element complexdatavalueele = complexvalueele
						.addElement(complexdatavalueq);
				Element referele = complexdatavalueele
						.addElement(executereferenceq);
				referele.addAttribute(hrefq,
						getValueByName(pi.getName(), parameternameandvalue));
				Element formatele = referele.addElement(formatq);
				String type = getComplexDataTypeByName(pi.getName(),
						parameternameandvalue);
				if (BaseTool.isNull(type)) {
					log.error("the paramter " + pi.getName() + "_datatype"
							+ " missed");
				} else {
					formatele.setText(type);
				}
			} else {
				Element literalvalueele = inputele.addElement(literalvalueq);
				literalvalueele.addAttribute("dataType", meta.get_type());
				literalvalueele.setText(getValueByName(pi.getName(),
						parameternameandvalue));
			}
		}

		List<IParameterInfo> outputPrarameters = operationInfo
				.getOutputParameters();
		IParameterInfo outputParameter;
		for (int i = 0; i < outputPrarameters.size(); i++) {
			outputParameter = outputPrarameters.get(i);
			Element outputele = dataoutputsele.addElement(outputq);
			Element outputnameele = outputele.addElement(identifierq);
			outputnameele.setText(outputParameter.getName());
		}
		log.info(executeele.asXML().replaceAll("&amp;", "&"));
		return executeele.asXML().replaceAll("&amp;", "&");
	}

	@Override
	public String sendRequest(String request) {
		if (serviceInfo == null) {
			serviceInfo = parseService();
		}
		return RequestMethodStore.POST(request, serviceInfo.getExecuteURI());
	}

	@SuppressWarnings("rawtypes")
	@Override
	public Map<String, String> parseResponse(String response) {
		Element respele = Dom4jXMLTool.readElementFromString(response.trim());
		Element processoutputsele = respele.element(processoutsq);
		if (processoutputsele == null) {
			JOptionPane.showMessageDialog(null,
					"Error occured:" + respele.asXML());
			return null;
		}
		Iterator outputit = processoutputsele.elementIterator(outputq);
		Map<String, String> respMap = new HashMap<String, String>();
		while (outputit.hasNext()) {
			Element outputele = (Element) outputit.next();
			Element complexvalueele = outputele.element(complexvalueq);
			if (!BaseTool.isNull(complexvalueele)) {
				Element complexdatavalueele = complexvalueele
						.element(complexdatavalueq);
				Element referele = complexdatavalueele
						.element(executereferenceq);
				String resulturi = referele.attributeValue(hrefq);
				Element formatele = referele.element(outputformatq);
				String format = formatele.getTextTrim();
				respMap.put("result_value", resulturi);
				respMap.put("result_datatype", format);
			}
		}
		return respMap;
	}

	@Override
	public String getAddress() {
		return service.getUrl();
	}

	@Override
	public IServiceInfo getServiceInfo() {
		return serviceInfo;
	}

	@Override
	public String getCapParameter() {
		return "?Request=GetCapabilities&Service=WPS";
	}

	@Override
	public String getDesParameter(String operationName) {
		return "?Request=DescribeProcess&Service=WPS&Version=0.4.0&Language=en&Identifier="
				+ operationName;
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	@Override
	public IOperationInfo getOperationInfo(String operationName) {
		if (serviceInfo == null) {
			if (parseService() == null) {
				return null;
			}
		}
		IOperationInfo operationInfo = serviceInfo
				.getOperationInfo(operationName);
		if (operationInfo != null) {
			return operationInfo;
		}
		Document doc = null;
		if (BaseTool.isNull(cacheDir)) {
			cacheDir = PathConf.getDataDir() + File.separator
					+ PathConf.Cache_Name + File.separator
					+ FileTool.getUniqueDirFromService(service);
		}

		String path = cacheDir + File.separator + operationName + ".xml";
		File file = new File(path);
		if (file.exists()) {
			SAXReader reader = new SAXReader();
			try {
				doc = reader.read(file);
			} catch (DocumentException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				doc = null;
			}
		}

		if (doc == null) {
			String getdescribeprocessrequest = new StringBuffer()
					.append(serviceInfo.getDescribeURI())
					.append("?Request=DescribeProcess&Service=").append("WPS")
					.append("&Version=0.4.0&Language=en&Identifier=")
					.append(operationName).toString();
			doc = RequestMethodStore.GetDoc(getdescribeprocessrequest);
			if (doc == null) {
				return null;
			}
		}
		Element describeEle = doc.getRootElement();
		if (describeEle == null) {
			log.error("Failed to get operation information.");
			return null;
		}

		OperationInfoImpl oi = new OperationInfoImpl();
		oi.setName(operationName);
		List<IParameterInfo> inputParametersInfos = new ArrayList<IParameterInfo>();

		OperationMeta operationMeta = new OperationMeta();
		Element processdescribeele = describeEle.element(processdescriptionq);
		Element titleele = processdescribeele.element(titleq);
		if (!BaseTool.isNull(titleele))
			operationMeta.set_title(titleele.getTextTrim());
		Element absele = processdescribeele.element(abstractq);
		if (!BaseTool.isNull(absele))
			operationMeta.set_abstract(absele.getTextTrim());
		oi.setMeta(operationMeta);
		Element datainputsele = processdescribeele.element(datainputsq);
		Iterator datainputsit = datainputsele.elementIterator(inputq);
		while (datainputsit.hasNext()) {
			Element inputele = (Element) datainputsit.next();
			ParameterMeta meta = new ParameterMeta();
			titleele = inputele.element(titleq);
			absele = inputele.element(abstractq);

			if (!BaseTool.isNull(titleele))
				meta.set_title(titleele.getTextTrim());
			if (!BaseTool.isNull(absele))
				meta.set_abstract(absele.getTextTrim());
			Element minioccurs = inputele.element(minimumq);
			Element maxioccurs = inputele.element(maxinumq);
			if (!BaseTool.isNull(minioccurs))
				meta.set_minoccurs(minioccurs.getTextTrim());
			if (!BaseTool.isNull(maxioccurs))
				meta.set_maxoccurs(maxioccurs.getTextTrim());

			Element identifierele = inputele.element(identifierq);
			String inputname = identifierele.getTextTrim();
			Element complexele = inputele.element(complexdataq);
			ParameterInfoImpl parameterInfo = new ParameterInfoImpl();
			parameterInfo.setName(inputname);

			parameterInfo.setMeta(meta);
			if (!BaseTool.isNull(complexele)) {
				meta.set_type("ComplexData");
			} else {
				Element literalele = inputele.element(literaldataq);
				Element datatypeele = literalele.element(datatypeq);
				String type = datatypeele.attributeValue(referenceq);
				meta.set_type(type);
			}
			inputParametersInfos.add(parameterInfo);
		}

		oi.setInputParameters(inputParametersInfos);

		// Output params.
		List<IParameterInfo> outPutParameters = new ArrayList<IParameterInfo>();
		Element outputsele = processdescribeele.element(processoutsq);
		Iterator<Element> outList = outputsele.elementIterator(outputq);
		while (outList.hasNext()) {
			Element outputele = outList.next();
			ParameterInfoImpl outPutParameter = new ParameterInfoImpl();
			ParameterMeta meta = new ParameterMeta();
			Element identifierele = outputele.element(identifierq);
			String outputname = identifierele.getTextTrim();
			Element complexoutputele = outputele.element(complexoutputq);
			outPutParameter.setName(outputname);
			titleele = outputele.element(titleq);
			absele = outputele.element(abstractq);
			if (!BaseTool.isNull(titleele))
				meta.set_title(titleele.getTextTrim());
			if (!BaseTool.isNull(absele))
				meta.set_abstract(absele.getTextTrim());
			if (!BaseTool.isNull(complexoutputele)) {
				meta.set_type("ComplexData");
			}
			outPutParameter.setMeta(meta);
			outPutParameters.add(outPutParameter);
		}
		oi.setOutputParameters(outPutParameters);

		serviceInfo.addOperationInfo(oi);
		// oi.setOuputParameter(outPutParameter);

		return oi;
	}

	/**
	 * Gets value from value map by name.
	 * 
	 * @param name
	 *            parameter name.
	 * @param namevaluemap
	 *            parameter value map.
	 * @return
	 */
	@SuppressWarnings("rawtypes")
	private String getComplexDataTypeByName(String name,
			Map<String, String> namevaluemap) {
		Iterator keyit = namevaluemap.keySet().iterator();
		String value = null;
		while (keyit.hasNext()) {
			String key = (String) keyit.next();
			String name2 = name + "_datatype";
			if (key.equals(name2)) {
				value = namevaluemap.get(key);
			}
		}
		return value;
	}

	/**
	 * Gets value from value map by name.
	 * 
	 * @param name
	 *            parameter name.
	 * @param namevaluemap
	 *            parameter value map.
	 * @return
	 */
	@SuppressWarnings("rawtypes")
	private String getValueByName(String name, Map<String, String> namevaluemap) {
		Iterator keyit = namevaluemap.keySet().iterator();
		String value = null;
		while (keyit.hasNext()) {
			String key = (String) keyit.next();
			// String name2 = key.split("\\#")[1];
			if (name.equals(key)) {
				value = namevaluemap.get(key);
			}
		}
		return value;
	}

}
