/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.starter;

import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.JOptionPane;

import cn.edu.whu.lmars.frame.ui.ClientFrame;
import cn.edu.whu.lmars.locale.LocaleKey;
import cn.edu.whu.lmars.locale.ResourceBundleFactory;
import cn.edu.whu.lmars.starter.ui.SelWSDialog;
import cn.edu.whu.lmars.starter.ui.WelcomeWin;
import cn.edu.whu.lmars.utils.HandleTool;
import conf.GlobeParam;
import conf.PathConf;

/**
 * Application launcher. Starts or Restarts the application .
 * 
 * @author Mingda Zhang
 * 
 */
public class Starter {
	private HandleTool handleTool = HandleTool.getInstance();
	private ResourceBundle rBundle = ResourceBundleFactory.getResourceBundle();

	/**
	 * Starts the application.
	 */
	public void start() {
		SelWSDialog dialog = new SelWSDialog(null);
		dialog.setVisible(true);
		String path = dialog.getPath();
		dialog.dispose();
		if (path == null || path.trim().equals("")) {
			return;
		}
		run(path);
	}

	/**
	 * Initialize the workspace.
	 * 
	 * @param path
	 *            the location of workspace
	 */
	private void run(String path) {

		new InitParam().execute();
		WelcomeWin splash = WelcomeWin.getInstance();
		splash.start();

		PathConf.WORK_PATH = path;
		splash.requestFocus();
		splash.toFront();
		splash.setProMessage(rBundle.getString(LocaleKey.Starter_Init_Check));
		splash.setProValue(10);
		if (!handleTool.checkDir(path)) {
			JOptionPane.showMessageDialog(null,
					rBundle.getString(LocaleKey.Starter_Init_Check_Msg_Fault));
			splash.stopPro();
			return;
		}
		ClientFrame client = new ClientFrame(
				rBundle.getString(LocaleKey.JFrame_Title));
		GlobeParam.setClient(client);
		splash.setProValue(100);
		splash.stopPro();
		splash.dispose();
		client.setVisible(true);
	}

	/**
	 * Restarts the application with a new workspace, switch the workspace.
	 */
	public void reStart() {
		SelWSDialog dialog = new SelWSDialog(null);
		dialog.setVisible(true);
		String path = dialog.getPath();
		dialog.dispose();
		if (path == null || path.trim().equals("")) {
			return;
		}
		if (path.equals(PathConf.WORK_PATH)) {
			return;
		}
		GlobeParam.getClient().dispose();
		run(path);
	}

	/**
	 * Application entry.
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			Locale.setDefault(new Locale("en","US"));
			new Starter().start();
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null,
					"Error occured:\n" + e.getMessage());
			System.out.print(e.getMessage());
			System.exit(-1);
		}

	}
}
