/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.starter.ui;

import java.awt.Font;
import java.awt.Rectangle;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ResourceBundle;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;

import cn.edu.whu.lmars.locale.LocaleKey;
import cn.edu.whu.lmars.locale.ResourceBundleFactory;
import cn.edu.whu.lmars.utils.WorkSpaceXML;
/**
 * Selects the workspace using this dialog.
 * 
 * @author Zhang Mingda
 *
 */

public class SelWSDialog extends JDialog implements ActionListener{

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JLabel jLabel = null;
	private JButton jButton = null;
	private String path = "";  //  @jve:decl-index=0:
	private JButton confirBtn = null;
	private JButton concel = null;
	private JComboBox jComboBox = null;
	private ResourceBundle rBundle = null;
	private WorkSpaceXML workSpaceXML;  
	private boolean value = false;
	public SelWSDialog(Window owner) {
		super(owner);
		initialize();
	}

	/**
	 * This method initializes the SelWSDialog	
	 * 	
	 */
	private void initialize() {
		rBundle = ResourceBundleFactory.getResourceBundle();
		this.setSize(586, 150);
		this.setTitle(rBundle.getString(LocaleKey.Starter_Dialog_Tile));
		this.setFont(new Font("Dialog", Font.PLAIN, 14));
		this.setContentPane(getJContentPane());
		
		String[] items = getItems();
		if (items!=null) {
			for(String item:items){
				jComboBox.addItem(item);
			}
			jComboBox.setSelectedIndex(0);
		}
		
		this.setLocationRelativeTo(null);
		this.setModal(true);
	}

	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jLabel = new JLabel();
			jLabel.setBounds(new Rectangle(10, 14, 90, 30));
			jLabel.setFont(new Font("Dialog", Font.BOLD, 14));
			jLabel.setText(rBundle.getString(LocaleKey.Starter_Dialog_Ws));
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(jLabel, null);
			jContentPane.add(getJButton(), null);
			jContentPane.add(getConfirBtn(), null);
			jContentPane.add(getConcel(), null);
			jContentPane.add(getJComboBox(), null);
		}
		return jContentPane;
	}

	private JButton getJButton() {
		if (jButton == null) {
			jButton = new JButton();
			jButton.setBounds(new Rectangle(442, 13, 103, 30));
			jButton.setText(rBundle.getString(LocaleKey.Button_Browse));
			jButton.setFont(new Font("Dialog", Font.BOLD, 14));
			jButton.addActionListener(this);
			jButton.setActionCommand("choose");
			
		}
		return jButton;
	}

	private JButton getConfirBtn() {
		if (confirBtn == null) {
			confirBtn = new JButton();
			confirBtn.setBounds(new Rectangle(274, 64, 69, 30));
			confirBtn.setText(rBundle.getString(LocaleKey.Button_OK));
			confirBtn.setFont(new Font("Dialog", Font.BOLD, 14));
			confirBtn.addActionListener(this);
			confirBtn.setActionCommand("confirm");
		}
		return confirBtn;
	}
	@Override
	public void actionPerformed(ActionEvent e) {
		String str = e.getActionCommand();
		
		if(str.equals("choose")){
			JFileChooser fileChooser = new JFileChooser();
			fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
			int returnVal = fileChooser.showSaveDialog(null);
			
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				File file = fileChooser.getSelectedFile();
				path = file.toString();
				jComboBox.addItem(path);
				jComboBox.setSelectedItem(path);
				if (workSpaceXML == null) {
					workSpaceXML = WorkSpaceXML.getInstance();
				}
				workSpaceXML.savePath(path);
			}else {
				System.out.println("concel");
			}
		}else if (str.equals("confirm")) {
			value = true;
			this.dispose();
		}else if (str.equals("concel")) {
			this.dispose();
		}
	}
	public String getPath(){
		if (value) {
			return jComboBox.getSelectedItem().toString();
		}
		return "";
	}

	/**
	 * This method initializes concel button	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getConcel() {
		if (concel == null) {
			concel = new JButton();
			concel.setBounds(new Rectangle(392, 62, 87, 30));
			concel.setText(rBundle.getString(LocaleKey.Button_Concel));
			concel.setFont(new Font("Dialog", Font.BOLD, 14));
			concel.setActionCommand("concel");
			concel.addActionListener(this);
		}
		return concel;
	}

	/**
	 * This method initializes jComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getJComboBox() {
		if (jComboBox == null) {
			jComboBox = new JComboBox();
			jComboBox.setBounds(new Rectangle(106, 13, 324, 29));
			jComboBox.setFont(new Font("Dialog", Font.BOLD, 14));
			jComboBox.setEditable(true);
		}
		return jComboBox;
	}
	private String[] getItems(){
		workSpaceXML = WorkSpaceXML.getInstance();
		String[] items = workSpaceXML.getPaths();
		return items;
		
	}
	
}  