/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.starter.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JProgressBar;
import javax.swing.JWindow;

import cn.edu.whu.lmars.locale.LocaleKey;
import cn.edu.whu.lmars.locale.ResourceBundleFactory;

/**
 * This Welcome Window will show when user starts the application as a thread.
 * Display the status of the initialization of the application.
 * 
 * @author Zhang Mingda
 * 
 */

public class WelcomeWin extends JWindow implements Runnable {
	private static final long serialVersionUID = 1L;
	Thread splashThread;
	JProgressBar progress;
	private int proValue = 0;
	private boolean flag = true;
	private static WelcomeWin welcomeWin = null;

	private WelcomeWin() {
		Container container = getContentPane();
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		URL url = this.getClass().getResource("/images/login1.jpg");
		ImageIcon imageIcon = new ImageIcon(url);
		container.add(new JLabel(imageIcon), BorderLayout.CENTER);
		progress = new JProgressBar(1, 100);
		progress.setStringPainted(true);
		progress.setString(ResourceBundleFactory.getResourceBundle().getString(
				LocaleKey.Starter_Init_Origin));
		progress.setBackground(Color.white);
		container.add(progress, BorderLayout.SOUTH);

		Dimension screen = getToolkit().getScreenSize();
		pack();
		setLocation((screen.width - getSize().width) / 2,
				(screen.height - getSize().height) / 2);
	}

	/**
	 * Gets an instance of the welcome window.
	 * 
	 * @return
	 */
	public static WelcomeWin getInstance() {
		if (welcomeWin == null) {
			welcomeWin = new WelcomeWin();
		}
		return welcomeWin;
	}

	/**
	 * Starts the initial thread
	 */
	public void start() {
		this.toFront();
		splashThread = new Thread(this);
		splashThread.start();
	}

	/**
	 * 
	 * @see Runnable#run();
	 */
	@Override
	public void run() {
		welcomeWin.toFront();
		setVisible(true);
		while (flag) {
			progress.setValue(proValue);
		}

		// When the end of the thread, destroy the window
		if (welcomeWin != null) {
			welcomeWin.dispose();
			welcomeWin = null;
		}
	}

	/**
	 * Set the value of initialization progress
	 * 
	 * @param value
	 */
	public void setProValue(int value) {
		this.proValue = value;
	}

	/**
	 * Stops the initial thread.
	 */
	public void stopPro() {
		this.flag = false;
	}

	/**
	 * Sets the message of initialization process.
	 * 
	 * @param message
	 *            the initial message
	 */
	public void setProMessage(String message) {
		progress.setString(message);
	}

}
