/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.tree.listeners;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.JOptionPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;

import cn.edu.whu.lmars.sensorweb.sos.beans.SOSSensor;
import cn.edu.whu.lmars.sensorweb.sos.utils.SensorUtility;
import cn.edu.whu.lmars.service.adapter.AdapterFactory;
import cn.edu.whu.lmars.service.adapter.base.IAdapter;
import cn.edu.whu.lmars.service.utils.NetUtil;
import cn.edu.whu.lmars.service.utils.ServiceCacheThread;
import cn.edu.whu.lmars.tree.nodes.ServiceTreeNode;
import cn.edu.whu.lmars.tree.nodes.TreeNodeImpl;
import cn.edu.whu.lmars.tree.ui.AddServiceDialog;
import cn.edu.whu.lmars.tree.ui.SearchDialog;
import cn.edu.whu.lmars.tree.utils.TreeConfigUtility;
import cn.edu.whu.lmars.tree.utils.TreeUtil;
import cn.edu.whu.lmars.worldwind.utils.WWDUtility;
import conf.GlobeParam;

/**
 * 
 * This listener receives action events from the tree list that contains services.
 * 
 * @author Mingda Zhang
 * 
 */

public class TreeActionListener implements ActionListener {

	private JTree tree = null;

	public TreeActionListener(JTree tree) {
		this.tree = tree;
	}

	/**
	 * Handles actions including adding, deleting, searching, displaying sensor.
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
	 */
	@Override
	public void actionPerformed(ActionEvent e) {

		DefaultMutableTreeNode targetNode = (DefaultMutableTreeNode) tree
				.getLastSelectedPathComponent();
		String comStr = e.getActionCommand();
		if (comStr.equals("add")) {
			TreeNodeImpl treeNode = (TreeNodeImpl) targetNode;
			AddServiceDialog dialog = new AddServiceDialog(
					GlobeParam.getClient(), treeNode, tree);
			dialog.setVisible(true);
		} else if (comStr.equals("delete")) {
			if (!(targetNode instanceof ServiceTreeNode)) {
				return;
			}
			ServiceTreeNode serviceNode = (ServiceTreeNode) targetNode;
			String url = serviceNode.getUrl();
			TreeConfigUtility treeConfig = new TreeConfigUtility();
			int reconfirm = JOptionPane.showConfirmDialog(
					GlobeParam.getClient(),
					"Are you sure to delete the service named "
							+ serviceNode.getName());
			if (reconfirm != JOptionPane.YES_OPTION) {
				return;
			}
			boolean flag = true;

			// delete the service from configure file
			switch (serviceNode.getServiceType()) {
			case WPS:
				flag = treeConfig.deleteWps(url);
				break;
			case WFS:
				flag = treeConfig.deleteWfs(url);
				break;
			case SOS:
				flag = treeConfig.deleteSos(url);
				break;
			case WCS:
				flag = treeConfig.deleteWcs(url);
				break;
			default:
				break;
			}

			// delete the service node from the tree
			if (flag) {
				((DefaultTreeModel) tree.getModel())
						.removeNodeFromParent(targetNode);
				JOptionPane.showMessageDialog(null,
						"Delete the service successfully");
			}
		} else if (comStr.equals("search")) {
			SearchDialog dialog = new SearchDialog(GlobeParam.getClient(),
					tree, targetNode);
			dialog.setVisible(true);
		} else if (comStr.equals("showSensor")) {
			TreeNodeImpl treeNodeImpl = (TreeNodeImpl) targetNode;
			TreeNode parentNode = treeNodeImpl.getParent();
			if (!(parentNode instanceof ServiceTreeNode)) {
				return;
			}
			ServiceTreeNode serviceNode = (ServiceTreeNode) parentNode;
			SOSSensor sensorBean = SensorUtility.parseSensor(serviceNode,
					treeNodeImpl.getName());
			WWDUtility.addBalloon(sensorBean);

		}else if (comStr.equals("update")) {
			if (!(targetNode instanceof ServiceTreeNode)) {
				JOptionPane.showMessageDialog(GlobeParam.getClient(), "This node can not be deleted.");
				return;
			}
			ServiceTreeNode serviceTreeNode = (ServiceTreeNode)targetNode;
			String url = serviceTreeNode.getUrl();
			if (!NetUtil.validUrl(url)) {
				JOptionPane.showMessageDialog(GlobeParam.getClient(), "Network can not be connected.");
				return;
			}
			IAdapter oldAdapter = AdapterFactory.getInstance().getAdapter(serviceTreeNode);
			IAdapter newAdapter = AdapterFactory.getInstance().createAdapter(serviceTreeNode,true);
			updateAdapter(oldAdapter, newAdapter,serviceTreeNode);
		}
	}
	
	/**
	 * Updates the adapter.
	 * @param oldAdapter
	 * 			old 
	 * @param newAdapter
	 */
	private void updateAdapter(IAdapter oldAdapter,IAdapter newAdapter,ServiceTreeNode serviceTreeNode){
		List<String> oldList = oldAdapter.getServiceInfo().getOperationList();
		List<String> newList = newAdapter.getServiceInfo().getOperationList();
		boolean changed = false;
		for(String str:oldList){
			if (newList.contains(str)) {
				continue;
			}
			changed = true;
			TreeNode node = TreeUtil.getChildNodeByName(serviceTreeNode, str);
			if (node == null) {
				continue;
			}
			((DefaultTreeModel) tree.getModel())
			.removeNodeFromParent((DefaultMutableTreeNode)node);
			//remove the operation from the serviceTreenode
			
		}
		
		for(String str:newList){
			if (oldList.contains(str)) {
				continue;
			}
			changed = true;
			TreeNodeImpl treeNodeImpl = new TreeNodeImpl(str);
			treeNodeImpl.setServiceType(serviceTreeNode.getServiceType());
			serviceTreeNode.add(treeNodeImpl);
		}
		
		if (changed) {
			this.tree.updateUI();
			AdapterFactory.getInstance().removeAdapter(oldAdapter);
			AdapterFactory.getInstance().addAdapter(newAdapter);
			String url = serviceTreeNode.getUrl();
			TreeConfigUtility treeConfig = new TreeConfigUtility();
			switch (serviceTreeNode.getServiceType()) {
			case WPS:
				treeConfig.deleteWps(url);
				treeConfig.addWpsService(serviceTreeNode, newList);
				break;
			case WFS:
				 treeConfig.deleteWfs(url);
				 treeConfig.addWfsService(serviceTreeNode, newList);
				break;
			case SOS:
				 treeConfig.deleteSos(url);
				 treeConfig.addSosService(serviceTreeNode, newList);
				break;
			case WCS:
				 treeConfig.deleteWcs(url);
				 treeConfig.addWcsService(serviceTreeNode, newList);
				break;
			default:
				break;
			}
			new ServiceCacheThread(serviceTreeNode,true).start();
			JOptionPane.showMessageDialog(GlobeParam.getClient(),"Update successfully.");
		}else {
			JOptionPane.showMessageDialog(GlobeParam.getClient(),"No Updates.");
		}
	}

}
