/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.tree.listeners;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import cn.edu.whu.lmars.tree.nodes.ServiceTreeNode;
import cn.edu.whu.lmars.tree.nodes.TreeNodeImpl;

/**
 * This listener receives mouse events on the tree panel. 
 * 
 * @author Mingda Zhang
 * 
 */
public class TreeMouseListener implements MouseListener {

	private JTree tree;
	private JPopupMenu addPopupMenu = null, servicePopupMenu = null,
			sensorPopupMenu = null;

	public TreeMouseListener(JTree tree) {
		this.tree = tree;
		initComp();
	}

	private void initComp() {
		addPopupMenu = createAddPopupMenu();
		servicePopupMenu = createServicePopupMenu();
		sensorPopupMenu = createSensorPopupMenu();
	}

	@Override
	public void mouseClicked(MouseEvent e) {

	}

	@Override
	public void mouseEntered(MouseEvent arg0) {

	}

	@Override
	public void mouseExited(MouseEvent arg0) {

	}

	/**
	 * Different JPopupMenu will show according the type of the tree node.
	 * 
	 * @see java.awt.event.MouseListener#mousePressed(MouseEvent)
	 */
	@Override
	public void mousePressed(MouseEvent e) {
		TreePath path = tree.getPathForLocation(e.getX(), e.getY());
		if (path == null) {
			return;
		}

		tree.setSelectionPath(path);

		if (e.getButton() == 3) {
			DefaultMutableTreeNode target = (DefaultMutableTreeNode) path
					.getLastPathComponent();
			if (!(target instanceof TreeNodeImpl)) {
				return;
			}
			TreeNodeImpl treeNode = (TreeNodeImpl) target;
			if (target instanceof ServiceTreeNode) {
				servicePopupMenu.show(tree, e.getX(), e.getY());
			}
			switch (treeNode.getServiceType()) {
			case SensorWeb:
			case WebData:
			case GeoProcessing:
				addPopupMenu.show(tree, e.getX(), e.getY());
				break;
			case SOS:
				if (!(treeNode instanceof ServiceTreeNode)) {
					sensorPopupMenu.show(tree, e.getX(), e.getY());
				}
				break;
			default:
				break;
			}
		}
	}

	@Override
	public void mouseReleased(MouseEvent arg0) {

	}

	/**
	 * Creates item.
	 * 
	 * @param name
	 *            the name of item.
	 * @param command
	 *            the action command of the item.
	 * @return
	 */
	private JMenuItem createItem(String name, String command) {
		TreeActionListener listener = new TreeActionListener(tree);
		JMenuItem item = new JMenuItem(name);
		item.setActionCommand(command);
		item.addActionListener(listener);
		return item;
	}

	/**
	 * Creates JPopupMenu used for service searching and deleting.
	 * 
	 * @return
	 */
	private JPopupMenu createServicePopupMenu() {
		JPopupMenu popupMenu = new JPopupMenu();
		popupMenu.add(createItem("search", "search"));
		popupMenu.add(createItem("update", "update"));
		popupMenu.add(createItem("delete", "delete"));
		return popupMenu;
	}

	/**
	 * Creates JPopupMenu used for service adding and searching.
	 * 
	 * @return
	 */
	private JPopupMenu createAddPopupMenu() {
		JPopupMenu popupMenu = new JPopupMenu();
		popupMenu.add(createItem("add", "add"));
		popupMenu.add(createItem("search", "search"));
		return popupMenu;
	}

	/**
	 * Creates JPopupMenu used for showing sensor.
	 * 
	 * @return
	 */
	private JPopupMenu createSensorPopupMenu() {
		JPopupMenu popupMenu = new JPopupMenu();
		popupMenu.add(createItem("show", "showSensor"));
		return popupMenu;
	}
}
