/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.tree.listeners.dragdrop;

import java.awt.Point;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.io.IOException;

import javax.swing.JOptionPane;
import javax.swing.tree.DefaultMutableTreeNode;

import org.apache.log4j.Logger;

import cn.edu.whu.lmars.figure.utils.FigureUtility;
import cn.edu.whu.lmars.tree.nodes.ServiceTreeNode;
import cn.edu.whu.lmars.tree.nodes.TreeNodeImpl;
import cn.edu.whu.lmars.utils.HandleTool;

/**
 * The implementor of DropTargetListener.
 * 
 * @see java.awt.dnd.DropTargetListener.
 * 
 * @author Mingda Zhang
 * 
 */

public class GeoDropTargetListener implements DropTargetListener {

	FigureUtility utility = FigureUtility.getInstance();
	private Logger logger = Logger.getLogger(GeoDragGestureListener.class);

	public void dragEnter(DropTargetDragEvent dtde) {

	}

	public void dragOver(DropTargetDragEvent dtde) {
	}

	public void dropActionChanged(DropTargetDragEvent dtde) {
	}

	public void dragExit(DropTargetEvent dte) {
	}

	/**
	 * Drags a tree node to workspace and creates a model, a figure and a
	 * default binding.
	 * 
	 * @see java.awt.dnd.DropTargetListener#drop(DropTargetDropEvent).
	 */
	public void drop(DropTargetDropEvent dtde) {
		Point point = dtde.getLocation();
		Transferable tr = dtde.getTransferable();
		DefaultMutableTreeNode parent = null;
		DefaultMutableTreeNode node = null;

		String s = "";
		try {
			if (tr.isDataFlavorSupported(DataFlavor.stringFlavor)) {

				s = tr.getTransferData(DataFlavor.stringFlavor).toString();
				node = (DefaultMutableTreeNode) tr
						.getTransferData(DataFlavor.stringFlavor);
				parent = (DefaultMutableTreeNode) node.getParent();
			}
		} catch (IOException ex) {
		} catch (UnsupportedFlavorException ex) {
		}

		if (s.trim().equals("") || s == null || node == null || parent == null) {
			return;
		}

		if (node instanceof ServiceTreeNode) {
			return;
		}
		if (node instanceof TreeNodeImpl && parent instanceof ServiceTreeNode) {
			TreeNodeImpl treeNode = (TreeNodeImpl) node;
			ServiceTreeNode serviceNode = (ServiceTreeNode) parent;
			switch (treeNode.getServiceType()) {
			case WPS:
				logger.info("Drag " + s + "@" + serviceNode.getUrl()
						+ " to workspace");
				HandleTool.getInstance().createModelByDrag(point, s,
						serviceNode);
				break;
			case SOS:
				HandleTool.getInstance().createModelByDrag(point, s,
						serviceNode);
				break;
			case SES:
				JOptionPane.showMessageDialog(null, "Drag ses");
				break;
			case WNS:
				JOptionPane.showMessageDialog(null, "Drag wns");
				break;
			case WFS:
				logger.info("Drag " + s + "@" + serviceNode.getUrl()
						+ " to workspace");
				HandleTool.getInstance().createModelByDrag(point, s,
						serviceNode);
				break;
			default:
				break;
			}
		}

	}

}