/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.tree.ui;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.List;
import java.util.ResourceBundle;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.SwingConstants;
import javax.swing.tree.DefaultTreeModel;

import org.apache.log4j.Logger;

import cn.edu.whu.lmars.locale.LocaleKey;
import cn.edu.whu.lmars.locale.ResourceBundleFactory;
import cn.edu.whu.lmars.service.adapter.AdapterFactory;
import cn.edu.whu.lmars.service.adapter.base.IAdapter;
import cn.edu.whu.lmars.service.adapter.base.IServiceInfo;
import cn.edu.whu.lmars.service.beans.ServiceType;
import cn.edu.whu.lmars.service.utils.ServiceCacheThread;
import cn.edu.whu.lmars.tree.nodes.ServiceTreeNode;
import cn.edu.whu.lmars.tree.nodes.TreeNodeImpl;
import cn.edu.whu.lmars.tree.utils.TreeConfigUtility;
import cn.edu.whu.lmars.utils.BaseTool;
import cn.edu.whu.lmars.utils.HandleTool;

/**
 * This dialog handles the adding service operation.
 * 
 * @author Mingda Zhang
 * 
 */
public class AddServiceDialog extends JDialog implements ActionListener {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JLabel serviceURLJL = null;
	private JLabel serviceType = null;
	private JTextField jTextField = null;
	private JButton confirmBtn = null;
	private JButton cancelBtn = null;
	private List<String> nodeList = null;
	private Logger logger = Logger.getLogger(AddServiceDialog.class);
	private JLabel serviceType1 = null;
	private JTextField displayT = null;
	private JLabel serviceType2 = null;
	private JComboBox serviceTypeCombox = null;
	private JComboBox serviceVersionCombox = null;
	private TreeNodeImpl treeNode;
	private JTree tree;
	private ResourceBundle rBundle;
	private String msg; // @jve:decl-index=0:

	/**
	 * Instantiates a AddServiceDialog.
	 * 
	 * @param owner
	 * @param treeNode
	 *            the target tree node
	 * @param tree
	 *            the target tree
	 */
	public AddServiceDialog(Frame owner, TreeNodeImpl treeNode, JTree tree) {
		super(owner);
		this.treeNode = treeNode;
		this.tree = tree;
		rBundle = ResourceBundleFactory.getResourceBundle();
		initialize();
	}

	/**
	 * This method initializes this dialog.
	 * 
	 */
	private void initialize() {
		this.setSize(453, 255);
		this.setFont(new Font("Dialog", Font.PLAIN, 14));
		msg = rBundle.getString(LocaleKey.Dialog_Service_Add_Title);
		this.setTitle(msg);
		this.setResizable(false);
		this.setVisible(false);
		this.setContentPane(getJContentPane());

		initComp();
		this.setLocationRelativeTo(null);
		this.setModal(true);
	}

	/**
	 * This method initializes the components according the type of tree node.
	 */
	private void initComp() {
		switch (treeNode.getServiceType()) {
		case GeoProcessing:
			serviceTypeCombox.addItem(ServiceType.WPS);
			serviceTypeCombox.setSelectedIndex(0);
			serviceTypeCombox.setEditable(false);
			serviceVersionCombox.addItem("0.4.0");
			serviceVersionCombox.addItem("1.0.0");
			break;
		case SensorWeb:
			serviceTypeCombox.addItem(ServiceType.SOS);
			// serviceTypeCombox.addItem(ServiceType.WNS);
			// serviceTypeCombox.addItem(ServiceType.SES);
			serviceVersionCombox.addItem("1.0.0");
			// serviceVersionCombox.addItem("Default");
			serviceVersionCombox.setEditable(false);
			break;
		case WebData:
			serviceTypeCombox.addItem(ServiceType.WFS);
			// serviceTypeCombox.addItem(ServiceType.WCS);
			// serviceVersionCombox.addItem("1.0.0");
			serviceVersionCombox.addItem("1.1.0");
			break;
		default:
			break;
		}
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			serviceType2 = new JLabel();
			serviceType2.setBounds(new Rectangle(9, 95, 85, 30));
			serviceType2.setToolTipText("the type of service");
			serviceType2.setHorizontalAlignment(SwingConstants.RIGHT);
			msg = rBundle.getString(LocaleKey.Dialog_Service_Add_Label_Version);
			serviceType2.setText(msg);
			serviceType2.setFont(new Font("Dialog", Font.BOLD, 14));
			serviceType1 = new JLabel();
			serviceType1.setBounds(new Rectangle(9, 135, 85, 30));
			serviceType1.setToolTipText("the type of service");
			msg = rBundle.getString(LocaleKey.Dialog_Service_Add_Label_Name);
			serviceType1.setText(msg);
			serviceType1.setHorizontalTextPosition(SwingConstants.RIGHT);
			serviceType1.setHorizontalAlignment(SwingConstants.RIGHT);
			serviceType1.setFont(new Font("Dialog", Font.BOLD, 14));
			serviceType = new JLabel();
			serviceType.setBounds(new Rectangle(9, 55, 85, 30));
			serviceType.setFont(new Font("Dialog", Font.BOLD, 14));
			serviceType.setToolTipText("the type of service");
			serviceType.setHorizontalAlignment(SwingConstants.RIGHT);
			msg = rBundle.getString(LocaleKey.Dialog_Service_Add_Label_Type);
			serviceType.setText(msg);
			serviceURLJL = new JLabel();
			msg = rBundle.getString(LocaleKey.Dialog_Service_Add_Label_Url);
			serviceURLJL.setText(msg);
			serviceURLJL.setFont(new Font("Dialog", Font.BOLD, 14));
			serviceURLJL.setToolTipText("The Service URL you want to add");
			serviceURLJL.setHorizontalAlignment(SwingConstants.RIGHT);
			serviceURLJL.setBounds(new Rectangle(9, 15, 85, 30));
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(serviceURLJL, BorderLayout.NORTH);
			jContentPane.add(serviceType, null);
			jContentPane.add(getJTextField(), null);
			jContentPane.add(getConfirmBtn(), null);
			jContentPane.add(getCancelBtn(), null);
			jContentPane.add(serviceType1, null);
			jContentPane.add(getDisplayT(), null);
			jContentPane.add(serviceType2, null);
			jContentPane.add(getServiceTypeCombox(), null);
			jContentPane.add(getServiceVersionCombox(), null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jTextField
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getJTextField() {
		if (jTextField == null) {
			jTextField = new JTextField();
			jTextField.setBounds(new Rectangle(105, 15, 310, 30));
			jTextField.setFont(new Font("Dialog", Font.PLAIN, 14));
		}
		return jTextField;
	}

	/**
	 * This method initializes confirmBtn
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getConfirmBtn() {
		if (confirmBtn == null) {
			confirmBtn = new JButton();
			confirmBtn.setBounds(new Rectangle(100, 178, 92, 30));
			confirmBtn.setActionCommand("confirm");
			confirmBtn.addActionListener(this);
			confirmBtn.setFont(new Font("Dialog", Font.BOLD, 14));
			msg = rBundle.getString(LocaleKey.Button_OK);
			confirmBtn.setText(msg);
		}
		return confirmBtn;
	}

	/**
	 * This method initializes cancelBtn
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getCancelBtn() {
		if (cancelBtn == null) {
			cancelBtn = new JButton();
			msg = rBundle.getString(LocaleKey.Button_Concel);
			cancelBtn.setText(msg);
			cancelBtn.addActionListener(this);
			cancelBtn.setActionCommand("cancel");
			cancelBtn.setBounds(new Rectangle(239, 178, 92, 30));
			cancelBtn.setFont(new Font("Dialog", Font.BOLD, 14));
		}
		return cancelBtn;
	}

	/**
	 * This method initializes serviceTypeCombox
	 * 
	 * @return javax.swing.JComboBox
	 */
	private JComboBox getServiceTypeCombox() {
		if (serviceTypeCombox == null) {
			serviceTypeCombox = new JComboBox();
			serviceTypeCombox.setBounds(new Rectangle(105, 55, 310, 30));
			serviceTypeCombox.setFont(new Font("Dialog", Font.BOLD, 14));
		}
		return serviceTypeCombox;
	}

	/**
	 * This method initializes nameJTextField
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getDisplayT() {
		if (displayT == null) {
			displayT = new JTextField();
			displayT.setBounds(new Rectangle(105, 135, 310, 30));
			displayT.setFont(new Font("Dialog", Font.PLAIN, 14));
		}
		return displayT;
	}

	/**
	 * This method initializes serviceVersionCombox
	 * 
	 * @return javax.swing.JComboBox
	 */
	private JComboBox getServiceVersionCombox() {
		if (serviceVersionCombox == null) {
			serviceVersionCombox = new JComboBox();
			serviceVersionCombox.setBounds(new Rectangle(105, 95, 310, 30));
			serviceVersionCombox.setFont(new Font("Dialog", Font.BOLD, 14));
		}
		return serviceVersionCombox;
	}

	/**
	 * 
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		String actionCom = e.getActionCommand();
		int counts = 0, state;
		URL urlStr;
		HttpURLConnection connection;
		boolean flag = false;
		if (actionCom.equals("confirm")) {
			String serviceUrl = jTextField.getText().toString();
			ServiceType serviceType = (ServiceType) serviceTypeCombox
					.getSelectedItem();
			String version = serviceVersionCombox.getSelectedItem().toString();
			String name = displayT.getText();

			if (BaseTool.isNull(serviceUrl)) {
				msg = rBundle
						.getString(LocaleKey.Dialog_Service_Add_Msg_Nul_Url);
				JOptionPane.showMessageDialog(null, msg);
				return;
			}
			if (BaseTool.isNull(name)) {
				msg = rBundle
						.getString(LocaleKey.Dialog_Service_Add_Msg_Nul_Url);
				JOptionPane.showMessageDialog(this, msg);
				return;
			}
			while (counts < 4) {
				try {
					urlStr = new URL(serviceUrl);
					connection = (HttpURLConnection) urlStr.openConnection();
					state = connection.getResponseCode();
					if (state == 200) {
						flag = true;
						break;
					}
				} catch (Exception ex) {
					counts++;
					logger.info("loop :" + counts);
					continue;
				}
			}
			if (!flag) {
				msg = rBundle
						.getString(LocaleKey.Dialog_Service_Add_Msg_Url_Wrong);
				JOptionPane.showMessageDialog(this, msg);
			}

			/*
			 * IService service = new ServiceModel();
			 * service.setServiceType(serviceType); service.setUrl(serviceUrl);
			 * service.setName(name); service.setVersion(version);
			 */
			ServiceTreeNode serviceNode = new ServiceTreeNode(name);
			serviceNode.setUrl(serviceUrl);
			serviceNode.setVersion(version);
			serviceNode.setName(name);
			serviceNode.setServiceType(serviceType);
			serviceNode.setId(HandleTool.getUUID());
			/*
			 * IAdapter adapter =
			 * AdapterFactory.getInstance().getAdapter(serviceNode);
			 * 
			 * if (adapter == null) { JOptionPane.showMessageDialog(this,
			 * "Can't parse this service"); return; } adapter.parseService();
			 */
			IAdapter adapter = AdapterFactory.getInstance().getAdapter(
					serviceNode);
			if (adapter == null) {
				JOptionPane.showMessageDialog(null, "Failed to get the suitable Adapter");
				return;
			}
			IServiceInfo serviceInfo = adapter.getServiceInfo();
			if (serviceInfo == null) {
				serviceInfo = adapter.parseService();
				if (serviceInfo == null) {
					JOptionPane.showMessageDialog(null, "Failed to parse the service.");
					return;
				}
			}
			nodeList = serviceInfo.getOperationList();
			ServiceCacheThread thread = new ServiceCacheThread(serviceNode,
					false);
			thread.start();

			boolean conflag = false;

			// add service to configure file
			TreeConfigUtility treeConfigUtility = new TreeConfigUtility();
			switch (serviceType) {
			case WPS:
				conflag = treeConfigUtility
						.addWpsService(serviceNode, nodeList);
				break;
			case SOS:
				conflag = treeConfigUtility.addSosService(serviceNode,
						nodeList);
				break;
			case WFS:
				conflag = treeConfigUtility
						.addWfsService(serviceNode, nodeList);
				break;
			case WCS:
				conflag = treeConfigUtility
						.addWcsService(serviceNode, nodeList);
				break;
			case WNS:
				break;
			default:
				break;
			}

			// add service node to the tree
			if (conflag) {
				serviceNode.setServiceType(serviceType);
				for (String nodeName : nodeList) {
					TreeNodeImpl node = new TreeNodeImpl(nodeName);
					node.setServiceType(serviceType);
					serviceNode.add(node);
				}
				((DefaultTreeModel) tree.getModel()).insertNodeInto(
						serviceNode, this.treeNode,
						this.treeNode.getChildCount());
				JOptionPane.showMessageDialog(this, "Add service successfully");
				logger.info("The following service to be added:");
				logger.info("url:" + serviceUrl + ",type:" + serviceType
						+ ",version:" + version + ",name:" + name);
			} else {
				JOptionPane.showMessageDialog(this, "Failed to add service");
			}

			dispose();
		} else {
			this.dispose();
		}
	}

}
