/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.tree.ui;

import java.awt.Font;
import java.awt.Frame;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.ResourceBundle;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import cn.edu.whu.lmars.locale.LocaleKey;
import cn.edu.whu.lmars.locale.ResourceBundleFactory;
import cn.edu.whu.lmars.tree.nodes.TreeNodeImpl;
import cn.edu.whu.lmars.utils.KMPMatch;

/**
 * This dialog is used to search operations in a service.
 * 
 * @author Mingda Zhang
 * 
 */

public class SearchDialog extends JDialog implements ActionListener,
		DocumentListener {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JTextField jTextField = null;
	private JButton confirmBtn = null;
	private JButton cancelBtn = null;
	private JLabel jLabel = null;
	private JTree tree = null;
	private String conStr = null;
	private DefaultMutableTreeNode selecedtNode;
	private ResourceBundle rBundle = null;
	private String msg;

	/**
	 * 
	 * Instantiates a SearchDialog.
	 * 
	 * @param owner
	 * @param tree
	 *            the target tree
	 * @param targetNode
	 *            the target tree node
	 */
	public SearchDialog(Frame owner, JTree tree,
			DefaultMutableTreeNode targetNode) {
		super(owner);
		this.selecedtNode = targetNode;
		this.tree = tree;
		rBundle = ResourceBundleFactory.getResourceBundle();
		initialize();
	}

	/**
	 * This method initializes this dialog.
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(378, 152);
		msg = rBundle.getString(LocaleKey.Dialog_Service_Search_Title);
		this.setTitle(msg);
		this.setFont(new Font("Dialog", Font.PLAIN, 14));
		this.setLocationRelativeTo(null);
		this.setContentPane(getJContentPane());
		this.setModal(false);
	}

	/**
	 * This method initializes jContentPane.
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jLabel = new JLabel();
			jLabel.setBounds(new Rectangle(10, 20, 106, 29));
			jLabel.setFont(new Font("Dialog", Font.BOLD, 14));
			msg = rBundle.getString(LocaleKey.Dialog_Service_Search_Label);
			jLabel.setText(msg);
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(getJTextField(), null);
			jContentPane.add(getConfirmBtn(), null);
			jContentPane.add(getCancelBtn(), null);
			jContentPane.add(jLabel, null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jTextField.
	 * 
	 * @return javax.swing.JTextField
	 */
	private JTextField getJTextField() {
		if (jTextField == null) {
			jTextField = new JTextField();
			jTextField.setBounds(new Rectangle(124, 17, 217, 34));
			jTextField.setFont(new Font("Dialog", Font.PLAIN, 14));
			jTextField.getDocument().addDocumentListener(this);
		}
		return jTextField;
	}

	/**
	 * This method initializes confirmBtn
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getConfirmBtn() {
		if (confirmBtn == null) {
			confirmBtn = new JButton();
			confirmBtn.setBounds(new Rectangle(47, 66, 86, 29));
			confirmBtn.setActionCommand("confirm");
			confirmBtn.addActionListener(this);
			confirmBtn.setFont(new Font("Dialog", Font.BOLD, 14));
			msg = rBundle.getString(LocaleKey.Button_OK);
			confirmBtn.setText(msg);
		}
		return confirmBtn;
	}

	/**
	 * This method initializes cancelBtn
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getCancelBtn() {
		if (cancelBtn == null) {
			cancelBtn = new JButton();
			cancelBtn.setBounds(new Rectangle(170, 65, 86, 29));
			cancelBtn.setActionCommand("cancel");
			msg = rBundle.getString(LocaleKey.Button_Concel);
			cancelBtn.setText(msg);
			cancelBtn.addActionListener(this);
			cancelBtn.setFont(new Font("Dialog", Font.BOLD, 14));
		}
		return cancelBtn;
	}

	/**
	 * This method is used to find matching tree nodes
	 * 
	 * @param treeNode
	 *            the target tree node to search
	 * @param searchStr
	 *            the string to be searched
	 */
	@SuppressWarnings("rawtypes")
	private void searchAllNodes(TreeNode treeNode, String searchStr) {
		if (!treeNode.isLeaf()) {
			for (Enumeration e = treeNode.children(); e.hasMoreElements();) {
				TreeNode n = (TreeNode) e.nextElement();
				searchAllNodes(n, searchStr);
			}
		} else {

			String str = treeNode.toString().toLowerCase();
			if (treeNode instanceof TreeNodeImpl) {
				str = ((TreeNodeImpl) treeNode).getName();
			}
			if (searchStr.equals("")) {
				return;
			}
			str = str.toLowerCase();
			int n = KMPMatch.searchKMP(str.toCharArray(),
					searchStr.toCharArray());
			if (n != -1) {
				DefaultTreeModel model = (DefaultTreeModel) tree.getModel();
				TreePath path = new TreePath(model.getPathToRoot(treeNode));
				tree.addSelectionPath(path);
				tree.scrollPathToVisible(path);
			}
		}
	}

	/**
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		String comStr = e.getActionCommand();
		if (comStr.equals("confirm")) {
			conStr = jTextField.getText();
			if (conStr == null && conStr.trim().equals("")) {
				return;
			}
			searchAllNodes(selecedtNode, conStr.toLowerCase());
			this.dispose();
		} else if (comStr.equals("cancel")) {
			this.dispose();
		}
	}

	/**
	 * 
	 * @see javax.swing.event.DocumentListener#insertUpdate(DocumentEvent)
	 */
	@Override
	public void insertUpdate(DocumentEvent e) {
		changedUpdate(e);
	}

	/**
	 * 
	 * @see javax.swing.event.DocumentListener#removeUpdate(DocumentEvent)
	 */
	@Override
	public void removeUpdate(DocumentEvent e) {
		changedUpdate(e);
	}

	/**
	 * 
	 * @see javax.swing.event.DocumentListener#changedUpdate(DocumentEvent)
	 */
	@Override
	public void changedUpdate(DocumentEvent e) {
		String str = jTextField.getText().trim();
		if (str.equals("")) {
			tree.clearSelection();
			return;
		}
		tree.clearSelection();
		searchAllNodes(selecedtNode, str.toLowerCase());
	}
}
