/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.tree.ui;


import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.tree.TreeNode;

import cn.edu.whu.lmars.service.beans.ServiceType;
import cn.edu.whu.lmars.tree.listeners.TreeMouseListener;
import cn.edu.whu.lmars.tree.nodes.ServiceTreeNode;
import cn.edu.whu.lmars.tree.nodes.TreeNodeImpl;
import cn.edu.whu.lmars.tree.utils.TreeCreate;

/**
 * Service container.
 * 
 * @author Mingda Zhang
 * 
 */
public class TreeScrollPanel extends JScrollPane {

	private static final long serialVersionUID = 1L;
	private JTree tree;
	public TreeScrollPanel() {
		super();
		initialize();
	}

	/**
	 * This method initializes the scroll panel.
	 * 
	 */
	private void initialize() {
		TreeCreate treeCreate = new TreeCreate();
		this.tree = treeCreate.execute();
		this.tree.addMouseListener(new TreeMouseListener(tree));
		this.getViewport().add(tree);
	}

	/**
	 * Gets WPS node.
	 * @return
	 */
	public List<ServiceTreeNode> getWPSServiceNode(){
		return getServiceNode(ServiceType.GeoProcessing, ServiceType.WPS);
	}
	
	/**
	 * Gets WFS node.
	 * @return
	 */
	public List<ServiceTreeNode> getWFSServiceNode(){
		return getServiceNode(ServiceType.WebData, ServiceType.WFS);
	}
	
	/**
	 * Gets SOS node.
	 * @return
	 */
	public List<ServiceTreeNode> getSOSServiceNode(){
		return getServiceNode(ServiceType.SensorWeb,ServiceType.SOS);
	}
	
	/**
	 * Gets service node.
	 * @param parentType
	 * 			the service type of parent.
	 * @param serviceType
	 * 			service type.
	 * @return
	 */
	@SuppressWarnings("rawtypes")
	private List<ServiceTreeNode> getServiceNode(ServiceType parentType,ServiceType serviceType){
		List<ServiceTreeNode> sosList = new ArrayList<ServiceTreeNode>(3);
		 TreeNode root = (TreeNode) tree.getModel().getRoot();
		 if (!root.isLeaf()) {
				for (Enumeration e = root.children(); e.hasMoreElements();) {
					TreeNode n = (TreeNode) e.nextElement();
					if (n instanceof TreeNodeImpl) {
						TreeNodeImpl nodeImpl = (TreeNodeImpl)n;
						if (nodeImpl.getServiceType()==parentType) {
							for (Enumeration node = nodeImpl.children(); node.hasMoreElements();) {
								Object object = node.nextElement();
								if (object instanceof ServiceTreeNode) {
									ServiceTreeNode serviceTreeNode = (ServiceTreeNode)object;
									if (serviceTreeNode.getServiceType()==serviceType) {
										sosList.add(serviceTreeNode);
									}
								}
								
							}
						}
						
					}
				}
		 }
		 return sosList;
	
	}
}
