/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.tree.utils;

import java.io.File;
import java.io.FileWriter;
import java.util.List;

import javax.swing.JOptionPane;

import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;

import cn.edu.whu.lmars.tree.nodes.ServiceTreeNode;
import conf.PathConf;

/**
 * 
 * This class operates configuration file of service.
 * 
 * @author Mingda Zhang
 * 
 */
public class TreeConfigUtility {
	private String confPath = PathConf.getServicConfDir();
	private String url = "url";
	private String id = "id";

	/**
	 * Writes document to configuration file.
	 * 
	 * @param document
	 * 
	 * @return
	 */
	public boolean writeXml(Document document) {
		boolean flag = false;
		try {
			XMLWriter output = new XMLWriter(new FileWriter(new File(confPath)));
			output.write(document);
			output.close();
			flag = true;
		} catch (Exception e) {
			e.printStackTrace();
			flag = false;
		}
		return flag;
	}

	/**
	 * Deletes service from configuration file.
	 * 
	 * @param xpath
	 * @return
	 */
	@SuppressWarnings("rawtypes")
	private boolean deleteService(String xpath) {
		boolean flag = false;
		SAXReader reader = new SAXReader();
		try {
			Document document = reader.read(new File(confPath));
			List nodes = document.selectNodes(xpath);
			if (nodes.size() == 0) {
				JOptionPane.showMessageDialog(null,
						"Error: no finding this service.");
				return false;
			}
			Element service = (Element) nodes.get(0);
			Element root = service.getParent();
			root.remove(service);

			flag = writeXml(document);
		} catch (Exception e) {
		}
		return flag;
	}

	/**
	 * Deletes WPS from configuration file.
	 * 
	 * @param url
	 *            wps address.
	 * @return
	 */
	public boolean deleteWps(String url) {
		String wpsXpath = "//WPSServices/WPSService[@" + this.url + "='" + url
				+ "']";
		return deleteService(wpsXpath);
	}

	/**
	 * Deletes SOS from configuration file.
	 * 
	 * @param url
	 *            sos address.
	 * @return
	 */
	public boolean deleteSos(String url) {
		String sosXpath = "//SensorWeb/SOSService[@" + this.url + "='" + url
				+ "']";
		return deleteService(sosXpath);
	}

	/**
	 * Deletes WFS from configuration file.
	 * 
	 * @param url
	 *            wfs address.
	 * @return
	 */
	public boolean deleteWfs(String url) {
		String wfsXpath = "//WebDatas/WFSService[@" + this.url + "='" + url
				+ "']";
		return deleteService(wfsXpath);
	}

	/**
	 * Deletes WCS from configuration file.
	 * 
	 * @param url
	 *            wcs address.
	 * @return
	 */
	public boolean deleteWcs(String url) {
		String wcsXpath = "//WebDatas/WCSService[@" + this.url + "='" + url
				+ "']";
		return deleteService(wcsXpath);
	}

	/**
	 * Adds SOS to configuration file.
	 * 
	 * @param serviceNode
	 *            service tree node
	 * @param nodeList
	 *            tree node list.
	 * @return
	 */
	public boolean addSosService(ServiceTreeNode serviceNode,
			List<String> nodeList) {
		String xpath = "//SensorWeb";
		String childName = "Sensor";
		String name = "SOSService";
		return addService(xpath, name, childName, serviceNode, nodeList);
	}

	/**
	 * Adds WPS to configuration file.
	 * 
	 * @param serviceNode
	 *            service tree node
	 * @param nodeList
	 *            tree node list.
	 * @return
	 */
	public boolean addWpsService(ServiceTreeNode serviceNode,
			List<String> nodeList) {
		String xpath = "//WPSServices";
		String childName = "Process";
		String name = "WPSService";
		return addService(xpath, name, childName, serviceNode, nodeList);
	}

	/**
	 * Adds WCS to configuration file.
	 * 
	 * @param serviceNode
	 *            service tree node
	 * @param nodeList
	 *            tree node list.
	 * @return
	 */
	public boolean addWcsService(ServiceTreeNode serviceNode,
			List<String> nodeList) {
		String xpath = "//WebDatas";
		String childName = "WCSData";
		String name = "WCSService";
		return addService(xpath, name, childName, serviceNode, nodeList);
	}

	/**
	 * Adds WFS to configuration file.
	 * 
	 * @param serviceNode
	 *            service tree node
	 * @param nodeList
	 *            tree node list.
	 * @return
	 */
	public boolean addWfsService(ServiceTreeNode serviceNode,
			List<String> nodeList) {
		String xpath = "//WebDatas";
		String childName = "WFSData";
		String name = "WFSService";
		return addService(xpath, name, childName, serviceNode, nodeList);
	}

	/**
	 * Adds service to configuration file.
	 * 
	 * @param xpath
	 *            xpath to locate service
	 * @param name
	 *            service name
	 * @param childName
	 *            child node name
	 * @param serviceNode
	 *            service tree node.
	 * @param nodeList
	 *            tree node list.
	 * @return
	 */
	private boolean addService(String xpath, String name, String childName,
			ServiceTreeNode serviceNode, List<String> nodeList) {
		SAXReader reader = new SAXReader();
		try {
			Document doc = reader.read(new File(confPath));
			Element wpsEle = (Element) doc.selectSingleNode(xpath);
			if (wpsEle == null) {
				return false;
			}
			Element serviceEle = wpsEle.addElement(name);

			serviceEle.addAttribute("version", serviceNode.getVersion());
			serviceEle.addAttribute("name", serviceNode.getName());
			serviceEle.addAttribute("url", serviceNode.getUrl());
			serviceEle.addAttribute(this.id, serviceNode.getId());

			for (String process : nodeList) {
				Element element = serviceEle.addElement(childName);
				element.setText(process);
			}
			XMLWriter output = new XMLWriter(new FileWriter(new File(
					PathConf.getServicConfDir())));
			output.write(doc);
			output.close();
		} catch (Exception e) {
			// TODO: handle exception
			return false;
		}
		return true;
	}
}
