/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.utils;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLConnection;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.log4j.Logger;

import cn.edu.whu.lmars.service.beans.ServiceType;

/**
 * Utility class.
 * 
 * @author Mingda Zhang
 * 
 */
public class BaseTool {

	public static Logger logger = Logger.getLogger(BaseTool.class);

	/**
	 * Parses long time to string
	 * 
	 * @param l
	 *            time long
	 */
	public static String getTimeStringByLong(long l) {
		String now = null;
		if (l == 0) {
			now = "";
		} else {
			Date d = new Date(l);
			SimpleDateFormat format = new SimpleDateFormat(
					"yyyy-MM-dd HH:mm:ss:SSS");
			now = format.format(d);
		}

		return now;
	}

	/**
	 * Gets time in SOS time format.
	 */
	public static String getTimeIntervalForSOSByStr() {
		String d = null;
		try {
			SimpleDateFormat sdf = new SimpleDateFormat(
					"yyyy-MM-dd]HH:mm:ss.SSS+00:00");
			Date date = new Date();
			d = sdf.format(date);
			String[] n = d.split("\\]");
			d = new StringBuffer().append(n[0]).append("T").append(n[1])
					.toString();
			long pretime = date.getTime() - 200;
			Date predate = new Date(pretime);
			String d2 = sdf.format(predate);
			n = d2.split("\\]");
			d = new StringBuffer().append(d).append(" ").append(n[0])
					.append("T").append(n[1]).toString();
		} catch (Exception e) {
			e.printStackTrace();
		}

		return d;
	}

	/**
	 * Gets SOS time stamp.
	 */
	public static String getTimeStampForSOSByStr() {
		String d = null;
		try {
			SimpleDateFormat sdf = new SimpleDateFormat(
					"yyyy-MM-dd]HH:mm:ss.SSS+00:00");
			d = sdf.format(new Date());
			String[] n = d.split("\\]");
			d = new StringBuffer().append(n[0]).append("T").append(n[1])
					.toString();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return d;
	}

	/**
	 * Gets time stamp by string.
	 * 
	 * @param s
	 */
	public static long getTimeStampByString(String s) {
		Date d = null;
		try {
			SimpleDateFormat sdf = new SimpleDateFormat(
					"yyyy-MM-dd HH:mm:ss:SSS");
			d = sdf.parse(s);
			// System.out.println(d.getTime());
		} catch (Exception e) {
			e.printStackTrace();
		}
		return d.getTime();
	}

	/**
	 * Get time string from double.
	 * 
	 * @param t
	 * @return
	 */
	public static String getTimeStringByDouble(double t) {
		String now = "";
		if (t != 0.0) {
			Date d = new Date((long) t);
			SimpleDateFormat format = new SimpleDateFormat(
					"yyyy-MM-dd HH:mm:ss:SSS");
			now = format.format(d);
		}
		return now;
	}

	/**
	 * Returns true, if the object is null or equals to empty string.
	 * 
	 * @param obj
	 * @return
	 */
	public static boolean isNull(Object obj) {
		boolean isnull = false;
		if (obj == null || obj == "" || "".equals(obj)) {
			isnull = true;
		}
		return isnull;
	}

	/**
	 * Downloads file from url.
	 * 
	 * @param fileURL
	 *            the file to be downloaded.
	 * @param path
	 *            the local location of file.
	 * @param targetName
	 *            the file name.
	 */
	public static boolean downloadFileByURL(String fileURL, String path,
			String targetName) {
		boolean issucess = false;
		Logger log = Logger.getLogger(BaseTool.class);
		try {
			URL url = new URL(fileURL);
			URLConnection cn = url.openConnection();
			BufferedReader response = new BufferedReader(new InputStreamReader(
					cn.getInputStream()));

			String result = "";
			String line;
			while ((line = response.readLine()) != null) {
				result = result + "\n" + line;
			}
			String engineResponse = result.toString();
			log.info(engineResponse);

			log.info(path + targetName);

			writeStringIntoFile(engineResponse, path, targetName);

		} catch (Exception e) {
			log.error(new StringBuffer()
					.append("Error in downloading file from ").append(fileURL)
					.append("\n").append(e.getLocalizedMessage()).toString());
			e.printStackTrace();
		}

		return issucess;
	}

	/**
	 * Downloads zip file.
	 * 
	 * @param savedir
	 * @param zipUrl
	 */
	public static void downZIP(String savedir, String zipUrl) {
		try {
			URLConnection conn = new URL(zipUrl).openConnection();
			InputStream is = conn.getInputStream();
			File file = new File(savedir);
			OutputStream os = new FileOutputStream(file);
			int len = -1;
			byte temp[] = new byte[1024];
			while ((len = is.read(temp)) != -1) {
				os.write(temp, 0, len);
			}
			is.close();
			os.close();
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	/**
	 * Downloads image.
	 * 
	 * @param savedir
	 * @param imgUrl
	 */
	public static void down(String savedir, String imgUrl) {
		Logger log = Logger.getLogger(BaseTool.class);
		File f = new File(savedir);
		byte[] buffer = new byte[8 * 1024];
		URL u;
		URLConnection connection = null;
		try {
			u = new URL(imgUrl);
			connection = u.openConnection();
		} catch (Exception e) {
			log.info("ERR:" + imgUrl);
			return;
		}
		connection.setReadTimeout(100000);
		InputStream is = null;
		FileOutputStream fos = null;
		try {
			f.createNewFile();
			is = connection.getInputStream();
			fos = new FileOutputStream(f);
			int len = 0;
			while ((len = is.read(buffer)) != -1) {
				fos.write(buffer, 0, len);
			}

		} catch (Exception e) {
			f.delete();
			e.printStackTrace();
		} finally {
			if (fos != null) {
				try {
					fos.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			if (is != null) {
				try {
					is.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		log.info(buffer);
		// buffer = null;
		// System.gc();
	}

	/**
	 * Writes string to file.
	 * 
	 * @param engineResponse
	 * @param path
	 * @param targetName
	 */
	public static void writeStringIntoFile(String engineResponse, String path,
			String targetName) {
		try {
			File nf = new File(path);
			if (!nf.exists()) {
				nf.mkdirs();
			}
			nf = new File(new StringBuffer().append(path).append(targetName)
					.toString());
			nf.createNewFile();
			FileWriter wr = new FileWriter(nf);
			wr.write(engineResponse);
			wr.close();
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	/**
	 * Gets service type from a string.
	 * 
	 * @param type
	 *            the type string.
	 * @return
	 */
	public static ServiceType getServiceTypeFromString(String type) {
		if (BaseTool.isNull(type)) {
			return null;
		}
		return Enum.valueOf(ServiceType.class, type.toUpperCase());
	}
}
