/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.utils;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Node;
import org.dom4j.XPath;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;

/**
 * Dom4j xml tool.
 * 
 * @author Mingda Zhang
 * 
 */
public class Dom4jXMLTool {
	private static Logger logger = Logger.getLogger(Dom4jXMLTool.class);

	/**
	 * Changes xml into indented format.
	 * 
	 * @param str
	 * @return
	 */
	public static String format(String str) {
		StringReader in = null;
		StringWriter out = null;
		String result = null;
		try {
			SAXReader reader = new SAXReader();
			str = str.trim();
			in = new StringReader(str);
			Document doc = reader.read(in);

			OutputFormat formate = OutputFormat.createPrettyPrint();

			out = new StringWriter();

			XMLWriter writer = new XMLWriter(out, formate);

			writer.write(doc);
			result = out.toString();
		} catch (Exception ioe) {
			logger.error("Errors occured when formatting the xml");
			result = null;
		} finally {
			try {
				if (in != null) {
					in.close();
				}
				if (out != null) {
					out.close();
				}
			} catch (Exception e) {
				logger.error("Errors occured when close the reader or writer");
			}
		}
		return result;
	}

	/**
	 * Selects elements from given element.
	 * 
	 * @param map
	 *            namespace map.
	 * @param xpathExpression
	 *            xpath expression.
	 * @param element
	 *            target element.
	 * @return
	 */
	@SuppressWarnings("rawtypes")
	public static List selectXmlNodes(Map<String, String> map,
			String xpathExpression, Object element) {
		XPath xPath = DocumentHelper.createXPath(xpathExpression);
		xPath.setNamespaceURIs(map);
		return xPath.selectNodes(element);
	}

	/**
	 * Selects an single element from given element.
	 * 
	 * @param map
	 *            namespace map.
	 * @param xpathExpression
	 *            xpath expression.
	 * @param element
	 *            target element.
	 * @return
	 */
	public static Node selectSingleXmlNode(Map<String, String> map,
			String xpathExpression, Object element) {
		XPath xPath = DocumentHelper.createXPath(xpathExpression);
		xPath.setNamespaceURIs(map);
		return xPath.selectSingleNode(element);
	}

	/**
	 * Writes document to file.
	 * 
	 * @param document
	 * @param file
	 * @return
	 */
	public static boolean writeXml(Document document, File file) {
		boolean flag = false;
		try {
			XMLWriter output = new XMLWriter(new FileWriter(file));
			output.write(document);
			output.close();
			flag = true;
		} catch (Exception e) {
			e.printStackTrace();
			flag = false;
		}
		return flag;
	}

	/**
	 * Parses string to xml element.
	 * 
	 * @param xmlstring
	 */
	public static Element readElementFromString(String xmlstring) {
		Element ele = null;
		try {
			Document doc = DocumentHelper.parseText(xmlstring);
			ele = doc.getRootElement();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return ele;
	}

	/**
	 * Returns Element from a file.
	 * 
	 * @param path
	 */
	public static Element readElementFromFile(String path) {
		Element resultele = null;
		SAXReader reader = new SAXReader();
		try {
			Document doc = reader.read(new File(path));
			resultele = doc.getRootElement();

		} catch (DocumentException e) {
			String xml = readStringFromFile(path);
			resultele = readElementFromString(xml.trim());
		}
		return resultele;
	}
	
	public static Document getDocFromFile(String path){
		SAXReader reader = new SAXReader();
		try {
			Document doc = reader.read(new File(path));
			return doc;
		} catch (DocumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * Returns string from a file.
	 * 
	 * @param path
	 */
	public static String readStringFromFile(String path) {
		StringBuffer strLine = new StringBuffer();
		try {
			FileInputStream fstream = new FileInputStream(path);
			DataInputStream in = new DataInputStream(fstream);
			FileReader fr = new FileReader(path);
			BufferedReader br = new BufferedReader(fr);
			String str = null;
			while ((str = br.readLine()) != null) {
				strLine.append(str);
			}
			in.close();
		} catch (Exception e) {// Catch exception if any
			System.err.println("Error: " + e.getMessage());
		}
		return strLine.toString();
	}
}
