/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.utils;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.io.SAXReader;

import cn.edu.whu.lmars.service.beans.IService;
import cn.edu.whu.lmars.service.beans.ServiceModel;
import conf.FileLoadProperty;
import conf.PathConf;

/**
 * Tools related to file operation.
 * 
 * @author Mingda Zhang
 * 
 */
public class FileTool {

	/** logger. */
	private static Logger logger = Logger.getLogger(FileTool.class);

	/**
	 * Gets suffix of a file from a file path.
	 * 
	 * @param path
	 *            file path.
	 * @return
	 */
	public static String getFileSuffix(String path) {
		int index = path.lastIndexOf(".");
		String suffix = path.substring(index + 1);
		return suffix.toLowerCase();
	}

	/**
	 * Gets suffix of a file.
	 * 
	 * @param file
	 *            target file
	 * @return
	 */
	public static String getFileSuffix(File file) {
		return getFileSuffix(file.getAbsolutePath());
	}

	/**
	 * Gets class name according file suffix.
	 * 
	 * @param baseDir
	 *            target file.
	 * @return
	 */
	public static File getAssignSuffixFile(File baseDir) {
		Properties properties = FileLoadProperty.getInstance()
				.getClassNameProperty();
		if (baseDir.isFile()) {
			return null;
		}
		File[] files = baseDir.listFiles();

		File file;
		String suffix;
		for (int i = 0; i < files.length; i++) {
			file = files[i];
			if (file.isDirectory()) {
				return getAssignSuffixFile(file);
			} else {
				suffix = getFileSuffix(file);
				for (Object set : properties.keySet()) {
					if (suffix.equals(set.toString())) {
						return file;
					}
				}
			}

		}
		return null;
	}

	/**
	 * Gets file name without suffix from a path.
	 * 
	 * @param path
	 *            file path.
	 * @return
	 */
	public static String getFileNameNoSuffix(String path) {
		if (path.endsWith("/") || path.endsWith("\\")) {
			path = path.substring(0, path.length() - 1);
		}
		int len = path.length();
		int p = path.lastIndexOf("/");
		if (p < 0)
			p = path.lastIndexOf("\\");
		int end = path.lastIndexOf(".");
		if (end > (p + 1) && end < len) {
			return path.substring(p + 1, end);
		} else {
			return null;
		}
	}

	/**
	 * Gets a file name from file path.
	 * 
	 * @param path
	 *            file path.
	 * @return
	 */
	public static String getFileName(String path) {
		if (path.endsWith("/") || path.endsWith("\\")) {
			path = path.substring(0, path.length() - 1);
		}
		int len = path.length();
		int p = path.lastIndexOf("/");
		if (p < 0)
			p = path.lastIndexOf("\\");
		return (p >= 0 && p + 1 < len) ? path.substring(p + 1, len) : null;
	}

	/**
	 * Returns string from resource.
	 * 
	 * @param path
	 *            the path of resource.
	 * @return
	 */
	public static String getStrFromResource(String path) {
		// InputStream
		// is=FileTool.class.getClass().getResourceAsStream("/resource/res.txt");
		StringBuffer buffer = new StringBuffer();
		InputStream is = FileTool.class.getClass().getResourceAsStream(path);
		BufferedReader br = new BufferedReader(new InputStreamReader(is));
		String str = "";
		try {
			while ((str = br.readLine()) != null) {
				buffer.append(str);
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		} finally {
			if (is != null) {
				try {
					is.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		return buffer.toString();
	}

	/**
	 * Returns Document from resource.
	 * 
	 * @param path
	 *            the path of resource.
	 */
	public static Document getDocFromResource(String path) {
		InputStream in = FileTool.class.getClass().getResourceAsStream(path);
		SAXReader reader = new SAXReader();
		try {
			Document doc = reader.read(in);
			return doc;
		} catch (DocumentException e1) {
			e1.printStackTrace();
			return null;
		} finally {
			if (in != null) {
				try {
					in.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}

	}

	/**
	 * Returns InputStream from resource.
	 * 
	 * @param path
	 *            the path of resource.
	 */
	public static InputStream getStreamFromResource(String path) {
		return FileTool.class.getClass().getResourceAsStream(path);
	}

	/**
	 * Gets unique direction from the information of service.
	 * 
	 * @param service
	 *            service information.
	 */
	public static String getUniqueDirFromService(IService service) {
		StringBuffer buffer = new StringBuffer();
		String url = service.getUrl();
		buffer.append(url.replace("://", "_").replace(":", ".")
				.replace("/", "_"));
		buffer.append("_");
		buffer.append(service.getVersion());
		return buffer.toString().trim();
	}

	public static String saveToText(List<String> stringList) {
		String path = PathConf.getDataDir() + File.separator
				+ HandleTool.getUUID() + ".txt";
		File file = new File(path);
		try {
			file.createNewFile();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
		OutputStreamWriter output;
		try {
			output = new OutputStreamWriter(new FileOutputStream(file), "UTF-8");
			BufferedWriter bw = new BufferedWriter(output);
			for (String str : stringList) {
				if (str.trim().equals("")) {
					continue;
				}
				bw.write(str);
				bw.write(";\n");
			}
			bw.flush();
			bw.close();
			output.close();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
		return file.getAbsolutePath();

	}

	/**
	 * Copies one file to another.
	 * @param src
	 * 			source file.
	 * @param dst
	 * 			target file.
	 * @return
	 */
	public static boolean copyFile(File src, File dst) {
		try {
			InputStream in = new FileInputStream(src);
			OutputStream out = new FileOutputStream(dst);

			byte[] buf = new byte[1024];
			int len;
			while ((len = in.read(buf)) > 0) {
				out.write(buf, 0, len);
			}
			in.close();
			out.close();
			logger.info("Copy file from "+src.getAbsolutePath()+" to "+dst.getAbsolutePath());
			return true;
		} catch (Exception e) {
//			e.printStackTrace();
			logger.error("Error occured when copy file from "+src.getAbsolutePath()+" to "+dst.getAbsolutePath());
			logger.error(e.getMessage());
			return false;
		}

	}

	/**
	 * Writes InputStream to the file.
	 * @param in
	 * 			input stream.
	 * @param dst
	 * 			target file.
	 * @return
	 */
	public static boolean copyFile(InputStream in, File dst) {
		try {
			OutputStream out = new FileOutputStream(dst);

			byte[] buf = new byte[1024];
			int len;
			while ((len = in.read(buf)) > 0) {
				out.write(buf, 0, len);
			}
			in.close();
			out.close();
			logger.info("Writes inputstream to "+dst.getAbsolutePath());
			return true;
		} catch (Exception e) {
			logger.error("Error occured when writes inputstrem to "+dst.getAbsolutePath());
			logger.error(e.getMessage());
			return false;
		}

	}
	public static void main(String[] args) {
		IService service = new ServiceModel();
		service.setUrl(" http://geopw.whu.edu.cn:8080/wps/WebProcessingService");
		service.setVersion("0.4.0");
		String str = getUniqueDirFromService(service);
		System.out.println(str);
	}
}
