/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.utils;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import javax.media.jai.JAI;
import javax.media.jai.RenderedOp;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileNameExtensionFilter;

import org.apache.log4j.Logger;

import cn.edu.whu.lmars.drawpanel.ui.JDrawPanel;
import cn.edu.whu.lmars.figure.beans.AbstractFigure;
import cn.edu.whu.lmars.figure.beans.DataFigure;
import cn.edu.whu.lmars.figure.beans.LinkFigure;
import cn.edu.whu.lmars.figure.beans.ProcessFigure;
import cn.edu.whu.lmars.figure.utils.FigureSave;
import cn.edu.whu.lmars.figure.utils.FigureStore;
import cn.edu.whu.lmars.figure.utils.FigureUtility;
import cn.edu.whu.lmars.figure.utils.LocXY;
import cn.edu.whu.lmars.figure.utils.OpenFigure;
import cn.edu.whu.lmars.model.AtomProcess;
import cn.edu.whu.lmars.model.ComplexParam;
import cn.edu.whu.lmars.model.IModel;
import cn.edu.whu.lmars.model.LiteralParam;
import cn.edu.whu.lmars.model.OutPutParam;
import cn.edu.whu.lmars.model.util.OpenModel;
import cn.edu.whu.lmars.model.util.PureModelStore;
import cn.edu.whu.lmars.model.util.SaveModel;
import cn.edu.whu.lmars.run.beans.ExecutionState;
import cn.edu.whu.lmars.run.bind.beans.BindAtom;
import cn.edu.whu.lmars.run.bind.beans.BindProcess;
import cn.edu.whu.lmars.run.bind.beans.BindStore;
import cn.edu.whu.lmars.run.bind.utils.BindTool;
import cn.edu.whu.lmars.service.adapter.AdapterFactory;
import cn.edu.whu.lmars.service.adapter.base.IAdapter;
import cn.edu.whu.lmars.service.adapter.base.IOperationInfo;
import cn.edu.whu.lmars.service.adapter.base.IParameterInfo;
import cn.edu.whu.lmars.service.beans.IService;

import com.sun.image.codec.jpeg.JPEGCodec;
import com.sun.image.codec.jpeg.JPEGImageEncoder;

import conf.GlobeParam;
import conf.PathConf;

/**
 * Utility class
 * 
 * @author Mingda Zhang
 * 
 */
public class HandleTool {
	private static HandleTool instance = null;
	/** logger */
	private Logger logger = Logger.getLogger(HandleTool.class);

	private HandleTool() {
	}

	/**
	 * Gets an instance of the class.
	 * 
	 * @return
	 */
	public static HandleTool getInstance() {
		if (instance == null) {
			instance = new HandleTool();
		}
		return instance;
	}

	/**
	 * Returns a id.
	 */
	public static String getUUID() {
		String str = UUID.randomUUID().toString();
		str = str.substring(0, 8) + str.substring(9, 13)
				+ str.substring(14, 18) + str.substring(19, 23)
				+ str.substring(24);
		return str;
	}

	/**
	 * Moves all the figures contained in the current JDrawPanel down.
	 * 
	 * @param num
	 *            The size of the movement
	 */
	public static void moveAllDown(int num) {
		JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
		FigureUtility.getInstance().moveAllDown(
				drawPanel.getFigureStore().getAllFigures(), num);
	}

	/**
	 * Moves all the figures contained in the current JDrawPanel right.
	 * 
	 * @param num
	 *            The size of the movement
	 */
	public static void moveAllRight(int num) {
		JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
		FigureUtility.getInstance().moveAllRight(
				drawPanel.getFigureStore().getAllFigures(), num);
	}

	/**
	 * Creates models by drag.
	 * 
	 * @param point
	 *            the release point of drag.
	 * @param operationName
	 *            the name of dragged operation.
	 * @param service
	 *            the information of dragged service.
	 * 
	 */
	public void createModelByDrag(Point point, String operationName,
			IService service) {
		LocXY locXY = new LocXY((int) point.getX(), (int) point.getY(), 90, 60);
		JDrawPanel drawPanel = GlobeParam.getCurrentJDrawPanel();
		IAdapter adapter = AdapterFactory.getInstance().getAdapter(service);
		if (adapter == null) {
			JOptionPane.showMessageDialog(null, "Failed to parse:\nservice:"
					+ service.getUrl() + "\ntype:" + service.getServiceType());
			return;
		}
		IOperationInfo operationInfo = adapter.getOperationInfo(operationName);
		if (operationInfo == null) {
			JOptionPane.showMessageDialog(null, "Failed to parse:\nservice:"
					+ service.getUrl() + "\noperation:" + operationName);
			return;
		}
		String modelName = "model name";
		switch (service.getServiceType()) {
		case SOS:
			modelName = "GetObservation";
			break;

		default:
			modelName = operationName;
			break;
		}
		String handleID = HandleTool.getUUID();
		AtomProcess atomProcess = new AtomProcess();
		atomProcess.setName(modelName);
		ProcessFigure processFigure = new ProcessFigure(locXY.X, locXY.Y,
				locXY.Width, locXY.Height);
		atomProcess.setUUID(handleID);
		int i = locXY.X - 120, j = locXY.Y - 100, width = locXY.Width, height = locXY.Height;
		if (i < 0)
			i = 0;
		if (j < 0)
			j = 0;
		String dataID;
		List<IParameterInfo> parameters = operationInfo
				.getInputParameterInfos();
		for (IParameterInfo parameter : parameters) {
			dataID = handleID + "#" + parameter.getName();
			if (parameter.getMeta().get_type().toLowerCase()
					.equals("complexdata")) {
				ComplexParam complexParam = new ComplexParam();
				complexParam.setName(parameter.getName());
				atomProcess.addComplexParam(complexParam);

				DataFigure dataFigure = new DataFigure(i, j, width, height);
				dataFigure.setModel(complexParam);
				dataFigure.setState("nul");
				dataFigure.setUuid(dataID);
				processFigure.addDataFigure(dataFigure);
			} else {
				LiteralParam param = new LiteralParam();
				param.setName(parameter.getName());
				String value = parameter.getMeta().get_allowValue();
				if (!BaseTool.isNull(value)) {
					param.setValue(value);
				}
				atomProcess.addLiteralParam(param);
			}
			j += height + 20;
		}

		// result data
		dataID = handleID + "#" + "result";
		OutPutParam outPut = new OutPutParam();
		outPut.setName("OutputData");
		atomProcess.setOutPutParam(outPut);
		DataFigure figure = new DataFigure(locXY.X + width + 30, locXY.Y,
				width, height);
		figure.setModel(outPut);
		figure.setState("res");
		figure.setUuid(dataID);
		processFigure.addDataFigure(figure);
		LinkFigure linkFigure = new LinkFigure(processFigure.pointR,
				figure.pointL, processFigure, figure);
		linkFigure.setUuid(figure.getUuid());
		processFigure.addLinkFigure(linkFigure);

		processFigure.setModel(atomProcess);
		processFigure.setUuid(handleID);
		FigureStore figureStore = drawPanel.getFigureStore();
		figureStore.addProcessFigure(processFigure);
		PureModelStore modelStore = drawPanel.getModelStore();
		modelStore.addAtomProcess(atomProcess);

		GlobeParam.getCurrentJDrawPanel().setChanged(true);
		HandleTool.refreshFigShow();

		// Binding
		BindStore bindStore = GlobeParam.getCurrentJDrawPanel().getBindStore();
		if (bindStore == null) {
			bindStore = new BindStore();
			GlobeParam.getCurrentJDrawPanel().setBindStore(bindStore);
		}
		BindProcess bindProcess = new BindProcess();
		bindProcess.setModelID(atomProcess.getUUID());
		BindAtom bindAtom = new BindAtom();
		bindAtom.setId(getUUID());
		bindAtom.setName(atomProcess.getName() + "_atom_Bind");
		bindAtom.setOperationName(operationName);
		bindAtom.setServiceType(service.getServiceType());
		bindAtom.setUrl(service.getUrl());
		bindAtom.setVersion(service.getVersion());
		for (IParameterInfo parameterInfo : operationInfo
				.getInputParameterInfos()) {
			bindAtom.addParamMapInfo(bindAtom.new ParamMapInfo(parameterInfo
					.getName(), parameterInfo.getName()));
		}
		bindProcess.addMapInfo(bindAtom);
		bindProcess.setCurMapID(bindAtom.getId());
		bindStore.addBindProcess(bindProcess);

		// atom_bind
		logger.info("Automatically bound");
	}

	/**
	 * Repaints the current JDrawPanel.
	 */
	public static void refreshFigShow() {
		GlobeParam.getCurrentJDrawPanel().repaint();
	}

	/**
	 * Gets the selected models from the JDrawPanel.
	 * 
	 * @param drawPanel
	 *            the current JDrawPanel.
	 * @return
	 */
	public List<IModel> getChoosenModels(JDrawPanel drawPanel) {
		FigureStore figureStore = drawPanel.getFigureStore();
		List<AbstractFigure> figures = figureStore.getAllFigures();
		List<IModel> models = new ArrayList<IModel>();
		for (AbstractFigure figure : figures) {
			if (figure.isChoosen()) {
				if (figure.getModel() != null) {
					models.add(figure.getModel());
				}
			}
		}
		return models;

	}

	/*
	 * JDrawPanel requests focus. public void getFocus() {
	 * GlobeParam.getCurrentJDrawPanel().requestFocus(); }
	 */

	/**
	 * Copies the content to the clipboard.
	 */
	public void setSysClipboardText(String writeMe) {
		Clipboard clip = Toolkit.getDefaultToolkit().getSystemClipboard();
		Transferable tText = new StringSelection(writeMe);
		clip.setContents(tText, null);
	}

	/**
	 * Locates the component to center of its parent component.
	 * 
	 * @param c
	 * @return
	 */
	public Dimension centerFrame(java.awt.Component c) {
		Dimension ownerSize;
		Point ownerLocation;
		if (c.getParent() == null) {
			ownerSize = Toolkit.getDefaultToolkit().getScreenSize();
			ownerLocation = new Point(0, 0);
		} else {
			ownerSize = c.getParent().getSize();
			ownerLocation = c.getParent().getLocation();
			if (ownerSize.height == 0 || ownerSize.width == 0
					|| !c.getParent().isVisible()) {
				ownerSize = Toolkit.getDefaultToolkit().getScreenSize();
				ownerLocation = new Point(0, 0);
			}
		}
		// Center the window
		Dimension frameSize = c.getSize();
		if (frameSize.height > ownerSize.height) {
			frameSize.height = ownerSize.height;
		}
		if (frameSize.width > ownerSize.width) {
			frameSize.width = ownerSize.width;
		}
		Dimension dimension = new Dimension(ownerLocation.x
				+ (ownerSize.width - frameSize.width) / 2, ownerLocation.y
				+ (ownerSize.height - frameSize.height) / 2);
		return dimension;
	}

	/**
	 * If the workspace doesn't exist, creates related files.
	 * 
	 * @param dir
	 *            workspace path.
	 * @return
	 */
	public boolean checkDir(String dir) {
		boolean flag = true;
		File dataFile = new File(PathConf.getDataDir());
		File modelFile = new File(PathConf.getModelDir());
		File configFile = new File(PathConf.getConfDir());
		File serviceFile = new File(PathConf.getServicConfDir());
		File tabFile = new File(PathConf.getTabConfDir());
		File locFile = new File(PathConf.getLocConfDir());
		try {
			if (!dataFile.exists()) {
				dataFile.mkdirs();
			}
			if (!modelFile.exists()) {
				modelFile.mkdirs();
			}
			if (!configFile.exists()) {
				configFile.mkdirs();
				serviceFile.createNewFile();
				tabFile.createNewFile();
				locFile.createNewFile();

				FileTool.copyFile(FileTool
						.getStreamFromResource(PathConf.SERVICE_CONFIG_MODEL),
						serviceFile);
				initGeoPWWPS();
				FileTool.copyFile(FileTool
						.getStreamFromResource(PathConf.TAB_CONFIG_MODEL),
						tabFile);
				FileTool.copyFile(FileTool
						.getStreamFromResource(PathConf.Loc_Config_Model),
						locFile);

				/*
				 * Document doc =
				 * FileTool.getDocFromResource(PathConf.SERVICE_CONFIG_MODEL);
				 * Dom4jXMLTool.writeXml(doc,serviceFile); doc =
				 * FileTool.getDocFromResource(PathConf.TAB_CONFIG_MODEL);
				 * Dom4jXMLTool.writeXml(doc,tabFile); doc =
				 * FileTool.getDocFromResource(PathConf.Loc_Config_Model);
				 * Dom4jXMLTool.writeXml(doc,locFile);
				 */
				/*
				 * copyFile(new File(PathConf.SERVICE_CONFIG_MODEL),
				 * serviceFile); copyFile(new File(PathConf.TAB_CONFIG_MODEL),
				 * tabFile); copyFile(new File(PathConf.Loc_Config_Model),
				 * locFile);
				 */
			} else {
				if (!serviceFile.exists()) {
					serviceFile.createNewFile();
					/*
					 * Document doc =
					 * FileTool.getDocFromResource(PathConf.SERVICE_CONFIG_MODEL
					 * ); Dom4jXMLTool.writeXml(doc,serviceFile);
					 */
					FileTool.copyFile(
							FileTool.getStreamFromResource(PathConf.SERVICE_CONFIG_MODEL),
							serviceFile);
					initGeoPWWPS();
					// copyFile(new File(PathConf.SERVICE_CONFIG_MODEL),
					// serviceFile);
				}
				if (!tabFile.exists()) {
					tabFile.createNewFile();
					/*
					 * Document doc =
					 * FileTool.getDocFromResource(PathConf.TAB_CONFIG_MODEL);
					 * Dom4jXMLTool.writeXml(doc,tabFile);
					 */
					FileTool.copyFile(FileTool
							.getStreamFromResource(PathConf.TAB_CONFIG_MODEL),
							tabFile);
					// copyFile(new File(PathConf.TAB_CONFIG_MODEL), tabFile);
				}
				if (!locFile.exists()) {
					locFile.createNewFile();
					/*
					 * Document doc =
					 * FileTool.getDocFromResource(PathConf.Loc_Config_Model);
					 * Dom4jXMLTool.writeXml(doc,locFile);
					 */
					// copyFile(new File(PathConf.Loc_Config_Model), locFile);
					FileTool.copyFile(FileTool
							.getStreamFromResource(PathConf.Loc_Config_Model),
							locFile);
				}

			}

		} catch (Exception e) {
			flag = false;
		}
		return flag;

	}

	private void initGeoPWWPS() {
		String dataDir = PathConf.getDataDir();
		String dir = dataDir + File.separator + PathConf.Cache_Name
				+ File.separator + PathConf.GEOPW_WPS_NAME;
		File dirFile = new File(dir);
		if (!dirFile.exists()) {
			dirFile.mkdirs();
		}
		boolean flag = true;
		File capFile = new File(dir + File.separator + "GetCapabilities.xml");
		InputStream in = FileTool.getStreamFromResource(PathConf.GEOPW_WPS_Src
				+ "GetCapabilities.xml");
		if (in != null) {
			if (!capFile.exists()) {
				try {
					capFile.createNewFile();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					flag = false;
					logger.error("Error occured when create file "
							+ capFile.getAbsolutePath());
				}
			}
			if (flag) {
				FileTool.copyFile(in, capFile);
			}
			if (in != null) {
				try {
					in.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					// e.printStackTrace();
				}
			}
		}

		InputStream algIn = FileTool
				.getStreamFromResource(PathConf.GEOPW_WPS_Algorithm);
		if (algIn != null) {
			try {
				InputStreamReader read = new InputStreamReader(algIn);
				BufferedReader bufferedReader = new BufferedReader(read);
				StringBuffer stringBuffer = new StringBuffer();
				String row;
				while ((row = bufferedReader.readLine()) != null) {
					stringBuffer.append(row);
				}
				String[] algorithms = stringBuffer.toString().split(",");
				try {
					bufferedReader.close();
					read.close();
					algIn.close();
				} catch (Exception e) {
					// TODO: handle exception
					logger.error("Error occured when close the inputstream");
				}

				for (String algorithm : algorithms) {
					algorithm = algorithm.trim();
					File algFile = new File(dir + File.separator + algorithm
							+ ".xml");
					flag = true;
					if (!algFile.exists()) {
						try {
							algFile.createNewFile();
						} catch (IOException e) {
							// TODO Auto-generated catch block
							flag = false;
							logger.error("Error occured when create file "
									+ capFile.getAbsolutePath());
						}
					}
					if (!flag) {
						continue;
					}

					InputStream inStream = FileTool
							.getStreamFromResource(PathConf.GEOPW_WPS_Src
									+ algorithm + ".xml");
					if (inStream != null) {
						FileTool.copyFile(inStream, algFile);
					}
					if (inStream != null) {
						try {
							inStream.close();
						} catch (IOException e) {
							// TODO Auto-generated catch block
							// e.printStackTrace();
						}
					}
				}
			} catch (Exception e) {
				// TODO: handle exception
			}
		}
	}

	/**
	 * Copies the contents of the file from one file to another.
	 * 
	 * @param inFile
	 *            source file.
	 * @param outFile
	 *            destination file.
	 * @return
	 */
	public boolean copyFile(File inFile, File outFile) {
		// long time=new Date().getTime();
		int length = 2097152;
		boolean flag = true;
		FileInputStream in;
		try {
			in = new FileInputStream(inFile);

			FileOutputStream out = new FileOutputStream(outFile);
			FileChannel inC = in.getChannel();
			FileChannel outC = out.getChannel();
			ByteBuffer b = null;
			while (true) {
				if (inC.position() == inC.size()) {
					inC.close();
					outC.close();
					return true;
				}
				if ((inC.size() - inC.position()) < length) {
					length = (int) (inC.size() - inC.position());
				} else
					length = 2097152;
				b = ByteBuffer.allocateDirect(length);
				inC.read(b);
				b.flip();
				outC.write(b);
				outC.force(false);

			}
		} catch (Exception e) {
			e.printStackTrace();
			flag = false;
		}
		return flag;
	}

	/**
	 * Changes the state of parameters in a model to 'value'.
	 * 
	 * @param processFigure
	 */
	public void changeNulToValue(ProcessFigure processFigure) {
		if (processFigure == null) {
			return;
		}
		int num = 1;
		int Px = processFigure.getX();
		int Py = processFigure.getY();
		int width = processFigure.getWidth();
		int heigh = processFigure.getHeight();
		int x = Px, y = Py;
		for (DataFigure dataFigure : processFigure.getDataFigures()) {
			if (dataFigure.getState().equals("nul")) {
				dataFigure.setState("val");
				// first one add to the left
				if (num == 1) {
					x = Px - width / 2 - width;
					y = Py;
					x = (x < 0 ? 0 : x);
				} else if (num == 2) { // second to the upleft
					x = Px;
					y = Py - heigh * 2;
					y = (y < 0 ? 0 : y);
				} else if (num == 3) {
					y = Py + heigh * 2;
					x = Px;
				} else if (num == 4) {
					x = Px - width / 2 - width;
					y = Py - heigh * 2;
					x = (x < 0 ? 0 : x);
					y = (y < 0 ? 0 : y);
				}
				num++;
				dataFigure.resize(x, y, width, heigh);
				LinkFigure linkFigure = new LinkFigure(dataFigure.pointR,
						processFigure.pointL, dataFigure, processFigure);
				linkFigure.setUuid(dataFigure.getUuid());
				processFigure.addLinkFigure(linkFigure);
			}
		}
		refreshFigShow();
	}

	/**
	 * Saves models. This method determines to create a new file or save to an
	 * existing file.
	 * 
	 * @param drawPanel
	 *            JDrawPanel contains target models.
	 */
	public void saveModel(JDrawPanel drawPanel) {
		String path = drawPanel.getModelPath();
		PureModelStore modelStore = drawPanel.getModelStore();
		FigureStore figureStore = drawPanel.getFigureStore();
		BindStore bindStore = drawPanel.getBindStore();
		boolean changed = drawPanel.isChanged();

		if (path == null || path.equals("")) {
			if (figureStore.getAllFigures().size() > 0) {
				int conf = JOptionPane.showConfirmDialog(
						GlobeParam.getClient(),
						"Saves models in '" + drawPanel.getName() + "' panel",
						"Save Models", JOptionPane.YES_NO_OPTION);
				if (conf == JOptionPane.YES_OPTION) {
					saveModelToNewFile(drawPanel);
				}
			}
		} else {
			if (changed) {
				int conf = JOptionPane.showConfirmDialog(
						GlobeParam.getClient(),
						"Models in '" + drawPanel.getName()
								+ "' have been modified. Save changed?",
						"Save Models", JOptionPane.YES_NO_OPTION);
				if (conf == JOptionPane.YES_OPTION) {
					saveModelToFile(path, figureStore, modelStore, bindStore);
					drawPanel.setChanged(false);
				}
			}
		}

	}

	/**
	 * Saves models to a new file.
	 * 
	 * @param drawPanel
	 * @return
	 */
	private boolean saveModelToNewFile(JDrawPanel drawPanel) {
		JFileChooser fileChooser;
		PureModelStore modelStore = drawPanel.getModelStore();
		FigureStore figureStore = drawPanel.getFigureStore();
		BindStore bindStore = drawPanel.getBindStore();
		if (PathConf.WORK_PATH.trim().equals("")) {
			fileChooser = new JFileChooser(".");
		} else {
			fileChooser = new JFileChooser(new File(PathConf.getModelDir()));
		}
		boolean flag = false;
		FileNameExtensionFilter filter = new FileNameExtensionFilter("*.del",
				"del");
		fileChooser.setFileFilter(filter);
		fileChooser.setAcceptAllFileFilterUsed(false);
		int returnVal = fileChooser.showSaveDialog(GlobeParam.getClient());
		boolean existFlag = false;
		boolean again = false;
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			File file = fileChooser.getSelectedFile();
			String path = file.toString();
			if (!path.endsWith(".del")) {
				path = path + ".del";
				file = new File(path);
			}

			existFlag = file.exists();
			if (existFlag) {
				int flag2 = JOptionPane.showConfirmDialog(
						GlobeParam.getClient(),
						"File exists. Do you wish to overwrite? ", "Question",
						JOptionPane.YES_NO_OPTION);
				if (flag2 != JOptionPane.YES_OPTION) {
					again = true;
				}
			}

			if (!again) {
				boolean flag2 = saveModelToFile(path, figureStore, modelStore,
						bindStore);
				if (flag2) {
					drawPanel.setChanged(false);
					JOptionPane.showMessageDialog(null,
							"Save Models successfully.");
				} else {
					JOptionPane.showMessageDialog(null, "Fail to save models.");
				}
				drawPanel.setModelPath(path);
			} else {
				flag = saveModelToNewFile(drawPanel);
				
			}
		} else {
			System.out.println("Cancel");
		}
		return flag;
	}

	/**
	 * Saves models, figures and binding models to file.
	 * 
	 * @param path
	 *            file path.
	 * @param figureStore
	 *            figures to save.
	 * @param modelStore
	 *            models to save.
	 * @param bindStore
	 *            binding models to save.
	 * @return
	 */
	private boolean saveModelToFile(String path, FigureStore figureStore,
			PureModelStore modelStore, BindStore bindStore) {

		boolean flag1 = new SaveModel().saveToFile(path, modelStore);
		String figurePath = path.replace(".del", ".gur");
		File figureFile = new File(figurePath);
		if (!figureFile.exists()) {
			try {
				figureFile.createNewFile();
			} catch (IOException e) {
				e.printStackTrace();
				return false;
			}
		}
		boolean flag2 = new FigureSave().saveToFile(figurePath, figureStore);
		String bindPath = path.replace(".del", ".bid");
		File bindFile = new File(bindPath);
		@SuppressWarnings("unused")
		boolean bindFlag = false;
		if (!bindFile.exists()) {
			try {
				bindFile.createNewFile();
				bindFlag = BindTool.saveBindStore(modelStore,bindStore, bindFile);
			} catch (IOException e) {
				e.printStackTrace();
			}
		} else {
			bindFlag = BindTool.saveBindStore(modelStore,bindStore, bindFile);
		}
		if (flag1 && flag2) {
			return true;
		}
		return false;
	}

	/**
	 * Creates models from model file.
	 * 
	 * @param drawPanel
	 *            the current JDrawPanel.
	 * @return
	 */
	public boolean openModel(JDrawPanel drawPanel) {
		JFileChooser fileChooser;
		if (PathConf.WORK_PATH.equals("")) {
			fileChooser = new JFileChooser(".");
		} else {
			fileChooser = new JFileChooser(new File(PathConf.getModelDir()));
		}
		boolean flag = false;
		FileNameExtensionFilter filter = new FileNameExtensionFilter("*.del",
				"del");
		fileChooser.setFileFilter(filter);

		int result = fileChooser.showOpenDialog(GlobeParam.getClient());
		if (result == JFileChooser.APPROVE_OPTION) {
			File file = fileChooser.getSelectedFile();
			// check the path
			String path = file.toString();
			boolean flag2 = path.endsWith(".del");
			if (flag2) {
				return addModelToPanel(path, drawPanel);
			} else {
				JOptionPane.showMessageDialog(GlobeParam.getClient(),
						"Please select the '*.del' file ");
			}
		} else {
			flag = false;
		}
		return flag;
	}

	/**
	 * Adds models to JDrawPanel.
	 * 
	 * @param path
	 *            the file path.
	 * @param drawPanel
	 *            the JDrawpanel
	 * @return
	 */
	public boolean addModelToPanel(String path, JDrawPanel drawPanel) {

		// Creates model container.
		PureModelStore modelStore = new OpenModel().open(path);
		String figurePath = path.replace(".del", ".gur");
		File figureFile = new File(figurePath);
		// Creates figure container.
		if (!figureFile.exists()) {
			JOptionPane
					.showMessageDialog(null,
							"Failed to open,because of lack of corresponding figure file ");
			return false;
		}
		if (modelStore == null) {
			JOptionPane.showMessageDialog(null, "Failed to open models.");
			return false;
		}
		FigureStore figureStore = new OpenFigure().open(figurePath, modelStore);
		String bindPath = path.replace(".del", ".bid");
		// Creates binding container.
		File bindFile = new File(bindPath);
		if (bindFile.exists()) {
			BindStore bindStore = BindTool.initBindStore(bindFile);
			if (bindStore != null) {
				drawPanel.setBindStore(bindStore);
			}
		}
		if (figureStore != null && modelStore != null) {
			drawPanel.setFigureStore(figureStore);
			FigureUtility figureUtility = FigureUtility.getInstance();
			figureUtility.adjustFigures(figureStore.getAllFigures());
			drawPanel.setModelStore(modelStore);
			drawPanel.setModelPath(path);
			figureUtility.reSizeDrawArea(drawPanel);
			drawPanel.repaint();
			return true;
		} else {
			JOptionPane.showMessageDialog(null, "Failed to open models.");
			return false;
		}

	}

	/**
	 * Clears the running information.
	 * 
	 * @param drawPanel
	 */
	public void clearResult(JDrawPanel drawPanel) {
		FigureStore figureStore = drawPanel.getFigureStore();

		List<AbstractFigure> figures = figureStore.getAllFigures();
		ProcessFigure processFigure;
		for (AbstractFigure figure : figures) {
			if (figure instanceof ProcessFigure) {
				processFigure = (ProcessFigure) figure;
				processFigure.setExecuteState(ExecutionState.READY);
			}
		}
	}

	/**
	 * Converts tif file to jpg file.
	 * 
	 * @param tifPath
	 *            The path of tif file.
	 * @return
	 */
	public static String tifToJpg(String tifPath) {
		int n = tifPath.lastIndexOf(".");
		String jpgPath = tifPath.substring(0, n) + ".jpg";
		RenderedOp src1 = JAI.create("fileload", tifPath);
		BufferedImage bufferedImage = src1.getAsBufferedImage();
		OutputStream os1;
		try {
			os1 = new FileOutputStream(jpgPath);
			JPEGImageEncoder enco = JPEGCodec.createJPEGEncoder(os1);
			enco.encode(bufferedImage);
			os1.close();
			System.out.println("Success!");
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
		return jpgPath;
	}
}
