/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.utils;

import cn.edu.whu.lmars.figure.beans.GeoMarkFigure;
import cn.edu.whu.lmars.figure.beans.GeoNodeFigure;
import cn.edu.whu.lmars.figure.beans.LinkFigure;

/**
 * Analyzes the positional relationship of mouse clicks and figures.
 * 
 * @author Mingda Zhang
 * 
 */
public class Judger {

	private static Judger instance = null;

	int x, y, width, height;

	/**
	 * Gets an instance of Judger.
	 * 
	 * @return
	 */
	public static Judger getInstance() {
		if (instance == null) {
			instance = new Judger();
		}
		return instance;
	}

	private Judger() {

	}

	/**
	 * Judges whether a click within a rectangle.
	 * 
	 * @param rect
	 *            rectangle
	 * @param x1
	 *            x coordinate value.
	 * @param y1
	 *            y coordinate value.
	 * @return
	 */
	public boolean rectPoint(GeoNodeFigure rect, int x1, int y1) {

		boolean flag = false;
		x = rect.getX();
		y = rect.getY();
		width = rect.getWidth();
		height = rect.getHeight();
		if ((x < x1) && (x + width > x1) && (y < y1) && (y + height > y1)) {
			flag = true;
		}
		return flag;
	}

	/**
	 * Judges whether the mouse pointer in the mark figure inside.
	 * 
	 * @param rect
	 *            mark figure.
	 * @param x1
	 *            x coordinate value.
	 * @param y1
	 *            y coordinate value.
	 * @return
	 */
	public boolean rectPoint(GeoMarkFigure rect, int x1, int y1) {
		boolean flag = false;
		x = rect.getX() - rect.getOffest();
		y = rect.getY() - rect.getOffest();
		width = rect.getWidth();
		height = rect.getWidth();
		if ((x < x1) && (x + width > x1) && (y < y1) && (y + height > y1)) {
			flag = true;
		}
		return flag;
	}

	/**
	 * Returns true if the mouse click one the line.
	 * 
	 * @param line
	 *            line.
	 * @param x
	 *            x coordinate value.
	 * @param y
	 *            y coordinate value.
	 * @return
	 */
	public boolean linePoint(LinkFigure line, int x, int y) {
		boolean flag = false;
		int bigX, smallX, bigY, smallY;
		int x1, y1, x2, y2;
		double a = 0, b = 0, c = 0, d = 0;

		x1 = line.getStartP().getX();
		y1 = line.getStartP().getY();
		x2 = line.getEndP().getX();
		y2 = line.getEndP().getY();

		smallX = x1 < x2 ? x1 : x2;
		bigX = x1 > x2 ? x1 : x2;
		smallY = y1 < y2 ? y1 : y2;
		bigY = y1 > y2 ? y1 : y2;

		if (x < smallX) {
			return false;
		}
		if (x > bigX) {
			return false;
		}
		if (y < smallY) {
			return false;
		}
		if (y > bigY) {
			return false;
		}

		if (x1 == x2) {
			d = Math.abs(y1 - y);
		} else {
			a = ((double) (y1 - y2)) / (x2 - x1);
			b = 1;
			c = ((double) (y2 - y1)) / (x2 - x1) * x1 - y1;
			d = Math.abs(a * x + b * y + c)
					/ Math.sqrt(Math.pow(a, 2) + Math.pow(b, 2));
		}
		if (d < 3) {
			flag = true;
		}
		return flag;
	}

}
