/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.utils;

/**
 * Matches strings using KMP matching algorithm.
 * 
 * @author Mingda Zhang
 * 
 */
public class KMPMatch {

	/**
	 * Gets roll back array.
	 * 
	 * @param pattern
	 * @return
	 */
	private static int[] getRollbackArray(char[] pattern) {
		int[] rollback = new int[pattern.length];
		for (int i = 0; i < pattern.length; i++) {
			rollback[i] = 0;
		}
		rollback[0] = -1;
		for (int i = 1; i < rollback.length; i++) {
			char prevChar = pattern[i - 1];
			int prevRollback = i - 1;
			while (prevRollback >= 0) {
				int previousRollBackIdx = rollback[prevRollback];
				if ((previousRollBackIdx == -1)
						|| (prevChar == pattern[previousRollBackIdx])) {
					rollback[i] = previousRollBackIdx + 1;
					break;
				} else {
					prevRollback = rollback[prevRollback];
				}
			}
		}
		return rollback;
	}

	/**
	 * Matches strings.
	 * 
	 * @param source
	 * 
	 * @param pattern
	 * @return
	 */
	public static int searchKMP(char[] source, char[] pattern) {
		if (source == null || source.length == 0 || pattern == null
				|| pattern.length == 0) {
			return -1;
		}
		// get the rollback array.
		int[] rollback = getRollbackArray(pattern);

		// incremental index of pattern. pointing the char to compare with.
		int currMatch = 0;
		int len = pattern.length;
		// i point the char to compare with
		for (int i = 0; i < source.length;) {
			// if current char match
			if ((currMatch == -1) || (source[i] == pattern[currMatch])) {
				/*
				 * then each of the indexes adding by one, moving to the next
				 * char for comparation. notice that if currMatch is -1, it
				 * means the first char in pattern can not be matched. so i add
				 * by one to move on. and currMatch add by one so its value is
				 * 0.
				 */
				i++;
				currMatch++;
				/*
				 * if reaches the end of pattern, then match success, return the
				 * index of first matched char.
				 */
				if (currMatch == len) {
					return i - len;
					// return i;
				}
			} else {
				/*
				 * if current char mismatch, then rollback the next char to
				 * compare in pattern.
				 */
				currMatch = rollback[currMatch];
			}
		}
		return -1;
	}

}
