/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.utils;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;

import org.apache.tools.zip.ZipEntry;
import org.apache.tools.zip.ZipFile;

/**
 * Unzips the file to the directory where the zip file with the same file name
 * 
 * @author Mingda Zhang
 * 
 */
public class UnZipFile {

	/** the unzip file */
	private String destDir = null;
	/** the zip file */
	private File targetFile;

	public UnZipFile(File zipfile) {
		this.targetFile = zipfile;
	}

	/**
	 * Executes unzip file.
	 * 
	 * @return
	 */
	@SuppressWarnings("rawtypes")
	public boolean execute() {
		// destDir = destDir.endsWith( "//" ) ? destDir : destDir + "//" ;
		byte b[] = new byte[1024];
		int length;
	//	String fileName;
		if (!targetFile.isFile()) {
			return false;
		} else {
			destDir = targetFile.getParent();
			/*fileName = targetFile.getName();
			fileName = fileName.substring(0, fileName.lastIndexOf("."));
			destDir += File.separator + fileName;*/
		//	File file = new File(destDir);
			/*int i = 0;
			while (file.exists()) {
				destDir += "_" + (i++);
				file = new File(destDir);
			}
			file.mkdir();*/
		}

		ZipFile zipFile;
		try {
			zipFile = new ZipFile(targetFile);
			Enumeration enumeration = zipFile.getEntries();
			ZipEntry zipEntry = null;

			while (enumeration.hasMoreElements()) {
				zipEntry = (ZipEntry) enumeration.nextElement();
				File loadFile = new File(destDir + File.separator
						+ zipEntry.getName());

				if (zipEntry.isDirectory()) {
					loadFile.mkdirs();
					destDir = loadFile.getAbsolutePath();
				} else {
					if (!loadFile.getParentFile().exists()){
						loadFile.getParentFile().mkdirs();
						destDir = loadFile.getParentFile().getAbsolutePath();
					}

					OutputStream outputStream = new FileOutputStream(loadFile);
					InputStream inputStream = zipFile.getInputStream(zipEntry);

					while ((length = inputStream.read(b)) > 0)
						outputStream.write(b, 0, length);

				}
			}
			System.out.println("Unzip file successfully.");
		} catch (IOException e) {
			e.printStackTrace();
			System.out.println(e.getMessage());
			return false;
		}
		return true;
	}

	/**
	 * Gets the unzip file directory.
	 * 
	 * @return
	 */
	public String getDestDir() {
		return destDir;
	}
}
