/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.utils;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;

import conf.PathConf;

/**
 * Operates workspace configuration file.
 * 
 * @author Mingda Zhang
 * 
 */

public class WorkSpaceXML {
	/** the path of workspace configuration file. */
	private String path = null;
	private Document document;
	private static WorkSpaceXML instance;

	private WorkSpaceXML() {
		
		File dir = new File(PathConf.getCurrentWorkingDirectory());
		if (!dir.exists()) {
			dir.mkdirs();
		}
		path = PathConf.getWorkspaceListPath();
		System.out.println(path);
		File file = new File(path);
		if(!file.exists()){
			try {
				file.createNewFile();
				Document doc = FileTool.getDocFromResource(PathConf.WorkSpacePath);
				Dom4jXMLTool.writeXml(doc, file);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				path = null;
			}
		}
	}

	public static WorkSpaceXML getInstance() {
		if (instance == null) {
			instance = new WorkSpaceXML();
		}
		return instance;
	}

	/**
	 * Gets all the workspace paths.
	 * 
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public String[] getPaths() {
		String[] items;
		document = Dom4jXMLTool.getDocFromFile(path);
		if (document == null) {
			return null;
		}
		Element root = document.getRootElement();
		List<Element> paths = root.selectNodes("//WS");
		int num = paths.size();
		if (num == 0) {
			return null;
		}
		items = new String[num];
		for (int i = 0; i < num; i++) {
			Element pathEle = paths.get(i);
			items[i] = pathEle.attributeValue("path");
		}
		return items;
	}

	/**
	 * Saves a new workspace path to configuration file.
	 * 
	 * @param newPath
	 */
	@SuppressWarnings("unchecked")
	public void savePath(String newPath) {
		if (document == null) {
			SAXReader reader = new SAXReader();
			try {
				document = reader.read(new File(path));
			} catch (DocumentException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				return;
			}
		}
		Element root = document.getRootElement();
		List<Element> nodes = root.selectNodes("//WS");
		for (Element node : nodes) {
			if (node.attributeValue("path").trim().equals(newPath)) {
				return;
			}
		}
		Element element = root.addElement("WS");
		element.addAttribute("path", newPath);
		writeXml(document);
	}

	/**
	 * Writes document to configuration file of workspace.
	 * 
	 * @param document
	 * @return
	 */
	public boolean writeXml(Document document) {
		boolean flag = false;
		try {
			XMLWriter output = new XMLWriter(new FileWriter(new File(path)));
			output.write(document);
			output.close();
			flag = true;
		} catch (Exception e) {
			e.printStackTrace();
			flag = false;
		}
		return flag;
	}

}
