/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.worldwind.balloon;

import gov.nasa.worldwind.View;
import gov.nasa.worldwind.avlist.AVKey;
import gov.nasa.worldwind.geom.Position;
import gov.nasa.worldwind.layers.RenderableLayer;
import gov.nasa.worldwind.render.AbstractBrowserBalloon;
import gov.nasa.worldwind.render.BalloonAttributes;
import gov.nasa.worldwind.render.BasicBalloonAttributes;
import gov.nasa.worldwind.render.GlobeBrowserBalloon;
import gov.nasa.worldwind.render.Material;
import gov.nasa.worldwind.render.PointPlacemark;
import gov.nasa.worldwind.render.PointPlacemarkAttributes;
import gov.nasa.worldwind.render.Size;
import gov.nasa.worldwindx.examples.util.BalloonController;
import gov.nasa.worldwindx.examples.util.HotSpotController;
import cn.edu.whu.lmars.sensorweb.sos.beans.SOSSensor;
import cn.edu.whu.lmars.sensorweb.sos.beans.SensorPosition;
import cn.edu.whu.lmars.sensorweb.sos.utils.SensorUtility;
import cn.edu.whu.lmars.worldwind.datatree.DataLayerInfo;
import cn.edu.whu.lmars.worldwind.ui.WWDPanel;
import cn.edu.whu.lmars.worldwind.utils.WWDUtility;

import conf.WWDConf;

/**
 * Adds a sensor balloon to the world wind.
 * 
 * @author Mingda Zhang
 * 
 */
public class SensorBalloon {
	private WWDPanel wwdPanel = null;
	@SuppressWarnings("unused")
	private HotSpotController hotSpotController;
	@SuppressWarnings("unused")
	private BalloonController balloonController;

	public SensorBalloon(WWDPanel wwdPanel) {
		this.wwdPanel = wwdPanel;

	}

	/**
	 * Adds SOS procedure to the world wind.
	 * 
	 * @param sensorBean
	 */
	public void add(SOSSensor sensorBean) {

		String htmlString = SensorUtility.sensorToHtml(sensorBean);
		SensorPosition sensorPosition = sensorBean.getSensorPosition();
		if (sensorPosition == null) {
			return;
		}
		Position balloonPosition = Position.fromDegrees(
				sensorPosition.getNorthing(), sensorPosition.getEasting());

		AbstractBrowserBalloon balloon = new GlobeBrowserBalloon(htmlString,
				balloonPosition);
		BalloonAttributes attrs = new BasicBalloonAttributes();
		attrs.setSize(new Size(Size.NATIVE_DIMENSION, 0d, null,
				Size.NATIVE_DIMENSION, 0d, null));
		balloon.setAttributes(attrs);

		// Create a placemark on the globe that the user can click to open the
		// balloon.
		PointPlacemark placemark = new PointPlacemark(balloonPosition);
		placemark.setValue(AVKey.BALLOON, balloon);
		PointPlacemarkAttributes attr = new PointPlacemarkAttributes();
		attr.setLabelColor("ffff0000");
		attr.setLineMaterial(Material.MAGENTA);
		attr.setLineWidth(2d);
		attr.setUsePointAsDefaultImage(true);
		attr.setScale(10d);
		placemark.setAttributes(attr);
		// Create a layer to display the balloons.
		RenderableLayer layer = new RenderableLayer();
		layer.setName(sensorBean.getProcedure());
		layer.addRenderable(balloon);
		layer.addRenderable(placemark);
		DataLayerInfo dataNode = new DataLayerInfo();
		dataNode.setName(layer.getName());
		WWDUtility.insertDataLayer(this.wwdPanel.getWwd(), layer, dataNode);
		View view = wwdPanel.getWwd().getView();
		view.goTo(balloonPosition, WWDConf.VIEW_ELEVATION);
		hotSpotController = new HotSpotController(this.wwdPanel.getWwd());
		balloonController = new BalloonController(this.wwdPanel.getWwd());
	}
}
