/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.worldwind.datatree;

import gov.nasa.worldwind.WorldWindow;
import gov.nasa.worldwind.layers.Layer;

import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import conf.GlobeParam;

import cn.edu.whu.lmars.utils.BaseTool;
import cn.edu.whu.lmars.worldwind.ui.OpacityController;
import cn.edu.whu.lmars.worldwind.utils.WWDUtility;

/**
 * A panel that contains layers added to the WorldWindow. Controls these data
 * layers.
 * 
 * @author Mingda Zhang
 * 
 */
@SuppressWarnings("serial")
public class DataLayerPanel extends JScrollPane implements MouseListener,
		TreeSelectionListener, ActionListener {
	/** data tree */
	private JTree jTree;
	/** tree root */
	private DefaultMutableTreeNode root;
	/** tree model */
	private DefaultTreeModel treeModel;
	private CheckTreeSelectionModel selectionModel = null;
	int hotspot = new JCheckBox().getPreferredSize().width;
	/** tree JPopupMenu */
	private JPopupMenu popupMenu = null;

	public DataLayerPanel() {
		initlize();
	}

	/**
	 * Initializes this panel.
	 */
	private void initlize() {
		this.setBorder(BorderFactory.createTitledBorder("Data Layers"));
		initTree();
		this.getViewport().add(jTree);
		// this.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
	}

	/**
	 * Initializes the tree.
	 */
	private void initTree() {
		DataLayerInfo dataNode = new DataLayerInfo();
		dataNode.setName("Added Data");
		root = new DataTreeNode(dataNode, null);
		jTree = new JTree(root);
		selectionModel = new CheckTreeSelectionModel(jTree.getModel());

		jTree.setRootVisible(true);

		DefaultTreeCellRenderer treeCellRenderer = new DefaultTreeCellRenderer();
		treeCellRenderer.setIcon(null);
		treeCellRenderer.setOpenIcon(null);
		treeCellRenderer.setLeafIcon(null);
		treeCellRenderer.setClosedIcon(null);
		treeCellRenderer.setBackground(SystemColor.control);
		treeCellRenderer.setBackgroundNonSelectionColor(SystemColor.control);
		jTree.setCellRenderer(new CheckTreeCellRenderer(treeCellRenderer,
				selectionModel));
		jTree.addMouseListener(this);
		jTree.setBackground(SystemColor.control);
		jTree.setRowHeight(30);
		jTree.setEditable(false);
		jTree.setDragEnabled(true);
		selectionModel.addTreeSelectionListener(this);
		treeModel = (DefaultTreeModel) jTree.getModel();
		jTree.setVisible(true);
	}

	/**
	 * Adds tree node to the data tree.
	 * 
	 * @param node
	 */
	public void addNode(DataTreeNode node) {
		if (treeModel == null) {
			treeModel = (DefaultTreeModel) jTree.getModel();
		}
		treeModel.insertNodeInto(node, root, root.getChildCount());
		TreeNode[] nodes = treeModel.getPathToRoot(node);
		TreePath path = new TreePath(nodes);
		jTree.setSelectionPath(path);
		jTree.scrollPathToVisible(path);
	}

	/**
	 * Deletes tree node from the data tree.
	 * 
	 * @param node
	 */
	public void deleteNode(DataTreeNode node) {
		if (treeModel == null) {
			treeModel = (DefaultTreeModel) jTree.getModel();
		}
		treeModel.removeNodeFromParent(node);
	}

	/**
	 * Selects or deselects a tree node.
	 */
	@Override
	public void mouseClicked(MouseEvent me) {
		// TODO Auto-generated method stub
		TreePath path = jTree.getPathForLocation(me.getX(), me.getY());
		if (path == null)
			return;
		if (me.getX() > jTree.getPathBounds(path).x + hotspot)
			return;

		boolean selected = selectionModel.isPathSelected(path, true);
		selectionModel.removeTreeSelectionListener(this);

		try {
			if (selected)
				selectionModel.removeSelectionPath(path);
			else
				selectionModel.addSelectionPath(path);
		} finally {
			selectionModel.addTreeSelectionListener(this);
			jTree.treeDidChange();
		}

	}

	/**
	 * Gets tree selection model.
	 */
	@Override
	public void valueChanged(TreeSelectionEvent e) {
		jTree.treeDidChange();
	}

	/**
	 * 
	 */
	@Override
	public void mousePressed(MouseEvent e) {
		if (popupMenu == null) {
			popupMenu = createJPopupMenu();
		}
		//Shows JPopuMenu.
		if (e.getButton() == 3) {
			popupMenu.show(jTree, e.getX(), e.getY());
		}
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	/**
	 * Creates JPopupMenu.
	 * 
	 * @return JPopupMenu
	 */
	private JPopupMenu createJPopupMenu() {
		popupMenu = new JPopupMenu();
		popupMenu.add(createItem("delete", "delete"));
		popupMenu.add(createItem("rename", "rename"));
		popupMenu.add(createItem("opacity controller", "opacity"));
		popupMenu.add(createItem("provenance", "provenance"));
		return popupMenu;
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	@Override
	public void mouseExited(MouseEvent e) {

	}

	/**
	 * Creates item for the JPopupMenu.
	 * 
	 * @param name
	 *            the name of item.
	 * @param command
	 *            the action command of the item.
	 * @return JMenuItem
	 */
	private JMenuItem createItem(String name, String command) {
		JMenuItem item = new JMenuItem(name);
		item.setActionCommand(command);
		item.addActionListener(this);
		return item;
	}

	public DataTreeNode contains(String path){
		for(int i=0;i< root.getChildCount();i++){
			DataTreeNode treeNode = (DataTreeNode)root.getChildAt(i);
			DataLayerInfo dataInfo = treeNode.getDataNode();
			if (path.equals(dataInfo.getPath())) {
				return treeNode;
			}
		}
		return null;
	}
	/**
	 * Handles the action of JPopupMenu.
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		String str = e.getActionCommand();
		Object obj = jTree.getLastSelectedPathComponent();
		if (!(obj instanceof DataTreeNode)) {
			return;
		}
		DataTreeNode treeNode = (DataTreeNode) obj;
		if (treeNode == root) {
			return;
		}
		// Deletes data layer.
		if (str.equals("delete")) {
			int flag = JOptionPane.showConfirmDialog(null, "Sure to delete?");
			if (flag == JOptionPane.OK_OPTION) {
				deleteNode(treeNode);
				Layer layer = treeNode.getLayer();
				if (layer != null) {
					WWDUtility.deleteLayer(layer, GlobeParam.getWwPanel()
							.getWwd());
				}
			}
			// Renames data layer.
		} else if (str.equals("rename")) {
			String name = JOptionPane.showInputDialog(null,
					"Input a new name please");
			if (!BaseTool.isNull(name)) {
				treeNode.setUserObject(name);
				treeNode.getDataNode().setName(name);
			}
			// Shows opacity controller for the data layer.
		} else if (str.equals("opacity")) {
			Layer layer = treeNode.getLayer();
			if (layer == null) {
				return;
			}
			WorldWindow wwd = GlobeParam.getWwPanel().getWwd();
			OpacityController controller = new OpacityController(
					GlobeParam.getClient(), layer, wwd);
			controller.setVisible(true);
			// Views the provenance of the data in modeling workspace.
		} else if (str.equals("provenance")) {
			DataLayerInfo dataNode = treeNode.getDataNode();
			WWDUtility.viewProv(dataNode);
		}
	}

	public JTree getjTree() {
		return jTree;
	}

}
