/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.worldwind.datatree;

import gov.nasa.worldwind.layers.Layer;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JCheckBox;
import javax.swing.tree.DefaultMutableTreeNode;

/**
 * Data tree node.
 * 
 * @author Mingda Zhang
 * 
 */
@SuppressWarnings("serial")
public class DataTreeNode extends DefaultMutableTreeNode {
	/** JCheckBox shows whether the data layer is selected */
	JCheckBox checkBox = null;
	/** Data layer */
	private Layer layer;
	/** Data node */
	private DataLayerInfo dataNode;

	public DataTreeNode(DataLayerInfo dataNode, Layer layer) {
		super(dataNode.getName());
		checkBox = new JCheckBox();
		this.layer = layer;
		this.dataNode = dataNode;
		checkBox.addItemListener(new CheckBoxListener());
	}

	/**
	 * Returns the JCheckBox in the tree node.
	 * 
	 * @return
	 */
	public JCheckBox getCheckBox() {
		return checkBox;
	}

	/**
	 * Listens for JCheckBox change.
	 * 
	 * @author Mingda Zhang
	 * 
	 */
	private class CheckBoxListener implements ItemListener {

		@Override
		public void itemStateChanged(ItemEvent e) {
			if (e.getSource() == checkBox) {
				if (checkBox.isSelected()) {
					if (layer != null) {
						layer.setEnabled(true);
					}
				} else {
					if (layer != null) {
						layer.setEnabled(false);
					}
				}
			}

		}

	}

	/**
	 * Gets the data layer.
	 * 
	 * @return
	 */
	public Layer getLayer() {
		return layer;
	}

	/**
	 * Gets data node.
	 * 
	 * @return
	 */
	public DataLayerInfo getDataNode() {
		return dataNode;
	}

	/**
	 * Sets data node.
	 * 
	 * @param dataNode
	 */
	public void setDataNode(DataLayerInfo dataNode) {
		this.dataNode = dataNode;
	}

}
