/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.worldwind.formats;

import java.io.File;
import java.util.Properties;

import cn.edu.whu.lmars.utils.FileTool;
import cn.edu.whu.lmars.utils.UnZipFile;

import conf.FileLoadProperty;

/**
 * Creates file loader according the type of a file.
 * 
 * @author Mingda Zhang
 * 
 */
public class FileLoadFactory {

	/**
	 * Returns file loader to loader the file.
	 * 
	 * @param file
	 *            file to be loaded.
	 * @return
	 */
	public static IFileLoad create(File file) {

		IFileLoad fileLoad = null;
		String suffix = FileTool.getFileSuffix(file);

		if (suffix.equals("zip")) {
			file = getLegalFile(file);
			suffix = FileTool.getFileSuffix(file);
		}
		if (file == null) {
			return null;
		}
		Properties properties = FileLoadProperty.getInstance()
				.getClassNameProperty();
		String className = properties.getProperty(suffix);

		try {
			fileLoad = (IFileLoad) Class.forName(className).newInstance();
			fileLoad.setFile(file);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}

		return fileLoad;
	}

	/**
	 * If file is a zip fomat, unzip this file.
	 * 
	 * @param zipFile
	 * @return
	 */
	private static File getLegalFile(File zipFile) {

		UnZipFile unZipFile = new UnZipFile(zipFile);
		boolean flag = unZipFile.execute();
		if (!flag) {
			return null;
		}
		String baseDir = unZipFile.getDestDir();
		return FileTool.getAssignSuffixFile(new File(baseDir));
	}
}
