/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.worldwind.formats;

import gov.nasa.worldwind.WorldWindow;
import gov.nasa.worldwind.layers.Layer;
import gov.nasa.worldwind.layers.RenderableLayer;
import gov.nasa.worldwind.ogc.kml.KMLAbstractFeature;
import gov.nasa.worldwind.ogc.kml.KMLAbstractView;
import gov.nasa.worldwind.ogc.kml.KMLRoot;
import gov.nasa.worldwind.ogc.kml.impl.KMLController;

import java.awt.Component;
import java.awt.Cursor;
import java.io.File;

import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

import conf.GlobeParam;

import cn.edu.whu.lmars.utils.BaseTool;
import cn.edu.whu.lmars.worldwind.datatree.DataLayerInfo;
import cn.edu.whu.lmars.worldwind.ui.WWDPanel;
import cn.edu.whu.lmars.worldwind.utils.WWDUtility;

/**
 * Loads kml file to world wind.
 * 
 * @author Mingda Zhang
 * 
 */
public class KMLFileLoad implements IFileLoad {

	private File file;
	private WWDPanel wwdPanel;
	private WorldWindow wwd;
	private String display;
	private KMLAbstractFeature abstractFeature;
	private DataLayerInfo dataNode;

	@Override
	public void setFile(File file) {
		this.file = file;
	}

	@Override
	public boolean execute() {

		try {
			final Layer layer = this.getKmlLayer();
			layer.setPickEnabled(true);

			// final View view = wwd.getView();
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					WWDUtility.insertDataLayer(wwdPanel.getWwd(), layer,
							getDataNode());
					if (abstractFeature != null) {
						KMLAbstractView kmlView = abstractFeature.getView();
						System.out.println(kmlView);
					}
					/*
					 * if (abstractFeature!=null && viewController!=null) {
					 * viewController
					 * .setObjectsToTrack(abstractFeature.getView()); }
					 */
					// wwdPanel.getLayers().add(layer);
					// wwdPanel.getLayerPanel().update(wwd);
					// view.goTo(box.get, WWDConf.VIEW_ELEVATION);

				}
			});
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					((Component) wwd).setCursor(Cursor.getDefaultCursor());
				}
			});
		}
		return true;
	}

	@Override
	public void setWWDPanel(WWDPanel wwdPanel) {
		this.wwdPanel = wwdPanel;
		this.wwd = wwdPanel.getWwd();
	}

	private Layer getKmlLayer() {
		KMLRoot kmlRoot = null;
		try {
			kmlRoot = KMLRoot.createAndParse(file);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
		display = WWDUtility.makeDisplayName(this.file, "kml layer");
		String name = JOptionPane.showInputDialog(GlobeParam.getClient(),
				"Input a name for this layer", display);
		if (!BaseTool.isNull(name)) {
			display = name;
		}
		getDataNode().setName(display);
		KMLController kmlController = new KMLController(kmlRoot);
		RenderableLayer layer = new RenderableLayer();
		layer.setName(display);
		layer.addRenderable(kmlController);
		abstractFeature = kmlRoot.getFeature();
		return layer;
	}

	@Override
	public void setDataNode(DataLayerInfo dataNode) {
		this.dataNode = dataNode;
	}

	@Override
	public DataLayerInfo getDataNode() {
		return dataNode;
	}

}
