/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.worldwind.formats;

import gov.nasa.worldwind.View;
import gov.nasa.worldwind.WorldWindow;
import gov.nasa.worldwind.formats.shapefile.Shapefile;
import gov.nasa.worldwind.geom.Position;
import gov.nasa.worldwind.layers.Layer;
import gov.nasa.worldwindx.examples.util.OpenStreetMapShapefileLoader;
import gov.nasa.worldwindx.examples.util.ShapefileLoader;

import java.awt.Component;
import java.awt.Cursor;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

import cn.edu.whu.lmars.utils.BaseTool;
import cn.edu.whu.lmars.worldwind.datatree.DataLayerInfo;
import cn.edu.whu.lmars.worldwind.ui.WWDPanel;
import cn.edu.whu.lmars.worldwind.utils.WWDUtility;

import conf.GlobeParam;
import conf.WWDConf;

/**
 * Loads ESRI Shapefile to world wind.
 * 
 * @author Mingda Zhang
 * 
 */
public class ShapeFileLoad implements IFileLoad {

	private File file;
	// private WWDPanel wwdPanel;
	private WorldWindow wwd;
	private double[] rectangle;
	private DataLayerInfo dataNode;

	@Override
	public void setFile(File file) {
		this.file = file;
	}

	@Override
	public boolean execute() {

		try {
			final List<Layer> layers = this.makeShapefileLayers();
			for (int i = 0; i < layers.size(); i++) {
				String name = WWDUtility
						.makeDisplayName(this.file, "shapefile");
				String name2 = JOptionPane.showInputDialog(
						GlobeParam.getClient(), "Input a for this layer", name);
				if (!BaseTool.isNull(name2)) {
					name = name2;
				}
				getDataNode().setName(name);
				layers.get(i).setName(
						i == 0 ? name : name + "-" + Integer.toString(i));
				layers.get(i).setPickEnabled(true);
			}

			final View view = wwd.getView();
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					for (Layer layer : layers) {
						WWDUtility.insertDataLayer(wwd, layer, getDataNode());
					}

					if (rectangle != null && rectangle.length == 4) {
						double lat = (rectangle[0] + rectangle[1]) / 2;
						double lnt = (rectangle[2] + rectangle[3]) / 2;
						Position position = Position.fromDegrees(lat, lnt);
						view.goTo(position, WWDConf.VIEW_ELEVATION);

					}
				}
			});
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					((Component) wwd).setCursor(Cursor.getDefaultCursor());
				}
			});
		}
		return true;
	}

	@Override
	public void setWWDPanel(WWDPanel wwdPanel) {
		// this.wwdPanel = wwdPanel;
		this.wwd = wwdPanel.getWwd();
	}

	private List<Layer> makeShapefileLayers() {

		Shapefile shp = new Shapefile(this.file);
		rectangle = shp.getBoundingRectangle();
		List<Layer> layers = new ArrayList<Layer>();

		if (OpenStreetMapShapefileLoader.isOSMPlacesSource(this.file)) {
			Layer layer = OpenStreetMapShapefileLoader
					.makeLayerFromOSMPlacesShapefile(shp);

			layers.add(layer);
		} else {
			ShapefileLoader loader = new ShapefileLoader();
			layers.add(loader.createLayerFromShapefile(shp));
		}
		if (shp != null) {
			shp.close();
		}
		return layers;
	}

	@Override
	public void setDataNode(DataLayerInfo dataNode) {
		this.dataNode = dataNode;
	}

	@Override
	public DataLayerInfo getDataNode() {
		return dataNode;
	}

}
