/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.worldwind.formats;

import gov.nasa.worldwind.WorldWind;
import gov.nasa.worldwind.WorldWindow;
import gov.nasa.worldwind.avlist.AVKey;
import gov.nasa.worldwind.avlist.AVList;
import gov.nasa.worldwind.data.DataRaster;
import gov.nasa.worldwind.data.DataRasterReader;
import gov.nasa.worldwind.data.DataRasterReaderFactory;
import gov.nasa.worldwind.geom.Sector;
import gov.nasa.worldwind.layers.SurfaceImageLayer;
import gov.nasa.worldwind.render.SurfaceImage;
import gov.nasa.worldwind.util.Logging;
import gov.nasa.worldwindx.examples.util.ExampleUtil;

import java.io.File;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import conf.GlobeParam;

import cn.edu.whu.lmars.utils.BaseTool;
import cn.edu.whu.lmars.utils.HandleTool;
import cn.edu.whu.lmars.worldwind.datatree.DataLayerInfo;
import cn.edu.whu.lmars.worldwind.ui.WWDPanel;
import cn.edu.whu.lmars.worldwind.utils.WWDUtility;

/**
 * Loads tif file to world Wind.
 * 
 * @author Mingda Zhang
 * 
 */
public class TifImageView implements IFileLoad {

	private SurfaceImageLayer layer;
	private Logger logger = Logger.getLogger(TifImageView.class);
	private File file;
	private WorldWindow wwd;
	private DataLayerInfo dataNode;

	/**
	 * Initializes the image layer.
	 */
	private void initImageLayer() {
		try {
			this.layer = new SurfaceImageLayer();
			this.layer.setOpacity(1);
			this.layer.setPickEnabled(true);
			String name = WWDUtility.makeDisplayName(this.file, "tif文件");
			String name2 = JOptionPane.showInputDialog(GlobeParam.getClient(),
					"Input a name for this layer", name);
			if (!BaseTool.isNull(name2)) {
				name = name2;
			}
			getDataNode().setName(name);
			this.layer.setName(name);
			WWDUtility.insertDataLayer(wwd, layer, getDataNode());
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("Failed to add image layer");
		}
	}

	@Override
	public void setFile(File file) {
		this.file = file;
	}

	@Override
	public boolean execute() {
		initImageLayer();
		DataRasterReaderFactory readerFactory = (DataRasterReaderFactory) WorldWind
				.createConfigurationComponent(AVKey.DATA_RASTER_READER_FACTORY_CLASS_NAME);
		DataRasterReader reader = readerFactory.findReaderFor(file, null);
		logger.info("Begin to load " + file.getAbsolutePath() + " to worldwind");
		try {
			AVList metadata = reader.readMetadata(file, null);
			if (metadata == null) {
				logger.error("Failed to get metadata");
				return false;
			}
			DataRaster[] rasters = reader.read(file, null);
			if (rasters == null || rasters.length == 0) {
				logger.error("Failed to read rasters");
			}
			final Sector sector = (Sector) rasters[0].getValue(AVKey.SECTOR);
			if (sector == null) {
				String msg = Logging.getMessage("DataRaster.MissingMetadata",
						AVKey.SECTOR);
				logger.error(msg);
			}

			String tifType = metadata.getStringValue(AVKey.PIXEL_FORMAT);
			logger.info("the type of tif is " + tifType);

			Thread t = new Thread(new Runnable() {
				public void run() {
					try {
						try {
							layer.addImage(file.getAbsolutePath());
						} catch (Exception e) {
							// TODO: handle exception
							// if failed to load the image directly, converts it
							// to jpg.
							String jpgPath = HandleTool.tifToJpg(file.getPath());
							SurfaceImage image = new SurfaceImage(jpgPath,
									sector);
							layer.addRenderable(image);
						} finally {
							wwd.redraw();
							ExampleUtil.goTo(wwd, sector);
						}

					} catch (Exception e) {
						logger.error(e.getMessage());
						e.printStackTrace();
					}
				}
			});
			t.start();

		} catch (Exception e) {
		}
		return true;
	}

	@Override
	public void setWWDPanel(WWDPanel wwdPanel) {
		this.wwd = wwdPanel.getWwd();
	}

	@Override
	public void setDataNode(DataLayerInfo dataNode) {
		this.dataNode = dataNode;
	}

	@Override
	public DataLayerInfo getDataNode() {
		return dataNode;
	}

}
