/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.worldwind.ui;

import gov.nasa.worldwind.WorldWindow;
import gov.nasa.worldwind.layers.Layer;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.Frame;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.WindowConstants;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Opacity Controller for data layer.
 * 
 * @author Mingda Zhang
 * 
 */
public class OpacityController extends JDialog {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private Layer layer = null;
	private JSlider opacitySlider = null;
	private WorldWindow wwd = null;

	/**
	 * @param owner
	 */
	public OpacityController(Frame owner, Layer layer, WorldWindow wwd) {
		super(owner);
		this.layer = layer;
		this.wwd = wwd;
		initialize();
	}

	/**
	 * This method initializes this dialog.
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(300, 96);
		this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		this.setTitle("Opacity Controller");
		this.setFont(new Font("Dialog", Font.PLAIN, 14));
		this.setContentPane(getJContentPane());
		this.setLocationRelativeTo(null);
		this.setModal(false);
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
		}
		jContentPane.add(getOpacityPanel(), BorderLayout.CENTER);
		return jContentPane;
	}

	/**
	 * 
	 * @return Opacity Slider to control the opacity of data layer.
	 */
	private JPanel getOpacityPanel() {
		this.opacitySlider = new JSlider();
		this.opacitySlider.setMaximum(100);
		this.opacitySlider.setValue((int) (layer.getOpacity() * 100));
		this.opacitySlider.setEnabled(true);
		this.opacitySlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				int value = opacitySlider.getValue();
				layer.setOpacity(value / 100d);
				wwd.redraw();
			}
		});
		JPanel opacityPanel = new JPanel(new BorderLayout(5, 5));
		opacityPanel.setBorder(new EmptyBorder(0, 10, 0, 0));
		opacityPanel.add(new JLabel("Opacity"), BorderLayout.WEST);
		opacityPanel.add(this.opacitySlider, BorderLayout.CENTER);
		return opacityPanel;
	}

}
