/****************************************************************************
 * Copyright (C) 2013
 * by State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 
 * Contact: Peng Yue
 * State Key Laboratory of Information Engineering in Surveying, Mapping and Remote Sensing(LIESMARS), Wuhan University
 * 129 Luoyu Road, Wuhan, Hubei, China, 430079
 * pyue@whu.edu.cn
 *
 * This program is free software; you can redistribute and/or modify it under 
 * the terms of the GNU General Public License version 2 as published by the 
 * Free Software Foundation.
 * Free Software Foundation web page, http://www.fsf.org.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * 
 * Authors: Mingda Zhang, Peng Yue
 * Created: 24.05.2013
 *****************************************************************************/
package cn.edu.whu.lmars.worldwind.ui;

import gov.nasa.worldwind.Model;
import gov.nasa.worldwind.WorldWind;
import gov.nasa.worldwind.avlist.AVKey;
import gov.nasa.worldwind.awt.WorldWindowGLCanvas;
import gov.nasa.worldwind.event.RenderingExceptionListener;
import gov.nasa.worldwind.event.SelectEvent;
import gov.nasa.worldwind.exception.WWAbsentRequirementException;
import gov.nasa.worldwind.layers.ViewControlsLayer;
import gov.nasa.worldwind.layers.ViewControlsSelectListener;
import gov.nasa.worldwind.layers.WorldMapLayer;
import gov.nasa.worldwind.util.StatusBar;
import gov.nasa.worldwindx.examples.ClickAndGoSelectListener;
import gov.nasa.worldwindx.examples.LayerPanel;
import gov.nasa.worldwindx.examples.util.HighlightController;
import gov.nasa.worldwindx.examples.util.ToolTipController;

import java.awt.BorderLayout;
import java.awt.Dimension;

import javax.swing.JOptionPane;
import javax.swing.JPanel;

import cn.edu.whu.lmars.worldwind.utils.WWDUtility;

/**
 * Gets a panel contains World Window.
 * 
 * @author Mingda Zhang
 * 
 */
public class WWDPanel extends JPanel {

	private static final long serialVersionUID = 1L;
	protected WorldWindowGLCanvas wwd;
	protected StatusBar statusBar;
	protected ToolTipController toolTipController;
	protected HighlightController highlightController;
	// private ArrayList<Layer> layers = new ArrayList<Layer>();
	private LayerPanel layerPanel;

	/**
	 * Instantiates a WWDPanel.
	 * 
	 * @param canvasSize
	 *            the size of world window
	 * @param includeStatusBar
	 *            whether to include a status bar.
	 */
	public WWDPanel(Dimension canvasSize, boolean includeStatusBar) {
		super(new BorderLayout());

		this.wwd = this.createWorldWindow();
		this.wwd.setPreferredSize(canvasSize);

		// Create the default model as described in the current worldwind
		// properties.
		Model m = (Model) WorldWind
				.createConfigurationComponent(AVKey.MODEL_CLASS_NAME);
		this.wwd.setModel(m);

		// Setup a select listener for the worldmap click-and-go feature
		this.wwd.addSelectListener(new ClickAndGoSelectListener(this.getWwd(),
				WorldMapLayer.class));

		this.add(this.wwd, BorderLayout.CENTER);
		layerPanel = new LayerPanel(wwd);
		if (includeStatusBar) {
			this.statusBar = new StatusBar();
			this.add(statusBar, BorderLayout.PAGE_END);
			this.statusBar.setEventSource(wwd);
		}

		// Add controllers to manage highlighting and tool tips.
		this.toolTipController = new ToolTipController(this.getWwd(),
				AVKey.DISPLAY_NAME, null);
		this.highlightController = new HighlightController(this.getWwd(),
				SelectEvent.ROLLOVER);

		ViewControlsLayer viewControlsLayer = new ViewControlsLayer();
		WWDUtility.insertBeforePlacenames(getWwd(), viewControlsLayer);
		this.getWwd()
				.addSelectListener(
						new ViewControlsSelectListener(this.getWwd(),
								viewControlsLayer));

		this.getWwd().addRenderingExceptionListener(
				new RenderingExceptionListener() {
					public void exceptionThrown(Throwable t) {
						if (t instanceof WWAbsentRequirementException) {
							String message = "Computer does not meet minimum graphics requirements.\n";
							message += "Please install up-to-date graphics driver and try again.\n";
							message += "Reason: " + t.getMessage() + "\n";
							message += "This program will end when you press OK.";

							JOptionPane.showMessageDialog(null, message,
									"Unable to Start Program",
									JOptionPane.ERROR_MESSAGE);
						}
					}
				});
	}

	/**
	 * Creates WorldWindow.
	 * 
	 * @return
	 */
	protected WorldWindowGLCanvas createWorldWindow() {
		return new WorldWindowGLCanvas();
	}

	/**
	 * Gets WorldWindow.
	 * 
	 * @return
	 */
	public WorldWindowGLCanvas getWwd() {
		return wwd;
	}

	/**
	 * Gets status bar.
	 * 
	 * @return
	 */
	public StatusBar getStatusBar() {
		return statusBar;
	}

	/**
	 * Gets layer panel of this world wind.
	 * 
	 * @return
	 */
	public LayerPanel getLayerPanel() {
		return layerPanel;
	}


}
